#include "UtsusemiWiringInfoEditorNeunet.hh"
//////////////////////////////////////////////////////////
UtsusemiWiringInfoEditorNeunet::UtsusemiWiringInfoEditorNeunet(){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiWiringInfoEditorNeunet::~UtsusemiWiringInfoEditorNeunet(){
}
//////////////////////////////////////////////////////////
void UtsusemiWiringInfoEditorNeunet::Initialize(){
    _MessageTag = "UtsusemiWiringInfoEditorNeunet::";
}
//////////////////////////////////////////////////////////
bool UtsusemiWiringInfoEditorNeunet::
SetDetParam( std::string params ){
    if (this->_WE==NULL){
        UtsusemiError(_MessageTag+"SetDetParam >> SetRunNo must be done at first.");
        return false;
    }

    if (this->_WE->SetDetParam( params )){
        _DetType = _WE->_detType;
        _withPH = _WE->_withPH;
        return true;
    }else{
        return false;
    }
}
