#include <string>
#include "AdvOperationType.hh"

const AdvOperationType UNKNOWN_OPERATION      = { UNKNOWN_OPERATION_ID,      string("unknown operation"),      };
const AdvOperationType SMOOTHING              = { SMOOTHING_ID,              string("smoothing"),              };
const AdvOperationType PEAK_SEARCH            = { PEAK_SEARCH_ID,            string("peak search"),            };
const AdvOperationType PEAK_FIT               = { PEAK_FIT_ID,            string("peak fit"),               };
const AdvOperationType MULTI_DATA_PEAK_FIT    = { MULTI_DATA_PEAK_FIT_ID,    string("multi data peak fit"),    };
const AdvOperationType BACK_GROUND_ESTIMATION = { BACK_GROUND_ESTIMATION_ID, string("back ground estimation"), };

