#include "UtsusemiRedisPubSubEventMonitorRPMT.hh"
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorRPMT::
UtsusemiRedisPubSubEventMonitorRPMT(){
    _MessageTag2 = "UtsusemiRedisPubSubEventMonitorRPMT::";
}
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorRPMT::
~UtsusemiRedisPubSubEventMonitorRPMT(){
}
//////////////////////////////////////////////////////////
void UtsusemiRedisPubSubEventMonitorRPMT::
OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId ){
    _EDC->SetElementContainerReadout( pixelId, ec, caseId );
    SetMeasTimeSliced( ec->PutHeaderPointer() );
    SetKickers( ec->PutHeaderPointer(), caseId );
}
