#!/usr/bin/python
import Manyo as m

API = m.AcquireNeutronSourceImageInformation()

API.SetSavePath("./Image")

API.GetImageInformation("BP")
API.GetImageInformation("CT01")
API.GetImageInformation("H2")
API.GetImageInformation("JP")
API.GetImageInformation("ST")
API.GetImageInformation("CT3H")
API.GetImageInformation("CT1D")

#API.GetImageInformation("CT01","2009/06/02 23:00:00","2009/06/03 01:00:00")
#API.GetImageInformation("CT01","2009/09/06 23:00:00")

#API.GetImageInformation("H2","2009/05/31 23:00:00","2009/06/01 01:00:00")
#API.GetImageInformation("H2","2009/09/06 23:00:00")
