#include "UtsusemiDetectorInfoEditorBase.hh"
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorBase::
UtsusemiDetectorInfoEditorBase(){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorBase::UtsusemiDetectorInfoEditorBase(std::string dfile, bool workAsReader):UtsusemiDetectorInfoEditorTemplate(dfile, workAsReader){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorBase::
~UtsusemiDetectorInfoEditorBase(){
}
//////////////////////////////////////////////////////////
void UtsusemiDetectorInfoEditorBase::
Initialize(){
    _MessageTag = "UtsusemiDetectorInfoEditorBase::";

}
