#ifndef KIKFCORRECTIONCHOPPER
#define KIKFCORRECTIONCHOPPER

#include "UtsusemiHeader.hh"

//////////////////////////////////
// kikf Correction
/////////////////////////////////

//! Function to correct ki/kf term 
/*! 
 * 
 * 
 *  
 * 
 */
class KiKfCorrection :
  public ManipulatorBase< ElementContainerMatrix >
{
private:

public:
    KiKfCorrection();
        //!< Constructor
        /*!<
         */
    KiKfCorrection(ElementContainerMatrix *ecm);
        //!< Constructor
        /*!<
         *  @param ecm ElementContainerMatrix for the target
         */
    ~KiKfCorrection();
        //!< Destructor
        /*!<
         */
    void KiKfCorrect();
        //!< executes correction.
        /*!< 
         *   @return None
         */
};
#endif
