#ifndef ECMSUBTRACTION
#define ECMSUBTRACTION

#include "OperatorBase.hh"
#include "ElementContainerMatrix.hh"
#include "SearchInHeader.hh"

////////////////////////////////////////
// ECM subtraction
////////////////////////////////////////

//! Function to subtract ElementContainerMatrix
/*!
 *
 *
 *
 */
class EcmSubtraction:
public OperatorBase< ElementContainerMatrix, ElementContainerMatrix >{
private:

protected:
    const std::string MessageTag;

public:
    EcmSubtraction();
        //!< Constructor
        /*!<
         */
    EcmSubtraction(ElementContainerMatrix* ecm0);
        //!< Constructor
        /*!<
         *  @param ecm0 ElementContainerMatrix for the target
         */
    virtual bool Subtract(ElementContainerMatrix* ecm1);
        //!< Sets ECM and subtract it
        /*!<
         *  @param ecm1 ElementContainerMatrix
         */
};
#endif
