#ifndef PARTICLETYPE_H
#define PARTICLETYPE_H

class AdvModel;
class AdvModelParticle;

class AdvModelParticleType
{
public:
        enum ParticleType {
            NONE                   =   0,
            SPHERE                 = 100,
            SPHERE_LAYER           = 101,
            ELLIPSOID_2            = 102,
            ELLIPSOID_3            = 103,
            PARALLELEPIPED         = 200,
            CYLINDER               = 300,
            CYLINDER_ELLIPTICAL    = 301,
            CYLINDER_HOLLOW        = 302,
            CYLINDER_THINROD       = 303,
            CYLINDER_THINDISK      = 304,
        };

protected:
        AdvModelParticle*  CreateParticle(AdvModel*,ParticleType);
};

#endif
