import Manyo.Utsusemi as mu

tt=mu.UtsusemiDetectorEfficiencyCorrection()
tt.Initialize( 99999, "SIK-SUS304-3He-PSD" )
tt.MakeApproximateData( "SIK-SUS304-3He-PSD_10k.dat", 0.1, 10000, 1000, -50.0, 50.0, 1000 )
del tt

#dfile = "DetectorInfo_200302.xml"
#tt=mu.UtsusemiDetectorEfficiencyCorrection()
#tt.Initialize( str(dfile), "SIK-SUS304-3He-PSD" )
#tt.MakeApproximateData( "SIK-SUS304-3He-PSD_T0.dat", 0.1, 1000, 1000, -50.0, 50.0, 1000 )
#del tt

