#!/usr/bin/python

from math import *
import numpy as np
import Manyo as m
import Functions

xmin = -5.0
xmax =  5.0
nDiv = 1000
p= [ 1.0, 0.0, 1.0 ]

gaussian = Functions.Gaussian()
delta = (xmax-xmin)/nDiv
for i in range(nDiv+1):
    x = xmin + delta*i
    print str(i).rjustify(4), str(x).rjustify(10), str(gauusian.eval(x, p)).rjustify(10)
