#ifndef PARAPETER_IO_CONST
#define PARAPETER_IO_CONST

#include <typeinfo>

#include "Header.hh"

#include ""

class ParameterIOConst {
    public:
        /** definition of parameter type ids */
        static enum KeyTypeID {
            BOOL,          /**< single boolean value */
            INT4,          /**< single Int4 type value */
            UINT4,         /**< single unsigned Int4 type vlaue */
            DOUBLE,        /**< single double type value */
            VECTOR_DOUBLE, /**< a vector of Double type elements */
            MATRIX_DOUBLE, /**< a matrix of Double type elements */
            PEAKDATA,      /**< a list of peaks */
            FUNC_LIST,     /**< vector of pointers of functions */
            MATRIX_FUNC_P, /**< matrix of pointers of functions */
        };

        /** parameter type string expression */
        const string KeyTypeStr[];
};

#endif //  PARAPETER_IO_CONST
