#ifndef UTSUSEMIEVENTDATAMONITORNEUNET
#define UTSUSEMIEVENTDATAMONITORNEUNET

#include "UtsusemiEventDataMonitorTemplate.hh"
#include "UtsusemiEventDataConverterNeunet.hh"
#include "T0TreatToolsNeunet.hh"

class UtsusemiEventDataMonitorNeunet:
    public UtsusemiEventDataMonitorTemplate<UtsusemiEventDataConverterNeunet>
//    public UtsusemiEventDataMonitorTemplate<UtsusemiEventDataConverterNeunet,T0TreatToolsNeunet>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiEventDataMonitorNeunet();
        //!< Constructor
        /*!<
         */
    ~UtsusemiEventDataMonitorNeunet();
        //!< Destructor
        /*!<
         */
    void OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId=1 );
    UtsusemiEventDataConverterNeunet* PutUtsusemiEventDataConverterNeunet(){ return _EDC; }
};
#endif
