#!/bin/bash

cd $1

echo IMPORT $1
echo `pwd`

echo "from ..MLF import *" > MLF/__init__.py
echo "from .MLF import *" >> MLF/__init__.py
echo "" >> MLF/__init__.py
echo "from ..Utsusemi import *" > Utsusemi/__init__.py
echo "from .Utsusemi import *" >> Utsusemi/__init__.py
echo "" >> Utsusemi/__init__.py
