#! /bin/bash

export PYENV_ROOT="${OPT}/.pyenv"
export PYPKG="${OPT}/py_pkg"
export PYTHONPATH="${PYPKG}/PySide2:${PYPKG}:${PYTHONPATH}"
export PATH="${PYENV_ROOT}/bin:${PYENV_ROOT}/shims:${PYPKG}/bin:${PATH}"
