#!/usr/bin/python
import Manyo as m
API = m.AcquireNeutronSourceTextInformation()
##################################################
def txtInfo4(sig,begin,end,step):
    firstTxt=[]
    secondTxt=[]
    print "txtInfo-4 >>>",sig,begin,end,step

    API.GetTextInformation(sig,begin,end,step)
    firstTxt = API.PutTextFirstInformation()
    secondTxt = API.PutTextSecondInformation() 

    return firstTxt,secondTxt
##################################################
def txtInfo2(sig,begin):
    firstTxt=[]
    secondTxt=[]
    print "txtInfo-2 >>>",sig,begin

    API.GetTextInformation(sig,begin)
    firstTxt = API.PutTextFirstInformation()
    secondTxt = API.PutTextSecondInformation() 

    return firstTxt,secondTxt
##################################################
def txtInfo1(sig):
    firstTxt=[]
    secondTxt=[]
    print "txtInfo-1 >>>",sig

    API.GetTextInformation(sig)
    firstTxt = API.PutTextFirstInformation()
    secondTxt = API.PutTextSecondInformation() 

    return firstTxt,secondTxt
##################################################

#firstStr, secondStr = txtInfo4("BS","2009/06/03 00:10:00","2009/06/03 03:00:00",1)
#firstStr, secondStr = txtInfo4("BS","2009/06/03 00:00:00","2009/06/03 10:00:00",0)
firstStr, secondStr = txtInfo2("BS","2009/06/03 00:00:00")
#firstStr, secondStr = txtInfo2("BS","")
#firstStr, secondStr = txtInfo1("BS")

print
print "      ##### result #####"
for i in range(len(firstStr)):
    print "[",i,"] =", firstStr[i],"<--->", secondStr[i]
print
