# Utsusemi software environment 
# (2014.09.30)

# BL info
XXX=SAS
BL=BL15

# Path to Manyo-lib
export MANYO_PATH=/opt/mlfsoft/manyo

# Path to python-Utsusemi
PYUTSUSEMI_PATH=/opt/mlfsoft/python-utsusemi

# Path to Emaki
EMAKIPY_PATH=/opt/mlfsoft/python-emaki

# External Library path
UTSUSEMI_EXT_LIBRARY_PATH=/usr/local/lib


# Utsusemi Environment
export UTSUSEMI_BASE_DIR=${PYUTSUSEMI_PATH}
export UTSUSEMI_SYS_NAME=${XXX}
export UTSUSEMI_DATA_DIR=/data
export UTSUSEMI_USR_DIR=${UTSUSEMI_BASE_DIR}/${XXX}
export UTSUSEMI_USR_PRIV_HOME=${HOME}



# Python Path
PYTHONPATH=${MANYO_PATH}:${PYUTSUSEMI_PATH}:${PYUTSUSEMI_PATH}/facade:${EMAKIPY_PATH}:${UTSUSEMI_USR_DIR}/facade
export PYTHONPATH

# Shared Library Path
export LD_LIBRARY_PATH=${MANYO_PATH}/lib:${UTSUSEMI_EXT_LIBRARY_PATH}

# Executable path
export PATH=${MANYO_PATH}/bin:${PATH}

echo '=============================='
echo ' Utsusemi Environment for '${XXX}
echo '=============================='
echo ' Ana   : Sequencer'
#echo ' Dview : EmakiDetectorViewer'
echo '=============================='

# Alias

alias Ana='python '${PYUTSUSEMI_PATH}'/ana/Sequencer2.py &'
#alias Dview='python '${EMAKIPY_PATH}'/EmakiDetectorViewer.py &'
