import Manyo.Utsusemi as mu

tt=mu.UtsusemiDetectorEfficiencyCorrection()
tt.Initialize( 99999, "AMR-SUS304-3He-PSD" )
tt.MakeApproximateData( "AMR-SUS304-3He-PSD.dat", 0.1, 1000, 1000, -50.0, 50.0, 1000 )
del tt
