# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'SequencerQ_Main.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGroupBox, QHeaderView,
    QLabel, QLineEdit, QMainWindow, QPushButton,
    QSizePolicy, QStatusBar, QTextBrowser, QTreeWidget,
    QTreeWidgetItem, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1174, 727)
        MainWindow.setUnifiedTitleAndToolBarOnMac(False)
        self.actionFile = QAction(MainWindow)
        self.actionFile.setObjectName(u"actionFile")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.SplashPanel = QLabel(self.centralwidget)
        self.SplashPanel.setObjectName(u"SplashPanel")
        self.SplashPanel.setGeometry(QRect(10, 10, 335, 50))
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(370, 10, 261, 61))
        self.btClear = QPushButton(self.groupBox)
        self.btClear.setObjectName(u"btClear")
        self.btClear.setGeometry(QRect(10, 25, 51, 32))
        self.btOpen = QPushButton(self.groupBox)
        self.btOpen.setObjectName(u"btOpen")
        self.btOpen.setGeometry(QRect(80, 25, 32, 32))
        self.btSaveas = QPushButton(self.groupBox)
        self.btSaveas.setObjectName(u"btSaveas")
        self.btSaveas.setGeometry(QRect(120, 25, 32, 32))
        self.btCCD = QPushButton(self.groupBox)
        self.btCCD.setObjectName(u"btCCD")
        self.btCCD.setGeometry(QRect(160, 25, 99, 32))
        self.groupBox_2 = QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(650, 10, 511, 61))
        self.btBegin = QPushButton(self.groupBox_2)
        self.btBegin.setObjectName(u"btBegin")
        self.btBegin.setGeometry(QRect(20, 25, 80, 32))
        self.btStop = QPushButton(self.groupBox_2)
        self.btStop.setObjectName(u"btStop")
        self.btStop.setGeometry(QRect(110, 25, 80, 32))
        self.btResume = QPushButton(self.groupBox_2)
        self.btResume.setObjectName(u"btResume")
        self.btResume.setGeometry(QRect(230, 25, 150, 32))
        self.btVis = QPushButton(self.groupBox_2)
        self.btVis.setObjectName(u"btVis")
        self.btVis.setGeometry(QRect(410, 25, 100, 32))
        self.line = QFrame(self.groupBox_2)
        self.line.setObjectName(u"line")
        self.line.setGeometry(QRect(390, 25, 20, 31))
        self.line.setFrameShape(QFrame.Shape.VLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)
        self.tree0 = QTreeWidget(self.centralwidget)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.tree0.setHeaderItem(__qtreewidgetitem)
        self.tree0.setObjectName(u"tree0")
        self.tree0.setGeometry(QRect(20, 70, 300, 320))
        self.tree0.setHeaderHidden(True)
        self.list1 = QTreeWidget(self.centralwidget)
        __qtreewidgetitem1 = QTreeWidgetItem()
        __qtreewidgetitem1.setText(0, u"1");
        self.list1.setHeaderItem(__qtreewidgetitem1)
        self.list1.setObjectName(u"list1")
        self.list1.setGeometry(QRect(370, 110, 800, 280))
        self.st0 = QLabel(self.centralwidget)
        self.st0.setObjectName(u"st0")
        self.st0.setGeometry(QRect(380, 70, 91, 17))
        self.btAdd = QPushButton(self.centralwidget)
        self.btAdd.setObjectName(u"btAdd")
        self.btAdd.setGeometry(QRect(330, 100, 32, 32))
        self.btRemove = QPushButton(self.centralwidget)
        self.btRemove.setObjectName(u"btRemove")
        self.btRemove.setGeometry(QRect(330, 150, 32, 32))
        self.btCopyAdd = QPushButton(self.centralwidget)
        self.btCopyAdd.setObjectName(u"btCopyAdd")
        self.btCopyAdd.setGeometry(QRect(330, 220, 32, 32))
        self.btUp = QPushButton(self.centralwidget)
        self.btUp.setObjectName(u"btUp")
        self.btUp.setGeometry(QRect(330, 270, 32, 32))
        self.btDown = QPushButton(self.centralwidget)
        self.btDown.setObjectName(u"btDown")
        self.btDown.setGeometry(QRect(330, 320, 32, 32))
        self.stCurrentDir = QLabel(self.centralwidget)
        self.stCurrentDir.setObjectName(u"stCurrentDir")
        self.stCurrentDir.setGeometry(QRect(470, 70, 671, 17))
        self.st1 = QLabel(self.centralwidget)
        self.st1.setObjectName(u"st1")
        self.st1.setGeometry(QRect(710, 90, 68, 17))
        self.status = QLabel(self.centralwidget)
        self.status.setObjectName(u"status")
        self.status.setGeometry(QRect(780, 90, 161, 17))
        self.groupBox_3 = QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(380, 390, 781, 311))
        self.st2 = QLabel(self.groupBox_3)
        self.st2.setObjectName(u"st2")
        self.st2.setGeometry(QRect(140, 30, 101, 17))
        self.retval = QLineEdit(self.groupBox_3)
        self.retval.setObjectName(u"retval")
        self.retval.setGeometry(QRect(240, 25, 150, 30))
        self.btSet = QPushButton(self.groupBox_3)
        self.btSet.setObjectName(u"btSet")
        self.btSet.setGeometry(QRect(590, 25, 120, 30))
        self.line_2 = QFrame(self.groupBox_3)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setGeometry(QRect(10, 50, 761, 16))
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)
        self.arg1 = QLineEdit(self.groupBox_3)
        self.arg1.setObjectName(u"arg1")
        self.arg1.setGeometry(QRect(10, 80, 240, 24))
        self.arg2 = QLineEdit(self.groupBox_3)
        self.arg2.setObjectName(u"arg2")
        self.arg2.setGeometry(QRect(270, 80, 240, 24))
        self.arg3 = QLineEdit(self.groupBox_3)
        self.arg3.setObjectName(u"arg3")
        self.arg3.setGeometry(QRect(530, 80, 240, 24))
        self.arg4 = QLineEdit(self.groupBox_3)
        self.arg4.setObjectName(u"arg4")
        self.arg4.setGeometry(QRect(10, 130, 240, 24))
        self.arg5 = QLineEdit(self.groupBox_3)
        self.arg5.setObjectName(u"arg5")
        self.arg5.setGeometry(QRect(270, 130, 240, 24))
        self.arg6 = QLineEdit(self.groupBox_3)
        self.arg6.setObjectName(u"arg6")
        self.arg6.setGeometry(QRect(530, 130, 240, 24))
        self.arg7 = QLineEdit(self.groupBox_3)
        self.arg7.setObjectName(u"arg7")
        self.arg7.setGeometry(QRect(10, 180, 240, 24))
        self.arg8 = QLineEdit(self.groupBox_3)
        self.arg8.setObjectName(u"arg8")
        self.arg8.setGeometry(QRect(270, 180, 240, 24))
        self.arg9 = QLineEdit(self.groupBox_3)
        self.arg9.setObjectName(u"arg9")
        self.arg9.setGeometry(QRect(530, 180, 240, 24))
        self.arg10 = QLineEdit(self.groupBox_3)
        self.arg10.setObjectName(u"arg10")
        self.arg10.setGeometry(QRect(10, 230, 240, 24))
        self.arg11 = QLineEdit(self.groupBox_3)
        self.arg11.setObjectName(u"arg11")
        self.arg11.setGeometry(QRect(270, 230, 240, 24))
        self.arg12 = QLineEdit(self.groupBox_3)
        self.arg12.setObjectName(u"arg12")
        self.arg12.setGeometry(QRect(530, 230, 240, 24))
        self.arg13 = QLineEdit(self.groupBox_3)
        self.arg13.setObjectName(u"arg13")
        self.arg13.setGeometry(QRect(10, 280, 240, 24))
        self.arg14 = QLineEdit(self.groupBox_3)
        self.arg14.setObjectName(u"arg14")
        self.arg14.setGeometry(QRect(270, 280, 240, 24))
        self.arg15 = QLineEdit(self.groupBox_3)
        self.arg15.setObjectName(u"arg15")
        self.arg15.setGeometry(QRect(530, 280, 240, 24))
        self.argst1 = QLabel(self.groupBox_3)
        self.argst1.setObjectName(u"argst1")
        self.argst1.setGeometry(QRect(20, 60, 231, 17))
        self.argst2 = QLabel(self.groupBox_3)
        self.argst2.setObjectName(u"argst2")
        self.argst2.setGeometry(QRect(280, 60, 231, 17))
        self.argst3 = QLabel(self.groupBox_3)
        self.argst3.setObjectName(u"argst3")
        self.argst3.setGeometry(QRect(540, 60, 231, 17))
        self.argst4 = QLabel(self.groupBox_3)
        self.argst4.setObjectName(u"argst4")
        self.argst4.setGeometry(QRect(20, 110, 231, 17))
        self.argst5 = QLabel(self.groupBox_3)
        self.argst5.setObjectName(u"argst5")
        self.argst5.setGeometry(QRect(280, 110, 231, 17))
        self.argst6 = QLabel(self.groupBox_3)
        self.argst6.setObjectName(u"argst6")
        self.argst6.setGeometry(QRect(540, 110, 231, 17))
        self.argst7 = QLabel(self.groupBox_3)
        self.argst7.setObjectName(u"argst7")
        self.argst7.setGeometry(QRect(20, 160, 231, 17))
        self.argst8 = QLabel(self.groupBox_3)
        self.argst8.setObjectName(u"argst8")
        self.argst8.setGeometry(QRect(280, 160, 231, 17))
        self.argst9 = QLabel(self.groupBox_3)
        self.argst9.setObjectName(u"argst9")
        self.argst9.setGeometry(QRect(540, 160, 231, 17))
        self.argst10 = QLabel(self.groupBox_3)
        self.argst10.setObjectName(u"argst10")
        self.argst10.setGeometry(QRect(20, 210, 231, 17))
        self.argst11 = QLabel(self.groupBox_3)
        self.argst11.setObjectName(u"argst11")
        self.argst11.setGeometry(QRect(280, 210, 231, 17))
        self.argst12 = QLabel(self.groupBox_3)
        self.argst12.setObjectName(u"argst12")
        self.argst12.setGeometry(QRect(540, 210, 231, 17))
        self.argst13 = QLabel(self.groupBox_3)
        self.argst13.setObjectName(u"argst13")
        self.argst13.setGeometry(QRect(20, 260, 231, 17))
        self.argst14 = QLabel(self.groupBox_3)
        self.argst14.setObjectName(u"argst14")
        self.argst14.setGeometry(QRect(280, 260, 231, 17))
        self.argst15 = QLabel(self.groupBox_3)
        self.argst15.setObjectName(u"argst15")
        self.argst15.setGeometry(QRect(540, 260, 231, 17))
        self.funcdoc = QTextBrowser(self.centralwidget)
        self.funcdoc.setObjectName(u"funcdoc")
        self.funcdoc.setGeometry(QRect(20, 400, 330, 280))
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionFile.setText(QCoreApplication.translate("MainWindow", u"File", None))
        self.SplashPanel.setText("")
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.btClear.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.btOpen.setText("")
        self.btSaveas.setText("")
        self.btCCD.setText(QCoreApplication.translate("MainWindow", u"Current Dir", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("MainWindow", u"Control", None))
        self.btBegin.setText(QCoreApplication.translate("MainWindow", u"Start", None))
        self.btStop.setText(QCoreApplication.translate("MainWindow", u"Stop", None))
        self.btResume.setText(QCoreApplication.translate("MainWindow", u"Resume", None))
        self.btVis.setText(QCoreApplication.translate("MainWindow", u"Visualize", None))
        self.st0.setText(QCoreApplication.translate("MainWindow", u"Current Dir :", None))
        self.btAdd.setText("")
        self.btRemove.setText("")
        self.btCopyAdd.setText("")
        self.btUp.setText("")
        self.btDown.setText("")
        self.stCurrentDir.setText("")
        self.st1.setText(QCoreApplication.translate("MainWindow", u"Status :", None))
        self.status.setText(QCoreApplication.translate("MainWindow", u"Executable", None))
        self.groupBox_3.setTitle("")
        self.st2.setText(QCoreApplication.translate("MainWindow", u"Return Label :", None))
        self.btSet.setText(QCoreApplication.translate("MainWindow", u"Set", None))
        self.argst1.setText(QCoreApplication.translate("MainWindow", u"Argument1", None))
        self.argst2.setText(QCoreApplication.translate("MainWindow", u"Argument2", None))
        self.argst3.setText(QCoreApplication.translate("MainWindow", u"Argument3", None))
        self.argst4.setText(QCoreApplication.translate("MainWindow", u"Argument4", None))
        self.argst5.setText(QCoreApplication.translate("MainWindow", u"Argument5", None))
        self.argst6.setText(QCoreApplication.translate("MainWindow", u"Argument6", None))
        self.argst7.setText(QCoreApplication.translate("MainWindow", u"Argument7", None))
        self.argst8.setText(QCoreApplication.translate("MainWindow", u"Argument8", None))
        self.argst9.setText(QCoreApplication.translate("MainWindow", u"Argument9", None))
        self.argst10.setText(QCoreApplication.translate("MainWindow", u"Argument10", None))
        self.argst11.setText(QCoreApplication.translate("MainWindow", u"Argument11", None))
        self.argst12.setText(QCoreApplication.translate("MainWindow", u"Argument12", None))
        self.argst13.setText(QCoreApplication.translate("MainWindow", u"Argument13", None))
        self.argst14.setText(QCoreApplication.translate("MainWindow", u"Argument14", None))
        self.argst15.setText(QCoreApplication.translate("MainWindow", u"Argument15", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'SequencerQ_Save.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QDialog, QLabel, QPushButton,
    QSizePolicy, QTextEdit, QWidget)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(342, 170)
        self.comment = QTextEdit(Dialog)
        self.comment.setObjectName(u"comment")
        self.comment.setGeometry(QRect(10, 40, 320, 80))
        self.label = QLabel(Dialog)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 10, 261, 17))
        self.btOk = QPushButton(Dialog)
        self.btOk.setObjectName(u"btOk")
        self.btOk.setGeometry(QRect(50, 130, 100, 32))
        self.btCancel = QPushButton(Dialog)
        self.btCancel.setObjectName(u"btCancel")
        self.btCancel.setGeometry(QRect(180, 130, 100, 32))

        self.retranslateUi(Dialog)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Dialog", None))
        self.label.setText(QCoreApplication.translate("Dialog", u"Comments for the sequence :", None))
        self.btOk.setText(QCoreApplication.translate("Dialog", u"OK", None))
        self.btCancel.setText(QCoreApplication.translate("Dialog", u"Cancel", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'SequencerQ_Vis.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGroupBox, QHeaderView,
    QLabel, QLineEdit, QMainWindow, QMenuBar,
    QPushButton, QSizePolicy, QStatusBar, QTreeWidget,
    QTreeWidgetItem, QWidget)

class Ui_visframe(object):
    def setupUi(self, visframe):
        if not visframe.objectName():
            visframe.setObjectName(u"visframe")
        visframe.resize(737, 340)
        self.centralwidget = QWidget(visframe)
        self.centralwidget.setObjectName(u"centralwidget")
        self.tree = QTreeWidget(self.centralwidget)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.tree.setHeaderItem(__qtreewidgetitem)
        self.tree.setObjectName(u"tree")
        self.tree.setGeometry(QRect(10, 10, 200, 265))
        self.tree.setHeaderHidden(True)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(220, 10, 391, 281))
        self.arg1 = QLineEdit(self.groupBox)
        self.arg1.setObjectName(u"arg1")
        self.arg1.setGeometry(QRect(280, 30, 100, 28))
        self.arg2 = QLineEdit(self.groupBox)
        self.arg2.setObjectName(u"arg2")
        self.arg2.setGeometry(QRect(280, 60, 100, 28))
        self.arg3 = QLineEdit(self.groupBox)
        self.arg3.setObjectName(u"arg3")
        self.arg3.setGeometry(QRect(280, 90, 100, 28))
        self.arg4 = QLineEdit(self.groupBox)
        self.arg4.setObjectName(u"arg4")
        self.arg4.setGeometry(QRect(280, 120, 100, 28))
        self.arg5 = QLineEdit(self.groupBox)
        self.arg5.setObjectName(u"arg5")
        self.arg5.setGeometry(QRect(280, 150, 100, 28))
        self.arg6 = QLineEdit(self.groupBox)
        self.arg6.setObjectName(u"arg6")
        self.arg6.setGeometry(QRect(280, 180, 100, 28))
        self.arg7 = QLineEdit(self.groupBox)
        self.arg7.setObjectName(u"arg7")
        self.arg7.setGeometry(QRect(280, 210, 100, 28))
        self.arg8 = QLineEdit(self.groupBox)
        self.arg8.setObjectName(u"arg8")
        self.arg8.setGeometry(QRect(280, 240, 100, 28))
        self.argst1 = QLabel(self.groupBox)
        self.argst1.setObjectName(u"argst1")
        self.argst1.setGeometry(QRect(10, 30, 101, 20))
        self.argst2 = QLabel(self.groupBox)
        self.argst2.setObjectName(u"argst2")
        self.argst2.setGeometry(QRect(10, 60, 101, 20))
        self.argst3 = QLabel(self.groupBox)
        self.argst3.setObjectName(u"argst3")
        self.argst3.setGeometry(QRect(10, 90, 101, 20))
        self.argst4 = QLabel(self.groupBox)
        self.argst4.setObjectName(u"argst4")
        self.argst4.setGeometry(QRect(10, 120, 101, 20))
        self.argst5 = QLabel(self.groupBox)
        self.argst5.setObjectName(u"argst5")
        self.argst5.setGeometry(QRect(10, 150, 101, 20))
        self.argst6 = QLabel(self.groupBox)
        self.argst6.setObjectName(u"argst6")
        self.argst6.setGeometry(QRect(10, 180, 101, 20))
        self.argst7 = QLabel(self.groupBox)
        self.argst7.setObjectName(u"argst7")
        self.argst7.setGeometry(QRect(10, 210, 101, 20))
        self.argst8 = QLabel(self.groupBox)
        self.argst8.setObjectName(u"argst8")
        self.argst8.setGeometry(QRect(10, 240, 101, 20))
        self.argsc1 = QComboBox(self.groupBox)
        self.argsc1.setObjectName(u"argsc1")
        self.argsc1.setGeometry(QRect(110, 30, 161, 27))
        self.argsc2 = QComboBox(self.groupBox)
        self.argsc2.setObjectName(u"argsc2")
        self.argsc2.setGeometry(QRect(110, 60, 161, 27))
        self.argsc3 = QComboBox(self.groupBox)
        self.argsc3.setObjectName(u"argsc3")
        self.argsc3.setGeometry(QRect(110, 90, 161, 27))
        self.argsc4 = QComboBox(self.groupBox)
        self.argsc4.setObjectName(u"argsc4")
        self.argsc4.setGeometry(QRect(110, 120, 161, 27))
        self.argsc5 = QComboBox(self.groupBox)
        self.argsc5.setObjectName(u"argsc5")
        self.argsc5.setGeometry(QRect(110, 150, 161, 27))
        self.argsc6 = QComboBox(self.groupBox)
        self.argsc6.setObjectName(u"argsc6")
        self.argsc6.setGeometry(QRect(110, 180, 161, 27))
        self.argsc7 = QComboBox(self.groupBox)
        self.argsc7.setObjectName(u"argsc7")
        self.argsc7.setGeometry(QRect(110, 210, 161, 27))
        self.argsc8 = QComboBox(self.groupBox)
        self.argsc8.setObjectName(u"argsc8")
        self.argsc8.setGeometry(QRect(110, 240, 161, 27))
        self.retval = QLineEdit(self.centralwidget)
        self.retval.setObjectName(u"retval")
        self.retval.setGeometry(QRect(620, 60, 100, 28))
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(620, 40, 91, 17))
        self.btExec = QPushButton(self.centralwidget)
        self.btExec.setObjectName(u"btExec")
        self.btExec.setGeometry(QRect(630, 110, 80, 32))
        self.btClose = QPushButton(self.centralwidget)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(640, 170, 80, 32))
        visframe.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(visframe)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 737, 24))
        visframe.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(visframe)
        self.statusbar.setObjectName(u"statusbar")
        visframe.setStatusBar(self.statusbar)

        self.retranslateUi(visframe)

        QMetaObject.connectSlotsByName(visframe)
    # setupUi

    def retranslateUi(self, visframe):
        visframe.setWindowTitle(QCoreApplication.translate("visframe", u"MainWindow", None))
        self.groupBox.setTitle(QCoreApplication.translate("visframe", u"Arguments", None))
        self.argst1.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst2.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst3.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst4.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst5.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst6.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst7.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.argst8.setText(QCoreApplication.translate("visframe", u"Argument", None))
        self.label.setText(QCoreApplication.translate("visframe", u"Return Label", None))
        self.btExec.setText(QCoreApplication.translate("visframe", u"Execute", None))
        self.btClose.setText(QCoreApplication.translate("visframe", u"Close", None))
    # retranslateUi

