#!/bin/bash

cd $1

echo IMPORT $1
echo `pwd`

echo "from ..MLF import *" > manyo/MLF/__init__.py
echo "from .MLF import *" >> manyo/MLF/__init__.py
echo "" >> manyo/MLF/__init__.py
echo "from ..Utsusemi import *" > Utsusemi/manyo/__init__.py
echo "from .Utsusemi import *" >> Utsusemi/manyo/__init__.py
echo "" >> Utsusemi/manyo/__init__.py
