#ifndef MESSAGE_TEST_CLASSH
#define MESSAGE_TEST_CLASSH

#include <typeinfo>
#include <string>
#include "Message.hh"


class MessageTestClass : public Message {

    private:
        static const string className;

    public :
        MessageTestClass() ;

        ~MessageTestClass() ;

        void method1() ;
};

#endif
