#include "UtsusemiEventCasesDecoderBase.hh"
//////////////////////////////////////////////////////////
UtsusemiEventCasesDecoderBase::
UtsusemiEventCasesDecoderBase()
{
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiEventCasesDecoderBase::
~UtsusemiEventCasesDecoderBase()
{
}
//////////////////////////////////////////////////////////
void UtsusemiEventCasesDecoderBase::
Initialize()
{
    _NumOfCases = 1;

}

/////////////////////////////////////////////////////////////
// Functions required by UtsusemiEventDataConverterTemplate
/////////////////////////////////////////////////////////////
void UtsusemiEventCasesDecoderBase::
ReadCaseInfoFile( std::string filepath ){
}
/////////////////////////////////////////////////////////////
void UtsusemiEventCasesDecoderBase::
ReadCaseEvent( UInt4 index, std::vector<std::string> evt_files, std::vector<std::string> t0b_files ){
}
//////////////////////////////////////////////////////////
void UtsusemiEventCasesDecoderBase::
ReadCaseEventFiles( UInt4 index, std::vector<std::string> evt_files, std::vector<std::string> t0b_files ){
//ReadCaseEventFiles( std::vector<std::string> files ){
}
//////////////////////////////////////////////////////////
Double UtsusemiEventCasesDecoderBase::
CalcIncrementPosi( Double* tof, UInt4* pixelId, UInt4* caseId )
{
    return (Double)(*tof);
}
//////////////////////////////////////////////////////////
UInt4 UtsusemiEventCasesDecoderBase::
GetCaseFromTof( const UInt4 pixelId, Double* tof, std::vector<UInt4>* caseVec )
{
    return caseVec->at(0);
}

//////////////////////////////////////////////////////////
std::vector<UInt4> UtsusemiEventCasesDecoderBase::
PutT0Index(){
    std::vector<UInt4> ret;
    return ret;
}
//////////////////////////////////////////////////////////
std::vector<Double> UtsusemiEventCasesDecoderBase::
PutT0ClockDiff(){
    std::vector<Double> ret;
    return ret;
}
//////////////////////////////////////////////////////////
std::vector< std::vector<UInt4>* >* UtsusemiEventCasesDecoderBase::
PutCaseTable(){
    return NULL;
}
//////////////////////////////////////////////////////////
void UtsusemiEventCasesDecoderBase::
CheckCaseTableWithT0Index(){
}
