#ifndef UTSUSEMIREDISPUBSUBEVENTMONITORRPMT
#define UTSUSEMIREDISPUBSUBEVENTMONITORRPMT

#include "UtsusemiRedisPubSubEventMonitorTemplate.hh"
#include "UtsusemiEventDataConverterRPMT.hh"


class UtsusemiRedisPubSubEventMonitorRPMT:
    public UtsusemiRedisPubSubEventMonitorTemplate<UtsusemiEventDataConverterRPMT>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiRedisPubSubEventMonitorRPMT();
        //!< Constructor
        /*!<
         */
    ~UtsusemiRedisPubSubEventMonitorRPMT();
        //!< Destructor
        /*!<
         */
    void OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId=1 );
    UtsusemiEventDataConverterRPMT* PutUtsusemiEventDataConverterRPMT(){ return _EDC; }
};
#endif
