#!/bin/bash
if [ ! -f disable-detection-of-gtk.patch ]; then
  curl http://vault.centos.org/7.7.1908/os/Source/SPackages/python-matplotlib-1.2.0-15.el7.src.rpm | rpm2cpio | cpio -idmv
  rm -f python-matplotlib.spec
fi
