# -*- coding: utf-8 -*-
"""
import sys
#from pyqtgraph.Qt import QtCore, QtGui, uic
try:
    import PySide6
except:
    import PySide2
import pyqtgraph as pg
import pyqtgraph.opengl as gl
import numpy as np
from OpenGL.GL import *
"""

class SliceData(object):
    def __init__(self):
        pass

        self.visible   = True
        self.axisTitle = "ax1"
        self.position  = "0" # スライス形式の表示を許可するため、文字列
        self.thickness = 1.0




