from uGao.MPlotJupyter import MPlotJupyter

class MPlotInDark(MPlotJupyter):
    """
    """
    def __init__(self, data=None, order=1, fTitle=None, sizeX=8.0, sizeY=5.0, dpi=90):
        """
        """
        super().__init__(data, order, fTitle, sizeX, sizeY, dpi)
