#ifndef UTSUSEMIEVENTDATACONVERTERSAMPLE
#define UTSUSEMIEVENTDATACONVERTERSAMPLE

#include "UtsusemiEventDataConverterTemplate.hh"
#include "UtsusemiNeutronEventDecoderBase.hh"
#include "UtsusemiEventCasesDecoderTemplate.hh"
#include "T0TreatToolsBase.hh"
#include "UtsusemiFilterBase.hh"

#include "Header.hh"
#include "HeaderBase.hh"
#include "ElementContainer.hh"
#include "ElementContainerArray.hh"
#include "ElementContainerMatrix.hh"
#include "StringTools.hh"
////////////////////////////////////
// UtsusemiEventDataConverterSample
///////////////////////////////////

//! Load Event Data and convert to Histogram for Inelastic scattering
/*!
 *
 */

class UtsusemiEventDataConverterSample:
    public UtsusemiEventDataConverterTemplate<UtsusemiNeutronEventDecoderBase,
                                              UtsusemiEventCasesDecoderTemplate<T0TreatToolsBase,UtsusemiFilterBase> >
{
private:
    std::string _MessageTag;

public:
    UtsusemiEventDataConverterSample();
        //!< Constructor
        /*!<
         */
    ~UtsusemiEventDataConverterSample();
        //!< Destructor
        /*!<
         */

};
#endif
