#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

import time
import os
import os.path
import Manyo as mm
import Manyo.SAS as ms
import Emaki.TransmittanceFitting as EMTF
import math

#import ana.Reduction.BaseCommands as BaseCom

# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
ecs = None
EC = None
ret = None
PyObj = None

#########################################
def PrintObject(obj):
    """
    Print object for debug.
    @param obj print to stdout
    """
    print(type(obj))
    try:
        print(obj)
    except:
        print("not printable")

#########################################
# Dictionary for entry the name of functions
_functions = {
              "PrintObject":PrintObject,
	      }

