class AdvModelInterferenceFunction
{

    enum  position_type {
      PAIR_POTENTIAL,
      LATTICE,
      NONE
    };

    enum  pair_potential_function {
      HARD_CORE,
      SQUARE_WELL,
      SCREENED_COULOMB
    };

    position_type  ptype;

};
