#!/bin/zsh

cd source/manyo
make clean -f Makefile.mac
chmod +x core/depend.mac
chmod +x MLF/depend.mac
cd core;./depend.mac;cd ../MLF;./depend.mac;cd ..
make install -f Makefile.mac NUM_MULTH=2 CXX=clang++
