# -*- coding: utf-8 -*-

import sys
import string
import math

import Manyo.core
import ModuleTest as ModTest


eca = ModTest.eca
ecs = ModTest.ecs
ecm = ModTest.ecm
ecd = ModTest.ecd
e2  = ModTest.ec0 * 2.0

#ModTest.PrintElementContainer( eca )
#ModTest.PrintElementContainer( ecs )
#ModTest.PrintElementContainer( ecm )
#ModTest.PrintElementContainer( ecd )
#ModTest.PrintElementContainer( ModTest.ec0 * 2.0 )

# python ModTestArithmeticOperations.py > `date +"%Y%m%d"`.txt
# などとして、diffで差分を監視すれば良い？
