#include "UtsusemiRedisPubSubEventMonitorReadout1d.hh"
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorReadout1d::
UtsusemiRedisPubSubEventMonitorReadout1d(){
    _MessageTag2 = "UtsusemiRedisPubSubEventMonitorReadout1d::";
}
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorReadout1d::
~UtsusemiRedisPubSubEventMonitorReadout1d(){
}
//////////////////////////////////////////////////////////
void UtsusemiRedisPubSubEventMonitorReadout1d::
OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId ){
    _EDC->SetElementContainerReadout( pixelId, ec, caseId );
    SetMeasTimeSliced( ec->PutHeaderPointer() );
    SetKickers( ec->PutHeaderPointer(), caseId );
}
