#include "AdvMultiDataMethod.hh"
#include <ios>
#include <ostream>

/**
 *  @author TANIMORI Souichirou, AdvanceSoft Corp.
 *  @version 0.0
 *  @since   0.7
 */


AdvMultiDataMethod::AdvMultiDataMethod() {
    //std::cout << "Debug: AdvMultiDataMethod::AdvMultiDataMethod(): called." << endl;
}


AdvMultiDataMethod::AdvMultiDataMethod(const AdvMultiDataMethod& method) {
    this->methodName      = method.methodName;
    this->valueOfLikehood = method.valueOfLikehood;
    //std::cout << "Debug: AdvMultiDataMethod::AdvMultiDataMethod(AdvMultiDataMethod &): called." << endl;
}

AdvMultiDataMethod::~AdvMultiDataMethod() {
}

AdvMultiDataMethod& AdvMultiDataMethod::operator = (const AdvMultiDataMethod &method) {
    this->methodName      = method.methodName;
    this->valueOfLikehood = method.valueOfLikehood;
    return *this;
}

/** 
 *  get the method name
 */
string AdvMultiDataMethod::getMethodName() const {
    return methodName;
}


/**
 *  get the value of likehood
 */
Double AdvMultiDataMethod::getLikehood() const {
    return valueOfLikehood;
}
