#ifndef UTSUSEMISUBTRACTDARKBACKGROUND
#define UTSUSEMISUBTRACTDARKBACKGROUND

#include "UtsusemiHeader.hh"

////////////////////////////////////
// UtsusemiSubtractDarkBackground
///////////////////////////////////

//!
/*!
 *
 */

class UtsusemiSubtractDarkBackground
{
private:
    std::string _MessageTag;

protected:

public:
    UtsusemiSubtractDarkBackground();
        //!< Constructor
        /*!<
         *    @retval None
         */
    ~UtsusemiSubtractDarkBackground();
        //!< Destructor
        /*!<
         */
    bool Execute(ElementContainerMatrix *sam, ElementContainerMatrix *bak);
        //!< Executes subtraction
        /*!< bool Execute(ElementContainerMatrix *sam, ElementContainerMatrix *bak)
         *
         *   @param sam (ElementContainermatrix) Sample data
         *   @param bak  (ElementContainermatrix) Background data
         *   @param normFact_ecm (Double) normalize factor for ecm
         *   @param normFact_bk  (Double) normalize factor for bk
         *   @retval true    : succeeded
         *   @retval false   : failed
         */
};
#endif
