#include "AdvConvolutionConsts.hh"

const string AdvConvolutionConsts::CONVOLUTION_TYPE("convolution type");;
const string AdvConvolutionConsts::ConvolutionTypeStr[] = {
    string("linewr"),
    string("cyclic"),
    //string("circular"),
    //string("periodic"),
};
const AdvConvolutionConsts::ConvolutionType AdvConvolutionConsts::DEFAULT_CONVOLUTION_TYPE=AdvConvolutionConsts::LINEAR;

const string  AdvConvolutionConsts::RES_INDEX("res index");
const UInt4   AdvConvolutionConsts::DEFAULT_RES_INDEX=0;

const string AdvConvolutionConsts::WIDTH_THRESHOLD("width threshold");
//static const Double DEFAULT_WIDTH_THRESHOLD=numeric_limits<Double>::epsilon;
const Double AdvConvolutionConsts::DEFAULT_WIDTH_THRESHOLD=1.0e-9; // - 120614 18:55
//const Double AdvConvolutionConsts::DEFAULT_WIDTH_THRESHOLD=1.0e-3;
