#include "UtsusemiEventDataMonitorMWPC.hh"
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorMWPC::
UtsusemiEventDataMonitorMWPC(){
    _MessageTag2 = "UtsusemiEventDataMonitorMWPC::";
    isReadout = true; //[inamura 170220]
}
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorMWPC::
~UtsusemiEventDataMonitorMWPC(){
}
