#ifndef CHECKDATAPROCESSEDCHOPPERS
#define CHECKDATAPROCESSEDCHOPPERS

#include "Header.hh"
#include "ElementContainer.hh"
#include "ElementContainerMatrix.hh"
#include "ElementContainerArray.hh"
#include "UtsusemiHeader.hh"

bool CheckDataProcessed( ElementContainerMatrix *ecm, string process );
void AddDataProcessed( ElementContainerMatrix *ecm, string process );
#endif
