#!/usr/bin/python

import Manyo
import BuildInFunc

paramForGaussian=Manyo.DoubleVector();
paramForGaussian.push_back(1.0)  # height
paramForGaussian.push_back(0.0)  # cneter
paramForGaussian.push_back(1.0)  # width

xmin=-5.0
xmax=5.0
div=100
delta=(xmax-xmin)/div

f=BuildInFunc.Lorentzian()
for i in range(div+1):
    x=xmin+delta*i
    g= f.gradient(x, paramForGaussian)
    print x, f.eval(x, paramForGaussian), f.der1st(x, paramForGaussian), f.der2nd(x, paramForGaussian), g[0], g[1], g[2]
    
