__AnaFacades__ = ["DR", "Cmm", "Com"]
__VisFacades__ = ["D1Plotter"]
"""
__Structures__ = [("File",["Cmm.SaveDataToManyoBinary",
                           "Cmm.LoadDataFromManyoBinary",
                           "Cmm.SaveDataToDAVEgrp",
                           "Cmm.SaveDataToDump",
                           "Cmm.LoadDataFromDump"]),
                  ("DataReduct",["Cmm.GetNeunetHist",
                                 "Cmm.GetMultiEi2",
                                 "DR.GetDataOfMonochroEi2",
                                 "DR.GetDataOfMonochroEi3",
                                 "DR.ToPowder",
                                 "DR.MakeRunList"])
                  ]
"""
