from __future__ import print_function
import Manyo.Utsusemi as mu


def UtsusemiVersion(isConsole=True):
    utsusemi_manyo_version = mu.UTSUSEMIMANYOVERSION
    try:
        utsusemi_manyo_version_rev = mu.UTSUSEMIMANYOVERSIONREV
    except:
        utsusemi_manyo_version_rev = 0

    ss = "---------------------------------------------\n"
    ss += "               Utsusemi Info\n"
    ss += "---------------------------------------------\n"
    if utsusemi_manyo_version_rev != 0:
        ss += "         version :  %s (rev %d)\n" % (
            utsusemi_manyo_version, utsusemi_manyo_version_rev)
    else:
        ss += "         version : %s \n" % (utsusemi_manyo_version)
    ss += "  Contact person : %s\n" % (mu.UTSUSEMICONTACTPERSON)
    ss += "          e-mail : %s\n" % (mu.UTSUSEMICONTACTEMAIL)
    ss += "---------------------------------------------\n"
    if isConsole:
        print(ss)
    else:
        return ss


UtsusemiVersion()
