#! /bin/bash

sudo rm -rf /Applications/Utsusemi*
rm -rf mlfdata/py_utsusemi mldata/python

mlfinit/setup_py_utsusemi.sh 
bash mlfscripts/build_utsusemi.sh; #bash mlfscripts/installer_utsusemi.sh ; cp mlfout/utsusemi_0.0.0_*.pkg ~/Desktop

