# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'test/uic007.ui',
# licensing of 'test/uic007.ui' applies.
#
# Created: Sun Feb 17 19:52:38 2019
#      by: pyside2-uic  running on PySide2 5.11.0
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(620, 372)
        MainWindow.setMaximumSize(QtCore.QSize(1200, 600))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(195, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 2, 4, 1, 1)
        self.browsePriv = QtWidgets.QPushButton(self.centralwidget)
        self.browsePriv.setMinimumSize(QtCore.QSize(80, 0))
        self.browsePriv.setMaximumSize(QtCore.QSize(80, 16777215))
        self.browsePriv.setObjectName("browsePriv")
        self.gridLayout.addWidget(self.browsePriv, 2, 5, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(self.centralwidget)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 8, 4, 1, 2)
        self.textPriv = QtWidgets.QTextEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textPriv.sizePolicy().hasHeightForWidth())
        self.textPriv.setSizePolicy(sizePolicy)
        self.textPriv.setMinimumSize(QtCore.QSize(0, 48))
        self.textPriv.setMaximumSize(QtCore.QSize(16777215, 48))
        self.textPriv.setObjectName("textPriv")
        self.gridLayout.addWidget(self.textPriv, 3, 3, 1, 3)
        self.browseData = QtWidgets.QPushButton(self.centralwidget)
        self.browseData.setObjectName("browseData")
        self.gridLayout.addWidget(self.browseData, 0, 5, 1, 1)
        self.beamList = QtWidgets.QListWidget(self.centralwidget)
        self.beamList.setMinimumSize(QtCore.QSize(250, 120))
        self.beamList.setMaximumSize(QtCore.QSize(250, 120))
        self.beamList.setObjectName("beamList")
        self.gridLayout.addWidget(self.beamList, 1, 0, 2, 2)
        self.label_5 = QtWidgets.QLabel(self.centralwidget)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 2, 3, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(195, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 4, 1, 1)
        self.checkYes = QtWidgets.QCheckBox(self.centralwidget)
        self.checkYes.setObjectName("checkYes")
        self.gridLayout.addWidget(self.checkYes, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setMinimumSize(QtCore.QSize(120, 24))
        self.label_6.setMaximumSize(QtCore.QSize(120, 24))
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 0, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(272, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 8, 0, 1, 4)
        self.textData = QtWidgets.QTextEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textData.sizePolicy().hasHeightForWidth())
        self.textData.setSizePolicy(sizePolicy)
        self.textData.setMinimumSize(QtCore.QSize(0, 48))
        self.textData.setMaximumSize(QtCore.QSize(16777215, 48))
        self.textData.setObjectName("textData")
        self.gridLayout.addWidget(self.textData, 1, 3, 1, 3)
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 0, 3, 1, 1)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 0, 2, 7, 1)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 6, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 3, 0, 1, 1)
        self.spinThreads = QtWidgets.QSpinBox(self.centralwidget)
        self.spinThreads.setMinimumSize(QtCore.QSize(60, 24))
        self.spinThreads.setMaximumSize(QtCore.QSize(60, 24))
        self.spinThreads.setObjectName("spinThreads")
        self.gridLayout.addWidget(self.spinThreads, 6, 1, 1, 1)
        self.checkDebug = QtWidgets.QCheckBox(self.centralwidget)
        self.checkDebug.setObjectName("checkDebug")
        self.gridLayout.addWidget(self.checkDebug, 4, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 4, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 620, 18))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.toolBar_2 = QtWidgets.QToolBar(MainWindow)
        self.toolBar_2.setObjectName("toolBar_2")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar_2)
        self.toolBar_3 = QtWidgets.QToolBar(MainWindow)
        self.toolBar_3.setObjectName("toolBar_3")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar_3)
        self.toolBar_4 = QtWidgets.QToolBar(MainWindow)
        self.toolBar_4.setObjectName("toolBar_4")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar_4)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setProperty("DlgSetUtsusemiEnv", QtWidgets.QApplication.translate("MainWindow", "MainWindow", None, -1))
        self.browsePriv.setText(QtWidgets.QApplication.translate("MainWindow", "Browse", None, -1))
        self.browseData.setText(QtWidgets.QApplication.translate("MainWindow", "Browse", None, -1))
        self.label_5.setText(QtWidgets.QApplication.translate("MainWindow", "USR_DIR", None, -1))
        self.checkYes.setText(QtWidgets.QApplication.translate("MainWindow", "Yes", None, -1))
        self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "Instrument", None, -1))
        self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "DATA_DIR", None, -1))
        self.label_3.setText(QtWidgets.QApplication.translate("MainWindow", "Number of threads", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("MainWindow", "Log quiet", None, -1))
        self.checkDebug.setText(QtWidgets.QApplication.translate("MainWindow", "Yes", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("MainWindow", "Utsusemi_debugmode", None, -1))
        self.toolBar.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "toolBar", None, -1))
        self.toolBar_2.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "toolBar_2", None, -1))
        self.toolBar_3.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "toolBar_3", None, -1))
        self.toolBar_4.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "toolBar_4", None, -1))

