#ifndef UTSUSEMIEVENTDATAMONITORMWPC
#define UTSUSEMIEVENTDATAMONITORMWPC

#include "UtsusemiEventDataMonitorTemplate.hh"
#include "UtsusemiEventDataConverterMWPC.hh"
#include "T0TreatToolsNeunet.hh"

class UtsusemiEventDataMonitorMWPC:
    public UtsusemiEventDataMonitorTemplate<UtsusemiEventDataConverterMWPC>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiEventDataMonitorMWPC();
        //!< Constructor
        /*!<
         */
    ~UtsusemiEventDataMonitorMWPC();
        //!< Destructor
        /*!<
         */
};
#endif
