#include "AdvModelParticleType.hh"
#include "AdvModelParticle.hh"

AdvModelParticle*  AdvModelParticleType::CreateParticle(AdvModel* model,ParticleType type)
{
    AdvModelParticle*  p = NULL;

    switch (type) {
        case NONE:
//            p = new AdvModelParticleNone ( model );
            break;
        case SPHERE:
            p = new AdvModelParticleSphere ( model );
            break;
        case SPHERE_LAYER:
            break;
        case ELLIPSOID_2:
            break;
        case ELLIPSOID_3:
            break;
        case PARALLELEPIPED:
            break;
        case CYLINDER:
            break;
        case CYLINDER_ELLIPTICAL:
            break;
        case CYLINDER_HOLLOW:
            break;
        case CYLINDER_THINROD:
            break;
        case CYLINDER_THINDISK:
            break;
        default:
            break;
    }

//    if (!p) p = new ModelParticleNone ( model );

    return p;
}
