#ifndef UTSUSEMIDETECTORINFOEDITORBASE
#define UTSUSEMIDETECTORINFOEDITORBASE

#include "UtsusemiHeader.hh"
#include "DetectorInfoEditorBase.hh"
#include "UtsusemiDetectorInfoEditorTemplate.hh"

//////////////////////////////////
// UtsusemiDetectorInfoEditorBase
/////////////////////////////////

//!
/*!
 *
 */

class UtsusemiDetectorInfoEditorBase:
    public UtsusemiDetectorInfoEditorTemplate< DetectorInfoEditorBase >
{
private:
    void Initialize();
    std::string _MessageTag;

protected:

public:
    UtsusemiDetectorInfoEditorBase();
        //!< Constructor
        /*!<
         */
    UtsusemiDetectorInfoEditorBase(std::string dfile, bool workAsReader);
        //!< Constructor with arguments
        /*!<
         *    @param dfile (std::string) path/to/DetectorInfo.xml to be loaded
         *    @param workAsReader (bool) swich the mode to work as a reader
         */
    ~UtsusemiDetectorInfoEditorBase();
        //!< Destructor
        /*!<
         */
    //////////////////////////////////////////////////////////////////////////
    // public variables
    //////////////////////////////////////////////////////////////////////////

    //////////////////////////////////////////////////////////////////////////
    // public functions
    //////////////////////////////////////////////////////////////////////////
};
#endif
