#! /bin/bash

sudo rm -rf /Applications/Manyo*
sudo rm -rf /Applications/Utsusemi*

bash mlfscripts/build_manyo.sh; bash mlfscripts/installer_manyo.sh ; cp mlfout/manyo_0.0.0_*.pkg ~/Desktop
bash mlfscripts/build_utsusemi.sh; bash mlfscripts/installer_utsusemi.sh ; cp mlfout/utsusemi_0.0.0_*.pkg ~/Desktop

