#include "UtsusemiEventDataMonitorNeunet.hh"
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorNeunet::
UtsusemiEventDataMonitorNeunet(){
    _MessageTag2 = "UtsusemiEventDataMonitorNeunet::";
}
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorNeunet::
~UtsusemiEventDataMonitorNeunet(){
}
//////////////////////////////////////////////////////////
void UtsusemiEventDataMonitorNeunet::
OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId ){
    _EDC->SetElementContainerNeunet( pixelId, ec, caseId );
    SetMeasTimeSliced( ec->PutHeaderPointer() );
    SetKickers( ec->PutHeaderPointer(), caseId );
}
