# Created by create_setenv.py at 2022-12-23 00:43:13
unset PREFIX
export MANYO_PATH=/Users/yinamura/mlfsoft/manyo

BREWPYPATH=/opt/homebrew/Cellar/python@3.10/3.10.9/Frameworks/Python.framework/Versions/Current
BREWPYINCPATH=${BREWPYPATH}/include/python3.10
BREWPYLIBPATH=${BREWPYPATH}/lib
export LIBRARY_PATH=${MANYO_PATH}/lib:/opt/homebrew/lib:/opt/homebrew/opt/openssl/lib:/opt/homebrew/Cellar/libomp/15.0.6/lib:${BREWPYLIBPATH}
export CPLUS_INCLUDE_PATH=/opt/homebrew/include/:/opt/homebrew/opt/openssl/include:/opt/homebrew/Cellar/libomp/15.0.6/include:${BREWPYINCPATH}:${MANYO_PATH}/include:${MANYO_PATH}/include/nexus
export DYLD_LIBRARY_PATH=${LIBRARY_PATH}
export INSTALL_PATH=${MANYO_PATH}
export PYTHONPATH=${MANYO_PATH}
export PyVer=3.10
