#!/usr/bin/python

import Manyo as m
import Peak as p

peak0 = p.Peak(p.SHOLDER_I, -2.5,  50000.0, 4.5, -7.0, -2.5)
peak1 = p.Peak(p.PEAK,       0.0, 100000.0, 2.5, -2.5,  2.5)
peak2 = p.Peak(p.SHOLDER_D,  2.5,  50000.0, 4.5,  2.5,  7.0)
p.Peak.title()
peak0.dump()
peak1.dump()
peak2.dump()

peak4 = p.Peak(peak1)
peak5 = peak2
peak4.dump()
peak5.dump()

v=peak0.toVector()
for i in range(p.Peak.shortSize):
    print "v[", i, "]=", v[i]
print ""

v=peak0.toFullVector()
for i in range(p.Peak.fullSize):
    print "v[", i, "]=", v[i]
print ""

print p.PEAK.value, p.PEAK.symbol


