#ifndef AdvSliceQECHOPPER
#define AdvSliceQECHOPPER

#include "Header.hh"
#include "OperatorBase.hh"
#include "ElementContainer.hh"
#include "ElementContainerArray.hh"
#include "ElementContainerMatrix.hh"
#include "HeaderBase.hh"
#include "StringTools.hh"

//////////////////////////////////
// Solid Angle Correction for DNA
// Advance soft 12/03/06
/////////////////////////////////

//!
/*!
 *
 *
 *
 *
 */
class AdvSliceQE :
  public ManipulatorBase< ElementContainerMatrix >
{
private:
    std::string commentHead;
    void Initialize();
 StringTools *stools;
public:
    AdvSliceQE();
        //!< Constructor
        /*!<
         */
        //!< Constructor
        /*!<
         *  @param ecm ElementContainerMatrix for the target
         */
    ~AdvSliceQE();
        //!< Destructor
        /*!<
         */

    ElementContainer Slice(ElementContainerArray *eca ,double lowerValue, double upperValue);


};

#endif
