#ifndef EFFICIENCYCORRECTIONTAKUMI
#define EFFICIENCYCORRECTIONTAKUMI

#include "Header.hh"
#include "UtsusemiHeader.hh"
#include "ManipulatorBase.hh"
#include "ElementContainerMatrix.hh"
#include "EmakiDetectorInfoEditorReadout.hh"

////////////////////////////////////////
// EfficiencyCorrectionTakumi
////////////////////////////////////////

//! Performe a detection efficiency corretion
/*!
 *
 */

class EfficiencyCorrectionTakumi:
    public ManipulatorBase< ElementContainerMatrix >{
private:
    std::string message_tag;

public:
    EfficiencyCorrectionTakumi();

    bool Execute(std::string filename);

};

#endif
