# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DetectMapQ.ui'
#
# Created: Mon Jan 20 19:07:46 2020
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_mapframe(object):
    def setupUi(self, mapframe):
        mapframe.setObjectName("mapframe")
        mapframe.resize(985, 778)
        self.centralwidget = QtGui.QWidget(mapframe)
        self.centralwidget.setObjectName("centralwidget")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(10, 10, 301, 51))
        self.groupBox.setObjectName("groupBox")
        self.tof1 = QtGui.QLineEdit(self.groupBox)
        self.tof1.setGeometry(QtCore.QRect(10, 20, 80, 25))
        self.tof1.setObjectName("tof1")
        self.tof2 = QtGui.QLineEdit(self.groupBox)
        self.tof2.setGeometry(QtCore.QRect(110, 20, 80, 25))
        self.tof2.setObjectName("tof2")
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setGeometry(QtCore.QRect(100, 20, 16, 25))
        self.label.setObjectName("label")
        self.btRedraw = QtGui.QPushButton(self.groupBox)
        self.btRedraw.setGeometry(QtCore.QRect(200, 20, 80, 30))
        self.btRedraw.setObjectName("btRedraw")
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setGeometry(QtCore.QRect(10, 80, 301, 51))
        self.groupBox_2.setObjectName("groupBox_2")
        self.psd0 = QtGui.QLineEdit(self.groupBox_2)
        self.psd0.setGeometry(QtCore.QRect(80, 20, 80, 25))
        self.psd0.setObjectName("psd0")
        self.bt2dPlot = QtGui.QPushButton(self.groupBox_2)
        self.bt2dPlot.setGeometry(QtCore.QRect(170, 20, 100, 30))
        self.bt2dPlot.setObjectName("bt2dPlot")
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setGeometry(QtCore.QRect(20, 20, 66, 17))
        self.label_2.setObjectName("label_2")
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setGeometry(QtCore.QRect(320, 10, 211, 121))
        self.groupBox_3.setObjectName("groupBox_3")
        self.label_3 = QtGui.QLabel(self.groupBox_3)
        self.label_3.setGeometry(QtCore.QRect(70, 30, 66, 17))
        self.label_3.setObjectName("label_3")
        self.label_4 = QtGui.QLabel(self.groupBox_3)
        self.label_4.setGeometry(QtCore.QRect(160, 30, 41, 17))
        self.label_4.setObjectName("label_4")
        self.label_5 = QtGui.QLabel(self.groupBox_3)
        self.label_5.setGeometry(QtCore.QRect(20, 50, 41, 25))
        self.label_5.setObjectName("label_5")
        self.label_6 = QtGui.QLabel(self.groupBox_3)
        self.label_6.setGeometry(QtCore.QRect(20, 90, 41, 25))
        self.label_6.setObjectName("label_6")
        self.PSD1 = QtGui.QLineEdit(self.groupBox_3)
        self.PSD1.setGeometry(QtCore.QRect(70, 50, 50, 25))
        self.PSD1.setObjectName("PSD1")
        self.PSD2 = QtGui.QLineEdit(self.groupBox_3)
        self.PSD2.setGeometry(QtCore.QRect(70, 90, 50, 25))
        self.PSD2.setObjectName("PSD2")
        self.pixel1 = QtGui.QLineEdit(self.groupBox_3)
        self.pixel1.setGeometry(QtCore.QRect(150, 50, 50, 25))
        self.pixel1.setObjectName("pixel1")
        self.pixel2 = QtGui.QLineEdit(self.groupBox_3)
        self.pixel2.setGeometry(QtCore.QRect(150, 90, 50, 25))
        self.pixel2.setObjectName("pixel2")
        self.label_7 = QtGui.QLabel(self.groupBox_3)
        self.label_7.setGeometry(QtCore.QRect(130, 50, 16, 25))
        self.label_7.setObjectName("label_7")
        self.label_8 = QtGui.QLabel(self.groupBox_3)
        self.label_8.setGeometry(QtCore.QRect(130, 90, 16, 25))
        self.label_8.setObjectName("label_8")
        self.cbXaxisOfPlot = QtGui.QComboBox(self.centralwidget)
        self.cbXaxisOfPlot.setGeometry(QtCore.QRect(560, 10, 100, 27))
        self.cbXaxisOfPlot.setObjectName("cbXaxisOfPlot")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.btAppend = QtGui.QPushButton(self.centralwidget)
        self.btAppend.setGeometry(QtCore.QRect(560, 40, 150, 27))
        self.btAppend.setObjectName("btAppend")
        self.btSumm = QtGui.QPushButton(self.centralwidget)
        self.btSumm.setGeometry(QtCore.QRect(560, 70, 150, 27))
        self.btSumm.setObjectName("btSumm")
        self.btNewPlot = QtGui.QPushButton(self.centralwidget)
        self.btNewPlot.setGeometry(QtCore.QRect(560, 100, 150, 27))
        self.btNewPlot.setObjectName("btNewPlot")
        self.label_9 = QtGui.QLabel(self.centralwidget)
        self.label_9.setGeometry(QtCore.QRect(720, 10, 66, 17))
        self.label_9.setObjectName("label_9")
        self.ckconv = QtGui.QCheckBox(self.centralwidget)
        self.ckconv.setGeometry(QtCore.QRect(810, 10, 151, 22))
        self.ckconv.setObjectName("ckconv")
        self.plotList = QtGui.QListWidget(self.centralwidget)
        self.plotList.setGeometry(QtCore.QRect(720, 30, 200, 115))
        self.plotList.setObjectName("plotList")
        self.slider = QtGui.QSlider(self.centralwidget)
        self.slider.setGeometry(QtCore.QRect(0, 230, 30, 350))
        self.slider.setProperty("value", 99)
        self.slider.setOrientation(QtCore.Qt.Vertical)
        self.slider.setInvertedAppearance(False)
        self.slider.setObjectName("slider")
        self.mapPanel = QtGui.QWidget(self.centralwidget)
        self.mapPanel.setGeometry(QtCore.QRect(30, 160, 950, 500))
        self.mapPanel.setObjectName("mapPanel")
        self.groupBox_4 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_4.setGeometry(QtCore.QRect(30, 670, 401, 51))
        self.groupBox_4.setObjectName("groupBox_4")
        self.ymin = QtGui.QLineEdit(self.groupBox_4)
        self.ymin.setGeometry(QtCore.QRect(20, 20, 100, 25))
        self.ymin.setObjectName("ymin")
        self.ymax = QtGui.QLineEdit(self.groupBox_4)
        self.ymax.setGeometry(QtCore.QRect(130, 20, 100, 25))
        self.ymax.setObjectName("ymax")
        self.label_10 = QtGui.QLabel(self.groupBox_4)
        self.label_10.setGeometry(QtCore.QRect(120, 20, 16, 25))
        self.label_10.setObjectName("label_10")
        self.btIntSet = QtGui.QPushButton(self.groupBox_4)
        self.btIntSet.setGeometry(QtCore.QRect(240, 20, 80, 30))
        self.btIntSet.setObjectName("btIntSet")
        self.btIntClear = QtGui.QPushButton(self.groupBox_4)
        self.btIntClear.setGeometry(QtCore.QRect(340, 20, 50, 30))
        self.btIntClear.setObjectName("btIntClear")
        self.infotxt = QtGui.QLabel(self.centralwidget)
        self.infotxt.setGeometry(QtCore.QRect(450, 670, 500, 55))
        self.infotxt.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.infotxt.setObjectName("infotxt")
        self.line = QtGui.QFrame(self.centralwidget)
        self.line.setGeometry(QtCore.QRect(0, 140, 981, 16))
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        mapframe.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(mapframe)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 985, 25))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuDisplay = QtGui.QMenu(self.menubar)
        self.menuDisplay.setObjectName("menuDisplay")
        mapframe.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(mapframe)
        self.statusbar.setObjectName("statusbar")
        mapframe.setStatusBar(self.statusbar)
        self.actionOpen = QtGui.QAction(mapframe)
        self.actionOpen.setObjectName("actionOpen")
        self.actionSave = QtGui.QAction(mapframe)
        self.actionSave.setObjectName("actionSave")
        self.actionSave_as = QtGui.QAction(mapframe)
        self.actionSave_as.setObjectName("actionSave_as")
        self.actionPrint = QtGui.QAction(mapframe)
        self.actionPrint.setObjectName("actionPrint")
        self.actionExit = QtGui.QAction(mapframe)
        self.actionExit.setObjectName("actionExit")
        self.actionNew_Plotter = QtGui.QAction(mapframe)
        self.actionNew_Plotter.setObjectName("actionNew_Plotter")
        self.actionLog_Scale = QtGui.QAction(mapframe)
        self.actionLog_Scale.setCheckable(True)
        self.actionLog_Scale.setObjectName("actionLog_Scale")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPrint)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menuDisplay.addAction(self.actionNew_Plotter)
        self.menuDisplay.addAction(self.actionLog_Scale)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuDisplay.menuAction())

        self.retranslateUi(mapframe)
        QtCore.QMetaObject.connectSlotsByName(mapframe)

    def retranslateUi(self, mapframe):
        mapframe.setWindowTitle(QtGui.QApplication.translate("mapframe", "Detect Map", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("mapframe", "X-axis Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.btRedraw.setText(QtGui.QApplication.translate("mapframe", "Redraw", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("mapframe", "GroupBox", None, QtGui.QApplication.UnicodeUTF8))
        self.bt2dPlot.setText(QtGui.QApplication.translate("mapframe", "2D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("mapframe", "DetNo. :", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("mapframe", "DetId Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("mapframe", "Detector", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("mapframe", "Pixel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("mapframe", "From", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("mapframe", "To", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(0, QtGui.QApplication.translate("mapframe", "TOF", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(1, QtGui.QApplication.translate("mapframe", "Detector", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(2, QtGui.QApplication.translate("mapframe", "Pixel", None, QtGui.QApplication.UnicodeUTF8))
        self.btAppend.setText(QtGui.QApplication.translate("mapframe", "Append >>", None, QtGui.QApplication.UnicodeUTF8))
        self.btSumm.setText(QtGui.QApplication.translate("mapframe", "Append Sum >>", None, QtGui.QApplication.UnicodeUTF8))
        self.btNewPlot.setText(QtGui.QApplication.translate("mapframe", "New Plotter", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("mapframe", "Plotters", None, QtGui.QApplication.UnicodeUTF8))
        self.ckconv.setText(QtGui.QApplication.translate("mapframe", "Convert toEnergy", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_4.setTitle(QtGui.QApplication.translate("mapframe", "Intensity Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.btIntSet.setText(QtGui.QApplication.translate("mapframe", "Apply", None, QtGui.QApplication.UnicodeUTF8))
        self.btIntClear.setText(QtGui.QApplication.translate("mapframe", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.infotxt.setText(QtGui.QApplication.translate("mapframe", "Det-ID", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("mapframe", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.menuDisplay.setTitle(QtGui.QApplication.translate("mapframe", "Display", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen.setText(QtGui.QApplication.translate("mapframe", "Open ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setText(QtGui.QApplication.translate("mapframe", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as.setText(QtGui.QApplication.translate("mapframe", "Save as ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrint.setText(QtGui.QApplication.translate("mapframe", "Print ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setText(QtGui.QApplication.translate("mapframe", "Exit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Plotter.setText(QtGui.QApplication.translate("mapframe", "New Plotter ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLog_Scale.setText(QtGui.QApplication.translate("mapframe", "Log Scale", None, QtGui.QApplication.UnicodeUTF8))

