#!/bin/bash

pyside6-uic D4mat2SlicerQ_Main.ui -o ui6_D4mat2SlicerQ.py
pyside6-uic D4mat2SlicerQ_Sub.ui -o ui_tmp1.py
pyside6-uic D4mat2SlicerQ_XtalParam.ui -o ui_tmp2.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide2 import QtCore, QtGui" >> ui6_D4mat2SlicerQ.py
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide2 import QtCore, QtGui" >> ui6_D4mat2SlicerQ.py

rm ui_tmp1.py ui_tmp2.py
