#ifndef FUNCCOMV_HH
#define FUNCCOMV_HH

#include "Header.hh"
#include "OperatorBase.hh"
#include "ElementContainer.hh"
#include "ElementContainerArray.hh"
#include "ElementContainerMatrix.hh"
#include <sstream>

class AdvFuncConv {

    public:
        AdvFuncConv();
        void SetResolutionEC(ElementContainer ec);
        vector <Double> ResolTestXbin(Double initial,Double final,Double delta);
        vector <Double> ResolTestYbin(Double initial,Double final,Double delta,Double a);
        void ReadResolutionFile();
        vector <Double> ResX;
        vector <Double> ResY;
};

#endif
