#include "AdvMultiDataLevmarConsts.hh"

/** class name */
const string AdvMultiDataLevmarConsts::className=string("AdvMultiDataLevmarConsts");

/** key for partial R factor */
const string AdvMultiDataLevmarConsts::PARTIAL_R_FACTOR=string("partial R factor");
/** key for partial residual error norm */
const string AdvMultiDataLevmarConsts::PARTIAL_RESIDUAL_ERR_NORM=string("partial residual error norm");
