#!/bin/bash

pyside-uic SequencerQ_LDR_Main.ui -o ui_SequencerQ_LDR.py
pyside-uic SequencerQ_Save.ui -o ui_tmp1.py
pyside-uic SequencerQ_Vis.ui -o ui_tmp2.py
pyside-uic SequencerQ_LDR_Serv.ui -o ui_tmp3.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_SequencerQ_LDR.py
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_SequencerQ_LDR.py
cat ui_tmp3.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_SequencerQ_LDR.py

cp  ui_SequencerQ_LDR.py ../
rm ui_tmp1.py ui_tmp2.py ui_tmp3.py ui_SequencerQ_LDR.py
