# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'SAS_UBSlicer.ui'
##
## Created by: Qt User Interface Compiler version 6.4.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGroupBox,
    QLabel, QLineEdit, QMainWindow, QMenu,
    QMenuBar, QPushButton, QSizePolicy, QStatusBar,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(594, 469)
        self.actionLoad = QAction(MainWindow)
        self.actionLoad.setObjectName(u"actionLoad")
        self.actionExit = QAction(MainWindow)
        self.actionExit.setObjectName(u"actionExit")
        self.actionLoad_Param = QAction(MainWindow)
        self.actionLoad_Param.setObjectName(u"actionLoad_Param")
        self.actionSave_Param = QAction(MainWindow)
        self.actionSave_Param.setObjectName(u"actionSave_Param")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.PanelProj = QGroupBox(self.centralwidget)
        self.PanelProj.setObjectName(u"PanelProj")
        self.PanelProj.setGeometry(QRect(10, 130, 541, 111))
        self.txt_proj_ax11 = QLineEdit(self.PanelProj)
        self.txt_proj_ax11.setObjectName(u"txt_proj_ax11")
        self.txt_proj_ax11.setGeometry(QRect(40, 40, 55, 20))
        self.label_pr5 = QLabel(self.PanelProj)
        self.label_pr5.setObjectName(u"label_pr5")
        self.label_pr5.setGeometry(QRect(10, 40, 31, 17))
        self.txt_proj_ax12 = QLineEdit(self.PanelProj)
        self.txt_proj_ax12.setObjectName(u"txt_proj_ax12")
        self.txt_proj_ax12.setGeometry(QRect(100, 40, 55, 20))
        self.txt_proj_ax13 = QLineEdit(self.PanelProj)
        self.txt_proj_ax13.setObjectName(u"txt_proj_ax13")
        self.txt_proj_ax13.setGeometry(QRect(160, 40, 55, 20))
        self.label_pr6 = QLabel(self.PanelProj)
        self.label_pr6.setObjectName(u"label_pr6")
        self.label_pr6.setGeometry(QRect(10, 60, 31, 17))
        self.txt_proj_ax23 = QLineEdit(self.PanelProj)
        self.txt_proj_ax23.setObjectName(u"txt_proj_ax23")
        self.txt_proj_ax23.setGeometry(QRect(160, 60, 55, 20))
        self.txt_proj_ax21 = QLineEdit(self.PanelProj)
        self.txt_proj_ax21.setObjectName(u"txt_proj_ax21")
        self.txt_proj_ax21.setGeometry(QRect(40, 60, 55, 20))
        self.txt_proj_ax22 = QLineEdit(self.PanelProj)
        self.txt_proj_ax22.setObjectName(u"txt_proj_ax22")
        self.txt_proj_ax22.setGeometry(QRect(100, 60, 55, 20))
        self.label_pr7 = QLabel(self.PanelProj)
        self.label_pr7.setObjectName(u"label_pr7")
        self.label_pr7.setGeometry(QRect(10, 80, 31, 17))
        self.txt_proj_t1 = QLineEdit(self.PanelProj)
        self.txt_proj_t1.setObjectName(u"txt_proj_t1")
        self.txt_proj_t1.setGeometry(QRect(240, 40, 91, 20))
        self.txt_proj_t2 = QLineEdit(self.PanelProj)
        self.txt_proj_t2.setObjectName(u"txt_proj_t2")
        self.txt_proj_t2.setGeometry(QRect(240, 60, 91, 20))
        self.label_pr1 = QLabel(self.PanelProj)
        self.label_pr1.setObjectName(u"label_pr1")
        self.label_pr1.setGeometry(QRect(40, 20, 55, 20))
        self.label_pr1.setAlignment(Qt.AlignCenter)
        self.label_pr2 = QLabel(self.PanelProj)
        self.label_pr2.setObjectName(u"label_pr2")
        self.label_pr2.setGeometry(QRect(100, 20, 55, 20))
        self.label_pr2.setAlignment(Qt.AlignCenter)
        self.label_pr3 = QLabel(self.PanelProj)
        self.label_pr3.setObjectName(u"label_pr3")
        self.label_pr3.setGeometry(QRect(160, 20, 55, 20))
        self.label_pr3.setAlignment(Qt.AlignCenter)
        self.label_pr4 = QLabel(self.PanelProj)
        self.label_pr4.setObjectName(u"label_pr4")
        self.label_pr4.setGeometry(QRect(244, 20, 91, 20))
        self.label_pr4.setAlignment(Qt.AlignCenter)
        self.txt_proj_t3 = QLineEdit(self.PanelProj)
        self.txt_proj_t3.setObjectName(u"txt_proj_t3")
        self.txt_proj_t3.setGeometry(QRect(240, 80, 91, 20))
        self.txt_proj_ax33 = QLineEdit(self.PanelProj)
        self.txt_proj_ax33.setObjectName(u"txt_proj_ax33")
        self.txt_proj_ax33.setGeometry(QRect(160, 80, 55, 20))
        self.txt_proj_ax31 = QLineEdit(self.PanelProj)
        self.txt_proj_ax31.setObjectName(u"txt_proj_ax31")
        self.txt_proj_ax31.setGeometry(QRect(40, 80, 55, 20))
        self.txt_proj_ax32 = QLineEdit(self.PanelProj)
        self.txt_proj_ax32.setObjectName(u"txt_proj_ax32")
        self.txt_proj_ax32.setGeometry(QRect(100, 80, 55, 20))
        self.btProj = QPushButton(self.PanelProj)
        self.btProj.setObjectName(u"btProj")
        self.btProj.setGeometry(QRect(420, 80, 99, 27))
        self.btClr_proj = QPushButton(self.PanelProj)
        self.btClr_proj.setObjectName(u"btClr_proj")
        self.btClr_proj.setGeometry(QRect(420, 40, 51, 27))
        self.txt_proj_u1 = QLineEdit(self.PanelProj)
        self.txt_proj_u1.setObjectName(u"txt_proj_u1")
        self.txt_proj_u1.setGeometry(QRect(340, 40, 61, 20))
        self.txt_proj_u2 = QLineEdit(self.PanelProj)
        self.txt_proj_u2.setObjectName(u"txt_proj_u2")
        self.txt_proj_u2.setGeometry(QRect(340, 60, 61, 20))
        self.txt_proj_u3 = QLineEdit(self.PanelProj)
        self.txt_proj_u3.setObjectName(u"txt_proj_u3")
        self.txt_proj_u3.setGeometry(QRect(340, 80, 61, 20))
        self.label_pr4_2 = QLabel(self.PanelProj)
        self.label_pr4_2.setObjectName(u"label_pr4_2")
        self.label_pr4_2.setGeometry(QRect(340, 20, 51, 20))
        self.label_pr4_2.setAlignment(Qt.AlignCenter)
        self.PanelSlice = QGroupBox(self.centralwidget)
        self.PanelSlice.setObjectName(u"PanelSlice")
        self.PanelSlice.setGeometry(QRect(10, 250, 571, 161))
        self.PanelSlice.setFlat(False)
        self.label_sl4 = QLabel(self.PanelSlice)
        self.label_sl4.setObjectName(u"label_sl4")
        self.label_sl4.setGeometry(QRect(0, 70, 31, 17))
        self.txt_sl_ax11 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax11.setObjectName(u"txt_sl_ax11")
        self.txt_sl_ax11.setGeometry(QRect(30, 70, 60, 20))
        self.label_sl5 = QLabel(self.PanelSlice)
        self.label_sl5.setObjectName(u"label_sl5")
        self.label_sl5.setGeometry(QRect(100, 70, 16, 17))
        self.txt_sl_ax12 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax12.setObjectName(u"txt_sl_ax12")
        self.txt_sl_ax12.setGeometry(QRect(120, 70, 60, 20))
        self.btAutoFillAx1 = QPushButton(self.PanelSlice)
        self.btAutoFillAx1.setObjectName(u"btAutoFillAx1")
        self.btAutoFillAx1.setGeometry(QRect(190, 70, 24, 24))
        self.cb_sl_ax1 = QComboBox(self.PanelSlice)
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.setObjectName(u"cb_sl_ax1")
        self.cb_sl_ax1.setGeometry(QRect(230, 70, 100, 25))
        self.txt_sl_ax13 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax13.setObjectName(u"txt_sl_ax13")
        self.txt_sl_ax13.setGeometry(QRect(340, 70, 60, 20))
        self.btAutoFillAx2 = QPushButton(self.PanelSlice)
        self.btAutoFillAx2.setObjectName(u"btAutoFillAx2")
        self.btAutoFillAx2.setGeometry(QRect(190, 100, 24, 24))
        self.label_sl7 = QLabel(self.PanelSlice)
        self.label_sl7.setObjectName(u"label_sl7")
        self.label_sl7.setGeometry(QRect(100, 100, 16, 17))
        self.txt_sl_ax22 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax22.setObjectName(u"txt_sl_ax22")
        self.txt_sl_ax22.setGeometry(QRect(120, 100, 60, 20))
        self.txt_sl_ax23 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax23.setObjectName(u"txt_sl_ax23")
        self.txt_sl_ax23.setGeometry(QRect(340, 100, 60, 20))
        self.cb_sl_ax2 = QComboBox(self.PanelSlice)
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.setObjectName(u"cb_sl_ax2")
        self.cb_sl_ax2.setGeometry(QRect(230, 100, 100, 25))
        self.label_sl6 = QLabel(self.PanelSlice)
        self.label_sl6.setObjectName(u"label_sl6")
        self.label_sl6.setGeometry(QRect(0, 100, 31, 17))
        self.txt_sl_ax21 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax21.setObjectName(u"txt_sl_ax21")
        self.txt_sl_ax21.setGeometry(QRect(30, 100, 60, 20))
        self.btAutoFillAx3 = QPushButton(self.PanelSlice)
        self.btAutoFillAx3.setObjectName(u"btAutoFillAx3")
        self.btAutoFillAx3.setGeometry(QRect(190, 130, 24, 24))
        self.label_sl9 = QLabel(self.PanelSlice)
        self.label_sl9.setObjectName(u"label_sl9")
        self.label_sl9.setGeometry(QRect(100, 130, 16, 17))
        self.txt_sl_ax32 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax32.setObjectName(u"txt_sl_ax32")
        self.txt_sl_ax32.setGeometry(QRect(120, 130, 60, 20))
        self.txt_sl_ax33 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax33.setObjectName(u"txt_sl_ax33")
        self.txt_sl_ax33.setGeometry(QRect(340, 130, 60, 20))
        self.cb_sl_ax3 = QComboBox(self.PanelSlice)
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.setObjectName(u"cb_sl_ax3")
        self.cb_sl_ax3.setGeometry(QRect(230, 130, 100, 25))
        self.label_sl8 = QLabel(self.PanelSlice)
        self.label_sl8.setObjectName(u"label_sl8")
        self.label_sl8.setGeometry(QRect(0, 130, 31, 17))
        self.txt_sl_ax31 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax31.setObjectName(u"txt_sl_ax31")
        self.txt_sl_ax31.setGeometry(QRect(30, 130, 60, 20))
        self.btAutoFill = QPushButton(self.PanelSlice)
        self.btAutoFill.setObjectName(u"btAutoFill")
        self.btAutoFill.setGeometry(QRect(130, 40, 51, 27))
        self.label_sl1 = QLabel(self.PanelSlice)
        self.label_sl1.setObjectName(u"label_sl1")
        self.label_sl1.setGeometry(QRect(60, 40, 51, 17))
        self.label_sl2 = QLabel(self.PanelSlice)
        self.label_sl2.setObjectName(u"label_sl2")
        self.label_sl2.setGeometry(QRect(240, 40, 81, 17))
        self.label_sl3 = QLabel(self.PanelSlice)
        self.label_sl3.setObjectName(u"label_sl3")
        self.label_sl3.setGeometry(QRect(350, 40, 51, 17))
        self.btSlice = QPushButton(self.PanelSlice)
        self.btSlice.setObjectName(u"btSlice")
        self.btSlice.setGeometry(QRect(420, 120, 120, 30))
        self.PanelUB = QGroupBox(self.centralwidget)
        self.PanelUB.setObjectName(u"PanelUB")
        self.PanelUB.setGeometry(QRect(10, 20, 541, 101))
        self.txt12_ub = QLineEdit(self.PanelUB)
        self.txt12_ub.setObjectName(u"txt12_ub")
        self.txt12_ub.setGeometry(QRect(170, 30, 60, 20))
        self.txt22_ub = QLineEdit(self.PanelUB)
        self.txt22_ub.setObjectName(u"txt22_ub")
        self.txt22_ub.setGeometry(QRect(170, 50, 60, 20))
        self.txt23_ub = QLineEdit(self.PanelUB)
        self.txt23_ub.setObjectName(u"txt23_ub")
        self.txt23_ub.setGeometry(QRect(240, 50, 60, 20))
        self.txt21_ub = QLineEdit(self.PanelUB)
        self.txt21_ub.setObjectName(u"txt21_ub")
        self.txt21_ub.setGeometry(QRect(100, 50, 60, 20))
        self.txt32_ub = QLineEdit(self.PanelUB)
        self.txt32_ub.setObjectName(u"txt32_ub")
        self.txt32_ub.setGeometry(QRect(170, 70, 60, 20))
        self.txt11_ub = QLineEdit(self.PanelUB)
        self.txt11_ub.setObjectName(u"txt11_ub")
        self.txt11_ub.setGeometry(QRect(100, 30, 60, 20))
        self.txt33_ub = QLineEdit(self.PanelUB)
        self.txt33_ub.setObjectName(u"txt33_ub")
        self.txt33_ub.setGeometry(QRect(240, 70, 60, 20))
        self.txt13_ub = QLineEdit(self.PanelUB)
        self.txt13_ub.setObjectName(u"txt13_ub")
        self.txt13_ub.setGeometry(QRect(240, 30, 60, 20))
        self.txt31_ub = QLineEdit(self.PanelUB)
        self.txt31_ub.setObjectName(u"txt31_ub")
        self.txt31_ub.setGeometry(QRect(100, 70, 60, 20))
        self.label_ub1 = QLabel(self.PanelUB)
        self.label_ub1.setObjectName(u"label_ub1")
        self.label_ub1.setGeometry(QRect(20, 30, 41, 21))
        self.label_ub2 = QLabel(self.PanelUB)
        self.label_ub2.setObjectName(u"label_ub2")
        self.label_ub2.setGeometry(QRect(20, 50, 41, 21))
        self.label_ub3 = QLabel(self.PanelUB)
        self.label_ub3.setObjectName(u"label_ub3")
        self.label_ub3.setGeometry(QRect(20, 70, 41, 21))
        self.label_ub6 = QLabel(self.PanelUB)
        self.label_ub6.setObjectName(u"label_ub6")
        self.label_ub6.setGeometry(QRect(360, 50, 41, 21))
        self.label_ub7 = QLabel(self.PanelUB)
        self.label_ub7.setObjectName(u"label_ub7")
        self.label_ub7.setGeometry(QRect(360, 70, 41, 21))
        self.label_ub5 = QLabel(self.PanelUB)
        self.label_ub5.setObjectName(u"label_ub5")
        self.label_ub5.setGeometry(QRect(360, 30, 41, 21))
        self.label_ub4 = QLabel(self.PanelUB)
        self.label_ub4.setObjectName(u"label_ub4")
        self.label_ub4.setGeometry(QRect(60, 50, 21, 17))
        self.line_ub1 = QFrame(self.PanelUB)
        self.line_ub1.setObjectName(u"line_ub1")
        self.line_ub1.setGeometry(QRect(10, 30, 3, 61))
        self.line_ub1.setFrameShape(QFrame.VLine)
        self.line_ub1.setFrameShadow(QFrame.Sunken)
        self.line_ub2 = QFrame(self.PanelUB)
        self.line_ub2.setObjectName(u"line_ub2")
        self.line_ub2.setGeometry(QRect(50, 30, 3, 61))
        self.line_ub2.setFrameShape(QFrame.VLine)
        self.line_ub2.setFrameShadow(QFrame.Sunken)
        self.line_ub3 = QFrame(self.PanelUB)
        self.line_ub3.setObjectName(u"line_ub3")
        self.line_ub3.setGeometry(QRect(90, 30, 3, 61))
        self.line_ub3.setFrameShape(QFrame.VLine)
        self.line_ub3.setFrameShadow(QFrame.Sunken)
        self.line_ub4 = QFrame(self.PanelUB)
        self.line_ub4.setObjectName(u"line_ub4")
        self.line_ub4.setGeometry(QRect(310, 30, 3, 61))
        self.line_ub4.setFrameShape(QFrame.VLine)
        self.line_ub4.setFrameShadow(QFrame.Sunken)
        self.label_ub8 = QLabel(self.PanelUB)
        self.label_ub8.setObjectName(u"label_ub8")
        self.label_ub8.setGeometry(QRect(320, 50, 21, 17))
        self.line_ub5 = QFrame(self.PanelUB)
        self.line_ub5.setObjectName(u"line_ub5")
        self.line_ub5.setGeometry(QRect(350, 30, 3, 61))
        self.line_ub5.setFrameShape(QFrame.VLine)
        self.line_ub5.setFrameShadow(QFrame.Sunken)
        self.line_ub6 = QFrame(self.PanelUB)
        self.line_ub6.setObjectName(u"line_ub6")
        self.line_ub6.setGeometry(QRect(380, 30, 3, 61))
        self.line_ub6.setFrameShape(QFrame.VLine)
        self.line_ub6.setFrameShadow(QFrame.Sunken)
        self.txt22_ub.raise_()
        self.txt23_ub.raise_()
        self.txt21_ub.raise_()
        self.txt32_ub.raise_()
        self.txt33_ub.raise_()
        self.txt31_ub.raise_()
        self.txt11_ub.raise_()
        self.txt12_ub.raise_()
        self.txt13_ub.raise_()
        self.label_ub1.raise_()
        self.label_ub2.raise_()
        self.label_ub3.raise_()
        self.label_ub6.raise_()
        self.label_ub7.raise_()
        self.label_ub5.raise_()
        self.label_ub4.raise_()
        self.line_ub1.raise_()
        self.line_ub2.raise_()
        self.line_ub3.raise_()
        self.line_ub4.raise_()
        self.label_ub8.raise_()
        self.line_ub5.raise_()
        self.line_ub6.raise_()
        self.label_runnotag = QLabel(self.centralwidget)
        self.label_runnotag.setObjectName(u"label_runnotag")
        self.label_runnotag.setGeometry(QRect(130, 0, 67, 17))
        self.label_runno = QLabel(self.centralwidget)
        self.label_runno.setObjectName(u"label_runno")
        self.label_runno.setGeometry(QRect(200, 0, 231, 17))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 594, 28))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.txt11_ub, self.txt12_ub)
        QWidget.setTabOrder(self.txt12_ub, self.txt13_ub)
        QWidget.setTabOrder(self.txt13_ub, self.txt21_ub)
        QWidget.setTabOrder(self.txt21_ub, self.txt22_ub)
        QWidget.setTabOrder(self.txt22_ub, self.txt23_ub)
        QWidget.setTabOrder(self.txt23_ub, self.txt31_ub)
        QWidget.setTabOrder(self.txt31_ub, self.txt32_ub)
        QWidget.setTabOrder(self.txt32_ub, self.txt33_ub)
        QWidget.setTabOrder(self.txt33_ub, self.txt_proj_ax11)
        QWidget.setTabOrder(self.txt_proj_ax11, self.txt_proj_ax12)
        QWidget.setTabOrder(self.txt_proj_ax12, self.txt_proj_ax13)
        QWidget.setTabOrder(self.txt_proj_ax13, self.txt_proj_ax21)
        QWidget.setTabOrder(self.txt_proj_ax21, self.txt_proj_ax22)
        QWidget.setTabOrder(self.txt_proj_ax22, self.txt_proj_ax23)
        QWidget.setTabOrder(self.txt_proj_ax23, self.txt_proj_ax31)
        QWidget.setTabOrder(self.txt_proj_ax31, self.txt_proj_ax32)
        QWidget.setTabOrder(self.txt_proj_ax32, self.txt_proj_ax33)
        QWidget.setTabOrder(self.txt_proj_ax33, self.txt_proj_t1)
        QWidget.setTabOrder(self.txt_proj_t1, self.txt_proj_u1)
        QWidget.setTabOrder(self.txt_proj_u1, self.txt_proj_t2)
        QWidget.setTabOrder(self.txt_proj_t2, self.txt_proj_u2)
        QWidget.setTabOrder(self.txt_proj_u2, self.txt_proj_t3)
        QWidget.setTabOrder(self.txt_proj_t3, self.txt_proj_u3)
        QWidget.setTabOrder(self.txt_proj_u3, self.btClr_proj)
        QWidget.setTabOrder(self.btClr_proj, self.btProj)
        QWidget.setTabOrder(self.btProj, self.btAutoFill)
        QWidget.setTabOrder(self.btAutoFill, self.txt_sl_ax11)
        QWidget.setTabOrder(self.txt_sl_ax11, self.txt_sl_ax12)
        QWidget.setTabOrder(self.txt_sl_ax12, self.btAutoFillAx1)
        QWidget.setTabOrder(self.btAutoFillAx1, self.cb_sl_ax1)
        QWidget.setTabOrder(self.cb_sl_ax1, self.txt_sl_ax13)
        QWidget.setTabOrder(self.txt_sl_ax13, self.txt_sl_ax21)
        QWidget.setTabOrder(self.txt_sl_ax21, self.txt_sl_ax22)
        QWidget.setTabOrder(self.txt_sl_ax22, self.btAutoFillAx2)
        QWidget.setTabOrder(self.btAutoFillAx2, self.cb_sl_ax2)
        QWidget.setTabOrder(self.cb_sl_ax2, self.txt_sl_ax23)
        QWidget.setTabOrder(self.txt_sl_ax23, self.txt_sl_ax31)
        QWidget.setTabOrder(self.txt_sl_ax31, self.txt_sl_ax32)
        QWidget.setTabOrder(self.txt_sl_ax32, self.btAutoFillAx3)
        QWidget.setTabOrder(self.btAutoFillAx3, self.cb_sl_ax3)
        QWidget.setTabOrder(self.cb_sl_ax3, self.txt_sl_ax33)
        QWidget.setTabOrder(self.txt_sl_ax33, self.btSlice)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionLoad)
        self.menuFile.addAction(self.actionLoad_Param)
        self.menuFile.addAction(self.actionSave_Param)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionLoad.setText(QCoreApplication.translate("MainWindow", u"Load ...", None))
        self.actionExit.setText(QCoreApplication.translate("MainWindow", u"Exit", None))
        self.actionLoad_Param.setText(QCoreApplication.translate("MainWindow", u"Load Param ...", None))
        self.actionSave_Param.setText(QCoreApplication.translate("MainWindow", u"Save Param ...", None))
        self.PanelProj.setTitle(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.label_pr5.setText(QCoreApplication.translate("MainWindow", u"Q1:", None))
        self.label_pr6.setText(QCoreApplication.translate("MainWindow", u"Q2:", None))
        self.label_pr7.setText(QCoreApplication.translate("MainWindow", u"Q3:", None))
        self.label_pr1.setText(QCoreApplication.translate("MainWindow", u"H", None))
        self.label_pr2.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_pr3.setText(QCoreApplication.translate("MainWindow", u"L", None))
        self.label_pr4.setText(QCoreApplication.translate("MainWindow", u"Axis Label", None))
        self.btProj.setText(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.btClr_proj.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.label_pr4_2.setText(QCoreApplication.translate("MainWindow", u"unit", None))
        self.PanelSlice.setTitle(QCoreApplication.translate("MainWindow", u"Slicing", None))
        self.label_sl4.setText(QCoreApplication.translate("MainWindow", u"Q1:", None))
        self.label_sl5.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.btAutoFillAx1.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.cb_sl_ax1.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax1.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax1.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.btAutoFillAx2.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label_sl7.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.cb_sl_ax2.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax2.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax2.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.label_sl6.setText(QCoreApplication.translate("MainWindow", u"Q2:", None))
        self.btAutoFillAx3.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label_sl9.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.cb_sl_ax3.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax3.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax3.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.label_sl8.setText(QCoreApplication.translate("MainWindow", u"Q3:", None))
        self.btAutoFill.setText(QCoreApplication.translate("MainWindow", u"Auto", None))
        self.label_sl1.setText(QCoreApplication.translate("MainWindow", u"Range", None))
        self.label_sl2.setText(QCoreApplication.translate("MainWindow", u"Plot Axis", None))
        self.label_sl3.setText(QCoreApplication.translate("MainWindow", u"Step", None))
        self.btSlice.setText(QCoreApplication.translate("MainWindow", u"Slice", None))
        self.PanelUB.setTitle(QCoreApplication.translate("MainWindow", u"UB matrix", None))
        self.label_ub1.setText(QCoreApplication.translate("MainWindow", u"Qx", None))
        self.label_ub2.setText(QCoreApplication.translate("MainWindow", u"Qy", None))
        self.label_ub3.setText(QCoreApplication.translate("MainWindow", u"Qz", None))
        self.label_ub6.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_ub7.setText(QCoreApplication.translate("MainWindow", u"L", None))
        self.label_ub5.setText(QCoreApplication.translate("MainWindow", u"H", None))
        self.label_ub4.setText(QCoreApplication.translate("MainWindow", u"=", None))
        self.label_ub8.setText(QCoreApplication.translate("MainWindow", u"X", None))
        self.label_runnotag.setText(QCoreApplication.translate("MainWindow", u"RunNo :", None))
        self.label_runno.setText(QCoreApplication.translate("MainWindow", u"None", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
    # retranslateUi

