#include <string>
#include "Header.hh"
#include "AdvMethodType.hh"

const AdvMethodType UNKNOWN_METHOD_TYPE  = { UNKNOWN_METHOD_ID,       std::string("unkown"),               std::string("unknown")   };
const AdvMethodType BSPLINE              = { BSPLINE_ID,              std::string("B spline"),             std::string("BS")        };
const AdvMethodType MOVING_AVERAGE       = { MOVING_AVERAGE_ID,       std::string("Moving Average"),       std::string("MA")        };
const AdvMethodType LEVMAR               = { LEVMAR_ID,               std::string("Levmar"),               std::string("LM")        };
const AdvMethodType NEW_LEVMAR           = { NEW_LEVMAR_ID,           std::string("New Levmar"),           std::string("NLM")       };
const AdvMethodType MULTIDATA_LEVMAR     = { MULTIDATA_LEVMAR_ID,     std::string("Multi Data Levmar"),    std::string("MDLM")      };
const AdvMethodType CONVOLUTION_BY_FFT   = { CONVOLUTION_BY_FFT_ID,   std::string("Convolution By FFT"),   std::string("CONVFFT")   };
const AdvMethodType DECONVOLUTION_BY_FFT = { DECONVOLUTION_BY_FFT_ID, std::string("Deconvolution By FFT"), std::string("DECONVFFT") };
const AdvMethodType EQUAL_SPACING        = { EQUAL_SPACING_ID,        std::string("Equal Spacing"),        std::string("ES")        };

