#include "UtsusemiRedisPubSubEventMonitorNeunet.hh"
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorNeunet::
UtsusemiRedisPubSubEventMonitorNeunet(){
    _MessageTag2 = "UtsusemiRedisPubSubEventMonitorNeunet::";
}
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorNeunet::
~UtsusemiRedisPubSubEventMonitorNeunet(){
}
//////////////////////////////////////////////////////////
void UtsusemiRedisPubSubEventMonitorNeunet::
OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId ){
    _EDC->SetElementContainerNeunet( pixelId, ec, caseId );
    SetMeasTimeSliced( ec->PutHeaderPointer() );
    SetKickers( ec->PutHeaderPointer(), caseId );
}
