# -*- coding: utf-8 -*-

import Manyo
import PlaneSlicer
import uGao.M2PlotPlusCui as mp
import sys


def main():
    argv = sys.argv
    argc = len(argv)

    orgX = float(argv[1])
    orgY = float(argv[2])
    orgZ = float(argv[3])
    org  = [orgX, orgY, orgZ]

    x0   = float(argv[4])
    x1   = float(argv[5])
    x2   = float(argv[6])
    x    = [x0, x1, x2]

    y0   = float(argv[7])
    y1   = float(argv[8])
    y2   = float(argv[9])
    y    = [y0, y1, y2]

    xbin = float(argv[10])
    ybin = float(argv[11])

    rangeMin = float(argv[12])
    rangeMax = float(argv[13])
    rngx = [rangeMin, rangeMax]

    rangeMin = float(argv[14])
    rangeMax = float(argv[15])
    rngy = [rangeMin, rangeMax]

    thick = float(argv[16])






if __name__ == '__main__':
    main()
