# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'D4matSlicerQ_Main.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QGroupBox,
    QHeaderView, QLabel, QLineEdit, QMainWindow,
    QMenu, QMenuBar, QPushButton, QSizePolicy,
    QStatusBar, QToolButton, QTreeWidget, QTreeWidgetItem,
    QWidget)

class Ui_D4matSlicerQtMain(object):
    def setupUi(self, D4matSlicerQtMain):
        if not D4matSlicerQtMain.objectName():
            D4matSlicerQtMain.setObjectName(u"D4matSlicerQtMain")
        D4matSlicerQtMain.resize(550, 720)
        self.actionMake_New_Matrix = QAction(D4matSlicerQtMain)
        self.actionMake_New_Matrix.setObjectName(u"actionMake_New_Matrix")
        self.actionOpen_Matrix = QAction(D4matSlicerQtMain)
        self.actionOpen_Matrix.setObjectName(u"actionOpen_Matrix")
        self.actionRebin_Matrix = QAction(D4matSlicerQtMain)
        self.actionRebin_Matrix.setObjectName(u"actionRebin_Matrix")
        self.actionSave_as_Text = QAction(D4matSlicerQtMain)
        self.actionSave_as_Text.setObjectName(u"actionSave_as_Text")
        self.actionExit = QAction(D4matSlicerQtMain)
        self.actionExit.setObjectName(u"actionExit")
        self.centralwidget = QWidget(D4matSlicerQtMain)
        self.centralwidget.setObjectName(u"centralwidget")
        self.panel1 = QGroupBox(self.centralwidget)
        self.panel1.setObjectName(u"panel1")
        self.panel1.setGeometry(QRect(10, 0, 530, 55))
        self.btMakeVirtualMat = QPushButton(self.panel1)
        self.btMakeVirtualMat.setObjectName(u"btMakeVirtualMat")
        self.btMakeVirtualMat.setGeometry(QRect(10, 20, 80, 25))
        self.btMakeVirtualMat.setAutoDefault(True)
        self.txtvmsg = QLabel(self.panel1)
        self.txtvmsg.setObjectName(u"txtvmsg")
        self.txtvmsg.setGeometry(QRect(100, 25, 421, 20))
        self.panel2 = QGroupBox(self.centralwidget)
        self.panel2.setObjectName(u"panel2")
        self.panel2.setGeometry(QRect(10, 55, 530, 55))
        self.panel2.setTitle(u"D4Matrix Files")
        self.panel2.setFlat(False)
        self.panel2.setCheckable(False)
        self.btOpenD4Matrix = QPushButton(self.panel2)
        self.btOpenD4Matrix.setObjectName(u"btOpenD4Matrix")
        self.btOpenD4Matrix.setGeometry(QRect(10, 20, 80, 25))
        self.btOpenD4Matrix.setAutoDefault(True)
        self.stPathToD4MatrixFiles = QLabel(self.panel2)
        self.stPathToD4MatrixFiles.setObjectName(u"stPathToD4MatrixFiles")
        self.stPathToD4MatrixFiles.setGeometry(QRect(100, 25, 421, 20))
        self.panel3 = QGroupBox(self.centralwidget)
        self.panel3.setObjectName(u"panel3")
        self.panel3.setGeometry(QRect(10, 110, 530, 131))
        self.label_2 = QLabel(self.panel3)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(80, 20, 60, 20))
        self.label_2.setAlignment(Qt.AlignCenter)
        self.label_3 = QLabel(self.panel3)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(210, 20, 60, 20))
        self.label_3.setAlignment(Qt.AlignCenter)
        self.label_4 = QLabel(self.panel3)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(300, 20, 60, 20))
        self.label_4.setAlignment(Qt.AlignCenter)
        self.v1txt = QLabel(self.panel3)
        self.v1txt.setObjectName(u"v1txt")
        self.v1txt.setGeometry(QRect(10, 40, 30, 20))
        self.txt1 = QLabel(self.panel3)
        self.txt1.setObjectName(u"txt1")
        self.txt1.setGeometry(QRect(40, 40, 60, 20))
        self.txt1.setAlignment(Qt.AlignCenter)
        self.label_6 = QLabel(self.panel3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(100, 40, 30, 20))
        self.label_6.setAlignment(Qt.AlignCenter)
        self.txt2 = QLabel(self.panel3)
        self.txt2.setObjectName(u"txt2")
        self.txt2.setGeometry(QRect(130, 40, 60, 20))
        self.txt2.setAlignment(Qt.AlignCenter)
        self.txt3 = QLabel(self.panel3)
        self.txt3.setObjectName(u"txt3")
        self.txt3.setGeometry(QRect(210, 40, 60, 20))
        self.txt3.setAlignment(Qt.AlignCenter)
        self.label_7 = QLabel(self.panel3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(100, 60, 30, 20))
        self.label_7.setAlignment(Qt.AlignCenter)
        self.txt5 = QLabel(self.panel3)
        self.txt5.setObjectName(u"txt5")
        self.txt5.setGeometry(QRect(130, 60, 60, 20))
        self.txt5.setAlignment(Qt.AlignCenter)
        self.txt4 = QLabel(self.panel3)
        self.txt4.setObjectName(u"txt4")
        self.txt4.setGeometry(QRect(40, 60, 60, 20))
        self.txt4.setAlignment(Qt.AlignCenter)
        self.v2txt = QLabel(self.panel3)
        self.v2txt.setObjectName(u"v2txt")
        self.v2txt.setGeometry(QRect(10, 60, 30, 20))
        self.txt6 = QLabel(self.panel3)
        self.txt6.setObjectName(u"txt6")
        self.txt6.setGeometry(QRect(210, 60, 60, 20))
        self.txt6.setAlignment(Qt.AlignCenter)
        self.label_8 = QLabel(self.panel3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(100, 80, 30, 20))
        self.label_8.setAlignment(Qt.AlignCenter)
        self.txt8 = QLabel(self.panel3)
        self.txt8.setObjectName(u"txt8")
        self.txt8.setGeometry(QRect(130, 80, 60, 20))
        self.txt8.setAlignment(Qt.AlignCenter)
        self.txt7 = QLabel(self.panel3)
        self.txt7.setObjectName(u"txt7")
        self.txt7.setGeometry(QRect(40, 80, 60, 20))
        self.txt7.setAlignment(Qt.AlignCenter)
        self.v3txt = QLabel(self.panel3)
        self.v3txt.setObjectName(u"v3txt")
        self.v3txt.setGeometry(QRect(10, 80, 30, 20))
        self.txt9 = QLabel(self.panel3)
        self.txt9.setObjectName(u"txt9")
        self.txt9.setGeometry(QRect(210, 80, 60, 20))
        self.txt9.setAlignment(Qt.AlignCenter)
        self.label_9 = QLabel(self.panel3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(100, 100, 30, 20))
        self.label_9.setAlignment(Qt.AlignCenter)
        self.txt11 = QLabel(self.panel3)
        self.txt11.setObjectName(u"txt11")
        self.txt11.setGeometry(QRect(130, 100, 60, 20))
        self.txt11.setAlignment(Qt.AlignCenter)
        self.txt10 = QLabel(self.panel3)
        self.txt10.setObjectName(u"txt10")
        self.txt10.setGeometry(QRect(40, 100, 60, 20))
        self.txt10.setAlignment(Qt.AlignCenter)
        self.v4txt = QLabel(self.panel3)
        self.v4txt.setObjectName(u"v4txt")
        self.v4txt.setGeometry(QRect(10, 100, 30, 20))
        self.txt12 = QLabel(self.panel3)
        self.txt12.setObjectName(u"txt12")
        self.txt12.setGeometry(QRect(210, 100, 60, 20))
        self.txt12.setAlignment(Qt.AlignCenter)
        self.v1title = QLineEdit(self.panel3)
        self.v1title.setObjectName(u"v1title")
        self.v1title.setGeometry(QRect(280, 40, 101, 20))
        self.v2title = QLineEdit(self.panel3)
        self.v2title.setObjectName(u"v2title")
        self.v2title.setGeometry(QRect(280, 60, 101, 20))
        self.v3title = QLineEdit(self.panel3)
        self.v3title.setObjectName(u"v3title")
        self.v3title.setGeometry(QRect(280, 80, 101, 20))
        self.v4title = QLineEdit(self.panel3)
        self.v4title.setObjectName(u"v4title")
        self.v4title.setGeometry(QRect(280, 100, 101, 20))
        self.btCloseMatrix = QPushButton(self.panel3)
        self.btCloseMatrix.setObjectName(u"btCloseMatrix")
        self.btCloseMatrix.setGeometry(QRect(464, 25, 60, 25))
        self.btCloseMatrix.setAutoDefault(True)
        self.panel31 = QGroupBox(self.panel3)
        self.panel31.setObjectName(u"panel31")
        self.panel31.setGeometry(QRect(460, 60, 71, 66))
        self.panel31.setFlat(False)
        self.btClearColumn = QPushButton(self.panel31)
        self.btClearColumn.setObjectName(u"btClearColumn")
        self.btClearColumn.setGeometry(QRect(5, 20, 60, 20))
        font = QFont()
        font.setPointSize(10)
        self.btClearColumn.setFont(font)
        self.btClearColumn.setAutoDefault(True)
        self.btSaveTitles = QPushButton(self.panel31)
        self.btSaveTitles.setObjectName(u"btSaveTitles")
        self.btSaveTitles.setGeometry(QRect(5, 40, 60, 20))
        self.btSaveTitles.setFont(font)
        self.btSaveTitles.setAutoDefault(True)
        self.v1unit = QLineEdit(self.panel3)
        self.v1unit.setObjectName(u"v1unit")
        self.v1unit.setGeometry(QRect(390, 40, 61, 20))
        self.v2unit = QLineEdit(self.panel3)
        self.v2unit.setObjectName(u"v2unit")
        self.v2unit.setGeometry(QRect(390, 60, 61, 20))
        self.v3unit = QLineEdit(self.panel3)
        self.v3unit.setObjectName(u"v3unit")
        self.v3unit.setGeometry(QRect(390, 80, 61, 20))
        self.v4unit = QLineEdit(self.panel3)
        self.v4unit.setObjectName(u"v4unit")
        self.v4unit.setGeometry(QRect(390, 100, 61, 20))
        self.label_17 = QLabel(self.panel3)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(390, 20, 60, 20))
        self.label_17.setAlignment(Qt.AlignCenter)
        self.panel4 = QGroupBox(self.centralwidget)
        self.panel4.setObjectName(u"panel4")
        self.panel4.setGeometry(QRect(10, 240, 530, 160))
        self.btAddSingleFile = QPushButton(self.panel4)
        self.btAddSingleFile.setObjectName(u"btAddSingleFile")
        self.btAddSingleFile.setGeometry(QRect(10, 130, 100, 25))
        self.btAddSingleFile.setAutoDefault(True)
        self.btAddFilesInDir = QPushButton(self.panel4)
        self.btAddFilesInDir.setObjectName(u"btAddFilesInDir")
        self.btAddFilesInDir.setGeometry(QRect(140, 130, 150, 25))
        self.btAddFilesInDir.setAutoDefault(True)
        self.btRemoveFile = QPushButton(self.panel4)
        self.btRemoveFile.setObjectName(u"btRemoveFile")
        self.btRemoveFile.setGeometry(QRect(420, 130, 100, 25))
        self.btRemoveFile.setAutoDefault(True)
        self.lbFileList = QTreeWidget(self.panel4)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.lbFileList.setHeaderItem(__qtreewidgetitem)
        self.lbFileList.setObjectName(u"lbFileList")
        self.lbFileList.setGeometry(QRect(10, 30, 510, 100))
        self.lbFileList.header().setVisible(False)
        self.panel5 = QGroupBox(self.centralwidget)
        self.panel5.setObjectName(u"panel5")
        self.panel5.setGeometry(QRect(10, 400, 530, 190))
        self.panel5.setStyleSheet(u"")
        self.panel5.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignVCenter)
        self.label_5 = QLabel(self.panel5)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(80, 30, 70, 20))
        self.label_5.setAlignment(Qt.AlignCenter)
        self.label_10 = QLabel(self.panel5)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(230, 30, 70, 20))
        self.label_10.setAlignment(Qt.AlignCenter)
        self.v1txt_2 = QLabel(self.panel5)
        self.v1txt_2.setObjectName(u"v1txt_2")
        self.v1txt_2.setGeometry(QRect(10, 55, 30, 20))
        self.v2txt_2 = QLabel(self.panel5)
        self.v2txt_2.setObjectName(u"v2txt_2")
        self.v2txt_2.setGeometry(QRect(10, 80, 30, 20))
        self.v4txt_2 = QLabel(self.panel5)
        self.v4txt_2.setObjectName(u"v4txt_2")
        self.v4txt_2.setGeometry(QRect(10, 130, 30, 20))
        self.v3txt_2 = QLabel(self.panel5)
        self.v3txt_2.setObjectName(u"v3txt_2")
        self.v3txt_2.setGeometry(QRect(10, 105, 30, 20))
        self.stxt1 = QLineEdit(self.panel5)
        self.stxt1.setObjectName(u"stxt1")
        self.stxt1.setGeometry(QRect(50, 55, 60, 20))
        self.label_11 = QLabel(self.panel5)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(110, 80, 30, 20))
        self.label_11.setAlignment(Qt.AlignCenter)
        self.label_12 = QLabel(self.panel5)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(110, 55, 30, 20))
        self.label_12.setAlignment(Qt.AlignCenter)
        self.label_13 = QLabel(self.panel5)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(110, 105, 30, 20))
        self.label_13.setAlignment(Qt.AlignCenter)
        self.label_14 = QLabel(self.panel5)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(110, 130, 30, 20))
        self.label_14.setAlignment(Qt.AlignCenter)
        self.stxt2 = QLineEdit(self.panel5)
        self.stxt2.setObjectName(u"stxt2")
        self.stxt2.setGeometry(QRect(140, 55, 60, 20))
        self.btAutoFillAx1 = QPushButton(self.panel5)
        self.btAutoFillAx1.setObjectName(u"btAutoFillAx1")
        self.btAutoFillAx1.setGeometry(QRect(210, 55, 32, 24))
        self.btAutoFillAx1.setAutoDefault(True)
        self.txtFold1 = QLineEdit(self.panel5)
        self.txtFold1.setObjectName(u"txtFold1")
        self.txtFold1.setGeometry(QRect(250, 55, 40, 20))
        self.v1cb = QComboBox(self.panel5)
        self.v1cb.addItem("")
        self.v1cb.addItem("")
        self.v1cb.addItem("")
        self.v1cb.addItem("")
        self.v1cb.setObjectName(u"v1cb")
        self.v1cb.setGeometry(QRect(370, 55, 80, 25))
        font1 = QFont()
        font1.setPointSize(10)
        font1.setBold(False)
        font1.setItalic(False)
        self.v1cb.setFont(font1)
        self.v1cb.setEditable(False)
        self.stxt3 = QLineEdit(self.panel5)
        self.stxt3.setObjectName(u"stxt3")
        self.stxt3.setEnabled(False)
        self.stxt3.setGeometry(QRect(460, 55, 60, 20))
        self.btAutoFillAx2 = QPushButton(self.panel5)
        self.btAutoFillAx2.setObjectName(u"btAutoFillAx2")
        self.btAutoFillAx2.setGeometry(QRect(210, 80, 32, 24))
        self.btAutoFillAx2.setAutoDefault(True)
        self.stxt5 = QLineEdit(self.panel5)
        self.stxt5.setObjectName(u"stxt5")
        self.stxt5.setGeometry(QRect(140, 80, 60, 20))
        self.v2cb = QComboBox(self.panel5)
        self.v2cb.addItem("")
        self.v2cb.addItem("")
        self.v2cb.addItem("")
        self.v2cb.addItem("")
        self.v2cb.setObjectName(u"v2cb")
        self.v2cb.setGeometry(QRect(370, 80, 80, 25))
        self.v2cb.setFont(font)
        self.v2cb.setEditable(False)
        self.txtFold2 = QLineEdit(self.panel5)
        self.txtFold2.setObjectName(u"txtFold2")
        self.txtFold2.setGeometry(QRect(250, 80, 40, 20))
        self.stxt6 = QLineEdit(self.panel5)
        self.stxt6.setObjectName(u"stxt6")
        self.stxt6.setEnabled(False)
        self.stxt6.setGeometry(QRect(460, 80, 60, 20))
        self.stxt4 = QLineEdit(self.panel5)
        self.stxt4.setObjectName(u"stxt4")
        self.stxt4.setGeometry(QRect(50, 80, 60, 20))
        self.btAutoFillAx3 = QPushButton(self.panel5)
        self.btAutoFillAx3.setObjectName(u"btAutoFillAx3")
        self.btAutoFillAx3.setGeometry(QRect(210, 105, 32, 24))
        self.btAutoFillAx3.setAutoDefault(True)
        self.stxt8 = QLineEdit(self.panel5)
        self.stxt8.setObjectName(u"stxt8")
        self.stxt8.setGeometry(QRect(140, 105, 60, 20))
        self.v3cb = QComboBox(self.panel5)
        self.v3cb.addItem("")
        self.v3cb.addItem("")
        self.v3cb.addItem("")
        self.v3cb.addItem("")
        self.v3cb.setObjectName(u"v3cb")
        self.v3cb.setGeometry(QRect(370, 105, 80, 25))
        self.v3cb.setFont(font)
        self.v3cb.setEditable(False)
        self.txtFold3 = QLineEdit(self.panel5)
        self.txtFold3.setObjectName(u"txtFold3")
        self.txtFold3.setGeometry(QRect(250, 105, 40, 20))
        self.stxt9 = QLineEdit(self.panel5)
        self.stxt9.setObjectName(u"stxt9")
        self.stxt9.setEnabled(False)
        self.stxt9.setGeometry(QRect(460, 105, 60, 20))
        self.stxt7 = QLineEdit(self.panel5)
        self.stxt7.setObjectName(u"stxt7")
        self.stxt7.setGeometry(QRect(50, 105, 60, 20))
        self.stxt10 = QLineEdit(self.panel5)
        self.stxt10.setObjectName(u"stxt10")
        self.stxt10.setGeometry(QRect(50, 130, 60, 20))
        self.stxt11 = QLineEdit(self.panel5)
        self.stxt11.setObjectName(u"stxt11")
        self.stxt11.setGeometry(QRect(140, 130, 60, 20))
        self.btAutoFillAx4 = QPushButton(self.panel5)
        self.btAutoFillAx4.setObjectName(u"btAutoFillAx4")
        self.btAutoFillAx4.setGeometry(QRect(210, 130, 32, 24))
        self.btAutoFillAx4.setAutoDefault(True)
        self.stxt12 = QLineEdit(self.panel5)
        self.stxt12.setObjectName(u"stxt12")
        self.stxt12.setEnabled(False)
        self.stxt12.setGeometry(QRect(460, 130, 60, 20))
        self.v4cb = QComboBox(self.panel5)
        self.v4cb.addItem("")
        self.v4cb.addItem("")
        self.v4cb.addItem("")
        self.v4cb.addItem("")
        self.v4cb.setObjectName(u"v4cb")
        self.v4cb.setGeometry(QRect(370, 130, 80, 25))
        self.v4cb.setFont(font)
        self.v4cb.setEditable(False)
        self.txtFold4 = QLineEdit(self.panel5)
        self.txtFold4.setObjectName(u"txtFold4")
        self.txtFold4.setGeometry(QRect(250, 130, 40, 20))
        self.btBack = QToolButton(self.panel5)
        self.btBack.setObjectName(u"btBack")
        self.btBack.setGeometry(QRect(250, 160, 25, 25))
        self.btBack.setArrowType(Qt.LeftArrow)
        self.btForward = QToolButton(self.panel5)
        self.btForward.setObjectName(u"btForward")
        self.btForward.setGeometry(QRect(290, 160, 25, 25))
        self.btForward.setArrowType(Qt.RightArrow)
        self.btSlice = QPushButton(self.panel5)
        self.btSlice.setObjectName(u"btSlice")
        self.btSlice.setGeometry(QRect(330, 160, 81, 27))
        self.btSlice.setAutoDefault(True)
        self.btSlice3D = QPushButton(self.panel5)
        self.btSlice3D.setObjectName(u"btSlice3D")
        self.btSlice3D.setGeometry(QRect(440, 160, 81, 27))
        self.btAutoFill = QPushButton(self.panel5)
        self.btAutoFill.setObjectName(u"btAutoFill")
        self.btAutoFill.setGeometry(QRect(140, 25, 51, 27))
        self.btAutoFill.setFont(font)
        self.btAutoFill.setAutoDefault(True)
        self.cbDiagFoldType = QComboBox(self.panel5)
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.addItem("")
        self.cbDiagFoldType.setObjectName(u"cbDiagFoldType")
        self.cbDiagFoldType.setGeometry(QRect(300, 25, 71, 27))
        self.cbDiagFoldType.setFont(font)
        self.ckbUseAx1Fold = QCheckBox(self.panel5)
        self.ckbUseAx1Fold.setObjectName(u"ckbUseAx1Fold")
        self.ckbUseAx1Fold.setGeometry(QRect(300, 55, 71, 22))
        self.ckbUseAx2Fold = QCheckBox(self.panel5)
        self.ckbUseAx2Fold.setObjectName(u"ckbUseAx2Fold")
        self.ckbUseAx2Fold.setGeometry(QRect(300, 80, 71, 22))
        self.ckbUseAx3Fold = QCheckBox(self.panel5)
        self.ckbUseAx3Fold.setObjectName(u"ckbUseAx3Fold")
        self.ckbUseAx3Fold.setGeometry(QRect(300, 105, 71, 22))
        self.ckbUseAx4Fold = QCheckBox(self.panel5)
        self.ckbUseAx4Fold.setObjectName(u"ckbUseAx4Fold")
        self.ckbUseAx4Fold.setGeometry(QRect(300, 130, 71, 22))
        self.label = QLabel(self.panel5)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(380, 30, 68, 17))
        self.label.setAlignment(Qt.AlignCenter)
        self.label_16 = QLabel(self.panel5)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(460, 30, 61, 17))
        self.label_16.setAlignment(Qt.AlignCenter)
        self.panel6 = QGroupBox(self.centralwidget)
        self.panel6.setObjectName(u"panel6")
        self.panel6.setGeometry(QRect(10, 590, 530, 60))
        self.btDumpToText = QPushButton(self.panel6)
        self.btDumpToText.setObjectName(u"btDumpToText")
        self.btDumpToText.setGeometry(QRect(20, 25, 121, 27))
        self.btDumpToBin = QPushButton(self.panel6)
        self.btDumpToBin.setObjectName(u"btDumpToBin")
        self.btDumpToBin.setGeometry(QRect(150, 25, 121, 27))
        self.btOut3DText = QPushButton(self.panel6)
        self.btOut3DText.setObjectName(u"btOut3DText")
        self.btOut3DText.setGeometry(QRect(330, 25, 121, 27))
        self.label_15 = QLabel(self.centralwidget)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(10, 650, 68, 17))
        self.txtStatus = QLabel(self.centralwidget)
        self.txtStatus.setObjectName(u"txtStatus")
        self.txtStatus.setGeometry(QRect(90, 650, 421, 17))
        D4matSlicerQtMain.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(D4matSlicerQtMain)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 550, 28))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        D4matSlicerQtMain.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(D4matSlicerQtMain)
        self.statusbar.setObjectName(u"statusbar")
        D4matSlicerQtMain.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.btMakeVirtualMat, self.btOpenD4Matrix)
        QWidget.setTabOrder(self.btOpenD4Matrix, self.v1title)
        QWidget.setTabOrder(self.v1title, self.v1unit)
        QWidget.setTabOrder(self.v1unit, self.v2title)
        QWidget.setTabOrder(self.v2title, self.v2unit)
        QWidget.setTabOrder(self.v2unit, self.v3title)
        QWidget.setTabOrder(self.v3title, self.v3unit)
        QWidget.setTabOrder(self.v3unit, self.v4title)
        QWidget.setTabOrder(self.v4title, self.v4unit)
        QWidget.setTabOrder(self.v4unit, self.btCloseMatrix)
        QWidget.setTabOrder(self.btCloseMatrix, self.btClearColumn)
        QWidget.setTabOrder(self.btClearColumn, self.btSaveTitles)
        QWidget.setTabOrder(self.btSaveTitles, self.lbFileList)
        QWidget.setTabOrder(self.lbFileList, self.btAddSingleFile)
        QWidget.setTabOrder(self.btAddSingleFile, self.btAddFilesInDir)
        QWidget.setTabOrder(self.btAddFilesInDir, self.btRemoveFile)
        QWidget.setTabOrder(self.btRemoveFile, self.btAutoFill)
        QWidget.setTabOrder(self.btAutoFill, self.cbDiagFoldType)
        QWidget.setTabOrder(self.cbDiagFoldType, self.stxt1)
        QWidget.setTabOrder(self.stxt1, self.stxt2)
        QWidget.setTabOrder(self.stxt2, self.btAutoFillAx1)
        QWidget.setTabOrder(self.btAutoFillAx1, self.txtFold1)
        QWidget.setTabOrder(self.txtFold1, self.ckbUseAx1Fold)
        QWidget.setTabOrder(self.ckbUseAx1Fold, self.v1cb)
        QWidget.setTabOrder(self.v1cb, self.stxt3)
        QWidget.setTabOrder(self.stxt3, self.stxt4)
        QWidget.setTabOrder(self.stxt4, self.stxt5)
        QWidget.setTabOrder(self.stxt5, self.btAutoFillAx2)
        QWidget.setTabOrder(self.btAutoFillAx2, self.txtFold2)
        QWidget.setTabOrder(self.txtFold2, self.ckbUseAx2Fold)
        QWidget.setTabOrder(self.ckbUseAx2Fold, self.v2cb)
        QWidget.setTabOrder(self.v2cb, self.stxt6)
        QWidget.setTabOrder(self.stxt6, self.stxt7)
        QWidget.setTabOrder(self.stxt7, self.stxt8)
        QWidget.setTabOrder(self.stxt8, self.btAutoFillAx3)
        QWidget.setTabOrder(self.btAutoFillAx3, self.txtFold3)
        QWidget.setTabOrder(self.txtFold3, self.ckbUseAx3Fold)
        QWidget.setTabOrder(self.ckbUseAx3Fold, self.v3cb)
        QWidget.setTabOrder(self.v3cb, self.stxt9)
        QWidget.setTabOrder(self.stxt9, self.stxt10)
        QWidget.setTabOrder(self.stxt10, self.stxt11)
        QWidget.setTabOrder(self.stxt11, self.btAutoFillAx4)
        QWidget.setTabOrder(self.btAutoFillAx4, self.txtFold4)
        QWidget.setTabOrder(self.txtFold4, self.ckbUseAx4Fold)
        QWidget.setTabOrder(self.ckbUseAx4Fold, self.v4cb)
        QWidget.setTabOrder(self.v4cb, self.stxt12)
        QWidget.setTabOrder(self.stxt12, self.btBack)
        QWidget.setTabOrder(self.btBack, self.btForward)
        QWidget.setTabOrder(self.btForward, self.btSlice)
        QWidget.setTabOrder(self.btSlice, self.btSlice3D)
        QWidget.setTabOrder(self.btSlice3D, self.btDumpToText)
        QWidget.setTabOrder(self.btDumpToText, self.btDumpToBin)
        QWidget.setTabOrder(self.btDumpToBin, self.btOut3DText)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionMake_New_Matrix)
        self.menuFile.addAction(self.actionOpen_Matrix)
        self.menuFile.addAction(self.actionRebin_Matrix)
        self.menuFile.addAction(self.actionSave_as_Text)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)

        self.retranslateUi(D4matSlicerQtMain)

        self.v1cb.setCurrentIndex(3)
        self.v2cb.setCurrentIndex(3)
        self.v3cb.setCurrentIndex(3)
        self.v4cb.setCurrentIndex(3)


        QMetaObject.connectSlotsByName(D4matSlicerQtMain)
    # setupUi

    def retranslateUi(self, D4matSlicerQtMain):
        D4matSlicerQtMain.setWindowTitle(QCoreApplication.translate("D4matSlicerQtMain", u"MainWindow", None))
        self.actionMake_New_Matrix.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Make New Matrix ...", None))
        self.actionOpen_Matrix.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Open Matrix ...", None))
        self.actionRebin_Matrix.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Rebin Matrix ...", None))
        self.actionSave_as_Text.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Save as Text ...", None))
        self.actionExit.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Exit", None))
        self.panel1.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"Make Virtual or New D4Matrix", None))
        self.btMakeVirtualMat.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Make", None))
        self.txtvmsg.setText(QCoreApplication.translate("D4matSlicerQtMain", u"No Info", None))
        self.btOpenD4Matrix.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Open", None))
        self.stPathToD4MatrixFiles.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Path to D4Matrix files", None))
        self.panel3.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"D4Matrix Info", None))
        self.label_2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Range", None))
        self.label_3.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Step", None))
        self.label_4.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Label", None))
        self.v1txt.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax1:", None))
        self.txt1.setText(QCoreApplication.translate("D4matSlicerQtMain", u"-1.0", None))
        self.label_6.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.txt2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.txt3.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.label_7.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.txt5.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.txt4.setText(QCoreApplication.translate("D4matSlicerQtMain", u"-1.0", None))
        self.v2txt.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax2:", None))
        self.txt6.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.label_8.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.txt8.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.txt7.setText(QCoreApplication.translate("D4matSlicerQtMain", u"-1.0", None))
        self.v3txt.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax3:", None))
        self.txt9.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.label_9.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.txt11.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.txt10.setText(QCoreApplication.translate("D4matSlicerQtMain", u"-1.0", None))
        self.v4txt.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax4:", None))
        self.txt12.setText(QCoreApplication.translate("D4matSlicerQtMain", u"1.0", None))
        self.btCloseMatrix.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Close", None))
        self.panel31.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"Label", None))
        self.btClearColumn.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Clear", None))
        self.btSaveTitles.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Save", None))
        self.label_17.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Unit", None))
        self.panel4.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"File Components", None))
        self.btAddSingleFile.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Add File", None))
        self.btAddFilesInDir.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Add Files in folder", None))
        self.btRemoveFile.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Remove File", None))
        self.panel5.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"Slice", None))
        self.label_5.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Range", None))
        self.label_10.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Folding", None))
        self.v1txt_2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax1:", None))
        self.v2txt_2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax2:", None))
        self.v4txt_2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax4:", None))
        self.v3txt_2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Ax3:", None))
        self.label_11.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.label_12.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.label_13.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.label_14.setText(QCoreApplication.translate("D4matSlicerQtMain", u"to", None))
        self.btAutoFillAx1.setText(QCoreApplication.translate("D4matSlicerQtMain", u"A", None))
        self.v1cb.setItemText(0, QCoreApplication.translate("D4matSlicerQtMain", u"X-Axis", None))
        self.v1cb.setItemText(1, QCoreApplication.translate("D4matSlicerQtMain", u"Y-Axis", None))
        self.v1cb.setItemText(2, QCoreApplication.translate("D4matSlicerQtMain", u"Z-Axis", None))
        self.v1cb.setItemText(3, QCoreApplication.translate("D4matSlicerQtMain", u"Thick", None))

        self.btAutoFillAx2.setText(QCoreApplication.translate("D4matSlicerQtMain", u"A", None))
        self.v2cb.setItemText(0, QCoreApplication.translate("D4matSlicerQtMain", u"X-Axis", None))
        self.v2cb.setItemText(1, QCoreApplication.translate("D4matSlicerQtMain", u"Y-Axis", None))
        self.v2cb.setItemText(2, QCoreApplication.translate("D4matSlicerQtMain", u"Z-Axis", None))
        self.v2cb.setItemText(3, QCoreApplication.translate("D4matSlicerQtMain", u"Thick", None))

        self.btAutoFillAx3.setText(QCoreApplication.translate("D4matSlicerQtMain", u"A", None))
        self.v3cb.setItemText(0, QCoreApplication.translate("D4matSlicerQtMain", u"X-Axis", None))
        self.v3cb.setItemText(1, QCoreApplication.translate("D4matSlicerQtMain", u"Y-Axis", None))
        self.v3cb.setItemText(2, QCoreApplication.translate("D4matSlicerQtMain", u"Z-Axis", None))
        self.v3cb.setItemText(3, QCoreApplication.translate("D4matSlicerQtMain", u"Thick", None))

        self.btAutoFillAx4.setText(QCoreApplication.translate("D4matSlicerQtMain", u"A", None))
        self.v4cb.setItemText(0, QCoreApplication.translate("D4matSlicerQtMain", u"X-Axis", None))
        self.v4cb.setItemText(1, QCoreApplication.translate("D4matSlicerQtMain", u"Y-Axis", None))
        self.v4cb.setItemText(2, QCoreApplication.translate("D4matSlicerQtMain", u"Z-Axis", None))
        self.v4cb.setItemText(3, QCoreApplication.translate("D4matSlicerQtMain", u"Thick", None))

        self.btBack.setText(QCoreApplication.translate("D4matSlicerQtMain", u"...", None))
        self.btForward.setText(QCoreApplication.translate("D4matSlicerQtMain", u"...", None))
        self.btSlice.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Slice2D", None))
        self.btSlice3D.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Slice3D", None))
        self.btAutoFill.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Auto", None))
        self.cbDiagFoldType.setItemText(0, QCoreApplication.translate("D4matSlicerQtMain", u"None", None))
        self.cbDiagFoldType.setItemText(1, QCoreApplication.translate("D4matSlicerQtMain", u"X > Y", None))
        self.cbDiagFoldType.setItemText(2, QCoreApplication.translate("D4matSlicerQtMain", u"X > -Y", None))
        self.cbDiagFoldType.setItemText(3, QCoreApplication.translate("D4matSlicerQtMain", u"Both", None))

        self.ckbUseAx1Fold.setText(QCoreApplication.translate("D4matSlicerQtMain", u"used", None))
        self.ckbUseAx2Fold.setText(QCoreApplication.translate("D4matSlicerQtMain", u"used", None))
        self.ckbUseAx3Fold.setText(QCoreApplication.translate("D4matSlicerQtMain", u"used", None))
        self.ckbUseAx4Fold.setText(QCoreApplication.translate("D4matSlicerQtMain", u"used", None))
        self.label.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Plot Axis", None))
        self.label_16.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Step", None))
        self.panel6.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"Output To File", None))
        self.btDumpToText.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Dump To Text", None))
        self.btDumpToBin.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Dump To Binary", None))
        self.btOut3DText.setText(QCoreApplication.translate("D4matSlicerQtMain", u"Out3D as Text", None))
        self.label_15.setText(QCoreApplication.translate("D4matSlicerQtMain", u"STATUS :", None))
        self.txtStatus.setText(QCoreApplication.translate("D4matSlicerQtMain", u"None", None))
        self.menuFile.setTitle(QCoreApplication.translate("D4matSlicerQtMain", u"File", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'D4matSlicerQ_Sub.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGroupBox, QLabel,
    QLineEdit, QMainWindow, QMenu, QMenuBar,
    QPushButton, QSizePolicy, QStatusBar, QWidget)

class Ui_D4matSlicerQtSub(object):
    def setupUi(self, D4matSlicerQtSub):
        if not D4matSlicerQtSub.objectName():
            D4matSlicerQtSub.setObjectName(u"D4matSlicerQtSub")
        D4matSlicerQtSub.resize(550, 709)
        self.actionLoad_XtalParams = QAction(D4matSlicerQtSub)
        self.actionLoad_XtalParams.setObjectName(u"actionLoad_XtalParams")
        self.actionSave_XtalParams = QAction(D4matSlicerQtSub)
        self.actionSave_XtalParams.setObjectName(u"actionSave_XtalParams")
        self.actionExit = QAction(D4matSlicerQtSub)
        self.actionExit.setObjectName(u"actionExit")
        self.centralwidget = QWidget(D4matSlicerQtSub)
        self.centralwidget.setObjectName(u"centralwidget")
        self.ParamFilePanel = QGroupBox(self.centralwidget)
        self.ParamFilePanel.setObjectName(u"ParamFilePanel")
        self.ParamFilePanel.setGeometry(QRect(400, 0, 140, 51))
        self.btLoadParams = QPushButton(self.ParamFilePanel)
        self.btLoadParams.setObjectName(u"btLoadParams")
        self.btLoadParams.setGeometry(QRect(10, 20, 55, 25))
        self.btSaveParams = QPushButton(self.ParamFilePanel)
        self.btSaveParams.setObjectName(u"btSaveParams")
        self.btSaveParams.setGeometry(QRect(70, 20, 55, 25))
        self.TitleLabel = QLabel(self.centralwidget)
        self.TitleLabel.setObjectName(u"TitleLabel")
        self.TitleLabel.setGeometry(QRect(10, 0, 321, 31))
        font = QFont()
        font.setPointSize(14)
        self.TitleLabel.setFont(font)
        self.SamplePanel = QGroupBox(self.centralwidget)
        self.SamplePanel.setObjectName(u"SamplePanel")
        self.SamplePanel.setGeometry(QRect(10, 50, 530, 160))
        self.SamplePanel.setFlat(False)
        self.SamplePanel.setCheckable(False)
        self.txt14 = QLineEdit(self.SamplePanel)
        self.txt14.setObjectName(u"txt14")
        self.txt14.setGeometry(QRect(310, 130, 60, 20))
        self.txt13 = QLineEdit(self.SamplePanel)
        self.txt13.setObjectName(u"txt13")
        self.txt13.setGeometry(QRect(150, 130, 60, 20))
        self.label15 = QLabel(self.SamplePanel)
        self.label15.setObjectName(u"label15")
        self.label15.setGeometry(QRect(0, 130, 68, 17))
        self.txt2 = QLineEdit(self.SamplePanel)
        self.txt2.setObjectName(u"txt2")
        self.txt2.setGeometry(QRect(260, 30, 60, 20))
        self.label7 = QLabel(self.SamplePanel)
        self.label7.setObjectName(u"label7")
        self.label7.setGeometry(QRect(190, 50, 65, 20))
        self.txt11 = QLineEdit(self.SamplePanel)
        self.txt11.setObjectName(u"txt11")
        self.txt11.setGeometry(QRect(260, 100, 60, 20))
        self.txt15 = QLineEdit(self.SamplePanel)
        self.txt15.setObjectName(u"txt15")
        self.txt15.setGeometry(QRect(460, 130, 60, 20))
        self.txt5 = QLineEdit(self.SamplePanel)
        self.txt5.setObjectName(u"txt5")
        self.txt5.setGeometry(QRect(260, 50, 60, 20))
        self.txt6 = QLineEdit(self.SamplePanel)
        self.txt6.setObjectName(u"txt6")
        self.txt6.setGeometry(QRect(410, 50, 60, 20))
        self.label4 = QLabel(self.SamplePanel)
        self.label4.setObjectName(u"label4")
        self.label4.setGeometry(QRect(190, 30, 65, 20))
        self.label6 = QLabel(self.SamplePanel)
        self.label6.setObjectName(u"label6")
        self.label6.setGeometry(QRect(30, 50, 65, 20))
        self.label14 = QLabel(self.SamplePanel)
        self.label14.setObjectName(u"label14")
        self.label14.setGeometry(QRect(340, 100, 65, 20))
        self.label18 = QLabel(self.SamplePanel)
        self.label18.setObjectName(u"label18")
        self.label18.setGeometry(QRect(380, 130, 31, 17))
        self.txt4 = QLineEdit(self.SamplePanel)
        self.txt4.setObjectName(u"txt4")
        self.txt4.setGeometry(QRect(100, 50, 60, 20))
        self.txt8 = QLineEdit(self.SamplePanel)
        self.txt8.setObjectName(u"txt8")
        self.txt8.setGeometry(QRect(260, 80, 60, 20))
        self.label8 = QLabel(self.SamplePanel)
        self.label8.setObjectName(u"label8")
        self.label8.setGeometry(QRect(340, 50, 65, 20))
        self.chRotateAxis3rd = QComboBox(self.SamplePanel)
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.addItem("")
        self.chRotateAxis3rd.setObjectName(u"chRotateAxis3rd")
        self.chRotateAxis3rd.setGeometry(QRect(410, 125, 50, 25))
        font1 = QFont()
        font1.setPointSize(10)
        self.chRotateAxis3rd.setFont(font1)
        self.chRotateAxis3rd.setEditable(False)
        self.label17 = QLabel(self.SamplePanel)
        self.label17.setObjectName(u"label17")
        self.label17.setGeometry(QRect(230, 130, 31, 17))
        self.txt1 = QLineEdit(self.SamplePanel)
        self.txt1.setObjectName(u"txt1")
        self.txt1.setGeometry(QRect(100, 30, 60, 20))
        self.txt9 = QLineEdit(self.SamplePanel)
        self.txt9.setObjectName(u"txt9")
        self.txt9.setGeometry(QRect(410, 80, 60, 20))
        self.txt12 = QLineEdit(self.SamplePanel)
        self.txt12.setObjectName(u"txt12")
        self.txt12.setGeometry(QRect(410, 100, 60, 20))
        self.label11 = QLabel(self.SamplePanel)
        self.label11.setObjectName(u"label11")
        self.label11.setGeometry(QRect(340, 80, 65, 20))
        self.label10 = QLabel(self.SamplePanel)
        self.label10.setObjectName(u"label10")
        self.label10.setGeometry(QRect(190, 80, 65, 20))
        self.chRotateAxis2nd = QComboBox(self.SamplePanel)
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.addItem("")
        self.chRotateAxis2nd.setObjectName(u"chRotateAxis2nd")
        self.chRotateAxis2nd.setGeometry(QRect(260, 125, 50, 25))
        self.chRotateAxis2nd.setFont(font1)
        self.chRotateAxis2nd.setEditable(False)
        self.label13 = QLabel(self.SamplePanel)
        self.label13.setObjectName(u"label13")
        self.label13.setGeometry(QRect(190, 100, 65, 20))
        self.chRotateAxis1st = QComboBox(self.SamplePanel)
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.addItem("")
        self.chRotateAxis1st.setObjectName(u"chRotateAxis1st")
        self.chRotateAxis1st.setGeometry(QRect(100, 125, 50, 25))
        self.chRotateAxis1st.setFont(font1)
        self.chRotateAxis1st.setEditable(False)
        self.label3 = QLabel(self.SamplePanel)
        self.label3.setObjectName(u"label3")
        self.label3.setGeometry(QRect(30, 30, 65, 20))
        self.txt3 = QLineEdit(self.SamplePanel)
        self.txt3.setObjectName(u"txt3")
        self.txt3.setGeometry(QRect(410, 30, 60, 20))
        self.label16 = QLabel(self.SamplePanel)
        self.label16.setObjectName(u"label16")
        self.label16.setGeometry(QRect(70, 130, 31, 17))
        self.txt7 = QLineEdit(self.SamplePanel)
        self.txt7.setObjectName(u"txt7")
        self.txt7.setGeometry(QRect(100, 80, 60, 20))
        self.label5 = QLabel(self.SamplePanel)
        self.label5.setObjectName(u"label5")
        self.label5.setGeometry(QRect(340, 30, 65, 20))
        self.txt10 = QLineEdit(self.SamplePanel)
        self.txt10.setObjectName(u"txt10")
        self.txt10.setGeometry(QRect(100, 100, 60, 20))
        self.label9 = QLabel(self.SamplePanel)
        self.label9.setObjectName(u"label9")
        self.label9.setGeometry(QRect(30, 80, 65, 20))
        self.label12 = QLabel(self.SamplePanel)
        self.label12.setObjectName(u"label12")
        self.label12.setGeometry(QRect(30, 100, 65, 20))
        self.txt14.raise_()
        self.txt13.raise_()
        self.label15.raise_()
        self.label7.raise_()
        self.txt11.raise_()
        self.txt15.raise_()
        self.txt5.raise_()
        self.txt6.raise_()
        self.label4.raise_()
        self.label6.raise_()
        self.label14.raise_()
        self.label18.raise_()
        self.txt4.raise_()
        self.txt8.raise_()
        self.label8.raise_()
        self.chRotateAxis3rd.raise_()
        self.label17.raise_()
        self.txt9.raise_()
        self.txt12.raise_()
        self.label11.raise_()
        self.label10.raise_()
        self.chRotateAxis2nd.raise_()
        self.label13.raise_()
        self.chRotateAxis1st.raise_()
        self.label3.raise_()
        self.label16.raise_()
        self.txt7.raise_()
        self.label5.raise_()
        self.txt10.raise_()
        self.label9.raise_()
        self.label12.raise_()
        self.txt1.raise_()
        self.txt2.raise_()
        self.txt3.raise_()
        self.ProjectionPanel = QGroupBox(self.centralwidget)
        self.ProjectionPanel.setObjectName(u"ProjectionPanel")
        self.ProjectionPanel.setGeometry(QRect(10, 210, 530, 135))
        self.ptxt1 = QLineEdit(self.ProjectionPanel)
        self.ptxt1.setObjectName(u"ptxt1")
        self.ptxt1.setGeometry(QRect(40, 40, 55, 20))
        self.label25 = QLabel(self.ProjectionPanel)
        self.label25.setObjectName(u"label25")
        self.label25.setGeometry(QRect(10, 40, 31, 17))
        self.ptxt2 = QLineEdit(self.ProjectionPanel)
        self.ptxt2.setObjectName(u"ptxt2")
        self.ptxt2.setGeometry(QRect(100, 40, 55, 20))
        self.ptxt3 = QLineEdit(self.ProjectionPanel)
        self.ptxt3.setObjectName(u"ptxt3")
        self.ptxt3.setGeometry(QRect(160, 40, 55, 20))
        self.ptxt4 = QLineEdit(self.ProjectionPanel)
        self.ptxt4.setObjectName(u"ptxt4")
        self.ptxt4.setGeometry(QRect(220, 40, 55, 20))
        self.ptxt9 = QLineEdit(self.ProjectionPanel)
        self.ptxt9.setObjectName(u"ptxt9")
        self.ptxt9.setGeometry(QRect(220, 60, 55, 20))
        self.label26 = QLabel(self.ProjectionPanel)
        self.label26.setObjectName(u"label26")
        self.label26.setGeometry(QRect(10, 60, 31, 17))
        self.ptxt8 = QLineEdit(self.ProjectionPanel)
        self.ptxt8.setObjectName(u"ptxt8")
        self.ptxt8.setGeometry(QRect(160, 60, 55, 20))
        self.ptxt6 = QLineEdit(self.ProjectionPanel)
        self.ptxt6.setObjectName(u"ptxt6")
        self.ptxt6.setGeometry(QRect(40, 60, 55, 20))
        self.ptxt7 = QLineEdit(self.ProjectionPanel)
        self.ptxt7.setObjectName(u"ptxt7")
        self.ptxt7.setGeometry(QRect(100, 60, 55, 20))
        self.label27 = QLabel(self.ProjectionPanel)
        self.label27.setObjectName(u"label27")
        self.label27.setGeometry(QRect(10, 80, 31, 17))
        self.label28 = QLabel(self.ProjectionPanel)
        self.label28.setObjectName(u"label28")
        self.label28.setGeometry(QRect(10, 100, 31, 17))
        self.ptxt5 = QLineEdit(self.ProjectionPanel)
        self.ptxt5.setObjectName(u"ptxt5")
        self.ptxt5.setGeometry(QRect(290, 40, 81, 20))
        self.ptxt10 = QLineEdit(self.ProjectionPanel)
        self.ptxt10.setObjectName(u"ptxt10")
        self.ptxt10.setGeometry(QRect(290, 60, 81, 20))
        self.label19 = QLabel(self.ProjectionPanel)
        self.label19.setObjectName(u"label19")
        self.label19.setGeometry(QRect(40, 20, 55, 20))
        self.label19.setAlignment(Qt.AlignCenter)
        self.label20 = QLabel(self.ProjectionPanel)
        self.label20.setObjectName(u"label20")
        self.label20.setGeometry(QRect(100, 20, 55, 20))
        self.label20.setAlignment(Qt.AlignCenter)
        self.label21 = QLabel(self.ProjectionPanel)
        self.label21.setObjectName(u"label21")
        self.label21.setGeometry(QRect(160, 20, 55, 20))
        self.label21.setAlignment(Qt.AlignCenter)
        self.label23 = QLabel(self.ProjectionPanel)
        self.label23.setObjectName(u"label23")
        self.label23.setGeometry(QRect(220, 20, 55, 20))
        self.label23.setAlignment(Qt.AlignCenter)
        self.label24 = QLabel(self.ProjectionPanel)
        self.label24.setObjectName(u"label24")
        self.label24.setGeometry(QRect(290, 20, 81, 20))
        self.label24.setAlignment(Qt.AlignCenter)
        self.ptxt16 = QLineEdit(self.ProjectionPanel)
        self.ptxt16.setObjectName(u"ptxt16")
        self.ptxt16.setGeometry(QRect(40, 100, 55, 20))
        self.ptxt18 = QLineEdit(self.ProjectionPanel)
        self.ptxt18.setObjectName(u"ptxt18")
        self.ptxt18.setGeometry(QRect(160, 100, 55, 20))
        self.ptxt17 = QLineEdit(self.ProjectionPanel)
        self.ptxt17.setObjectName(u"ptxt17")
        self.ptxt17.setGeometry(QRect(100, 100, 55, 20))
        self.ptxt14 = QLineEdit(self.ProjectionPanel)
        self.ptxt14.setObjectName(u"ptxt14")
        self.ptxt14.setGeometry(QRect(220, 80, 55, 20))
        self.ptxt15 = QLineEdit(self.ProjectionPanel)
        self.ptxt15.setObjectName(u"ptxt15")
        self.ptxt15.setGeometry(QRect(290, 80, 81, 20))
        self.ptxt13 = QLineEdit(self.ProjectionPanel)
        self.ptxt13.setObjectName(u"ptxt13")
        self.ptxt13.setGeometry(QRect(160, 80, 55, 20))
        self.ptxt11 = QLineEdit(self.ProjectionPanel)
        self.ptxt11.setObjectName(u"ptxt11")
        self.ptxt11.setGeometry(QRect(40, 80, 55, 20))
        self.ptxt19 = QLineEdit(self.ProjectionPanel)
        self.ptxt19.setObjectName(u"ptxt19")
        self.ptxt19.setGeometry(QRect(220, 100, 55, 20))
        self.ptxt12 = QLineEdit(self.ProjectionPanel)
        self.ptxt12.setObjectName(u"ptxt12")
        self.ptxt12.setGeometry(QRect(100, 80, 55, 20))
        self.ptxt20 = QLineEdit(self.ProjectionPanel)
        self.ptxt20.setObjectName(u"ptxt20")
        self.ptxt20.setGeometry(QRect(290, 100, 81, 20))
        self.btClr = QPushButton(self.ProjectionPanel)
        self.btClr.setObjectName(u"btClr")
        self.btClr.setGeometry(QRect(460, 40, 60, 27))
        self.ptxt21 = QLineEdit(self.ProjectionPanel)
        self.ptxt21.setObjectName(u"ptxt21")
        self.ptxt21.setGeometry(QRect(380, 40, 61, 20))
        self.ptxt22 = QLineEdit(self.ProjectionPanel)
        self.ptxt22.setObjectName(u"ptxt22")
        self.ptxt22.setGeometry(QRect(380, 60, 61, 20))
        self.ptxt23 = QLineEdit(self.ProjectionPanel)
        self.ptxt23.setObjectName(u"ptxt23")
        self.ptxt23.setGeometry(QRect(380, 80, 61, 20))
        self.ptxt24 = QLineEdit(self.ProjectionPanel)
        self.ptxt24.setObjectName(u"ptxt24")
        self.ptxt24.setGeometry(QRect(380, 100, 61, 20))
        self.label24_2 = QLabel(self.ProjectionPanel)
        self.label24_2.setObjectName(u"label24_2")
        self.label24_2.setGeometry(QRect(380, 20, 61, 20))
        self.label24_2.setAlignment(Qt.AlignCenter)
        self.CondPanel = QGroupBox(self.centralwidget)
        self.CondPanel.setObjectName(u"CondPanel")
        self.CondPanel.setGeometry(QRect(10, 340, 530, 130))
        self.label = QLabel(self.CondPanel)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(0, 20, 71, 17))
        self.txEi = QLineEdit(self.CondPanel)
        self.txEi.setObjectName(u"txEi")
        self.txEi.setGeometry(QRect(70, 20, 71, 21))
        self.txminhw = QLineEdit(self.CondPanel)
        self.txminhw.setObjectName(u"txminhw")
        self.txminhw.setGeometry(QRect(170, 40, 71, 21))
        self.txmaxhw = QLineEdit(self.CondPanel)
        self.txmaxhw.setObjectName(u"txmaxhw")
        self.txmaxhw.setGeometry(QRect(260, 40, 71, 21))
        self.txdeltahw = QLineEdit(self.CondPanel)
        self.txdeltahw.setObjectName(u"txdeltahw")
        self.txdeltahw.setGeometry(QRect(430, 40, 71, 21))
        self.txminphi = QLineEdit(self.CondPanel)
        self.txminphi.setObjectName(u"txminphi")
        self.txminphi.setGeometry(QRect(170, 70, 71, 21))
        self.txmaxphi = QLineEdit(self.CondPanel)
        self.txmaxphi.setObjectName(u"txmaxphi")
        self.txmaxphi.setGeometry(QRect(260, 70, 71, 21))
        self.txdeltaphi = QLineEdit(self.CondPanel)
        self.txdeltaphi.setObjectName(u"txdeltaphi")
        self.txdeltaphi.setGeometry(QRect(430, 70, 71, 21))
        self.txrunno = QLineEdit(self.CondPanel)
        self.txrunno.setObjectName(u"txrunno")
        self.txrunno.setGeometry(QRect(340, 100, 81, 21))
        self.label_2 = QLabel(self.CondPanel)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 40, 151, 21))
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_3 = QLabel(self.CondPanel)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(10, 70, 151, 21))
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_4 = QLabel(self.CondPanel)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(240, 40, 21, 17))
        self.label_5 = QLabel(self.CondPanel)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(240, 70, 21, 17))
        self.label_6 = QLabel(self.CondPanel)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(330, 40, 91, 21))
        self.label_6.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_7 = QLabel(self.CondPanel)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(330, 70, 91, 21))
        self.label_7.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_8 = QLabel(self.CondPanel)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(10, 100, 321, 21))
        self.label_8.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.RangePanel = QGroupBox(self.centralwidget)
        self.RangePanel.setObjectName(u"RangePanel")
        self.RangePanel.setGeometry(QRect(10, 470, 530, 130))
        self.dtxt1 = QLineEdit(self.RangePanel)
        self.dtxt1.setObjectName(u"dtxt1")
        self.dtxt1.setGeometry(QRect(60, 40, 60, 20))
        self.dtxt2 = QLineEdit(self.RangePanel)
        self.dtxt2.setObjectName(u"dtxt2")
        self.dtxt2.setGeometry(QRect(140, 40, 60, 20))
        self.dtxt3 = QLineEdit(self.RangePanel)
        self.dtxt3.setObjectName(u"dtxt3")
        self.dtxt3.setGeometry(QRect(240, 40, 60, 20))
        self.dtxt4 = QLineEdit(self.RangePanel)
        self.dtxt4.setObjectName(u"dtxt4")
        self.dtxt4.setGeometry(QRect(60, 60, 60, 20))
        self.dtxt5 = QLineEdit(self.RangePanel)
        self.dtxt5.setObjectName(u"dtxt5")
        self.dtxt5.setGeometry(QRect(140, 60, 60, 20))
        self.dtxt6 = QLineEdit(self.RangePanel)
        self.dtxt6.setObjectName(u"dtxt6")
        self.dtxt6.setGeometry(QRect(240, 60, 60, 20))
        self.dtxt7 = QLineEdit(self.RangePanel)
        self.dtxt7.setObjectName(u"dtxt7")
        self.dtxt7.setGeometry(QRect(60, 80, 60, 20))
        self.dtxt8 = QLineEdit(self.RangePanel)
        self.dtxt8.setObjectName(u"dtxt8")
        self.dtxt8.setGeometry(QRect(140, 80, 60, 20))
        self.dtxt9 = QLineEdit(self.RangePanel)
        self.dtxt9.setObjectName(u"dtxt9")
        self.dtxt9.setGeometry(QRect(240, 80, 60, 20))
        self.dtxt10 = QLineEdit(self.RangePanel)
        self.dtxt10.setObjectName(u"dtxt10")
        self.dtxt10.setGeometry(QRect(60, 100, 60, 20))
        self.dtxt11 = QLineEdit(self.RangePanel)
        self.dtxt11.setObjectName(u"dtxt11")
        self.dtxt11.setGeometry(QRect(140, 100, 60, 20))
        self.dtxt12 = QLineEdit(self.RangePanel)
        self.dtxt12.setObjectName(u"dtxt12")
        self.dtxt12.setGeometry(QRect(240, 100, 60, 20))
        self.label_9 = QLabel(self.RangePanel)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(10, 40, 41, 17))
        self.label_9.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_10 = QLabel(self.RangePanel)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(10, 60, 41, 17))
        self.label_10.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_11 = QLabel(self.RangePanel)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(10, 80, 41, 17))
        self.label_11.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_12 = QLabel(self.RangePanel)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(10, 100, 41, 17))
        self.label_12.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_13 = QLabel(self.RangePanel)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(110, 20, 51, 17))
        self.label_13.setAlignment(Qt.AlignCenter)
        self.label_14 = QLabel(self.RangePanel)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(120, 40, 21, 20))
        self.label_14.setAlignment(Qt.AlignCenter)
        self.label_15 = QLabel(self.RangePanel)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(120, 60, 21, 20))
        self.label_15.setAlignment(Qt.AlignCenter)
        self.label_16 = QLabel(self.RangePanel)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(120, 80, 21, 20))
        self.label_16.setAlignment(Qt.AlignCenter)
        self.label_17 = QLabel(self.RangePanel)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(120, 100, 21, 20))
        self.label_17.setAlignment(Qt.AlignCenter)
        self.label_18 = QLabel(self.RangePanel)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setGeometry(QRect(250, 20, 51, 17))
        self.label_18.setAlignment(Qt.AlignCenter)
        self.VirtualD4MatPanel = QGroupBox(self.centralwidget)
        self.VirtualD4MatPanel.setObjectName(u"VirtualD4MatPanel")
        self.VirtualD4MatPanel.setGeometry(QRect(350, 490, 171, 101))
        self.btFillRange = QPushButton(self.VirtualD4MatPanel)
        self.btFillRange.setObjectName(u"btFillRange")
        self.btFillRange.setGeometry(QRect(10, 30, 131, 27))
        self.btMakeVirtualMatrix = QPushButton(self.VirtualD4MatPanel)
        self.btMakeVirtualMatrix.setObjectName(u"btMakeVirtualMatrix")
        self.btMakeVirtualMatrix.setGeometry(QRect(10, 70, 151, 27))
        self.NewD4MatPanel = QGroupBox(self.centralwidget)
        self.NewD4MatPanel.setObjectName(u"NewD4MatPanel")
        self.NewD4MatPanel.setGeometry(QRect(10, 600, 371, 61))
        self.btEstimateD4MatSize = QPushButton(self.NewD4MatPanel)
        self.btEstimateD4MatSize.setObjectName(u"btEstimateD4MatSize")
        self.btEstimateD4MatSize.setGeometry(QRect(20, 25, 121, 27))
        self.btMakeNewD4Mat = QPushButton(self.NewD4MatPanel)
        self.btMakeNewD4Mat.setObjectName(u"btMakeNewD4Mat")
        self.btMakeNewD4Mat.setGeometry(QRect(200, 22, 151, 36))
        self.btClose = QPushButton(self.centralwidget)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(438, 626, 71, 31))
        D4matSlicerQtSub.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(D4matSlicerQtSub)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 550, 28))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        D4matSlicerQtSub.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(D4matSlicerQtSub)
        self.statusbar.setObjectName(u"statusbar")
        D4matSlicerQtSub.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.btLoadParams, self.btSaveParams)
        QWidget.setTabOrder(self.btSaveParams, self.txt1)
        QWidget.setTabOrder(self.txt1, self.txt2)
        QWidget.setTabOrder(self.txt2, self.txt3)
        QWidget.setTabOrder(self.txt3, self.txt4)
        QWidget.setTabOrder(self.txt4, self.txt5)
        QWidget.setTabOrder(self.txt5, self.txt6)
        QWidget.setTabOrder(self.txt6, self.txt7)
        QWidget.setTabOrder(self.txt7, self.txt8)
        QWidget.setTabOrder(self.txt8, self.txt9)
        QWidget.setTabOrder(self.txt9, self.txt10)
        QWidget.setTabOrder(self.txt10, self.txt11)
        QWidget.setTabOrder(self.txt11, self.txt12)
        QWidget.setTabOrder(self.txt12, self.chRotateAxis1st)
        QWidget.setTabOrder(self.chRotateAxis1st, self.txt13)
        QWidget.setTabOrder(self.txt13, self.chRotateAxis2nd)
        QWidget.setTabOrder(self.chRotateAxis2nd, self.txt14)
        QWidget.setTabOrder(self.txt14, self.chRotateAxis3rd)
        QWidget.setTabOrder(self.chRotateAxis3rd, self.txt15)
        QWidget.setTabOrder(self.txt15, self.ptxt1)
        QWidget.setTabOrder(self.ptxt1, self.ptxt2)
        QWidget.setTabOrder(self.ptxt2, self.ptxt3)
        QWidget.setTabOrder(self.ptxt3, self.ptxt4)
        QWidget.setTabOrder(self.ptxt4, self.ptxt21)
        QWidget.setTabOrder(self.ptxt21, self.ptxt5)
        QWidget.setTabOrder(self.ptxt5, self.ptxt6)
        QWidget.setTabOrder(self.ptxt6, self.ptxt7)
        QWidget.setTabOrder(self.ptxt7, self.ptxt8)
        QWidget.setTabOrder(self.ptxt8, self.ptxt22)
        QWidget.setTabOrder(self.ptxt22, self.ptxt9)
        QWidget.setTabOrder(self.ptxt9, self.ptxt10)
        QWidget.setTabOrder(self.ptxt10, self.ptxt11)
        QWidget.setTabOrder(self.ptxt11, self.ptxt12)
        QWidget.setTabOrder(self.ptxt12, self.ptxt13)
        QWidget.setTabOrder(self.ptxt13, self.ptxt23)
        QWidget.setTabOrder(self.ptxt23, self.ptxt14)
        QWidget.setTabOrder(self.ptxt14, self.ptxt15)
        QWidget.setTabOrder(self.ptxt15, self.ptxt16)
        QWidget.setTabOrder(self.ptxt16, self.ptxt17)
        QWidget.setTabOrder(self.ptxt17, self.ptxt18)
        QWidget.setTabOrder(self.ptxt18, self.ptxt19)
        QWidget.setTabOrder(self.ptxt19, self.ptxt20)
        QWidget.setTabOrder(self.ptxt20, self.ptxt24)
        QWidget.setTabOrder(self.ptxt24, self.btClr)
        QWidget.setTabOrder(self.btClr, self.txEi)
        QWidget.setTabOrder(self.txEi, self.txminhw)
        QWidget.setTabOrder(self.txminhw, self.txmaxhw)
        QWidget.setTabOrder(self.txmaxhw, self.txdeltahw)
        QWidget.setTabOrder(self.txdeltahw, self.txminphi)
        QWidget.setTabOrder(self.txminphi, self.txmaxphi)
        QWidget.setTabOrder(self.txmaxphi, self.txdeltaphi)
        QWidget.setTabOrder(self.txdeltaphi, self.txrunno)
        QWidget.setTabOrder(self.txrunno, self.dtxt1)
        QWidget.setTabOrder(self.dtxt1, self.dtxt2)
        QWidget.setTabOrder(self.dtxt2, self.dtxt3)
        QWidget.setTabOrder(self.dtxt3, self.dtxt4)
        QWidget.setTabOrder(self.dtxt4, self.dtxt5)
        QWidget.setTabOrder(self.dtxt5, self.dtxt6)
        QWidget.setTabOrder(self.dtxt6, self.dtxt7)
        QWidget.setTabOrder(self.dtxt7, self.dtxt8)
        QWidget.setTabOrder(self.dtxt8, self.dtxt9)
        QWidget.setTabOrder(self.dtxt9, self.dtxt10)
        QWidget.setTabOrder(self.dtxt10, self.dtxt11)
        QWidget.setTabOrder(self.dtxt11, self.dtxt12)
        QWidget.setTabOrder(self.dtxt12, self.btFillRange)
        QWidget.setTabOrder(self.btFillRange, self.btMakeVirtualMatrix)
        QWidget.setTabOrder(self.btMakeVirtualMatrix, self.btEstimateD4MatSize)
        QWidget.setTabOrder(self.btEstimateD4MatSize, self.btMakeNewD4Mat)
        QWidget.setTabOrder(self.btMakeNewD4Mat, self.btClose)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionLoad_XtalParams)
        self.menuFile.addAction(self.actionSave_XtalParams)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)

        self.retranslateUi(D4matSlicerQtSub)

        QMetaObject.connectSlotsByName(D4matSlicerQtSub)
    # setupUi

    def retranslateUi(self, D4matSlicerQtSub):
        D4matSlicerQtSub.setWindowTitle(QCoreApplication.translate("D4matSlicerQtSub", u"MainWindow", None))
        self.actionLoad_XtalParams.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Load XtalParams ...", None))
        self.actionSave_XtalParams.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Save XtalParams ...", None))
        self.actionExit.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Exit", None))
        self.ParamFilePanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Param File ...", None))
        self.btLoadParams.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Load", None))
        self.btSaveParams.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Save", None))
        self.TitleLabel.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Making Virtual or New D4Matrix", None))
        self.SamplePanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Sample Information", None))
        self.txt14.setText("")
        self.label15.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Rotation", None))
        self.label7.setText(QCoreApplication.translate("D4matSlicerQtSub", u"beta:", None))
        self.label4.setText(QCoreApplication.translate("D4matSlicerQtSub", u"b (1/A):", None))
        self.label6.setText(QCoreApplication.translate("D4matSlicerQtSub", u"alpha:", None))
        self.label14.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Vl:", None))
        self.label18.setText(QCoreApplication.translate("D4matSlicerQtSub", u"3rd:", None))
        self.label8.setText(QCoreApplication.translate("D4matSlicerQtSub", u"gamma:", None))
        self.chRotateAxis3rd.setItemText(0, "")
        self.chRotateAxis3rd.setItemText(1, QCoreApplication.translate("D4matSlicerQtSub", u"Y", None))
        self.chRotateAxis3rd.setItemText(2, QCoreApplication.translate("D4matSlicerQtSub", u"Z", None))
        self.chRotateAxis3rd.setItemText(3, QCoreApplication.translate("D4matSlicerQtSub", u"X", None))

        self.label17.setText(QCoreApplication.translate("D4matSlicerQtSub", u"2nd:", None))
        self.label11.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Ul:", None))
        self.label10.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Uk:", None))
        self.chRotateAxis2nd.setItemText(0, "")
        self.chRotateAxis2nd.setItemText(1, QCoreApplication.translate("D4matSlicerQtSub", u"Y", None))
        self.chRotateAxis2nd.setItemText(2, QCoreApplication.translate("D4matSlicerQtSub", u"Z", None))
        self.chRotateAxis2nd.setItemText(3, QCoreApplication.translate("D4matSlicerQtSub", u"X", None))

        self.label13.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Vk:", None))
        self.chRotateAxis1st.setItemText(0, "")
        self.chRotateAxis1st.setItemText(1, QCoreApplication.translate("D4matSlicerQtSub", u"Y", None))
        self.chRotateAxis1st.setItemText(2, QCoreApplication.translate("D4matSlicerQtSub", u"Z", None))
        self.chRotateAxis1st.setItemText(3, QCoreApplication.translate("D4matSlicerQtSub", u"X", None))

        self.label3.setText(QCoreApplication.translate("D4matSlicerQtSub", u"a (1/A):", None))
        self.label16.setText(QCoreApplication.translate("D4matSlicerQtSub", u"1st:", None))
        self.label5.setText(QCoreApplication.translate("D4matSlicerQtSub", u"c (1/A):", None))
        self.label9.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Uh:", None))
        self.label12.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Vh:", None))
        self.ProjectionPanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Projection", None))
        self.label25.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax1:", None))
        self.label26.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax2:", None))
        self.label27.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax3:", None))
        self.label28.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax4:", None))
        self.label19.setText(QCoreApplication.translate("D4matSlicerQtSub", u"a*", None))
        self.label20.setText(QCoreApplication.translate("D4matSlicerQtSub", u"b*", None))
        self.label21.setText(QCoreApplication.translate("D4matSlicerQtSub", u"c*", None))
        self.label23.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Energy", None))
        self.label24.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Axis Label", None))
        self.btClr.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Clear", None))
        self.label24_2.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Unit", None))
        self.CondPanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Measurement Conditions", None))
        self.label.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Ei (meV) :", None))
        self.label_2.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Range (meV) :", None))
        self.label_3.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Rotation Range (deg) :", None))
        self.label_4.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_5.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_6.setText(QCoreApplication.translate("D4matSlicerQtSub", u"bin (meV) :", None))
        self.label_7.setText(QCoreApplication.translate("D4matSlicerQtSub", u"step (deg) :", None))
        self.label_8.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Detector's Structure : import from run No =", None))
        self.RangePanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"D4 Matrix", None))
        self.label_9.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax1 :", None))
        self.label_10.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax2 :", None))
        self.label_11.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax3 :", None))
        self.label_12.setText(QCoreApplication.translate("D4matSlicerQtSub", u"ax4 :", None))
        self.label_13.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Range", None))
        self.label_14.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_15.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_16.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_17.setText(QCoreApplication.translate("D4matSlicerQtSub", u"to", None))
        self.label_18.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Step", None))
        self.VirtualD4MatPanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Virtual D4Mat", None))
        self.btFillRange.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Estimate Ranges", None))
        self.btMakeVirtualMatrix.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Make Virtual D4Mat", None))
        self.NewD4MatPanel.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"Make New D4Mat", None))
        self.btEstimateD4MatSize.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Estimate size", None))
        self.btMakeNewD4Mat.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Make New D4Mat", None))
        self.btClose.setText(QCoreApplication.translate("D4matSlicerQtSub", u"Close", None))
        self.menuFile.setTitle(QCoreApplication.translate("D4matSlicerQtSub", u"File", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'D4matSlicerQ_MakeNewD4mat.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGroupBox, QLabel, QLineEdit,
    QMainWindow, QMenuBar, QPushButton, QSizePolicy,
    QStatusBar, QWidget)

class Ui_NewD4MatrixFrame(object):
    def setupUi(self, NewD4MatrixFrame):
        if not NewD4MatrixFrame.objectName():
            NewD4MatrixFrame.setObjectName(u"NewD4MatrixFrame")
        NewD4MatrixFrame.resize(470, 223)
        self.centralwidget = QWidget(NewD4MatrixFrame)
        self.centralwidget.setObjectName(u"centralwidget")
        self.RangePanel = QGroupBox(self.centralwidget)
        self.RangePanel.setObjectName(u"RangePanel")
        self.RangePanel.setGeometry(QRect(0, 0, 451, 131))
        self.dtxt1 = QLineEdit(self.RangePanel)
        self.dtxt1.setObjectName(u"dtxt1")
        self.dtxt1.setGeometry(QRect(50, 40, 60, 20))
        self.dtxt2 = QLineEdit(self.RangePanel)
        self.dtxt2.setObjectName(u"dtxt2")
        self.dtxt2.setGeometry(QRect(130, 40, 60, 20))
        self.dtxt3 = QLineEdit(self.RangePanel)
        self.dtxt3.setObjectName(u"dtxt3")
        self.dtxt3.setGeometry(QRect(210, 40, 60, 20))
        self.dtxt4 = QLineEdit(self.RangePanel)
        self.dtxt4.setObjectName(u"dtxt4")
        self.dtxt4.setGeometry(QRect(50, 60, 60, 20))
        self.dtxt5 = QLineEdit(self.RangePanel)
        self.dtxt5.setObjectName(u"dtxt5")
        self.dtxt5.setGeometry(QRect(130, 60, 60, 20))
        self.dtxt6 = QLineEdit(self.RangePanel)
        self.dtxt6.setObjectName(u"dtxt6")
        self.dtxt6.setGeometry(QRect(210, 60, 60, 20))
        self.dtxt7 = QLineEdit(self.RangePanel)
        self.dtxt7.setObjectName(u"dtxt7")
        self.dtxt7.setGeometry(QRect(50, 80, 60, 20))
        self.dtxt8 = QLineEdit(self.RangePanel)
        self.dtxt8.setObjectName(u"dtxt8")
        self.dtxt8.setGeometry(QRect(130, 80, 60, 20))
        self.dtxt9 = QLineEdit(self.RangePanel)
        self.dtxt9.setObjectName(u"dtxt9")
        self.dtxt9.setGeometry(QRect(210, 80, 60, 20))
        self.dtxt10 = QLineEdit(self.RangePanel)
        self.dtxt10.setObjectName(u"dtxt10")
        self.dtxt10.setGeometry(QRect(50, 100, 60, 20))
        self.dtxt11 = QLineEdit(self.RangePanel)
        self.dtxt11.setObjectName(u"dtxt11")
        self.dtxt11.setGeometry(QRect(130, 100, 60, 20))
        self.dtxt12 = QLineEdit(self.RangePanel)
        self.dtxt12.setObjectName(u"dtxt12")
        self.dtxt12.setGeometry(QRect(210, 100, 60, 20))
        self.label_9 = QLabel(self.RangePanel)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(0, 40, 41, 17))
        self.label_9.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_10 = QLabel(self.RangePanel)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(0, 60, 41, 17))
        self.label_10.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_11 = QLabel(self.RangePanel)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(0, 80, 41, 17))
        self.label_11.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_12 = QLabel(self.RangePanel)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(0, 100, 41, 17))
        self.label_12.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.label_13 = QLabel(self.RangePanel)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(100, 20, 51, 17))
        self.label_13.setAlignment(Qt.AlignCenter)
        self.label_14 = QLabel(self.RangePanel)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(110, 40, 21, 20))
        self.label_14.setAlignment(Qt.AlignCenter)
        self.label_15 = QLabel(self.RangePanel)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(110, 60, 21, 20))
        self.label_15.setAlignment(Qt.AlignCenter)
        self.label_16 = QLabel(self.RangePanel)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(110, 80, 21, 20))
        self.label_16.setAlignment(Qt.AlignCenter)
        self.label_17 = QLabel(self.RangePanel)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(110, 100, 21, 20))
        self.label_17.setAlignment(Qt.AlignCenter)
        self.label_18 = QLabel(self.RangePanel)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setGeometry(QRect(210, 20, 51, 17))
        self.label_18.setAlignment(Qt.AlignCenter)
        self.v1title = QLineEdit(self.RangePanel)
        self.v1title.setObjectName(u"v1title")
        self.v1title.setGeometry(QRect(290, 40, 81, 21))
        self.v2title = QLineEdit(self.RangePanel)
        self.v2title.setObjectName(u"v2title")
        self.v2title.setGeometry(QRect(290, 60, 81, 21))
        self.v3title = QLineEdit(self.RangePanel)
        self.v3title.setObjectName(u"v3title")
        self.v3title.setGeometry(QRect(290, 80, 81, 21))
        self.v4title = QLineEdit(self.RangePanel)
        self.v4title.setObjectName(u"v4title")
        self.v4title.setGeometry(QRect(290, 100, 81, 21))
        self.label = QLabel(self.RangePanel)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(290, 20, 68, 17))
        self.label.setAlignment(Qt.AlignCenter)
        self.v1unit = QLineEdit(self.RangePanel)
        self.v1unit.setObjectName(u"v1unit")
        self.v1unit.setGeometry(QRect(380, 40, 61, 21))
        self.v2unit = QLineEdit(self.RangePanel)
        self.v2unit.setObjectName(u"v2unit")
        self.v2unit.setGeometry(QRect(380, 60, 61, 21))
        self.v3unit = QLineEdit(self.RangePanel)
        self.v3unit.setObjectName(u"v3unit")
        self.v3unit.setGeometry(QRect(380, 80, 61, 21))
        self.v4unit = QLineEdit(self.RangePanel)
        self.v4unit.setObjectName(u"v4unit")
        self.v4unit.setGeometry(QRect(380, 100, 61, 21))
        self.label_2 = QLabel(self.RangePanel)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(370, 20, 68, 17))
        self.label_2.setAlignment(Qt.AlignCenter)
        self.btLoadParams = QPushButton(self.centralwidget)
        self.btLoadParams.setObjectName(u"btLoadParams")
        self.btLoadParams.setGeometry(QRect(10, 140, 140, 30))
        self.btMakeNewMatrix = QPushButton(self.centralwidget)
        self.btMakeNewMatrix.setObjectName(u"btMakeNewMatrix")
        self.btMakeNewMatrix.setGeometry(QRect(170, 140, 140, 30))
        self.btClose = QPushButton(self.centralwidget)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(380, 140, 80, 30))
        NewD4MatrixFrame.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(NewD4MatrixFrame)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 470, 28))
        NewD4MatrixFrame.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(NewD4MatrixFrame)
        self.statusbar.setObjectName(u"statusbar")
        NewD4MatrixFrame.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.dtxt1, self.dtxt2)
        QWidget.setTabOrder(self.dtxt2, self.dtxt3)
        QWidget.setTabOrder(self.dtxt3, self.v1title)
        QWidget.setTabOrder(self.v1title, self.v1unit)
        QWidget.setTabOrder(self.v1unit, self.dtxt4)
        QWidget.setTabOrder(self.dtxt4, self.dtxt5)
        QWidget.setTabOrder(self.dtxt5, self.dtxt6)
        QWidget.setTabOrder(self.dtxt6, self.v2title)
        QWidget.setTabOrder(self.v2title, self.v2unit)
        QWidget.setTabOrder(self.v2unit, self.dtxt7)
        QWidget.setTabOrder(self.dtxt7, self.dtxt8)
        QWidget.setTabOrder(self.dtxt8, self.dtxt9)
        QWidget.setTabOrder(self.dtxt9, self.v3title)
        QWidget.setTabOrder(self.v3title, self.v3unit)
        QWidget.setTabOrder(self.v3unit, self.dtxt10)
        QWidget.setTabOrder(self.dtxt10, self.dtxt11)
        QWidget.setTabOrder(self.dtxt11, self.dtxt12)
        QWidget.setTabOrder(self.dtxt12, self.v4title)
        QWidget.setTabOrder(self.v4title, self.v4unit)
        QWidget.setTabOrder(self.v4unit, self.btLoadParams)
        QWidget.setTabOrder(self.btLoadParams, self.btMakeNewMatrix)
        QWidget.setTabOrder(self.btMakeNewMatrix, self.btClose)

        self.retranslateUi(NewD4MatrixFrame)

        QMetaObject.connectSlotsByName(NewD4MatrixFrame)
    # setupUi

    def retranslateUi(self, NewD4MatrixFrame):
        NewD4MatrixFrame.setWindowTitle(QCoreApplication.translate("NewD4MatrixFrame", u"MainWindow", None))
        self.RangePanel.setTitle(QCoreApplication.translate("NewD4MatrixFrame", u"D4 Matrix", None))
        self.label_9.setText(QCoreApplication.translate("NewD4MatrixFrame", u"ax1 :", None))
        self.label_10.setText(QCoreApplication.translate("NewD4MatrixFrame", u"ax2 :", None))
        self.label_11.setText(QCoreApplication.translate("NewD4MatrixFrame", u"ax3 :", None))
        self.label_12.setText(QCoreApplication.translate("NewD4MatrixFrame", u"ax4 :", None))
        self.label_13.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Range", None))
        self.label_14.setText(QCoreApplication.translate("NewD4MatrixFrame", u"to", None))
        self.label_15.setText(QCoreApplication.translate("NewD4MatrixFrame", u"to", None))
        self.label_16.setText(QCoreApplication.translate("NewD4MatrixFrame", u"to", None))
        self.label_17.setText(QCoreApplication.translate("NewD4MatrixFrame", u"to", None))
        self.label_18.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Step", None))
        self.label.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Title", None))
        self.label_2.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Unit", None))
        self.btLoadParams.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Load param file", None))
        self.btMakeNewMatrix.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Make New D4Mat", None))
        self.btClose.setText(QCoreApplication.translate("NewD4MatrixFrame", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'D4matSlicerQ_Rebin.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGroupBox, QLabel, QLineEdit,
    QMainWindow, QMenuBar, QPushButton, QSizePolicy,
    QSpinBox, QStatusBar, QWidget)

class Ui_D4matSlicerQRebin(object):
    def setupUi(self, D4matSlicerQRebin):
        if not D4matSlicerQRebin.objectName():
            D4matSlicerQRebin.setObjectName(u"D4matSlicerQRebin")
        D4matSlicerQRebin.resize(510, 249)
        self.centralwidget = QWidget(D4matSlicerQRebin)
        self.centralwidget.setObjectName(u"centralwidget")
        self.panel3 = QGroupBox(self.centralwidget)
        self.panel3.setObjectName(u"panel3")
        self.panel3.setGeometry(QRect(0, 0, 501, 171))
        self.label_2 = QLabel(self.panel3)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(100, 20, 60, 20))
        self.label_2.setAlignment(Qt.AlignCenter)
        self.label_3 = QLabel(self.panel3)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(230, 20, 60, 20))
        self.label_3.setAlignment(Qt.AlignCenter)
        self.label_4 = QLabel(self.panel3)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(400, 20, 60, 20))
        self.label_4.setAlignment(Qt.AlignCenter)
        self.v1txt = QLabel(self.panel3)
        self.v1txt.setObjectName(u"v1txt")
        self.v1txt.setGeometry(QRect(10, 40, 30, 20))
        self.txt1 = QLabel(self.panel3)
        self.txt1.setObjectName(u"txt1")
        self.txt1.setGeometry(QRect(50, 40, 60, 20))
        self.txt1.setAlignment(Qt.AlignCenter)
        self.label_6 = QLabel(self.panel3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(110, 40, 30, 20))
        self.label_6.setAlignment(Qt.AlignCenter)
        self.txt2 = QLabel(self.panel3)
        self.txt2.setObjectName(u"txt2")
        self.txt2.setGeometry(QRect(140, 40, 60, 20))
        self.txt2.setAlignment(Qt.AlignCenter)
        self.txt3 = QLabel(self.panel3)
        self.txt3.setObjectName(u"txt3")
        self.txt3.setGeometry(QRect(230, 40, 60, 20))
        self.txt3.setAlignment(Qt.AlignCenter)
        self.label_7 = QLabel(self.panel3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(110, 70, 30, 20))
        self.label_7.setAlignment(Qt.AlignCenter)
        self.txt5 = QLabel(self.panel3)
        self.txt5.setObjectName(u"txt5")
        self.txt5.setGeometry(QRect(140, 70, 60, 20))
        self.txt5.setAlignment(Qt.AlignCenter)
        self.txt4 = QLabel(self.panel3)
        self.txt4.setObjectName(u"txt4")
        self.txt4.setGeometry(QRect(50, 70, 60, 20))
        self.txt4.setAlignment(Qt.AlignCenter)
        self.v2txt = QLabel(self.panel3)
        self.v2txt.setObjectName(u"v2txt")
        self.v2txt.setGeometry(QRect(10, 70, 30, 20))
        self.txt6 = QLabel(self.panel3)
        self.txt6.setObjectName(u"txt6")
        self.txt6.setGeometry(QRect(230, 70, 60, 20))
        self.txt6.setAlignment(Qt.AlignCenter)
        self.label_8 = QLabel(self.panel3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(110, 100, 30, 20))
        self.label_8.setAlignment(Qt.AlignCenter)
        self.txt8 = QLabel(self.panel3)
        self.txt8.setObjectName(u"txt8")
        self.txt8.setGeometry(QRect(140, 100, 60, 20))
        self.txt8.setAlignment(Qt.AlignCenter)
        self.txt7 = QLabel(self.panel3)
        self.txt7.setObjectName(u"txt7")
        self.txt7.setGeometry(QRect(50, 100, 60, 20))
        self.txt7.setAlignment(Qt.AlignCenter)
        self.v3txt = QLabel(self.panel3)
        self.v3txt.setObjectName(u"v3txt")
        self.v3txt.setGeometry(QRect(10, 100, 30, 20))
        self.txt9 = QLabel(self.panel3)
        self.txt9.setObjectName(u"txt9")
        self.txt9.setGeometry(QRect(230, 100, 60, 20))
        self.txt9.setAlignment(Qt.AlignCenter)
        self.label_9 = QLabel(self.panel3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(110, 130, 30, 20))
        self.label_9.setAlignment(Qt.AlignCenter)
        self.txt11 = QLabel(self.panel3)
        self.txt11.setObjectName(u"txt11")
        self.txt11.setGeometry(QRect(140, 130, 60, 20))
        self.txt11.setAlignment(Qt.AlignCenter)
        self.txt10 = QLabel(self.panel3)
        self.txt10.setObjectName(u"txt10")
        self.txt10.setGeometry(QRect(50, 130, 60, 20))
        self.txt10.setAlignment(Qt.AlignCenter)
        self.v4txt = QLabel(self.panel3)
        self.v4txt.setObjectName(u"v4txt")
        self.v4txt.setGeometry(QRect(10, 130, 30, 20))
        self.txt12 = QLabel(self.panel3)
        self.txt12.setObjectName(u"txt12")
        self.txt12.setGeometry(QRect(230, 130, 60, 20))
        self.txt12.setAlignment(Qt.AlignCenter)
        self.v1title = QLineEdit(self.panel3)
        self.v1title.setObjectName(u"v1title")
        self.v1title.setGeometry(QRect(380, 40, 120, 20))
        self.v2title = QLineEdit(self.panel3)
        self.v2title.setObjectName(u"v2title")
        self.v2title.setGeometry(QRect(380, 70, 120, 20))
        self.v3title = QLineEdit(self.panel3)
        self.v3title.setObjectName(u"v3title")
        self.v3title.setGeometry(QRect(380, 100, 120, 20))
        self.v4title = QLineEdit(self.panel3)
        self.v4title.setObjectName(u"v4title")
        self.v4title.setGeometry(QRect(380, 130, 120, 20))
        self.spn1StepRatio = QSpinBox(self.panel3)
        self.spn1StepRatio.setObjectName(u"spn1StepRatio")
        self.spn1StepRatio.setGeometry(QRect(300, 40, 61, 25))
        self.spn2StepRatio = QSpinBox(self.panel3)
        self.spn2StepRatio.setObjectName(u"spn2StepRatio")
        self.spn2StepRatio.setGeometry(QRect(300, 70, 61, 25))
        self.spn3StepRatio = QSpinBox(self.panel3)
        self.spn3StepRatio.setObjectName(u"spn3StepRatio")
        self.spn3StepRatio.setGeometry(QRect(300, 100, 61, 25))
        self.spn4StepRatio = QSpinBox(self.panel3)
        self.spn4StepRatio.setObjectName(u"spn4StepRatio")
        self.spn4StepRatio.setGeometry(QRect(300, 130, 61, 25))
        self.label = QLabel(self.panel3)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(300, 20, 51, 17))
        self.label.setAlignment(Qt.AlignCenter)
        self.label_5 = QLabel(self.panel3)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(290, 40, 10, 25))
        self.label_10 = QLabel(self.panel3)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(290, 70, 10, 25))
        self.label_11 = QLabel(self.panel3)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(290, 100, 10, 25))
        self.label_12 = QLabel(self.panel3)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(290, 130, 10, 25))
        self.btRebinMatrix = QPushButton(self.centralwidget)
        self.btRebinMatrix.setObjectName(u"btRebinMatrix")
        self.btRebinMatrix.setGeometry(QRect(170, 170, 140, 30))
        self.btClose = QPushButton(self.centralwidget)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(438, 170, 61, 27))
        D4matSlicerQRebin.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(D4matSlicerQRebin)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 510, 25))
        D4matSlicerQRebin.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(D4matSlicerQRebin)
        self.statusbar.setObjectName(u"statusbar")
        D4matSlicerQRebin.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.spn1StepRatio, self.v1title)
        QWidget.setTabOrder(self.v1title, self.spn2StepRatio)
        QWidget.setTabOrder(self.spn2StepRatio, self.v2title)
        QWidget.setTabOrder(self.v2title, self.spn3StepRatio)
        QWidget.setTabOrder(self.spn3StepRatio, self.v3title)
        QWidget.setTabOrder(self.v3title, self.spn4StepRatio)
        QWidget.setTabOrder(self.spn4StepRatio, self.v4title)
        QWidget.setTabOrder(self.v4title, self.btRebinMatrix)
        QWidget.setTabOrder(self.btRebinMatrix, self.btClose)

        self.retranslateUi(D4matSlicerQRebin)

        QMetaObject.connectSlotsByName(D4matSlicerQRebin)
    # setupUi

    def retranslateUi(self, D4matSlicerQRebin):
        D4matSlicerQRebin.setWindowTitle(QCoreApplication.translate("D4matSlicerQRebin", u"MainWindow", None))
        self.panel3.setTitle(QCoreApplication.translate("D4matSlicerQRebin", u"D4Matrix Info", None))
        self.label_2.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Range", None))
        self.label_3.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Step", None))
        self.label_4.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Title", None))
        self.v1txt.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Ax1:", None))
        self.txt1.setText(QCoreApplication.translate("D4matSlicerQRebin", u"-1.0", None))
        self.label_6.setText(QCoreApplication.translate("D4matSlicerQRebin", u"to", None))
        self.txt2.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.txt3.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.label_7.setText(QCoreApplication.translate("D4matSlicerQRebin", u"to", None))
        self.txt5.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.txt4.setText(QCoreApplication.translate("D4matSlicerQRebin", u"-1.0", None))
        self.v2txt.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Ax2:", None))
        self.txt6.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.label_8.setText(QCoreApplication.translate("D4matSlicerQRebin", u"to", None))
        self.txt8.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.txt7.setText(QCoreApplication.translate("D4matSlicerQRebin", u"-1.0", None))
        self.v3txt.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Ax3:", None))
        self.txt9.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.label_9.setText(QCoreApplication.translate("D4matSlicerQRebin", u"to", None))
        self.txt11.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.txt10.setText(QCoreApplication.translate("D4matSlicerQRebin", u"-1.0", None))
        self.v4txt.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Ax4:", None))
        self.txt12.setText(QCoreApplication.translate("D4matSlicerQRebin", u"1.0", None))
        self.label.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Ratio", None))
        self.label_5.setText(QCoreApplication.translate("D4matSlicerQRebin", u"X", None))
        self.label_10.setText(QCoreApplication.translate("D4matSlicerQRebin", u"X", None))
        self.label_11.setText(QCoreApplication.translate("D4matSlicerQRebin", u"X", None))
        self.label_12.setText(QCoreApplication.translate("D4matSlicerQRebin", u"X", None))
        self.btRebinMatrix.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Rebin D4Mat", None))
        self.btClose.setText(QCoreApplication.translate("D4matSlicerQRebin", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgSaveAsText.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDialog,
    QFrame, QLabel, QLineEdit, QPushButton,
    QSizePolicy, QWidget)

class Ui_DgSaveAsText(object):
    def setupUi(self, DgSaveAsText):
        if not DgSaveAsText.objectName():
            DgSaveAsText.setObjectName(u"DgSaveAsText")
        DgSaveAsText.resize(400, 197)
        self.frame = QFrame(DgSaveAsText)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(10, 10, 371, 121))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.ckIgnoreMask = QCheckBox(self.frame)
        self.ckIgnoreMask.setObjectName(u"ckIgnoreMask")
        self.ckIgnoreMask.setGeometry(QRect(10, 10, 181, 22))
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(50, 50, 181, 17))
        self.chMaskValue = QComboBox(self.frame)
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.setObjectName(u"chMaskValue")
        self.chMaskValue.setGeometry(QRect(220, 45, 131, 27))
        self.txMaskValue = QLineEdit(self.frame)
        self.txMaskValue.setObjectName(u"txMaskValue")
        self.txMaskValue.setGeometry(QRect(220, 80, 131, 27))
        self.btSave = QPushButton(DgSaveAsText)
        self.btSave.setObjectName(u"btSave")
        self.btSave.setGeometry(QRect(30, 150, 121, 27))
        self.btCancel = QPushButton(DgSaveAsText)
        self.btCancel.setObjectName(u"btCancel")
        self.btCancel.setGeometry(QRect(240, 150, 121, 27))
        QWidget.setTabOrder(self.ckIgnoreMask, self.chMaskValue)
        QWidget.setTabOrder(self.chMaskValue, self.txMaskValue)
        QWidget.setTabOrder(self.txMaskValue, self.btSave)
        QWidget.setTabOrder(self.btSave, self.btCancel)

        self.retranslateUi(DgSaveAsText)

        QMetaObject.connectSlotsByName(DgSaveAsText)
    # setupUi

    def retranslateUi(self, DgSaveAsText):
        DgSaveAsText.setWindowTitle(QCoreApplication.translate("DgSaveAsText", u"Dialog", None))
        self.ckIgnoreMask.setText(QCoreApplication.translate("DgSaveAsText", u"Ignore Masked Points", None))
        self.label.setText(QCoreApplication.translate("DgSaveAsText", u"Replace MaskValue with", None))
        self.chMaskValue.setItemText(0, QCoreApplication.translate("DgSaveAsText", u"Value Below", None))
        self.chMaskValue.setItemText(1, QCoreApplication.translate("DgSaveAsText", u"0", None))
        self.chMaskValue.setItemText(2, QCoreApplication.translate("DgSaveAsText", u"-1", None))
        self.chMaskValue.setItemText(3, QCoreApplication.translate("DgSaveAsText", u"Original Value", None))

        self.btSave.setText(QCoreApplication.translate("DgSaveAsText", u"Save", None))
        self.btCancel.setText(QCoreApplication.translate("DgSaveAsText", u"Cancel", None))
    # retranslateUi

