from .core import *

__version__ = "0.4"

def ShowVersion():
    print "Manyo Library   ( ver.", __version__, ")"


def test():

    import sys
    import os
    sys.path.append( os.environ.get( "MANYO_PATH", "" ) + "/sample" )
    os.system( "python $MANYO_PATH/sample/ModuleTestHistOperationsLight.py" )


#    import sys
#    import os
#    sys.path.append( os.environ.get( "MANYO_PATH", "" ) + "/sample" )
#    import Test
#    TestVal = Test.TestVal
#    print "TestVal =", TestVal
#    print "If the value shown above is 0.0, Manyo-Lib is working well."


