#ifndef MULTI_DATA_LEVMAR_CONSTS
#define MULTI_DATA_LEVMAR_CONSTS
#include "AdvLevmarConsts.hh"

class AdvMultiDataLevmarConsts : public AdvLevmarConsts {
    private:
        static const string className; // =string("AdvMultiDataLevmarConsts")

    public:
        static const string PARTIAL_R_FACTOR;
        static const string PARTIAL_RESIDUAL_ERR_NORM;
};
#endif //  MULTI_DATA_LEVMAR_CONSTS
