#ifndef LEVMAR_FUNC_H
#define LEVMAR_FUNC_H

//#ifndef SWIGPYTHON

//#include <vector>
#include "Header.hh"

#ifdef __cplusplus 
extern "C" {
#endif

/** evaluate values of an objective function for levmar */
void evalFunc(Double *p,Double *y, Int4 m, Int4 n, void *adata);

/** evaluate the Jaccobian of an objective function for levmar */
void evalJaccobian(double *p, double *j, int m, int n, void *adata);

#ifdef __cplusplus
}
#endif


//#endif // SWIGPYTHON

#endif // LEVMAR_FUNC_H
