#ifndef WRITETEXTFILE
#define WRITETEXTFILE

#include "Header.hh"



class WriteTextFile
{
private:
  FILE *fp;
public:
  WriteTextFile( std::string FileName );
  /*!< Constructor. "FileName" is the target file name. */
 ~WriteTextFile();
  /*!< Destructor. The file descriptor created in this class
   will be closed in this method. */

  void Write( std::string st );
  /*!< The value of the argument is appended to the target file
   defined in the constructor. "\n" should be added at the tail end of
   the string. */

  void Write( std::vector<std::string> st );
  /*!< Each element of the argument of this method is passed to
    "Write(std::string)". Please read the manual of it.
   */
};
#endif
