#ifndef CONVERTTOSPECHOPPERS
#define CONVERTTOSPECHOPPERS

#include "UtsusemiHeader.hh"

//////////////////////////////////
// Conversion to SPE
/////////////////////////////////

//! Function to convert histogram data to SPE 
/*! 
 * 
 *  Now this just divides Intensity with width of hw
 *  in ElementContainer. 
 *  In future, other function will be included.
 */
class ConvertToSPEChoppers :
  public ManipulatorBase< ElementContainerMatrix >
{
private:

public:
    ConvertToSPEChoppers();
        //!< Constructor
        /*!<
         */
    ConvertToSPEChoppers(ElementContainerMatrix *ecm);
        //!< Constructor
        /*!<
         *  @param ecm ElementContainerMatrix for the target
         */
    ~ConvertToSPEChoppers();
        //!< Destructor
        /*!<
         */
    void ToSPE();
        //!< Convert to SPE
        /*!<
         *  @return None
         */
};
#endif
