#! /bin/bash

sudo rm -rf /Applications/Manyo*
rm -rf mlfdata/python mlfdata/py_manyo

mlfinit/setup_py_manyo.sh 
bash mlfscripts/build_manyo.sh; bash mlfscripts/installer_manyo.sh ; cp mlfout/manyo_0.0.0_*.pkg ~/Desktop

