#include <ios>
#include <ostream>
#include <iostream>
#include <iomanip>

#include <stirng>
#include <vector>

#include "Header.hh"

#include "PeakData.hh"
#include "FuncBase.hh"

#include "ContainerBaseTemplate.hh"

class ParamContainer {
    private:
        static const string className; // = "ParamContainer"
    private:
        ContainerBase< Bool >  boolMap;
        ContainerBase< Int4 >  int4Map;
        ContainerBase< UInt4 > uint4Map;
        ContainerBase< Double >                   doubleMap;
        ContainerBase< vector<Double> >           vectorMap;
        ContainerBase< vector< vector<Double> > > matrixMap;
        ContainerBase< PeakData >  peaDataMap;
}
