#include "AdvMethod.hh"
#include <ios>
#include <ostream>

/** @author TANIMORI Souichirou, AdvanceSoft Corp.
 *  @version 0.0
 */


AdvMethod::AdvMethod() {
    //std::cout << "Debug: AdvMethod::AdvMethod(): called." << endl;
}


AdvMethod::AdvMethod(const AdvMethod& method) {
    this->methodName      = method.methodName;
    this->valueOfLikehood = method.valueOfLikehood;
    //std::cout << "Debug: AdvMethod::AdvMethod(AdvMethod &): called." << endl;
}

AdvMethod::~AdvMethod() {
}

AdvMethod& AdvMethod::operator = (const AdvMethod &method) {
    this->methodName      = method.methodName;
    this->valueOfLikehood = method.valueOfLikehood;
    return *this;
}

/** 
 *  get the method name
 */
string AdvMethod::getMethodName() const {
    return methodName;
}


/**
 *  get the value of likehood
 */
Double AdvMethod::getLikehood() const {
    return valueOfLikehood;
}
