# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'SequencerQ_Main.ui'
#
# Created: Thu Apr  5 09:45:29 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1174, 755)
        MainWindow.setUnifiedTitleAndToolBarOnMac(False)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.SplashPanel = QtGui.QLabel(self.centralwidget)
        self.SplashPanel.setGeometry(QtCore.QRect(10, 10, 335, 50))
        self.SplashPanel.setText("")
        self.SplashPanel.setObjectName("SplashPanel")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(370, 10, 261, 51))
        self.groupBox.setObjectName("groupBox")
        self.btClear = QtGui.QPushButton(self.groupBox)
        self.btClear.setGeometry(QtCore.QRect(10, 15, 51, 32))
        self.btClear.setObjectName("btClear")
        self.btOpen = QtGui.QPushButton(self.groupBox)
        self.btOpen.setGeometry(QtCore.QRect(80, 15, 32, 32))
        self.btOpen.setText("")
        self.btOpen.setObjectName("btOpen")
        self.btSaveas = QtGui.QPushButton(self.groupBox)
        self.btSaveas.setGeometry(QtCore.QRect(120, 15, 32, 32))
        self.btSaveas.setText("")
        self.btSaveas.setObjectName("btSaveas")
        self.btCCD = QtGui.QPushButton(self.groupBox)
        self.btCCD.setGeometry(QtCore.QRect(160, 15, 99, 32))
        self.btCCD.setObjectName("btCCD")
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setGeometry(QtCore.QRect(650, 10, 511, 51))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btBegin = QtGui.QPushButton(self.groupBox_2)
        self.btBegin.setGeometry(QtCore.QRect(20, 15, 80, 32))
        self.btBegin.setObjectName("btBegin")
        self.btStop = QtGui.QPushButton(self.groupBox_2)
        self.btStop.setGeometry(QtCore.QRect(110, 15, 80, 32))
        self.btStop.setObjectName("btStop")
        self.btResume = QtGui.QPushButton(self.groupBox_2)
        self.btResume.setGeometry(QtCore.QRect(230, 15, 150, 32))
        self.btResume.setObjectName("btResume")
        self.btVis = QtGui.QPushButton(self.groupBox_2)
        self.btVis.setGeometry(QtCore.QRect(410, 15, 100, 32))
        self.btVis.setObjectName("btVis")
        self.line = QtGui.QFrame(self.groupBox_2)
        self.line.setGeometry(QtCore.QRect(390, 20, 20, 31))
        self.line.setFrameShape(QtGui.QFrame.VLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.tree0 = QtGui.QTreeWidget(self.centralwidget)
        self.tree0.setGeometry(QtCore.QRect(20, 70, 300, 320))
        self.tree0.setHeaderHidden(True)
        self.tree0.setObjectName("tree0")
        self.tree0.headerItem().setText(0, "1")
        self.list1 = QtGui.QTreeWidget(self.centralwidget)
        self.list1.setGeometry(QtCore.QRect(370, 110, 800, 280))
        self.list1.setObjectName("list1")
        self.list1.headerItem().setText(0, "1")
        self.st0 = QtGui.QLabel(self.centralwidget)
        self.st0.setGeometry(QtCore.QRect(380, 70, 91, 17))
        self.st0.setObjectName("st0")
        self.btAdd = QtGui.QPushButton(self.centralwidget)
        self.btAdd.setGeometry(QtCore.QRect(330, 100, 32, 32))
        self.btAdd.setText("")
        self.btAdd.setObjectName("btAdd")
        self.btRemove = QtGui.QPushButton(self.centralwidget)
        self.btRemove.setGeometry(QtCore.QRect(330, 150, 32, 32))
        self.btRemove.setText("")
        self.btRemove.setObjectName("btRemove")
        self.btCopyAdd = QtGui.QPushButton(self.centralwidget)
        self.btCopyAdd.setGeometry(QtCore.QRect(330, 220, 32, 32))
        self.btCopyAdd.setText("")
        self.btCopyAdd.setObjectName("btCopyAdd")
        self.btUp = QtGui.QPushButton(self.centralwidget)
        self.btUp.setGeometry(QtCore.QRect(330, 270, 32, 32))
        self.btUp.setText("")
        self.btUp.setObjectName("btUp")
        self.btDown = QtGui.QPushButton(self.centralwidget)
        self.btDown.setGeometry(QtCore.QRect(330, 320, 32, 32))
        self.btDown.setText("")
        self.btDown.setObjectName("btDown")
        self.stCurrentDir = QtGui.QLabel(self.centralwidget)
        self.stCurrentDir.setGeometry(QtCore.QRect(470, 70, 671, 17))
        self.stCurrentDir.setText("")
        self.stCurrentDir.setObjectName("stCurrentDir")
        self.st1 = QtGui.QLabel(self.centralwidget)
        self.st1.setGeometry(QtCore.QRect(710, 90, 68, 17))
        self.st1.setObjectName("st1")
        self.status = QtGui.QLabel(self.centralwidget)
        self.status.setGeometry(QtCore.QRect(780, 90, 161, 17))
        self.status.setObjectName("status")
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setGeometry(QtCore.QRect(380, 400, 781, 301))
        self.groupBox_3.setTitle("")
        self.groupBox_3.setObjectName("groupBox_3")
        self.st2 = QtGui.QLabel(self.groupBox_3)
        self.st2.setGeometry(QtCore.QRect(140, 20, 101, 17))
        self.st2.setObjectName("st2")
        self.retval = QtGui.QLineEdit(self.groupBox_3)
        self.retval.setGeometry(QtCore.QRect(240, 10, 150, 32))
        self.retval.setObjectName("retval")
        self.btSet = QtGui.QPushButton(self.groupBox_3)
        self.btSet.setGeometry(QtCore.QRect(590, 10, 120, 32))
        self.btSet.setObjectName("btSet")
        self.line_2 = QtGui.QFrame(self.groupBox_3)
        self.line_2.setGeometry(QtCore.QRect(10, 40, 761, 16))
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.arg1 = QtGui.QLineEdit(self.groupBox_3)
        self.arg1.setGeometry(QtCore.QRect(10, 70, 240, 24))
        self.arg1.setObjectName("arg1")
        self.arg2 = QtGui.QLineEdit(self.groupBox_3)
        self.arg2.setGeometry(QtCore.QRect(270, 70, 240, 24))
        self.arg2.setObjectName("arg2")
        self.arg3 = QtGui.QLineEdit(self.groupBox_3)
        self.arg3.setGeometry(QtCore.QRect(530, 70, 240, 24))
        self.arg3.setObjectName("arg3")
        self.arg4 = QtGui.QLineEdit(self.groupBox_3)
        self.arg4.setGeometry(QtCore.QRect(10, 120, 240, 24))
        self.arg4.setObjectName("arg4")
        self.arg5 = QtGui.QLineEdit(self.groupBox_3)
        self.arg5.setGeometry(QtCore.QRect(270, 120, 240, 24))
        self.arg5.setObjectName("arg5")
        self.arg6 = QtGui.QLineEdit(self.groupBox_3)
        self.arg6.setGeometry(QtCore.QRect(530, 120, 240, 24))
        self.arg6.setObjectName("arg6")
        self.arg7 = QtGui.QLineEdit(self.groupBox_3)
        self.arg7.setGeometry(QtCore.QRect(10, 170, 240, 24))
        self.arg7.setObjectName("arg7")
        self.arg8 = QtGui.QLineEdit(self.groupBox_3)
        self.arg8.setGeometry(QtCore.QRect(270, 170, 240, 24))
        self.arg8.setObjectName("arg8")
        self.arg9 = QtGui.QLineEdit(self.groupBox_3)
        self.arg9.setGeometry(QtCore.QRect(530, 170, 240, 24))
        self.arg9.setObjectName("arg9")
        self.arg10 = QtGui.QLineEdit(self.groupBox_3)
        self.arg10.setGeometry(QtCore.QRect(10, 220, 240, 24))
        self.arg10.setObjectName("arg10")
        self.arg11 = QtGui.QLineEdit(self.groupBox_3)
        self.arg11.setGeometry(QtCore.QRect(270, 220, 240, 24))
        self.arg11.setObjectName("arg11")
        self.arg12 = QtGui.QLineEdit(self.groupBox_3)
        self.arg12.setGeometry(QtCore.QRect(530, 220, 240, 24))
        self.arg12.setObjectName("arg12")
        self.arg13 = QtGui.QLineEdit(self.groupBox_3)
        self.arg13.setGeometry(QtCore.QRect(10, 270, 240, 24))
        self.arg13.setObjectName("arg13")
        self.arg14 = QtGui.QLineEdit(self.groupBox_3)
        self.arg14.setGeometry(QtCore.QRect(270, 270, 240, 24))
        self.arg14.setObjectName("arg14")
        self.arg15 = QtGui.QLineEdit(self.groupBox_3)
        self.arg15.setGeometry(QtCore.QRect(530, 270, 240, 24))
        self.arg15.setObjectName("arg15")
        self.argst1 = QtGui.QLabel(self.groupBox_3)
        self.argst1.setGeometry(QtCore.QRect(20, 50, 231, 17))
        self.argst1.setObjectName("argst1")
        self.argst2 = QtGui.QLabel(self.groupBox_3)
        self.argst2.setGeometry(QtCore.QRect(280, 50, 231, 17))
        self.argst2.setObjectName("argst2")
        self.argst3 = QtGui.QLabel(self.groupBox_3)
        self.argst3.setGeometry(QtCore.QRect(540, 50, 231, 17))
        self.argst3.setObjectName("argst3")
        self.argst4 = QtGui.QLabel(self.groupBox_3)
        self.argst4.setGeometry(QtCore.QRect(20, 100, 231, 17))
        self.argst4.setObjectName("argst4")
        self.argst5 = QtGui.QLabel(self.groupBox_3)
        self.argst5.setGeometry(QtCore.QRect(280, 100, 231, 17))
        self.argst5.setObjectName("argst5")
        self.argst6 = QtGui.QLabel(self.groupBox_3)
        self.argst6.setGeometry(QtCore.QRect(540, 100, 231, 17))
        self.argst6.setObjectName("argst6")
        self.argst7 = QtGui.QLabel(self.groupBox_3)
        self.argst7.setGeometry(QtCore.QRect(20, 150, 231, 17))
        self.argst7.setObjectName("argst7")
        self.argst8 = QtGui.QLabel(self.groupBox_3)
        self.argst8.setGeometry(QtCore.QRect(280, 150, 231, 17))
        self.argst8.setObjectName("argst8")
        self.argst9 = QtGui.QLabel(self.groupBox_3)
        self.argst9.setGeometry(QtCore.QRect(540, 150, 231, 17))
        self.argst9.setObjectName("argst9")
        self.argst10 = QtGui.QLabel(self.groupBox_3)
        self.argst10.setGeometry(QtCore.QRect(20, 200, 231, 17))
        self.argst10.setObjectName("argst10")
        self.argst11 = QtGui.QLabel(self.groupBox_3)
        self.argst11.setGeometry(QtCore.QRect(280, 200, 231, 17))
        self.argst11.setObjectName("argst11")
        self.argst12 = QtGui.QLabel(self.groupBox_3)
        self.argst12.setGeometry(QtCore.QRect(540, 200, 231, 17))
        self.argst12.setObjectName("argst12")
        self.argst13 = QtGui.QLabel(self.groupBox_3)
        self.argst13.setGeometry(QtCore.QRect(20, 250, 231, 17))
        self.argst13.setObjectName("argst13")
        self.argst14 = QtGui.QLabel(self.groupBox_3)
        self.argst14.setGeometry(QtCore.QRect(280, 250, 231, 17))
        self.argst14.setObjectName("argst14")
        self.argst15 = QtGui.QLabel(self.groupBox_3)
        self.argst15.setGeometry(QtCore.QRect(540, 250, 231, 17))
        self.argst15.setObjectName("argst15")
        self.funcdoc = QtGui.QTextBrowser(self.centralwidget)
        self.funcdoc.setGeometry(QtCore.QRect(20, 400, 330, 280))
        self.funcdoc.setObjectName("funcdoc")
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionFile = QtGui.QAction(MainWindow)
        self.actionFile.setObjectName("actionFile")

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "MainWindow", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.btClear.setText(QtGui.QApplication.translate("MainWindow", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.btCCD.setText(QtGui.QApplication.translate("MainWindow", "Current Dir", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("MainWindow", "Control", None, QtGui.QApplication.UnicodeUTF8))
        self.btBegin.setText(QtGui.QApplication.translate("MainWindow", "Start", None, QtGui.QApplication.UnicodeUTF8))
        self.btStop.setText(QtGui.QApplication.translate("MainWindow", "Stop", None, QtGui.QApplication.UnicodeUTF8))
        self.btResume.setText(QtGui.QApplication.translate("MainWindow", "Resume", None, QtGui.QApplication.UnicodeUTF8))
        self.btVis.setText(QtGui.QApplication.translate("MainWindow", "Visualize", None, QtGui.QApplication.UnicodeUTF8))
        self.st0.setText(QtGui.QApplication.translate("MainWindow", "Current Dir :", None, QtGui.QApplication.UnicodeUTF8))
        self.st1.setText(QtGui.QApplication.translate("MainWindow", "Status :", None, QtGui.QApplication.UnicodeUTF8))
        self.status.setText(QtGui.QApplication.translate("MainWindow", "Executable", None, QtGui.QApplication.UnicodeUTF8))
        self.st2.setText(QtGui.QApplication.translate("MainWindow", "Return Label :", None, QtGui.QApplication.UnicodeUTF8))
        self.btSet.setText(QtGui.QApplication.translate("MainWindow", "Set", None, QtGui.QApplication.UnicodeUTF8))
        self.argst1.setText(QtGui.QApplication.translate("MainWindow", "Argument1", None, QtGui.QApplication.UnicodeUTF8))
        self.argst2.setText(QtGui.QApplication.translate("MainWindow", "Argument2", None, QtGui.QApplication.UnicodeUTF8))
        self.argst3.setText(QtGui.QApplication.translate("MainWindow", "Argument3", None, QtGui.QApplication.UnicodeUTF8))
        self.argst4.setText(QtGui.QApplication.translate("MainWindow", "Argument4", None, QtGui.QApplication.UnicodeUTF8))
        self.argst5.setText(QtGui.QApplication.translate("MainWindow", "Argument5", None, QtGui.QApplication.UnicodeUTF8))
        self.argst6.setText(QtGui.QApplication.translate("MainWindow", "Argument6", None, QtGui.QApplication.UnicodeUTF8))
        self.argst7.setText(QtGui.QApplication.translate("MainWindow", "Argument7", None, QtGui.QApplication.UnicodeUTF8))
        self.argst8.setText(QtGui.QApplication.translate("MainWindow", "Argument8", None, QtGui.QApplication.UnicodeUTF8))
        self.argst9.setText(QtGui.QApplication.translate("MainWindow", "Argument9", None, QtGui.QApplication.UnicodeUTF8))
        self.argst10.setText(QtGui.QApplication.translate("MainWindow", "Argument10", None, QtGui.QApplication.UnicodeUTF8))
        self.argst11.setText(QtGui.QApplication.translate("MainWindow", "Argument11", None, QtGui.QApplication.UnicodeUTF8))
        self.argst12.setText(QtGui.QApplication.translate("MainWindow", "Argument12", None, QtGui.QApplication.UnicodeUTF8))
        self.argst13.setText(QtGui.QApplication.translate("MainWindow", "Argument13", None, QtGui.QApplication.UnicodeUTF8))
        self.argst14.setText(QtGui.QApplication.translate("MainWindow", "Argument14", None, QtGui.QApplication.UnicodeUTF8))
        self.argst15.setText(QtGui.QApplication.translate("MainWindow", "Argument15", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFile.setText(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'SequencerQ_Save.ui'
#
# Created: Thu Apr  5 09:45:29 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(342, 170)
        self.comment = QtGui.QTextEdit(Dialog)
        self.comment.setGeometry(QtCore.QRect(10, 40, 320, 80))
        self.comment.setObjectName("comment")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(10, 10, 261, 17))
        self.label.setObjectName("label")
        self.btOk = QtGui.QPushButton(Dialog)
        self.btOk.setGeometry(QtCore.QRect(50, 130, 100, 32))
        self.btOk.setObjectName("btOk")
        self.btCancel = QtGui.QPushButton(Dialog)
        self.btCancel.setGeometry(QtCore.QRect(180, 130, 100, 32))
        self.btCancel.setObjectName("btCancel")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Comments for the sequence :", None, QtGui.QApplication.UnicodeUTF8))
        self.btOk.setText(QtGui.QApplication.translate("Dialog", "OK", None, QtGui.QApplication.UnicodeUTF8))
        self.btCancel.setText(QtGui.QApplication.translate("Dialog", "Cancel", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'SequencerQ_Vis.ui'
#
# Created: Thu Apr  5 09:45:29 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_visframe(object):
    def setupUi(self, visframe):
        visframe.setObjectName("visframe")
        visframe.resize(558, 333)
        self.centralwidget = QtGui.QWidget(visframe)
        self.centralwidget.setObjectName("centralwidget")
        self.tree = QtGui.QTreeWidget(self.centralwidget)
        self.tree.setGeometry(QtCore.QRect(10, 10, 200, 265))
        self.tree.setHeaderHidden(True)
        self.tree.setObjectName("tree")
        self.tree.headerItem().setText(0, "1")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(220, 10, 221, 261))
        self.groupBox.setObjectName("groupBox")
        self.arg1 = QtGui.QLineEdit(self.groupBox)
        self.arg1.setGeometry(QtCore.QRect(110, 20, 100, 28))
        self.arg1.setObjectName("arg1")
        self.arg2 = QtGui.QLineEdit(self.groupBox)
        self.arg2.setGeometry(QtCore.QRect(110, 50, 100, 28))
        self.arg2.setObjectName("arg2")
        self.arg3 = QtGui.QLineEdit(self.groupBox)
        self.arg3.setGeometry(QtCore.QRect(110, 80, 100, 28))
        self.arg3.setObjectName("arg3")
        self.arg4 = QtGui.QLineEdit(self.groupBox)
        self.arg4.setGeometry(QtCore.QRect(110, 110, 100, 28))
        self.arg4.setObjectName("arg4")
        self.arg5 = QtGui.QLineEdit(self.groupBox)
        self.arg5.setGeometry(QtCore.QRect(110, 140, 100, 28))
        self.arg5.setObjectName("arg5")
        self.arg6 = QtGui.QLineEdit(self.groupBox)
        self.arg6.setGeometry(QtCore.QRect(110, 170, 100, 28))
        self.arg6.setObjectName("arg6")
        self.arg7 = QtGui.QLineEdit(self.groupBox)
        self.arg7.setGeometry(QtCore.QRect(110, 200, 100, 28))
        self.arg7.setObjectName("arg7")
        self.arg8 = QtGui.QLineEdit(self.groupBox)
        self.arg8.setGeometry(QtCore.QRect(110, 230, 100, 28))
        self.arg8.setObjectName("arg8")
        self.argst1 = QtGui.QLabel(self.groupBox)
        self.argst1.setGeometry(QtCore.QRect(0, 30, 101, 20))
        self.argst1.setObjectName("argst1")
        self.argst2 = QtGui.QLabel(self.groupBox)
        self.argst2.setGeometry(QtCore.QRect(0, 60, 101, 20))
        self.argst2.setObjectName("argst2")
        self.argst3 = QtGui.QLabel(self.groupBox)
        self.argst3.setGeometry(QtCore.QRect(0, 90, 101, 20))
        self.argst3.setObjectName("argst3")
        self.argst4 = QtGui.QLabel(self.groupBox)
        self.argst4.setGeometry(QtCore.QRect(0, 120, 101, 20))
        self.argst4.setObjectName("argst4")
        self.argst5 = QtGui.QLabel(self.groupBox)
        self.argst5.setGeometry(QtCore.QRect(0, 150, 101, 20))
        self.argst5.setObjectName("argst5")
        self.argst6 = QtGui.QLabel(self.groupBox)
        self.argst6.setGeometry(QtCore.QRect(0, 180, 101, 20))
        self.argst6.setObjectName("argst6")
        self.argst7 = QtGui.QLabel(self.groupBox)
        self.argst7.setGeometry(QtCore.QRect(0, 210, 101, 20))
        self.argst7.setObjectName("argst7")
        self.argst8 = QtGui.QLabel(self.groupBox)
        self.argst8.setGeometry(QtCore.QRect(0, 240, 101, 20))
        self.argst8.setObjectName("argst8")
        self.retval = QtGui.QLineEdit(self.centralwidget)
        self.retval.setGeometry(QtCore.QRect(450, 60, 100, 28))
        self.retval.setObjectName("retval")
        self.label = QtGui.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(450, 40, 91, 17))
        self.label.setObjectName("label")
        self.btExec = QtGui.QPushButton(self.centralwidget)
        self.btExec.setGeometry(QtCore.QRect(468, 110, 80, 32))
        self.btExec.setObjectName("btExec")
        self.btClose = QtGui.QPushButton(self.centralwidget)
        self.btClose.setGeometry(QtCore.QRect(470, 170, 80, 32))
        self.btClose.setObjectName("btClose")
        visframe.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(visframe)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 558, 25))
        self.menubar.setObjectName("menubar")
        visframe.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(visframe)
        self.statusbar.setObjectName("statusbar")
        visframe.setStatusBar(self.statusbar)

        self.retranslateUi(visframe)
        QtCore.QMetaObject.connectSlotsByName(visframe)

    def retranslateUi(self, visframe):
        visframe.setWindowTitle(QtGui.QApplication.translate("visframe", "MainWindow", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("visframe", "Arguments", None, QtGui.QApplication.UnicodeUTF8))
        self.argst1.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst2.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst3.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst4.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst5.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst6.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst7.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst8.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("visframe", "Return Label", None, QtGui.QApplication.UnicodeUTF8))
        self.btExec.setText(QtGui.QApplication.translate("visframe", "Execute", None, QtGui.QApplication.UnicodeUTF8))
        self.btClose.setText(QtGui.QApplication.translate("visframe", "Close", None, QtGui.QApplication.UnicodeUTF8))

