
import Manyo as mm
import Manyo.Utsusemi as mu

Wfile = "/home/yinamura/dev/ManyoRelease/yinamura_SIK000373_DI_hostpc.xml"
Dfile = "/home/yinamura/dev/ManyoRelease/DetectorInfo.xml"

InstCode = "SIK"
RunNumber = 373
DataPath = "/data"
T0DataPath = "/home/yinamura/ana/tmp"

evt = mu.UtsusemiEventDataConverter3HePsd()
evt.LoadParamFiles( Wfile, Dfile )
evt.SetHistAllocation()

ecm = mm.ElementContainerMatrix()

evt.SetElementContainerMatrix( ecm, InstCode, RunNumber, DataPath, T0DataPath )
