# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DetectorViewer.ui',
# licensing of 'DetectorViewer.ui' applies.
#
# Created: Tue Jun 18 14:40:01 2019
#      by: pyside2-uic  running on PySide2 5.12.3
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 140)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(800, 140))
        MainWindow.setMaximumSize(QtCore.QSize(800, 140))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/ugao_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btn_open = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_open.sizePolicy().hasHeightForWidth())
        self.btn_open.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/folder_open.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_open.setIcon(icon1)
        self.btn_open.setIconSize(QtCore.QSize(32, 32))
        self.btn_open.setObjectName("btn_open")
        self.horizontalLayout_4.addWidget(self.btn_open)
        self.grp_x_rng = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.grp_x_rng.sizePolicy().hasHeightForWidth())
        self.grp_x_rng.setSizePolicy(sizePolicy)
        self.grp_x_rng.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.grp_x_rng.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.grp_x_rng.setFlat(False)
        self.grp_x_rng.setObjectName("grp_x_rng")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.grp_x_rng)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(10, 10, 331, 61))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_x_start = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_x_start.setObjectName("label_x_start")
        self.horizontalLayout.addWidget(self.label_x_start)
        self.txt_x_start = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.txt_x_start.setObjectName("txt_x_start")
        self.horizontalLayout.addWidget(self.txt_x_start)
        self.label_x_end = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_x_end.setObjectName("label_x_end")
        self.horizontalLayout.addWidget(self.label_x_end)
        self.txt_x_end = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.txt_x_end.setObjectName("txt_x_end")
        self.horizontalLayout.addWidget(self.txt_x_end)
        self.btn_x_apply = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.btn_x_apply.setObjectName("btn_x_apply")
        self.horizontalLayout.addWidget(self.btn_x_apply)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_4.addWidget(self.grp_x_rng)
        self.grp_cb = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.grp_cb.sizePolicy().hasHeightForWidth())
        self.grp_cb.setSizePolicy(sizePolicy)
        self.grp_cb.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.grp_cb.setObjectName("grp_cb")
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.grp_cb)
        self.verticalLayoutWidget_2.setGeometry(QtCore.QRect(10, 10, 321, 61))
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_cb_min = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_cb_min.setObjectName("label_cb_min")
        self.horizontalLayout_2.addWidget(self.label_cb_min)
        self.txt_cb_min = QtWidgets.QLineEdit(self.verticalLayoutWidget_2)
        self.txt_cb_min.setObjectName("txt_cb_min")
        self.horizontalLayout_2.addWidget(self.txt_cb_min)
        self.label_cb_max = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_cb_max.setObjectName("label_cb_max")
        self.horizontalLayout_2.addWidget(self.label_cb_max)
        self.txt_cb_max = QtWidgets.QLineEdit(self.verticalLayoutWidget_2)
        self.txt_cb_max.setObjectName("txt_cb_max")
        self.horizontalLayout_2.addWidget(self.txt_cb_max)
        self.btn_cb_apply = QtWidgets.QPushButton(self.verticalLayoutWidget_2)
        self.btn_cb_apply.setObjectName("btn_cb_apply")
        self.horizontalLayout_2.addWidget(self.btn_cb_apply)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4.addWidget(self.grp_cb)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_4)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 17))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuPlot = QtWidgets.QMenu(self.menubar)
        self.menuPlot.setObjectName("menuPlot")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuWindow = QtWidgets.QMenu(self.menubar)
        self.menuWindow.setObjectName("menuWindow")
        self.menuOthers = QtWidgets.QMenu(self.menubar)
        self.menuOthers.setObjectName("menuOthers")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen = QtWidgets.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionLoad_data = QtWidgets.QAction(MainWindow)
        self.actionLoad_data.setObjectName("actionLoad_data")
        self.actionTwo_theta = QtWidgets.QAction(MainWindow)
        self.actionTwo_theta.setObjectName("actionTwo_theta")
        self.actionFormula = QtWidgets.QAction(MainWindow)
        self.actionFormula.setObjectName("actionFormula")
        self.actionPicker_tolerance = QtWidgets.QAction(MainWindow)
        self.actionPicker_tolerance.setObjectName("actionPicker_tolerance")
        self.actionMask_view = QtWidgets.QAction(MainWindow)
        self.actionMask_view.setCheckable(True)
        self.actionMask_view.setChecked(True)
        self.actionMask_view.setObjectName("actionMask_view")
        self.actionPixel_information = QtWidgets.QAction(MainWindow)
        self.actionPixel_information.setObjectName("actionPixel_information")
        self.actionCreate_dummy_data = QtWidgets.QAction(MainWindow)
        self.actionCreate_dummy_data.setObjectName("actionCreate_dummy_data")
        self.actionHelp = QtWidgets.QAction(MainWindow)
        self.actionHelp.setObjectName("actionHelp")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionExit)
        self.menuPlot.addAction(self.actionLoad_data)
        self.menuPlot.addAction(self.actionTwo_theta)
        self.menuPlot.addAction(self.actionFormula)
        self.menuEdit.addAction(self.actionPicker_tolerance)
        self.menuEdit.addAction(self.actionMask_view)
        self.menuWindow.addAction(self.actionPixel_information)
        self.menuOthers.addAction(self.actionCreate_dummy_data)
        self.menuHelp.addAction(self.actionHelp)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuPlot.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuWindow.menuAction())
        self.menubar.addAction(self.menuOthers.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "DetectorViewer", None, -1))
        self.btn_open.setText(QtWidgets.QApplication.translate("MainWindow", "Open", None, -1))
        self.grp_x_rng.setTitle(QtWidgets.QApplication.translate("MainWindow", "X range", None, -1))
        self.label_x_start.setText(QtWidgets.QApplication.translate("MainWindow", "Start", None, -1))
        self.label_x_end.setText(QtWidgets.QApplication.translate("MainWindow", "End", None, -1))
        self.btn_x_apply.setText(QtWidgets.QApplication.translate("MainWindow", "Apply", None, -1))
        self.grp_cb.setTitle(QtWidgets.QApplication.translate("MainWindow", "Color bar", None, -1))
        self.label_cb_min.setText(QtWidgets.QApplication.translate("MainWindow", "Min.", None, -1))
        self.label_cb_max.setText(QtWidgets.QApplication.translate("MainWindow", "Max.", None, -1))
        self.btn_cb_apply.setText(QtWidgets.QApplication.translate("MainWindow", "Apply", None, -1))
        self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.menuPlot.setTitle(QtWidgets.QApplication.translate("MainWindow", "Plot", None, -1))
        self.menuEdit.setTitle(QtWidgets.QApplication.translate("MainWindow", "Edit", None, -1))
        self.menuWindow.setTitle(QtWidgets.QApplication.translate("MainWindow", "Window", None, -1))
        self.menuOthers.setTitle(QtWidgets.QApplication.translate("MainWindow", "Others", None, -1))
        self.menuHelp.setTitle(QtWidgets.QApplication.translate("MainWindow", "Help", None, -1))
        self.actionOpen.setText(QtWidgets.QApplication.translate("MainWindow", "Open", None, -1))
        self.actionExit.setText(QtWidgets.QApplication.translate("MainWindow", "Exit", None, -1))
        self.actionLoad_data.setText(QtWidgets.QApplication.translate("MainWindow", "Load data", None, -1))
        self.actionTwo_theta.setText(QtWidgets.QApplication.translate("MainWindow", "Two-theta", None, -1))
        self.actionFormula.setText(QtWidgets.QApplication.translate("MainWindow", "Formula", None, -1))
        self.actionPicker_tolerance.setText(QtWidgets.QApplication.translate("MainWindow", "Picker tolerance", None, -1))
        self.actionMask_view.setText(QtWidgets.QApplication.translate("MainWindow", "Mask view", None, -1))
        self.actionPixel_information.setText(QtWidgets.QApplication.translate("MainWindow", "Pixel information", None, -1))
        self.actionCreate_dummy_data.setText(QtWidgets.QApplication.translate("MainWindow", "Create dummy data", None, -1))
        self.actionHelp.setText(QtWidgets.QApplication.translate("MainWindow", "Help", None, -1))
        self.actionAbout.setText(QtWidgets.QApplication.translate("MainWindow", "About", None, -1))


# Form implementation generated from reading ui file 'PixelInfo.ui',
# licensing of 'PixelInfo.ui' applies.
#
# Created: Tue Jun 18 14:40:01 2019
#      by: pyside2-uic  running on PySide2 5.12.3
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(320, 480)
        self.gridLayoutWidget = QtWidgets.QWidget(Dialog)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(10, 10, 291, 411))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.txt_pixel = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_pixel.setObjectName("txt_pixel")
        self.gridLayout.addWidget(self.txt_pixel, 11, 0, 1, 1)
        self.txt_intensity_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_intensity_val.setText("")
        self.txt_intensity_val.setObjectName("txt_intensity_val")
        self.gridLayout.addWidget(self.txt_intensity_val, 0, 1, 1, 1)
        self.txt_y = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_y.setObjectName("txt_y")
        self.gridLayout.addWidget(self.txt_y, 2, 0, 1, 1)
        self.txt_x = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_x.setObjectName("txt_x")
        self.gridLayout.addWidget(self.txt_x, 1, 0, 1, 1)
        self.txt_y_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_y_val.setText("")
        self.txt_y_val.setObjectName("txt_y_val")
        self.gridLayout.addWidget(self.txt_y_val, 2, 1, 1, 1)
        self.txt_x_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_x_val.setText("")
        self.txt_x_val.setObjectName("txt_x_val")
        self.gridLayout.addWidget(self.txt_x_val, 1, 1, 1, 1)
        self.txt_rel_y_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_y_val.setText("")
        self.txt_rel_y_val.setObjectName("txt_rel_y_val")
        self.gridLayout.addWidget(self.txt_rel_y_val, 7, 1, 1, 1)
        self.txt_z = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_z.setObjectName("txt_z")
        self.gridLayout.addWidget(self.txt_z, 3, 0, 1, 1)
        self.txt_rel_x = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_x.setObjectName("txt_rel_x")
        self.gridLayout.addWidget(self.txt_rel_x, 6, 0, 1, 1)
        self.txt_rel_z = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_z.setObjectName("txt_rel_z")
        self.gridLayout.addWidget(self.txt_rel_z, 8, 0, 1, 1)
        self.txt_azimuth = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_azimuth.setObjectName("txt_azimuth")
        self.gridLayout.addWidget(self.txt_azimuth, 5, 0, 1, 1)
        self.txt_azimuth_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_azimuth_val.setText("")
        self.txt_azimuth_val.setObjectName("txt_azimuth_val")
        self.gridLayout.addWidget(self.txt_azimuth_val, 5, 1, 1, 1)
        self.txt_rel_y = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_y.setObjectName("txt_rel_y")
        self.gridLayout.addWidget(self.txt_rel_y, 7, 0, 1, 1)
        self.txt_rel_x_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_x_val.setText("")
        self.txt_rel_x_val.setObjectName("txt_rel_x_val")
        self.gridLayout.addWidget(self.txt_rel_x_val, 6, 1, 1, 1)
        self.txt_z_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_z_val.setText("")
        self.txt_z_val.setObjectName("txt_z_val")
        self.gridLayout.addWidget(self.txt_z_val, 3, 1, 1, 1)
        self.txt_tth = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_tth.setObjectName("txt_tth")
        self.gridLayout.addWidget(self.txt_tth, 4, 0, 1, 1)
        self.txt_tth_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_tth_val.setText("")
        self.txt_tth_val.setObjectName("txt_tth_val")
        self.gridLayout.addWidget(self.txt_tth_val, 4, 1, 1, 1)
        self.txt_intensity = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_intensity.setObjectName("txt_intensity")
        self.gridLayout.addWidget(self.txt_intensity, 0, 0, 1, 1)
        self.txt_bank = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_bank.setObjectName("txt_bank")
        self.gridLayout.addWidget(self.txt_bank, 9, 0, 1, 1)
        self.txt_position_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_position_val.setText("")
        self.txt_position_val.setObjectName("txt_position_val")
        self.gridLayout.addWidget(self.txt_position_val, 11, 1, 1, 1)
        self.txt_rel_z_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_rel_z_val.setText("")
        self.txt_rel_z_val.setObjectName("txt_rel_z_val")
        self.gridLayout.addWidget(self.txt_rel_z_val, 8, 1, 1, 1)
        self.txt_bank_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_bank_val.setText("")
        self.txt_bank_val.setObjectName("txt_bank_val")
        self.gridLayout.addWidget(self.txt_bank_val, 9, 1, 1, 1)
        self.txt_det_id = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_det_id.setObjectName("txt_det_id")
        self.gridLayout.addWidget(self.txt_det_id, 10, 0, 1, 1)
        self.txt_det_id_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_det_id_val.setText("")
        self.txt_det_id_val.setObjectName("txt_det_id_val")
        self.gridLayout.addWidget(self.txt_det_id_val, 10, 1, 1, 1)
        self.txt_point = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_point.setObjectName("txt_point")
        self.gridLayout.addWidget(self.txt_point, 12, 0, 1, 1)
        self.txt_point_val = QtWidgets.QLabel(self.gridLayoutWidget)
        self.txt_point_val.setText("")
        self.txt_point_val.setObjectName("txt_point_val")
        self.gridLayout.addWidget(self.txt_point_val, 12, 1, 1, 1)
        self.horizontalLayoutWidget = QtWidgets.QWidget(Dialog)
        self.horizontalLayoutWidget.setGeometry(QtCore.QRect(10, 420, 291, 41))
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout.setSpacing(20)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_new_plot = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.btn_new_plot.setObjectName("btn_new_plot")
        self.horizontalLayout.addWidget(self.btn_new_plot)
        self.btn_append_plot = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.btn_append_plot.setObjectName("btn_append_plot")
        self.horizontalLayout.addWidget(self.btn_append_plot)
        self.btn_2d_plot = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.btn_2d_plot.setObjectName("btn_2d_plot")
        self.horizontalLayout.addWidget(self.btn_2d_plot)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtWidgets.QApplication.translate("Dialog", "Dialog", None, -1))
        self.txt_pixel.setText(QtWidgets.QApplication.translate("Dialog", "Pixel position", None, -1))
        self.txt_y.setText(QtWidgets.QApplication.translate("Dialog", "Y", None, -1))
        self.txt_x.setText(QtWidgets.QApplication.translate("Dialog", "X", None, -1))
        self.txt_z.setText(QtWidgets.QApplication.translate("Dialog", "Z", None, -1))
        self.txt_rel_x.setText(QtWidgets.QApplication.translate("Dialog", "Relative X", None, -1))
        self.txt_rel_z.setText(QtWidgets.QApplication.translate("Dialog", "Relative Z", None, -1))
        self.txt_azimuth.setText(QtWidgets.QApplication.translate("Dialog", "Azimuth", None, -1))
        self.txt_rel_y.setText(QtWidgets.QApplication.translate("Dialog", "Relative Y", None, -1))
        self.txt_tth.setText(QtWidgets.QApplication.translate("Dialog", "Two theta", None, -1))
        self.txt_intensity.setText(QtWidgets.QApplication.translate("Dialog", "Intensity", None, -1))
        self.txt_bank.setText(QtWidgets.QApplication.translate("Dialog", "Bank name and ID", None, -1))
        self.txt_det_id.setText(QtWidgets.QApplication.translate("Dialog", "Detector ID", None, -1))
        self.txt_point.setText(QtWidgets.QApplication.translate("Dialog", "Point ID", None, -1))
        self.btn_new_plot.setText(QtWidgets.QApplication.translate("Dialog", "New plot", None, -1))
        self.btn_append_plot.setText(QtWidgets.QApplication.translate("Dialog", "Append plot", None, -1))
        self.btn_2d_plot.setText(QtWidgets.QApplication.translate("Dialog", "2D plot", None, -1))

try:
    import DetectorViewer_rc
except:
    import uGao.DetectorViewer_rc
