#ifndef UTSUSEMIWIRINGINFOEDITORBASE
#define UTSUSEMIWIRINGINFOEDITORBASE

#include "Header.hh"
#include  <boost/filesystem.hpp>
#include "UtsusemiHeader.hh"
#include "StringTools.hh"
#include "WiringInfoEditorBase.hh"
#include "UtsusemiWiringInfoEditorTemplate.hh"

class UtsusemiWiringInfoEditorBase:
    public UtsusemiWiringInfoEditorTemplate< WiringInfoEditorBase >
{
private:
    void Initialize();
    std::string _MessageTag;

protected:


public:
    UtsusemiWiringInfoEditorBase();
        //!< Constructor
        /*!<
         */
    ~UtsusemiWiringInfoEditorBase();
        //!< Destructor
        /*!<
         */
    //////////////////////////////////////////////////////////////////////////
    // public variables
    //////////////////////////////////////////////////////////////////////////
    bool _Status;
    bool _withPH;
    //////////////////////////////////////////////////////////////////////////
    // public functions
    //////////////////////////////////////////////////////////////////////////

};
#endif
