#include <string>
#include "Header.hh"
#include "AdvMethodType.hh"

const AdvMethodType UNKNOWN_METHOD_TYPE  = { UNKNOWN_METHOD_ID,       string("unkown"),               string("unknown")   };
const AdvMethodType BSPLINE              = { BSPLINE_ID,              string("B spline"),             string("BS")        };
const AdvMethodType MOVING_AVERAGE       = { MOVING_AVERAGE_ID,       string("Moving Average"),       string("MA")        };
const AdvMethodType LEVMAR               = { LEVMAR_ID,               string("Levmar"),               string("LM")        };
const AdvMethodType NEW_LEVMAR           = { NEW_LEVMAR_ID,           string("New Levmar"),           string("NLM")       };
const AdvMethodType MULTIDATA_LEVMAR     = { MULTIDATA_LEVMAR_ID,     string("Multi Data Levmar"),    string("MDLM")      };
const AdvMethodType CONVOLUTION_BY_FFT   = { CONVOLUTION_BY_FFT_ID,   string("Convolution By FFT"),   string("CONVFFT")   };
const AdvMethodType DECONVOLUTION_BY_FFT = { DECONVOLUTION_BY_FFT_ID, string("Deconvolution By FFT"), string("DECONVFFT") };
const AdvMethodType EQUAL_SPACING        = { EQUAL_SPACING_ID,        string("Equal Spacing"),        string("ES")        };

