#ifndef UTSUSEMIEVENTDATAMONITORRPMT
#define UTSUSEMIEVENTDATAMONITORRPMT

#include "UtsusemiEventDataMonitorTemplate.hh"
#include "UtsusemiEventDataConverterRPMT.hh"
#include "T0TreatToolsNeunet.hh"

class UtsusemiEventDataMonitorRPMT:
    public UtsusemiEventDataMonitorTemplate<UtsusemiEventDataConverterRPMT>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiEventDataMonitorRPMT();
        //!< Constructor
        /*!<
         */
    ~UtsusemiEventDataMonitorRPMT();
        //!< Destructor
        /*!<
         */
};
#endif
