#!/bin/bash

pyside6-uic VisContQ_Main.ui -o ui6_VisContQ.py
pyside6-uic VisContQ_DataOperationDialog.ui -o ui_tmp1.py
pyside6-uic VisContQ_SettingDialog.ui -o ui_tmp2.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide2 import QtCore, QtGui, QtWidgets" >> ui6_VisContQ.py
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide2 import QtCore, QtGui, QtWidgets" >> ui6_VisContQ.py

rm ui_tmp1.py ui_tmp2.py
