from uGao.U2IFJupyter import U2IFJupyter

class U2IFInDark(U2IFJupyter):
    """
    U2IFInDark class

    This enables to save plot file using u2dplot of uGao without GUI

    import uGao.U2IFInDark as mp
    p = mp.U2IFInDark(sizeX=10.0, sizeY=5.0)
    p.SetKeys("index", "index", "Intensity")
    p.SetData(dat)
    p.SetRange(False, 0.0, 40.0)
    p.SetTitles("Test in Dark", "")
    p.SetLabels("QA", "hw")
    p.SaveFile("test_InDark.png")
    """
    def __init__(self, data=None, orderNo=1, sizeX=7.0, sizeY=7.0, dpi=100):
        super().__init__(data, orderNo, sizeX, sizeY, dpi)

