# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'FastPlot.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_FastPlot(object):
    def setupUi(self, FastPlot):
        if not FastPlot.objectName():
            FastPlot.setObjectName(u"FastPlot")
        FastPlot.resize(750, 579)
        FastPlot.setIconSize(QSize(24, 24))
        self.centralwidget = QWidget(FastPlot)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayoutWidget = QWidget(self.centralwidget)
        self.verticalLayoutWidget.setObjectName(u"verticalLayoutWidget")
        self.verticalLayoutWidget.setGeometry(QRect(0, 0, 751, 531))
        self.topLayout = QVBoxLayout(self.verticalLayoutWidget)
        self.topLayout.setObjectName(u"topLayout")
        self.topLayout.setContentsMargins(0, 0, 0, 0)
        self.ButtonFrame = QFrame(self.verticalLayoutWidget)
        self.ButtonFrame.setObjectName(u"ButtonFrame")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ButtonFrame.sizePolicy().hasHeightForWidth())
        self.ButtonFrame.setSizePolicy(sizePolicy)
        self.ButtonFrame.setMinimumSize(QSize(0, 40))
        self.ButtonFrame.setFrameShape(QFrame.StyledPanel)
        self.ButtonFrame.setFrameShadow(QFrame.Raised)
        self.btPrev = QPushButton(self.ButtonFrame)
        self.btPrev.setObjectName(u"btPrev")
        self.btPrev.setGeometry(QRect(30, 10, 121, 27))
        self.btNxt = QPushButton(self.ButtonFrame)
        self.btNxt.setObjectName(u"btNxt")
        self.btNxt.setGeometry(QRect(170, 10, 98, 27))
        self.strNum = QLabel(self.ButtonFrame)
        self.strNum.setObjectName(u"strNum")
        self.strNum.setEnabled(True)
        self.strNum.setGeometry(QRect(300, 10, 180, 20))
        self.strXY = QLabel(self.ButtonFrame)
        self.strXY.setObjectName(u"strXY")
        self.strXY.setEnabled(True)
        self.strXY.setGeometry(QRect(480, 10, 251, 21))

        self.topLayout.addWidget(self.ButtonFrame)

        FastPlot.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(FastPlot)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setEnabled(True)
        self.menubar.setGeometry(QRect(0, 0, 750, 25))
        FastPlot.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(FastPlot)
        self.statusbar.setObjectName(u"statusbar")
        FastPlot.setStatusBar(self.statusbar)

        self.retranslateUi(FastPlot)

        QMetaObject.connectSlotsByName(FastPlot)
    # setupUi

    def retranslateUi(self, FastPlot):
        FastPlot.setWindowTitle(QCoreApplication.translate("FastPlot", u"MainWindow", None))
        self.btPrev.setText(QCoreApplication.translate("FastPlot", u"<< Previous", None))
        self.btNxt.setText(QCoreApplication.translate("FastPlot", u"Next >>", None))
        self.strNum.setText(QCoreApplication.translate("FastPlot", u"Page: 0/0 (0/0)", None))
        self.strXY.setText(QCoreApplication.translate("FastPlot", u"TextLabel", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgPlotPtn.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgPlotPtn(object):
    def setupUi(self, DgPlotPtn):
        if not DgPlotPtn.objectName():
            DgPlotPtn.setObjectName(u"DgPlotPtn")
        DgPlotPtn.resize(423, 389)
        self.ckAutoApply = QCheckBox(DgPlotPtn)
        self.ckAutoApply.setObjectName(u"ckAutoApply")
        self.ckAutoApply.setGeometry(QRect(170, 10, 111, 22))
        self.btSaveDefAtt = QPushButton(DgPlotPtn)
        self.btSaveDefAtt.setObjectName(u"btSaveDefAtt")
        self.btSaveDefAtt.setGeometry(QRect(300, 10, 111, 27))
        self.Linepanel = QFrame(DgPlotPtn)
        self.Linepanel.setObjectName(u"Linepanel")
        self.Linepanel.setGeometry(QRect(0, 40, 421, 91))
        self.Linepanel.setFrameShape(QFrame.WinPanel)
        self.Linepanel.setFrameShadow(QFrame.Raised)
        self.label = QLabel(self.Linepanel)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 10, 81, 21))
        self.cbLineStyle = QComboBox(self.Linepanel)
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.setObjectName(u"cbLineStyle")
        self.cbLineStyle.setGeometry(QRect(110, 10, 111, 27))
        self.ckHist = QCheckBox(self.Linepanel)
        self.ckHist.setObjectName(u"ckHist")
        self.ckHist.setGeometry(QRect(250, 10, 131, 22))
        self.label_2 = QLabel(self.Linepanel)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 50, 81, 17))
        self.spLineWidth = QSpinBox(self.Linepanel)
        self.spLineWidth.setObjectName(u"spLineWidth")
        self.spLineWidth.setGeometry(QRect(110, 50, 60, 21))
        self.label_3 = QLabel(self.Linepanel)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(210, 50, 51, 17))
        self.cbLineColor = QComboBox(self.Linepanel)
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.addItem("")
        self.cbLineColor.setObjectName(u"cbLineColor")
        self.cbLineColor.setGeometry(QRect(260, 50, 131, 27))
        self.Linepanel_2 = QFrame(DgPlotPtn)
        self.Linepanel_2.setObjectName(u"Linepanel_2")
        self.Linepanel_2.setGeometry(QRect(0, 140, 421, 91))
        self.Linepanel_2.setFrameShape(QFrame.WinPanel)
        self.Linepanel_2.setFrameShadow(QFrame.Raised)
        self.label_4 = QLabel(self.Linepanel_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(10, 10, 81, 21))
        self.cbSymbolType = QComboBox(self.Linepanel_2)
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.addItem("")
        self.cbSymbolType.setObjectName(u"cbSymbolType")
        self.cbSymbolType.setGeometry(QRect(110, 10, 111, 27))
        self.ckFilledFace = QCheckBox(self.Linepanel_2)
        self.ckFilledFace.setObjectName(u"ckFilledFace")
        self.ckFilledFace.setGeometry(QRect(250, 10, 131, 22))
        self.label_5 = QLabel(self.Linepanel_2)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(10, 50, 81, 17))
        self.spSymbolSize = QSpinBox(self.Linepanel_2)
        self.spSymbolSize.setObjectName(u"spSymbolSize")
        self.spSymbolSize.setGeometry(QRect(110, 50, 60, 21))
        self.label_6 = QLabel(self.Linepanel_2)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(210, 50, 51, 17))
        self.cbSymbolColor = QComboBox(self.Linepanel_2)
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.addItem("")
        self.cbSymbolColor.setObjectName(u"cbSymbolColor")
        self.cbSymbolColor.setGeometry(QRect(260, 50, 131, 27))
        self.Linepanel_3 = QFrame(DgPlotPtn)
        self.Linepanel_3.setObjectName(u"Linepanel_3")
        self.Linepanel_3.setGeometry(QRect(0, 240, 421, 91))
        self.Linepanel_3.setFrameShape(QFrame.WinPanel)
        self.Linepanel_3.setFrameShadow(QFrame.Raised)
        self.ckErrorBar = QCheckBox(self.Linepanel_3)
        self.ckErrorBar.setObjectName(u"ckErrorBar")
        self.ckErrorBar.setGeometry(QRect(10, 10, 131, 22))
        self.label_8 = QLabel(self.Linepanel_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(10, 50, 81, 17))
        self.spCapSize = QSpinBox(self.Linepanel_3)
        self.spCapSize.setObjectName(u"spCapSize")
        self.spCapSize.setGeometry(QRect(110, 50, 60, 21))
        self.label_9 = QLabel(self.Linepanel_3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(210, 50, 51, 17))
        self.cbErrorColor = QComboBox(self.Linepanel_3)
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.addItem("")
        self.cbErrorColor.setObjectName(u"cbErrorColor")
        self.cbErrorColor.setGeometry(QRect(260, 50, 131, 27))
        self.btClose = QPushButton(DgPlotPtn)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(40, 350, 81, 27))
        self.btApplyThis = QPushButton(DgPlotPtn)
        self.btApplyThis.setObjectName(u"btApplyThis")
        self.btApplyThis.setGeometry(QRect(220, 350, 81, 27))
        self.btApply = QPushButton(DgPlotPtn)
        self.btApply.setObjectName(u"btApply")
        self.btApply.setGeometry(QRect(320, 350, 81, 27))
        QWidget.setTabOrder(self.ckAutoApply, self.btSaveDefAtt)
        QWidget.setTabOrder(self.btSaveDefAtt, self.cbLineStyle)
        QWidget.setTabOrder(self.cbLineStyle, self.ckHist)
        QWidget.setTabOrder(self.ckHist, self.spLineWidth)
        QWidget.setTabOrder(self.spLineWidth, self.cbLineColor)
        QWidget.setTabOrder(self.cbLineColor, self.cbSymbolType)
        QWidget.setTabOrder(self.cbSymbolType, self.ckFilledFace)
        QWidget.setTabOrder(self.ckFilledFace, self.spSymbolSize)
        QWidget.setTabOrder(self.spSymbolSize, self.cbSymbolColor)
        QWidget.setTabOrder(self.cbSymbolColor, self.ckErrorBar)
        QWidget.setTabOrder(self.ckErrorBar, self.spCapSize)
        QWidget.setTabOrder(self.spCapSize, self.cbErrorColor)
        QWidget.setTabOrder(self.cbErrorColor, self.btClose)
        QWidget.setTabOrder(self.btClose, self.btApplyThis)
        QWidget.setTabOrder(self.btApplyThis, self.btApply)

        self.retranslateUi(DgPlotPtn)

        QMetaObject.connectSlotsByName(DgPlotPtn)
    # setupUi

    def retranslateUi(self, DgPlotPtn):
        DgPlotPtn.setWindowTitle(QCoreApplication.translate("DgPlotPtn", u"Set Plotting Style", None))
        self.ckAutoApply.setText(QCoreApplication.translate("DgPlotPtn", u"Auto Apply", None))
        self.btSaveDefAtt.setText(QCoreApplication.translate("DgPlotPtn", u"Save as default", None))
        self.label.setText(QCoreApplication.translate("DgPlotPtn", u"Line Style: ", None))
        self.cbLineStyle.setItemText(0, QCoreApplication.translate("DgPlotPtn", u"Line", None))
        self.cbLineStyle.setItemText(1, QCoreApplication.translate("DgPlotPtn", u"Dotted", None))
        self.cbLineStyle.setItemText(2, QCoreApplication.translate("DgPlotPtn", u"Dashed", None))
        self.cbLineStyle.setItemText(3, QCoreApplication.translate("DgPlotPtn", u"Dot-dashed", None))
        self.cbLineStyle.setItemText(4, QCoreApplication.translate("DgPlotPtn", u"None", None))

        self.ckHist.setText(QCoreApplication.translate("DgPlotPtn", u"Histogram", None))
        self.label_2.setText(QCoreApplication.translate("DgPlotPtn", u"Line Width: ", None))
        self.label_3.setText(QCoreApplication.translate("DgPlotPtn", u"Color:", None))
        self.cbLineColor.setItemText(0, QCoreApplication.translate("DgPlotPtn", u"Auto", None))
        self.cbLineColor.setItemText(1, QCoreApplication.translate("DgPlotPtn", u"Blue", None))
        self.cbLineColor.setItemText(2, QCoreApplication.translate("DgPlotPtn", u"Green", None))
        self.cbLineColor.setItemText(3, QCoreApplication.translate("DgPlotPtn", u"Red", None))
        self.cbLineColor.setItemText(4, QCoreApplication.translate("DgPlotPtn", u"Cyan", None))
        self.cbLineColor.setItemText(5, QCoreApplication.translate("DgPlotPtn", u"Magenta", None))
        self.cbLineColor.setItemText(6, QCoreApplication.translate("DgPlotPtn", u"Yellow", None))
        self.cbLineColor.setItemText(7, QCoreApplication.translate("DgPlotPtn", u"Black", None))

        self.label_4.setText(QCoreApplication.translate("DgPlotPtn", u"Symbol:", None))
        self.cbSymbolType.setItemText(0, QCoreApplication.translate("DgPlotPtn", u"None", None))
        self.cbSymbolType.setItemText(1, QCoreApplication.translate("DgPlotPtn", u"Dot", None))
        self.cbSymbolType.setItemText(2, QCoreApplication.translate("DgPlotPtn", u"Circle", None))
        self.cbSymbolType.setItemText(3, QCoreApplication.translate("DgPlotPtn", u"Square", None))
        self.cbSymbolType.setItemText(4, QCoreApplication.translate("DgPlotPtn", u"Triangle", None))
        self.cbSymbolType.setItemText(5, QCoreApplication.translate("DgPlotPtn", u"Diamond", None))
        self.cbSymbolType.setItemText(6, QCoreApplication.translate("DgPlotPtn", u"Cross", None))
        self.cbSymbolType.setItemText(7, QCoreApplication.translate("DgPlotPtn", u"Crisscross", None))

        self.ckFilledFace.setText(QCoreApplication.translate("DgPlotPtn", u"Filled face", None))
        self.label_5.setText(QCoreApplication.translate("DgPlotPtn", u"Symbol Size:", None))
        self.label_6.setText(QCoreApplication.translate("DgPlotPtn", u"Color:", None))
        self.cbSymbolColor.setItemText(0, QCoreApplication.translate("DgPlotPtn", u"Auto", None))
        self.cbSymbolColor.setItemText(1, QCoreApplication.translate("DgPlotPtn", u"Blue", None))
        self.cbSymbolColor.setItemText(2, QCoreApplication.translate("DgPlotPtn", u"Green", None))
        self.cbSymbolColor.setItemText(3, QCoreApplication.translate("DgPlotPtn", u"Red", None))
        self.cbSymbolColor.setItemText(4, QCoreApplication.translate("DgPlotPtn", u"Cyan", None))
        self.cbSymbolColor.setItemText(5, QCoreApplication.translate("DgPlotPtn", u"Magenta", None))
        self.cbSymbolColor.setItemText(6, QCoreApplication.translate("DgPlotPtn", u"Yellow", None))
        self.cbSymbolColor.setItemText(7, QCoreApplication.translate("DgPlotPtn", u"Black", None))

        self.ckErrorBar.setText(QCoreApplication.translate("DgPlotPtn", u"With Error Bar", None))
        self.label_8.setText(QCoreApplication.translate("DgPlotPtn", u"Cap Size:", None))
        self.label_9.setText(QCoreApplication.translate("DgPlotPtn", u"Color:", None))
        self.cbErrorColor.setItemText(0, QCoreApplication.translate("DgPlotPtn", u"Auto", None))
        self.cbErrorColor.setItemText(1, QCoreApplication.translate("DgPlotPtn", u"Blue", None))
        self.cbErrorColor.setItemText(2, QCoreApplication.translate("DgPlotPtn", u"Green", None))
        self.cbErrorColor.setItemText(3, QCoreApplication.translate("DgPlotPtn", u"Red", None))
        self.cbErrorColor.setItemText(4, QCoreApplication.translate("DgPlotPtn", u"Cyan", None))
        self.cbErrorColor.setItemText(5, QCoreApplication.translate("DgPlotPtn", u"Magenta", None))
        self.cbErrorColor.setItemText(6, QCoreApplication.translate("DgPlotPtn", u"Yellow", None))
        self.cbErrorColor.setItemText(7, QCoreApplication.translate("DgPlotPtn", u"Black", None))

        self.btClose.setText(QCoreApplication.translate("DgPlotPtn", u"Close", None))
        self.btApplyThis.setText(QCoreApplication.translate("DgPlotPtn", u"Apply", None))
        self.btApply.setText(QCoreApplication.translate("DgPlotPtn", u"Apply All", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgScaleLabel.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgScaleLabel(object):
    def setupUi(self, DgScaleLabel):
        if not DgScaleLabel.objectName():
            DgScaleLabel.setObjectName(u"DgScaleLabel")
        DgScaleLabel.resize(462, 204)
        self.frame = QFrame(DgScaleLabel)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(0, 0, 461, 161))
        self.frame.setFrameShape(QFrame.WinPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.layoutWidget = QWidget(self.frame)
        self.layoutWidget.setObjectName(u"layoutWidget")
        self.layoutWidget.setGeometry(QRect(20, 50, 211, 29))
        self.horizontalLayout_3 = QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.label_3 = QLabel(self.layoutWidget)
        self.label_3.setObjectName(u"label_3")

        self.horizontalLayout_3.addWidget(self.label_3)

        self.ylabel = QLineEdit(self.layoutWidget)
        self.ylabel.setObjectName(u"ylabel")

        self.horizontalLayout_3.addWidget(self.ylabel)

        self.layoutWidget_2 = QWidget(self.frame)
        self.layoutWidget_2.setObjectName(u"layoutWidget_2")
        self.layoutWidget_2.setGeometry(QRect(260, 50, 191, 29))
        self.horizontalLayout_4 = QHBoxLayout(self.layoutWidget_2)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.label_4 = QLabel(self.layoutWidget_2)
        self.label_4.setObjectName(u"label_4")

        self.horizontalLayout_4.addWidget(self.label_4)

        self.yunit = QLineEdit(self.layoutWidget_2)
        self.yunit.setObjectName(u"yunit")

        self.horizontalLayout_4.addWidget(self.yunit)

        self.label_5 = QLabel(self.frame)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(20, 100, 71, 17))
        self.spFS = QSpinBox(self.frame)
        self.spFS.setObjectName(u"spFS")
        self.spFS.setGeometry(QRect(80, 120, 60, 27))
        self.label_6 = QLabel(self.frame)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(180, 100, 91, 17))
        self.layoutWidget1 = QWidget(self.frame)
        self.layoutWidget1.setObjectName(u"layoutWidget1")
        self.layoutWidget1.setGeometry(QRect(260, 10, 191, 29))
        self.horizontalLayout = QHBoxLayout(self.layoutWidget1)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label_2 = QLabel(self.layoutWidget1)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout.addWidget(self.label_2)

        self.xunit = QLineEdit(self.layoutWidget1)
        self.xunit.setObjectName(u"xunit")

        self.horizontalLayout.addWidget(self.xunit)

        self.layoutWidget2 = QWidget(self.frame)
        self.layoutWidget2.setObjectName(u"layoutWidget2")
        self.layoutWidget2.setGeometry(QRect(20, 11, 211, 29))
        self.horizontalLayout_2 = QHBoxLayout(self.layoutWidget2)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.label = QLabel(self.layoutWidget2)
        self.label.setObjectName(u"label")

        self.horizontalLayout_2.addWidget(self.label)

        self.xlabel = QLineEdit(self.layoutWidget2)
        self.xlabel.setObjectName(u"xlabel")

        self.horizontalLayout_2.addWidget(self.xlabel)

        self.cbFont = QComboBox(self.frame)
        self.cbFont.setObjectName(u"cbFont")
        self.cbFont.setGeometry(QRect(240, 120, 201, 27))
        self.btCancel = QPushButton(DgScaleLabel)
        self.btCancel.setObjectName(u"btCancel")
        self.btCancel.setGeometry(QRect(270, 170, 98, 27))
        self.btOK = QPushButton(DgScaleLabel)
        self.btOK.setObjectName(u"btOK")
        self.btOK.setGeometry(QRect(90, 170, 98, 27))
        QWidget.setTabOrder(self.xlabel, self.xunit)
        QWidget.setTabOrder(self.xunit, self.ylabel)
        QWidget.setTabOrder(self.ylabel, self.yunit)
        QWidget.setTabOrder(self.yunit, self.spFS)
        QWidget.setTabOrder(self.spFS, self.cbFont)
        QWidget.setTabOrder(self.cbFont, self.btOK)
        QWidget.setTabOrder(self.btOK, self.btCancel)

        self.retranslateUi(DgScaleLabel)

        QMetaObject.connectSlotsByName(DgScaleLabel)
    # setupUi

    def retranslateUi(self, DgScaleLabel):
        DgScaleLabel.setWindowTitle(QCoreApplication.translate("DgScaleLabel", u"Scale Labels Setting", None))
        self.label_3.setText(QCoreApplication.translate("DgScaleLabel", u"Y Label:", None))
        self.ylabel.setText(QCoreApplication.translate("DgScaleLabel", u"Counts", None))
        self.label_4.setText(QCoreApplication.translate("DgScaleLabel", u"Y Unit:", None))
        self.yunit.setText(QCoreApplication.translate("DgScaleLabel", u"Counts", None))
        self.label_5.setText(QCoreApplication.translate("DgScaleLabel", u"Font Size:", None))
        self.label_6.setText(QCoreApplication.translate("DgScaleLabel", u"Font Type:", None))
        self.label_2.setText(QCoreApplication.translate("DgScaleLabel", u"X Unit:", None))
        self.xunit.setText(QCoreApplication.translate("DgScaleLabel", u"micro sec", None))
        self.label.setText(QCoreApplication.translate("DgScaleLabel", u"X Label:", None))
        self.xlabel.setText(QCoreApplication.translate("DgScaleLabel", u"Time", None))
        self.btCancel.setText(QCoreApplication.translate("DgScaleLabel", u"Cancel", None))
        self.btOK.setText(QCoreApplication.translate("DgScaleLabel", u"OK", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgSetting.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgSetting(object):
    def setupUi(self, DgSetting):
        if not DgSetting.objectName():
            DgSetting.setObjectName(u"DgSetting")
        DgSetting.setWindowModality(Qt.NonModal)
        DgSetting.resize(394, 486)
        DgSetting.setContextMenuPolicy(Qt.DefaultContextMenu)
        DgSetting.setModal(False)
        self.scpanel = QFrame(DgSetting)
        self.scpanel.setObjectName(u"scpanel")
        self.scpanel.setGeometry(QRect(0, 0, 391, 271))
        self.scpanel.setFrameShape(QFrame.WinPanel)
        self.scpanel.setFrameShadow(QFrame.Raised)
        self.scpanel.setLineWidth(1)
        self.scpanel_2 = QFrame(self.scpanel)
        self.scpanel_2.setObjectName(u"scpanel_2")
        self.scpanel_2.setGeometry(QRect(10, 10, 371, 121))
        self.scpanel_2.setFrameShape(QFrame.WinPanel)
        self.scpanel_2.setFrameShadow(QFrame.Raised)
        self.scpanel_2.setLineWidth(1)
        self.label = QLabel(self.scpanel_2)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 10, 66, 17))
        self.ckXAuto = QCheckBox(self.scpanel_2)
        self.ckXAuto.setObjectName(u"ckXAuto")
        self.ckXAuto.setGeometry(QRect(10, 30, 97, 22))
        self.ckXAuto.setChecked(True)
        self.ckLogX = QCheckBox(self.scpanel_2)
        self.ckLogX.setObjectName(u"ckLogX")
        self.ckLogX.setGeometry(QRect(10, 90, 97, 22))
        self.label_2 = QLabel(self.scpanel_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(30, 60, 66, 17))
        self.label_3 = QLabel(self.scpanel_2)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(210, 60, 16, 17))
        self.x0 = QLineEdit(self.scpanel_2)
        self.x0.setObjectName(u"x0")
        self.x0.setGeometry(QRect(100, 60, 101, 21))
        self.x1 = QLineEdit(self.scpanel_2)
        self.x1.setObjectName(u"x1")
        self.x1.setGeometry(QRect(230, 60, 101, 21))
        self.btXApply = QPushButton(self.scpanel_2)
        self.btXApply.setObjectName(u"btXApply")
        self.btXApply.setGeometry(QRect(290, 90, 71, 27))
        self.scpanel_3 = QFrame(self.scpanel)
        self.scpanel_3.setObjectName(u"scpanel_3")
        self.scpanel_3.setGeometry(QRect(10, 140, 371, 121))
        self.scpanel_3.setFrameShape(QFrame.WinPanel)
        self.scpanel_3.setFrameShadow(QFrame.Raised)
        self.scpanel_3.setLineWidth(1)
        self.label_4 = QLabel(self.scpanel_3)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(10, 10, 66, 17))
        self.ckYAuto = QCheckBox(self.scpanel_3)
        self.ckYAuto.setObjectName(u"ckYAuto")
        self.ckYAuto.setGeometry(QRect(10, 30, 97, 22))
        self.ckYAuto.setChecked(True)
        self.ckLogY = QCheckBox(self.scpanel_3)
        self.ckLogY.setObjectName(u"ckLogY")
        self.ckLogY.setGeometry(QRect(10, 90, 97, 22))
        self.label_5 = QLabel(self.scpanel_3)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(30, 60, 66, 17))
        self.label_6 = QLabel(self.scpanel_3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(210, 60, 16, 17))
        self.y0 = QLineEdit(self.scpanel_3)
        self.y0.setObjectName(u"y0")
        self.y0.setGeometry(QRect(100, 60, 101, 21))
        self.y1 = QLineEdit(self.scpanel_3)
        self.y1.setObjectName(u"y1")
        self.y1.setGeometry(QRect(230, 60, 101, 21))
        self.btYApply = QPushButton(self.scpanel_3)
        self.btYApply.setObjectName(u"btYApply")
        self.btYApply.setGeometry(QRect(290, 90, 71, 27))
        self.stpanel = QFrame(DgSetting)
        self.stpanel.setObjectName(u"stpanel")
        self.stpanel.setGeometry(QRect(0, 280, 391, 101))
        self.stpanel.setFrameShape(QFrame.WinPanel)
        self.stpanel.setFrameShadow(QFrame.Raised)
        self.stpanel.setLineWidth(1)
        self.ckOver = QCheckBox(self.stpanel)
        self.ckOver.setObjectName(u"ckOver")
        self.ckOver.setGeometry(QRect(10, 10, 97, 22))
        self.label_7 = QLabel(self.stpanel)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(30, 40, 66, 17))
        self.spX = QSpinBox(self.stpanel)
        self.spX.setObjectName(u"spX")
        self.spX.setGeometry(QRect(110, 40, 60, 21))
        self.spY = QSpinBox(self.stpanel)
        self.spY.setObjectName(u"spY")
        self.spY.setGeometry(QRect(270, 40, 60, 21))
        self.label_8 = QLabel(self.stpanel)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(200, 40, 66, 17))
        self.label_9 = QLabel(self.stpanel)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(20, 70, 91, 17))
        self.spN = QSpinBox(self.stpanel)
        self.spN.setObjectName(u"spN")
        self.spN.setGeometry(QRect(110, 70, 60, 21))
        self.fspanel = QFrame(DgSetting)
        self.fspanel.setObjectName(u"fspanel")
        self.fspanel.setGeometry(QRect(0, 390, 391, 51))
        self.fspanel.setFrameShape(QFrame.WinPanel)
        self.fspanel.setFrameShadow(QFrame.Raised)
        self.fspanel.setLineWidth(1)
        self.label_10 = QLabel(self.fspanel)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(10, 10, 71, 20))
        self.spFS = QSpinBox(self.fspanel)
        self.spFS.setObjectName(u"spFS")
        self.spFS.setGeometry(QRect(80, 10, 60, 21))
        self.label_11 = QLabel(self.fspanel)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(150, 10, 81, 17))
        self.label_12 = QLabel(self.fspanel)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(20, 90, 91, 17))
        self.spN_2 = QSpinBox(self.fspanel)
        self.spN_2.setObjectName(u"spN_2")
        self.spN_2.setGeometry(QRect(110, 90, 60, 21))
        self.cbFontBox = QComboBox(self.fspanel)
        self.cbFontBox.setObjectName(u"cbFontBox")
        self.cbFontBox.setGeometry(QRect(230, 10, 151, 27))
        self.btClose = QPushButton(DgSetting)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(150, 450, 98, 27))
        QWidget.setTabOrder(self.ckXAuto, self.x0)
        QWidget.setTabOrder(self.x0, self.x1)
        QWidget.setTabOrder(self.x1, self.ckLogX)
        QWidget.setTabOrder(self.ckLogX, self.btXApply)
        QWidget.setTabOrder(self.btXApply, self.ckYAuto)
        QWidget.setTabOrder(self.ckYAuto, self.y0)
        QWidget.setTabOrder(self.y0, self.y1)
        QWidget.setTabOrder(self.y1, self.ckLogY)
        QWidget.setTabOrder(self.ckLogY, self.btYApply)
        QWidget.setTabOrder(self.btYApply, self.ckOver)
        QWidget.setTabOrder(self.ckOver, self.spX)
        QWidget.setTabOrder(self.spX, self.spY)
        QWidget.setTabOrder(self.spY, self.spN)
        QWidget.setTabOrder(self.spN, self.spFS)
        QWidget.setTabOrder(self.spFS, self.cbFontBox)
        QWidget.setTabOrder(self.cbFontBox, self.spN_2)
        QWidget.setTabOrder(self.spN_2, self.btClose)

        self.retranslateUi(DgSetting)

        QMetaObject.connectSlotsByName(DgSetting)
    # setupUi

    def retranslateUi(self, DgSetting):
        DgSetting.setWindowTitle(QCoreApplication.translate("DgSetting", u"Set Parameter", None))
        self.label.setText(QCoreApplication.translate("DgSetting", u"X axis", None))
        self.ckXAuto.setText(QCoreApplication.translate("DgSetting", u"Auto Scale", None))
        self.ckLogX.setText(QCoreApplication.translate("DgSetting", u"Log Scale", None))
        self.label_2.setText(QCoreApplication.translate("DgSetting", u"X Range :", None))
        self.label_3.setText(QCoreApplication.translate("DgSetting", u"-", None))
        self.btXApply.setText(QCoreApplication.translate("DgSetting", u"Apply", None))
        self.label_4.setText(QCoreApplication.translate("DgSetting", u"Y axis", None))
        self.ckYAuto.setText(QCoreApplication.translate("DgSetting", u"Auto Scale", None))
        self.ckLogY.setText(QCoreApplication.translate("DgSetting", u"Log Scale", None))
        self.label_5.setText(QCoreApplication.translate("DgSetting", u"Y Range :", None))
        self.label_6.setText(QCoreApplication.translate("DgSetting", u"-", None))
        self.btYApply.setText(QCoreApplication.translate("DgSetting", u"Apply", None))
        self.ckOver.setText(QCoreApplication.translate("DgSetting", u"Overlay", None))
        self.label_7.setText(QCoreApplication.translate("DgSetting", u"X offset :", None))
        self.label_8.setText(QCoreApplication.translate("DgSetting", u"Y offset :", None))
        self.label_9.setText(QCoreApplication.translate("DgSetting", u"Max Traces :", None))
        self.label_10.setText(QCoreApplication.translate("DgSetting", u"Font size :", None))
        self.label_11.setText(QCoreApplication.translate("DgSetting", u"Font Type :", None))
        self.label_12.setText(QCoreApplication.translate("DgSetting", u"Max Traces :", None))
        self.btClose.setText(QCoreApplication.translate("DgSetting", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgTitle.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgTitle(object):
    def setupUi(self, DgTitle):
        if not DgTitle.objectName():
            DgTitle.setObjectName(u"DgTitle")
        DgTitle.resize(470, 259)
        self.frame = QFrame(DgTitle)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(0, 0, 471, 221))
        self.frame.setFrameShape(QFrame.WinPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(20, 10, 71, 17))
        self.tx1 = QLineEdit(self.frame)
        self.tx1.setObjectName(u"tx1")
        self.tx1.setGeometry(QRect(10, 30, 451, 27))
        self.label_2 = QLabel(self.frame)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(20, 70, 81, 17))
        self.label_3 = QLabel(self.frame)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(20, 170, 71, 17))
        self.spFS = QSpinBox(self.frame)
        self.spFS.setObjectName(u"spFS")
        self.spFS.setGeometry(QRect(80, 190, 60, 27))
        self.label_4 = QLabel(self.frame)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(180, 170, 91, 17))
        self.tx2 = QPlainTextEdit(self.frame)
        self.tx2.setObjectName(u"tx2")
        self.tx2.setGeometry(QRect(20, 90, 431, 61))
        self.cbFont = QComboBox(self.frame)
        self.cbFont.setObjectName(u"cbFont")
        self.cbFont.setGeometry(QRect(240, 190, 211, 27))
        self.btOK = QPushButton(DgTitle)
        self.btOK.setObjectName(u"btOK")
        self.btOK.setGeometry(QRect(90, 230, 98, 27))
        self.btCancel = QPushButton(DgTitle)
        self.btCancel.setObjectName(u"btCancel")
        self.btCancel.setGeometry(QRect(270, 230, 98, 27))
        QWidget.setTabOrder(self.tx1, self.tx2)
        QWidget.setTabOrder(self.tx2, self.spFS)
        QWidget.setTabOrder(self.spFS, self.cbFont)
        QWidget.setTabOrder(self.cbFont, self.btOK)
        QWidget.setTabOrder(self.btOK, self.btCancel)

        self.retranslateUi(DgTitle)

        QMetaObject.connectSlotsByName(DgTitle)
    # setupUi

    def retranslateUi(self, DgTitle):
        DgTitle.setWindowTitle(QCoreApplication.translate("DgTitle", u"Titles Setting", None))
        self.label.setText(QCoreApplication.translate("DgTitle", u"Main Title", None))
        self.label_2.setText(QCoreApplication.translate("DgTitle", u"Sub Title", None))
        self.label_3.setText(QCoreApplication.translate("DgTitle", u"Font Size:", None))
        self.label_4.setText(QCoreApplication.translate("DgTitle", u"Font Type:", None))
        self.btOK.setText(QCoreApplication.translate("DgTitle", u"OK", None))
        self.btCancel.setText(QCoreApplication.translate("DgTitle", u"Cancel", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'FloatBox.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_FloatBox(object):
    def setupUi(self, FloatBox):
        if not FloatBox.objectName():
            FloatBox.setObjectName(u"FloatBox")
        FloatBox.resize(474, 362)
        self.centralwidget = QWidget(FloatBox)
        self.centralwidget.setObjectName(u"centralwidget")
        self.frame = QFrame(self.centralwidget)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(10, 10, 451, 281))
        self.frame.setFrameShape(QFrame.WinPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(20, 10, 111, 17))
        self.spNo = QSpinBox(self.frame)
        self.spNo.setObjectName(u"spNo")
        self.spNo.setGeometry(QRect(10, 240, 60, 27))
        self.btPlot = QPushButton(self.frame)
        self.btPlot.setObjectName(u"btPlot")
        self.btPlot.setGeometry(QRect(330, 50, 101, 27))
        self.btChange = QPushButton(self.frame)
        self.btChange.setObjectName(u"btChange")
        self.btChange.setGeometry(QRect(330, 240, 111, 27))
        self.txEdit = QLineEdit(self.frame)
        self.txEdit.setObjectName(u"txEdit")
        self.txEdit.setGeometry(QRect(80, 240, 241, 27))
        self.specList = QListWidget(self.frame)
        self.specList.setObjectName(u"specList")
        self.specList.setGeometry(QRect(10, 30, 311, 201))
        self.btUp = QPushButton(self.frame)
        self.btUp.setObjectName(u"btUp")
        self.btUp.setGeometry(QRect(330, 100, 25, 25))
        self.btDown = QPushButton(self.frame)
        self.btDown.setObjectName(u"btDown")
        self.btDown.setGeometry(QRect(330, 130, 25, 25))
        self.btDelete = QPushButton(self.frame)
        self.btDelete.setObjectName(u"btDelete")
        self.btDelete.setGeometry(QRect(400, 130, 25, 25))
        self.btDuplicate = QPushButton(self.frame)
        self.btDuplicate.setObjectName(u"btDuplicate")
        self.btDuplicate.setGeometry(QRect(330, 170, 81, 27))
        self.txDupFactor = QLineEdit(self.frame)
        self.txDupFactor.setObjectName(u"txDupFactor")
        self.txDupFactor.setGeometry(QRect(330, 200, 113, 27))
        self.btClose = QPushButton(self.centralwidget)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(190, 300, 98, 27))
        FloatBox.setCentralWidget(self.centralwidget)
        self.statusbar = QStatusBar(FloatBox)
        self.statusbar.setObjectName(u"statusbar")
        FloatBox.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.specList, self.btPlot)
        QWidget.setTabOrder(self.btPlot, self.btUp)
        QWidget.setTabOrder(self.btUp, self.btDown)
        QWidget.setTabOrder(self.btDown, self.btDelete)
        QWidget.setTabOrder(self.btDelete, self.btDuplicate)
        QWidget.setTabOrder(self.btDuplicate, self.txDupFactor)
        QWidget.setTabOrder(self.txDupFactor, self.spNo)
        QWidget.setTabOrder(self.spNo, self.txEdit)
        QWidget.setTabOrder(self.txEdit, self.btChange)
        QWidget.setTabOrder(self.btChange, self.btClose)

        self.retranslateUi(FloatBox)

        QMetaObject.connectSlotsByName(FloatBox)
    # setupUi

    def retranslateUi(self, FloatBox):
        FloatBox.setWindowTitle(QCoreApplication.translate("FloatBox", u"Plotting Data", None))
        self.label.setText(QCoreApplication.translate("FloatBox", u"No. Trace Label", None))
        self.btPlot.setText(QCoreApplication.translate("FloatBox", u"Plot Pattern...", None))
        self.btChange.setText(QCoreApplication.translate("FloatBox", u"Change Label", None))
        self.btUp.setText("")
        self.btDown.setText("")
        self.btDelete.setText("")
        self.btDuplicate.setText(QCoreApplication.translate("FloatBox", u"Duplicate", None))
        self.txDupFactor.setText(QCoreApplication.translate("FloatBox", u"1.0", None))
        self.btClose.setText(QCoreApplication.translate("FloatBox", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgSaveAsText.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgSaveAsText(object):
    def setupUi(self, DgSaveAsText):
        if not DgSaveAsText.objectName():
            DgSaveAsText.setObjectName(u"DgSaveAsText")
        DgSaveAsText.resize(282, 201)
        self.frame = QFrame(DgSaveAsText)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(10, 10, 261, 141))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.ckIgnoreMask = QCheckBox(self.frame)
        self.ckIgnoreMask.setObjectName(u"ckIgnoreMask")
        self.ckIgnoreMask.setGeometry(QRect(20, 10, 201, 22))
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(40, 40, 181, 17))
        self.chMaskValue = QComboBox(self.frame)
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.setObjectName(u"chMaskValue")
        self.chMaskValue.setGeometry(QRect(110, 60, 131, 27))
        self.txMaskValue = QLineEdit(self.frame)
        self.txMaskValue.setObjectName(u"txMaskValue")
        self.txMaskValue.setGeometry(QRect(110, 100, 131, 27))
        self.btSave = QPushButton(DgSaveAsText)
        self.btSave.setObjectName(u"btSave")
        self.btSave.setGeometry(QRect(20, 160, 121, 31))
        self.btCancel = QPushButton(DgSaveAsText)
        self.btCancel.setObjectName(u"btCancel")
        self.btCancel.setGeometry(QRect(190, 160, 81, 27))
        QWidget.setTabOrder(self.ckIgnoreMask, self.chMaskValue)
        QWidget.setTabOrder(self.chMaskValue, self.txMaskValue)
        QWidget.setTabOrder(self.txMaskValue, self.btSave)
        QWidget.setTabOrder(self.btSave, self.btCancel)

        self.retranslateUi(DgSaveAsText)

        QMetaObject.connectSlotsByName(DgSaveAsText)
    # setupUi

    def retranslateUi(self, DgSaveAsText):
        DgSaveAsText.setWindowTitle(QCoreApplication.translate("DgSaveAsText", u"Save as Text file", None))
        self.ckIgnoreMask.setText(QCoreApplication.translate("DgSaveAsText", u"Ignore Masked Points", None))
        self.label.setText(QCoreApplication.translate("DgSaveAsText", u"Replace Mask Value with", None))
        self.chMaskValue.setItemText(0, QCoreApplication.translate("DgSaveAsText", u"Value below", None))
        self.chMaskValue.setItemText(1, QCoreApplication.translate("DgSaveAsText", u"0", None))
        self.chMaskValue.setItemText(2, QCoreApplication.translate("DgSaveAsText", u"-1", None))
        self.chMaskValue.setItemText(3, QCoreApplication.translate("DgSaveAsText", u"Original Value", None))

        self.btSave.setText(QCoreApplication.translate("DgSaveAsText", u"Save", None))
        self.btCancel.setText(QCoreApplication.translate("DgSaveAsText", u"Cancel", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgFitting.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgFitting(object):
    def setupUi(self, DgFitting):
        if not DgFitting.objectName():
            DgFitting.setObjectName(u"DgFitting")
        DgFitting.resize(426, 569)
        self.frame = QFrame(DgFitting)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(10, 10, 411, 71))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.cbTrace = QComboBox(self.frame)
        self.cbTrace.setObjectName(u"cbTrace")
        self.cbTrace.setGeometry(QRect(80, 10, 221, 27))
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(0, 0, 66, 17))
        self.label_2 = QLabel(self.frame)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(120, 40, 41, 25))
        self.txRangeMin = QLineEdit(self.frame)
        self.txRangeMin.setObjectName(u"txRangeMin")
        self.txRangeMin.setGeometry(QRect(160, 40, 100, 25))
        self.txRangeMax = QLineEdit(self.frame)
        self.txRangeMax.setObjectName(u"txRangeMax")
        self.txRangeMax.setGeometry(QRect(300, 40, 100, 25))
        self.label_3 = QLabel(self.frame)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(280, 40, 21, 25))
        self.frame_2 = QFrame(DgFitting)
        self.frame_2.setObjectName(u"frame_2")
        self.frame_2.setGeometry(QRect(10, 90, 411, 101))
        self.frame_2.setFrameShape(QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QFrame.Raised)
        self.label_4 = QLabel(self.frame_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(0, 0, 71, 17))
        self.cbFunctions = QComboBox(self.frame_2)
        self.cbFunctions.addItem("")
        self.cbFunctions.addItem("")
        self.cbFunctions.addItem("")
        self.cbFunctions.addItem("")
        self.cbFunctions.addItem("")
        self.cbFunctions.setObjectName(u"cbFunctions")
        self.cbFunctions.setGeometry(QRect(10, 20, 261, 27))
        self.pnlFunction = QLabel(self.frame_2)
        self.pnlFunction.setObjectName(u"pnlFunction")
        self.pnlFunction.setGeometry(QRect(10, 50, 391, 50))
        self.btLoadFunc = QPushButton(self.frame_2)
        self.btLoadFunc.setObjectName(u"btLoadFunc")
        self.btLoadFunc.setGeometry(QRect(280, 20, 121, 27))
        self.frame_3 = QFrame(DgFitting)
        self.frame_3.setObjectName(u"frame_3")
        self.frame_3.setGeometry(QRect(10, 200, 411, 321))
        self.frame_3.setFrameShape(QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QFrame.Raised)
        self.label_5 = QLabel(self.frame_3)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(0, 0, 101, 17))
        self.txA1 = QLineEdit(self.frame_3)
        self.txA1.setObjectName(u"txA1")
        self.txA1.setGeometry(QRect(40, 50, 100, 25))
        self.label_6 = QLabel(self.frame_3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(10, 50, 25, 25))
        self.ckA1 = QCheckBox(self.frame_3)
        self.ckA1.setObjectName(u"ckA1")
        self.ckA1.setGeometry(QRect(150, 50, 21, 25))
        self.txA1r = QLabel(self.frame_3)
        self.txA1r.setObjectName(u"txA1r")
        self.txA1r.setGeometry(QRect(180, 50, 100, 25))
        self.label_8 = QLabel(self.frame_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(10, 80, 25, 25))
        self.txA2 = QLineEdit(self.frame_3)
        self.txA2.setObjectName(u"txA2")
        self.txA2.setGeometry(QRect(40, 80, 100, 25))
        self.ckA2 = QCheckBox(self.frame_3)
        self.ckA2.setObjectName(u"ckA2")
        self.ckA2.setGeometry(QRect(150, 80, 21, 25))
        self.txA2r = QLabel(self.frame_3)
        self.txA2r.setObjectName(u"txA2r")
        self.txA2r.setGeometry(QRect(180, 80, 100, 25))
        self.txA3 = QLineEdit(self.frame_3)
        self.txA3.setObjectName(u"txA3")
        self.txA3.setGeometry(QRect(40, 110, 100, 25))
        self.txA4 = QLineEdit(self.frame_3)
        self.txA4.setObjectName(u"txA4")
        self.txA4.setGeometry(QRect(40, 140, 100, 25))
        self.txA5 = QLineEdit(self.frame_3)
        self.txA5.setObjectName(u"txA5")
        self.txA5.setGeometry(QRect(40, 170, 100, 25))
        self.txA6 = QLineEdit(self.frame_3)
        self.txA6.setObjectName(u"txA6")
        self.txA6.setGeometry(QRect(40, 200, 100, 25))
        self.txA7 = QLineEdit(self.frame_3)
        self.txA7.setObjectName(u"txA7")
        self.txA7.setGeometry(QRect(40, 230, 100, 25))
        self.txA8 = QLineEdit(self.frame_3)
        self.txA8.setObjectName(u"txA8")
        self.txA8.setGeometry(QRect(40, 260, 100, 25))
        self.label_10 = QLabel(self.frame_3)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(10, 110, 25, 25))
        self.label_11 = QLabel(self.frame_3)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(10, 140, 25, 25))
        self.label_12 = QLabel(self.frame_3)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(10, 170, 25, 25))
        self.label_13 = QLabel(self.frame_3)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(10, 200, 25, 25))
        self.label_14 = QLabel(self.frame_3)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(10, 230, 25, 25))
        self.label_15 = QLabel(self.frame_3)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(10, 260, 25, 25))
        self.ckA3 = QCheckBox(self.frame_3)
        self.ckA3.setObjectName(u"ckA3")
        self.ckA3.setGeometry(QRect(150, 110, 21, 25))
        self.ckA4 = QCheckBox(self.frame_3)
        self.ckA4.setObjectName(u"ckA4")
        self.ckA4.setGeometry(QRect(150, 140, 21, 25))
        self.ckA5 = QCheckBox(self.frame_3)
        self.ckA5.setObjectName(u"ckA5")
        self.ckA5.setGeometry(QRect(150, 170, 21, 25))
        self.ckA6 = QCheckBox(self.frame_3)
        self.ckA6.setObjectName(u"ckA6")
        self.ckA6.setGeometry(QRect(150, 200, 21, 25))
        self.ckA7 = QCheckBox(self.frame_3)
        self.ckA7.setObjectName(u"ckA7")
        self.ckA7.setGeometry(QRect(150, 230, 21, 25))
        self.ckA8 = QCheckBox(self.frame_3)
        self.ckA8.setObjectName(u"ckA8")
        self.ckA8.setGeometry(QRect(150, 260, 21, 25))
        self.txA3r = QLabel(self.frame_3)
        self.txA3r.setObjectName(u"txA3r")
        self.txA3r.setGeometry(QRect(180, 110, 100, 25))
        self.txA4r = QLabel(self.frame_3)
        self.txA4r.setObjectName(u"txA4r")
        self.txA4r.setGeometry(QRect(180, 140, 100, 25))
        self.txA5r = QLabel(self.frame_3)
        self.txA5r.setObjectName(u"txA5r")
        self.txA5r.setGeometry(QRect(180, 170, 100, 25))
        self.txA6r = QLabel(self.frame_3)
        self.txA6r.setObjectName(u"txA6r")
        self.txA6r.setGeometry(QRect(180, 200, 100, 25))
        self.txA7r = QLabel(self.frame_3)
        self.txA7r.setObjectName(u"txA7r")
        self.txA7r.setGeometry(QRect(180, 230, 100, 25))
        self.txA8r = QLabel(self.frame_3)
        self.txA8r.setObjectName(u"txA8r")
        self.txA8r.setGeometry(QRect(180, 260, 100, 25))
        self.label_22 = QLabel(self.frame_3)
        self.label_22.setObjectName(u"label_22")
        self.label_22.setGeometry(QRect(50, 20, 91, 21))
        self.label_23 = QLabel(self.frame_3)
        self.label_23.setObjectName(u"label_23")
        self.label_23.setGeometry(QRect(150, 20, 31, 21))
        self.label_24 = QLabel(self.frame_3)
        self.label_24.setObjectName(u"label_24")
        self.label_24.setGeometry(QRect(180, 20, 91, 21))
        self.txA1e = QLabel(self.frame_3)
        self.txA1e.setObjectName(u"txA1e")
        self.txA1e.setGeometry(QRect(290, 50, 120, 25))
        self.txA2e = QLabel(self.frame_3)
        self.txA2e.setObjectName(u"txA2e")
        self.txA2e.setGeometry(QRect(290, 80, 120, 25))
        self.txA3e = QLabel(self.frame_3)
        self.txA3e.setObjectName(u"txA3e")
        self.txA3e.setGeometry(QRect(290, 110, 120, 25))
        self.txA4e = QLabel(self.frame_3)
        self.txA4e.setObjectName(u"txA4e")
        self.txA4e.setGeometry(QRect(290, 140, 120, 25))
        self.txA5e = QLabel(self.frame_3)
        self.txA5e.setObjectName(u"txA5e")
        self.txA5e.setGeometry(QRect(290, 170, 120, 25))
        self.txA6e = QLabel(self.frame_3)
        self.txA6e.setObjectName(u"txA6e")
        self.txA6e.setGeometry(QRect(290, 200, 120, 25))
        self.txA7e = QLabel(self.frame_3)
        self.txA7e.setObjectName(u"txA7e")
        self.txA7e.setGeometry(QRect(290, 230, 120, 25))
        self.txA8e = QLabel(self.frame_3)
        self.txA8e.setObjectName(u"txA8e")
        self.txA8e.setGeometry(QRect(290, 260, 120, 25))
        self.txChisq = QLabel(self.frame_3)
        self.txChisq.setObjectName(u"txChisq")
        self.txChisq.setGeometry(QRect(240, 290, 120, 25))
        self.label_9 = QLabel(self.frame_3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(180, 290, 68, 25))
        self.label_7 = QLabel(self.frame_3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(290, 20, 68, 21))
        self.btCheck = QPushButton(DgFitting)
        self.btCheck.setObjectName(u"btCheck")
        self.btCheck.setGeometry(QRect(10, 530, 61, 27))
        self.btFitting = QPushButton(DgFitting)
        self.btFitting.setObjectName(u"btFitting")
        self.btFitting.setGeometry(QRect(100, 530, 121, 31))
        self.btClose = QPushButton(DgFitting)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(360, 540, 51, 27))
        QWidget.setTabOrder(self.cbTrace, self.txRangeMin)
        QWidget.setTabOrder(self.txRangeMin, self.txRangeMax)
        QWidget.setTabOrder(self.txRangeMax, self.cbFunctions)
        QWidget.setTabOrder(self.cbFunctions, self.btLoadFunc)
        QWidget.setTabOrder(self.btLoadFunc, self.txA1)
        QWidget.setTabOrder(self.txA1, self.ckA1)
        QWidget.setTabOrder(self.ckA1, self.txA2)
        QWidget.setTabOrder(self.txA2, self.ckA2)
        QWidget.setTabOrder(self.ckA2, self.txA3)
        QWidget.setTabOrder(self.txA3, self.ckA3)
        QWidget.setTabOrder(self.ckA3, self.txA4)
        QWidget.setTabOrder(self.txA4, self.ckA4)
        QWidget.setTabOrder(self.ckA4, self.txA5)
        QWidget.setTabOrder(self.txA5, self.ckA5)
        QWidget.setTabOrder(self.ckA5, self.txA6)
        QWidget.setTabOrder(self.txA6, self.ckA6)
        QWidget.setTabOrder(self.ckA6, self.txA7)
        QWidget.setTabOrder(self.txA7, self.ckA7)
        QWidget.setTabOrder(self.ckA7, self.txA8)
        QWidget.setTabOrder(self.txA8, self.ckA8)
        QWidget.setTabOrder(self.ckA8, self.btCheck)
        QWidget.setTabOrder(self.btCheck, self.btFitting)
        QWidget.setTabOrder(self.btFitting, self.btClose)

        self.retranslateUi(DgFitting)

        QMetaObject.connectSlotsByName(DgFitting)
    # setupUi

    def retranslateUi(self, DgFitting):
        DgFitting.setWindowTitle(QCoreApplication.translate("DgFitting", u"Fitting", None))
        self.label.setText(QCoreApplication.translate("DgFitting", u"Data", None))
        self.label_2.setText(QCoreApplication.translate("DgFitting", u"From", None))
        self.txRangeMin.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txRangeMax.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.label_3.setText(QCoreApplication.translate("DgFitting", u"To", None))
        self.label_4.setText(QCoreApplication.translate("DgFitting", u"Function", None))
        self.cbFunctions.setItemText(0, QCoreApplication.translate("DgFitting", u"Linear", None))
        self.cbFunctions.setItemText(1, QCoreApplication.translate("DgFitting", u"Linear + Gauss", None))
        self.cbFunctions.setItemText(2, QCoreApplication.translate("DgFitting", u"Linear + Lorentz", None))
        self.cbFunctions.setItemText(3, QCoreApplication.translate("DgFitting", u"Linear + Gauss + Lorents", None))
        self.cbFunctions.setItemText(4, QCoreApplication.translate("DgFitting", u"Linear + Gauss + Gauss", None))

        self.pnlFunction.setText("")
        self.btLoadFunc.setText(QCoreApplication.translate("DgFitting", u"Load Func File", None))
        self.label_5.setText(QCoreApplication.translate("DgFitting", u"Parameters", None))
        self.txA1.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.label_6.setText(QCoreApplication.translate("DgFitting", u"A1", None))
        self.ckA1.setText("")
        self.txA1r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.label_8.setText(QCoreApplication.translate("DgFitting", u"A2", None))
        self.txA2.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.ckA2.setText("")
        self.txA2r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA3.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txA4.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txA5.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txA6.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txA7.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.txA8.setText(QCoreApplication.translate("DgFitting", u"0.0", None))
        self.label_10.setText(QCoreApplication.translate("DgFitting", u"A3", None))
        self.label_11.setText(QCoreApplication.translate("DgFitting", u"A4", None))
        self.label_12.setText(QCoreApplication.translate("DgFitting", u"A5", None))
        self.label_13.setText(QCoreApplication.translate("DgFitting", u"A6", None))
        self.label_14.setText(QCoreApplication.translate("DgFitting", u"A7", None))
        self.label_15.setText(QCoreApplication.translate("DgFitting", u"A8", None))
        self.ckA3.setText("")
        self.ckA4.setText("")
        self.ckA5.setText("")
        self.ckA6.setText("")
        self.ckA7.setText("")
        self.ckA8.setText("")
        self.txA3r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA4r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA5r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA6r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA7r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA8r.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.label_22.setText(QCoreApplication.translate("DgFitting", u"Initial Value", None))
        self.label_23.setText(QCoreApplication.translate("DgFitting", u"Fix", None))
        self.label_24.setText(QCoreApplication.translate("DgFitting", u"Result Value", None))
        self.txA1e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA2e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA3e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA4e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA5e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA6e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA7e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txA8e.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.txChisq.setText(QCoreApplication.translate("DgFitting", u"TextLabel", None))
        self.label_9.setText(QCoreApplication.translate("DgFitting", u"ChiSq =", None))
        self.label_7.setText(QCoreApplication.translate("DgFitting", u"Error", None))
        self.btCheck.setText(QCoreApplication.translate("DgFitting", u"Check", None))
        self.btFitting.setText(QCoreApplication.translate("DgFitting", u"Fitting", None))
        self.btClose.setText(QCoreApplication.translate("DgFitting", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgGridSetting.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgGridSetting(object):
    def setupUi(self, DgGridSetting):
        if not DgGridSetting.objectName():
            DgGridSetting.setObjectName(u"DgGridSetting")
        DgGridSetting.resize(452, 252)
        self.frame = QFrame(DgGridSetting)
        self.frame.setObjectName(u"frame")
        self.frame.setGeometry(QRect(10, 10, 431, 91))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.label = QLabel(self.frame)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 0, 131, 17))
        self.label_3 = QLabel(self.frame)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(20, 30, 90, 25))
        self.label_4 = QLabel(self.frame)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(20, 60, 90, 25))
        self.label_5 = QLabel(self.frame)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(220, 30, 51, 25))
        self.cbXlineColor = QComboBox(self.frame)
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.addItem("")
        self.cbXlineColor.setObjectName(u"cbXlineColor")
        self.cbXlineColor.setGeometry(QRect(270, 30, 141, 27))
        self.spXlineWidth = QSpinBox(self.frame)
        self.spXlineWidth.setObjectName(u"spXlineWidth")
        self.spXlineWidth.setGeometry(QRect(110, 30, 60, 27))
        self.cbXlineStyle = QComboBox(self.frame)
        self.cbXlineStyle.addItem("")
        self.cbXlineStyle.addItem("")
        self.cbXlineStyle.addItem("")
        self.cbXlineStyle.addItem("")
        self.cbXlineStyle.addItem("")
        self.cbXlineStyle.setObjectName(u"cbXlineStyle")
        self.cbXlineStyle.setGeometry(QRect(110, 60, 141, 27))
        self.frame_2 = QFrame(DgGridSetting)
        self.frame_2.setObjectName(u"frame_2")
        self.frame_2.setGeometry(QRect(10, 110, 431, 91))
        self.frame_2.setFrameShape(QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QFrame.Raised)
        self.label_2 = QLabel(self.frame_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 0, 91, 17))
        self.label_6 = QLabel(self.frame_2)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(20, 30, 90, 25))
        self.label_7 = QLabel(self.frame_2)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(20, 60, 90, 25))
        self.spYlineWidth = QSpinBox(self.frame_2)
        self.spYlineWidth.setObjectName(u"spYlineWidth")
        self.spYlineWidth.setGeometry(QRect(110, 30, 60, 27))
        self.label_8 = QLabel(self.frame_2)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(210, 30, 51, 25))
        self.cbYlineColor = QComboBox(self.frame_2)
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.addItem("")
        self.cbYlineColor.setObjectName(u"cbYlineColor")
        self.cbYlineColor.setGeometry(QRect(270, 30, 141, 27))
        self.cbYlineStyle = QComboBox(self.frame_2)
        self.cbYlineStyle.addItem("")
        self.cbYlineStyle.addItem("")
        self.cbYlineStyle.addItem("")
        self.cbYlineStyle.addItem("")
        self.cbYlineStyle.addItem("")
        self.cbYlineStyle.setObjectName(u"cbYlineStyle")
        self.cbYlineStyle.setGeometry(QRect(110, 60, 141, 27))
        self.btClose = QPushButton(DgGridSetting)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(30, 210, 81, 27))
        QWidget.setTabOrder(self.spXlineWidth, self.cbXlineColor)
        QWidget.setTabOrder(self.cbXlineColor, self.cbXlineStyle)
        QWidget.setTabOrder(self.cbXlineStyle, self.spYlineWidth)
        QWidget.setTabOrder(self.spYlineWidth, self.cbYlineColor)
        QWidget.setTabOrder(self.cbYlineColor, self.cbYlineStyle)
        QWidget.setTabOrder(self.cbYlineStyle, self.btClose)

        self.retranslateUi(DgGridSetting)

        QMetaObject.connectSlotsByName(DgGridSetting)
    # setupUi

    def retranslateUi(self, DgGridSetting):
        DgGridSetting.setWindowTitle(QCoreApplication.translate("DgGridSetting", u"Grid Style Setting", None))
        self.label.setText(QCoreApplication.translate("DgGridSetting", u"X Grid Style", None))
        self.label_3.setText(QCoreApplication.translate("DgGridSetting", u"Line Width : ", None))
        self.label_4.setText(QCoreApplication.translate("DgGridSetting", u"Line Style : ", None))
        self.label_5.setText(QCoreApplication.translate("DgGridSetting", u"Color:", None))
        self.cbXlineColor.setItemText(0, QCoreApplication.translate("DgGridSetting", u"Blue", None))
        self.cbXlineColor.setItemText(1, QCoreApplication.translate("DgGridSetting", u"Green", None))
        self.cbXlineColor.setItemText(2, QCoreApplication.translate("DgGridSetting", u"Red", None))
        self.cbXlineColor.setItemText(3, QCoreApplication.translate("DgGridSetting", u"Cyan", None))
        self.cbXlineColor.setItemText(4, QCoreApplication.translate("DgGridSetting", u"Magenta", None))
        self.cbXlineColor.setItemText(5, QCoreApplication.translate("DgGridSetting", u"Yellow", None))
        self.cbXlineColor.setItemText(6, QCoreApplication.translate("DgGridSetting", u"Black", None))

        self.cbXlineStyle.setItemText(0, QCoreApplication.translate("DgGridSetting", u"Line", None))
        self.cbXlineStyle.setItemText(1, QCoreApplication.translate("DgGridSetting", u"Dotted", None))
        self.cbXlineStyle.setItemText(2, QCoreApplication.translate("DgGridSetting", u"Dashed", None))
        self.cbXlineStyle.setItemText(3, QCoreApplication.translate("DgGridSetting", u"Dot-dashed", None))
        self.cbXlineStyle.setItemText(4, QCoreApplication.translate("DgGridSetting", u"None", None))

        self.label_2.setText(QCoreApplication.translate("DgGridSetting", u"Y Grid Style", None))
        self.label_6.setText(QCoreApplication.translate("DgGridSetting", u"Line Width : ", None))
        self.label_7.setText(QCoreApplication.translate("DgGridSetting", u"Line Style : ", None))
        self.label_8.setText(QCoreApplication.translate("DgGridSetting", u"Color:", None))
        self.cbYlineColor.setItemText(0, QCoreApplication.translate("DgGridSetting", u"Blue", None))
        self.cbYlineColor.setItemText(1, QCoreApplication.translate("DgGridSetting", u"Green", None))
        self.cbYlineColor.setItemText(2, QCoreApplication.translate("DgGridSetting", u"Red", None))
        self.cbYlineColor.setItemText(3, QCoreApplication.translate("DgGridSetting", u"Cyan", None))
        self.cbYlineColor.setItemText(4, QCoreApplication.translate("DgGridSetting", u"Magenta", None))
        self.cbYlineColor.setItemText(5, QCoreApplication.translate("DgGridSetting", u"Yellow", None))
        self.cbYlineColor.setItemText(6, QCoreApplication.translate("DgGridSetting", u"Black", None))

        self.cbYlineStyle.setItemText(0, QCoreApplication.translate("DgGridSetting", u"Line", None))
        self.cbYlineStyle.setItemText(1, QCoreApplication.translate("DgGridSetting", u"Dotted", None))
        self.cbYlineStyle.setItemText(2, QCoreApplication.translate("DgGridSetting", u"Dashed", None))
        self.cbYlineStyle.setItemText(3, QCoreApplication.translate("DgGridSetting", u"Dot-dashed", None))
        self.cbYlineStyle.setItemText(4, QCoreApplication.translate("DgGridSetting", u"None", None))

        self.btClose.setText(QCoreApplication.translate("DgGridSetting", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgTickStyleSetting.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgTickStyleSetting(object):
    def setupUi(self, DgTickStyleSetting):
        if not DgTickStyleSetting.objectName():
            DgTickStyleSetting.setObjectName(u"DgTickStyleSetting")
        DgTickStyleSetting.resize(320, 260)
        self.ckTopSideOn = QCheckBox(DgTickStyleSetting)
        self.ckTopSideOn.setObjectName(u"ckTopSideOn")
        self.ckTopSideOn.setGeometry(QRect(10, 10, 151, 22))
        self.ckRightSideOn = QCheckBox(DgTickStyleSetting)
        self.ckRightSideOn.setObjectName(u"ckRightSideOn")
        self.ckRightSideOn.setGeometry(QRect(160, 10, 141, 22))
        self.label = QLabel(DgTickStyleSetting)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(20, 50, 81, 17))
        self.line = QFrame(DgTickStyleSetting)
        self.line.setObjectName(u"line")
        self.line.setGeometry(QRect(10, 30, 301, 16))
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.label_2 = QLabel(DgTickStyleSetting)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(130, 50, 68, 17))
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.txMajorLength = QLineEdit(DgTickStyleSetting)
        self.txMajorLength.setObjectName(u"txMajorLength")
        self.txMajorLength.setGeometry(QRect(200, 50, 100, 25))
        self.label_3 = QLabel(DgTickStyleSetting)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(130, 80, 68, 17))
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.txMajorWidth = QLineEdit(DgTickStyleSetting)
        self.txMajorWidth.setObjectName(u"txMajorWidth")
        self.txMajorWidth.setGeometry(QRect(200, 80, 100, 25))
        self.ckUseMinorTick = QCheckBox(DgTickStyleSetting)
        self.ckUseMinorTick.setObjectName(u"ckUseMinorTick")
        self.ckUseMinorTick.setGeometry(QRect(20, 110, 111, 22))
        self.label_4 = QLabel(DgTickStyleSetting)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(130, 110, 68, 17))
        self.label_4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.txMinorLength = QLineEdit(DgTickStyleSetting)
        self.txMinorLength.setObjectName(u"txMinorLength")
        self.txMinorLength.setGeometry(QRect(200, 110, 100, 25))
        self.label_5 = QLabel(DgTickStyleSetting)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(130, 140, 68, 17))
        self.label_5.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.txMinorWidth = QLineEdit(DgTickStyleSetting)
        self.txMinorWidth.setObjectName(u"txMinorWidth")
        self.txMinorWidth.setGeometry(QRect(200, 140, 100, 25))
        self.btApply = QPushButton(DgTickStyleSetting)
        self.btApply.setObjectName(u"btApply")
        self.btApply.setGeometry(QRect(200, 170, 100, 30))
        self.line_2 = QFrame(DgTickStyleSetting)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setGeometry(QRect(10, 200, 301, 16))
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.btRevert = QPushButton(DgTickStyleSetting)
        self.btRevert.setObjectName(u"btRevert")
        self.btRevert.setGeometry(QRect(40, 220, 60, 30))
        self.btClose = QPushButton(DgTickStyleSetting)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(220, 220, 60, 30))

        self.retranslateUi(DgTickStyleSetting)

        QMetaObject.connectSlotsByName(DgTickStyleSetting)
    # setupUi

    def retranslateUi(self, DgTickStyleSetting):
        DgTickStyleSetting.setWindowTitle(QCoreApplication.translate("DgTickStyleSetting", u"Ticks Style Setting", None))
        self.ckTopSideOn.setText(QCoreApplication.translate("DgTickStyleSetting", u"Top Side Ticks", None))
        self.ckRightSideOn.setText(QCoreApplication.translate("DgTickStyleSetting", u"Right Side Ticks", None))
        self.label.setText(QCoreApplication.translate("DgTickStyleSetting", u"Major Ticks", None))
        self.label_2.setText(QCoreApplication.translate("DgTickStyleSetting", u"Length :", None))
        self.label_3.setText(QCoreApplication.translate("DgTickStyleSetting", u"Width :", None))
        self.ckUseMinorTick.setText(QCoreApplication.translate("DgTickStyleSetting", u"Minor Ticks", None))
        self.label_4.setText(QCoreApplication.translate("DgTickStyleSetting", u"Length :", None))
        self.label_5.setText(QCoreApplication.translate("DgTickStyleSetting", u"Width :", None))
        self.btApply.setText(QCoreApplication.translate("DgTickStyleSetting", u"Apply", None))
        self.btRevert.setText(QCoreApplication.translate("DgTickStyleSetting", u"Revert", None))
        self.btClose.setText(QCoreApplication.translate("DgTickStyleSetting", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'DgCalcDS.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_DgCalcDS(object):
    def setupUi(self, DgCalcDS):
        if not DgCalcDS.objectName():
            DgCalcDS.setObjectName(u"DgCalcDS")
        DgCalcDS.resize(256, 260)
        self.L1txt = QLineEdit(DgCalcDS)
        self.L1txt.setObjectName(u"L1txt")
        self.L1txt.setGeometry(QRect(50, 10, 81, 31))
        self.label = QLabel(DgCalcDS)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 20, 31, 17))
        self.label_2 = QLabel(DgCalcDS)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(140, 20, 41, 17))
        self.btApply = QPushButton(DgCalcDS)
        self.btApply.setObjectName(u"btApply")
        self.btApply.setGeometry(QRect(190, 10, 51, 27))
        self.btClose = QPushButton(DgCalcDS)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(90, 230, 61, 27))
        self.line = QFrame(DgCalcDS)
        self.line.setObjectName(u"line")
        self.line.setGeometry(QRect(90, 40, 161, 16))
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.line_2 = QFrame(DgCalcDS)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setGeometry(QRect(0, 210, 251, 16))
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.label_3 = QLabel(DgCalcDS)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(0, 40, 91, 17))
        self.txtIndex = QLabel(DgCalcDS)
        self.txtIndex.setObjectName(u"txtIndex")
        self.txtIndex.setGeometry(QRect(10, 60, 91, 141))
        self.txtResults = QLabel(DgCalcDS)
        self.txtResults.setObjectName(u"txtResults")
        self.txtResults.setGeometry(QRect(100, 60, 151, 141))

        self.retranslateUi(DgCalcDS)

        QMetaObject.connectSlotsByName(DgCalcDS)
    # setupUi

    def retranslateUi(self, DgCalcDS):
        DgCalcDS.setWindowTitle(QCoreApplication.translate("DgCalcDS", u"TOF conversions", None))
        self.label.setText(QCoreApplication.translate("DgCalcDS", u"L1 = ", None))
        self.label_2.setText(QCoreApplication.translate("DgCalcDS", u"[mm]", None))
        self.btApply.setText(QCoreApplication.translate("DgCalcDS", u"Apply", None))
        self.btClose.setText(QCoreApplication.translate("DgCalcDS", u"Close", None))
        self.label_3.setText(QCoreApplication.translate("DgCalcDS", u"Information", None))
        self.txtIndex.setText(QCoreApplication.translate("DgCalcDS", u"TextLabel", None))
        self.txtResults.setText(QCoreApplication.translate("DgCalcDS", u"TextLabel", None))
    # retranslateUi

