# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'SQW.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(434, 720)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.errorstr = QLabel(self.centralwidget)
        self.errorstr.setObjectName(u"errorstr")
        self.errorstr.setGeometry(QRect(30, 6, 371, 21))
        font = QFont()
        font.setPointSize(16)
        self.errorstr.setFont(font)
        self.errorstr.setAlignment(Qt.AlignCenter)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 30, 411, 91))
        self.rb_Si111 = QRadioButton(self.groupBox)
        self.rb_Si111.setObjectName(u"rb_Si111")
        self.rb_Si111.setGeometry(QRect(20, 20, 121, 23))
        self.dsp_Si111 = QDoubleSpinBox(self.groupBox)
        self.dsp_Si111.setObjectName(u"dsp_Si111")
        self.dsp_Si111.setGeometry(QRect(140, 20, 65, 20))
        self.dsp_Si111.setMinimum(0.010000000000000)
        self.dsp_Si111.setMaximum(9.990000000000000)
        self.dsp_Si111.setSingleStep(0.010000000000000)
        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(220, 22, 151, 17))
        self.rb_Si333 = QRadioButton(self.groupBox)
        self.rb_Si333.setObjectName(u"rb_Si333")
        self.rb_Si333.setGeometry(QRect(20, 40, 121, 23))
        self.rb_Si311 = QRadioButton(self.groupBox)
        self.rb_Si311.setObjectName(u"rb_Si311")
        self.rb_Si311.setGeometry(QRect(20, 60, 121, 23))
        self.dsp_Si333 = QDoubleSpinBox(self.groupBox)
        self.dsp_Si333.setObjectName(u"dsp_Si333")
        self.dsp_Si333.setGeometry(QRect(140, 40, 65, 20))
        self.dsp_Si333.setMinimum(0.010000000000000)
        self.dsp_Si333.setMaximum(9.990000000000000)
        self.dsp_Si333.setSingleStep(0.010000000000000)
        self.dsp_Si311 = QDoubleSpinBox(self.groupBox)
        self.dsp_Si311.setObjectName(u"dsp_Si311")
        self.dsp_Si311.setGeometry(QRect(140, 60, 65, 20))
        self.dsp_Si311.setMinimum(0.010000000000000)
        self.dsp_Si311.setMaximum(9.990000000000000)
        self.dsp_Si311.setSingleStep(0.010000000000000)
        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(220, 42, 151, 17))
        self.label_3 = QLabel(self.groupBox)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(220, 62, 151, 17))
        self.groupBox_2 = QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(10, 130, 411, 61))
        self.run = QSpinBox(self.groupBox_2)
        self.run.setObjectName(u"run")
        self.run.setGeometry(QRect(10, 30, 81, 26))
        self.Ch = QComboBox(self.groupBox_2)
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.addItem("")
        self.Ch.setObjectName(u"Ch")
        self.Ch.setGeometry(QRect(190, 30, 61, 25))
        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(100, 35, 101, 17))
        self.bt_ReadTemp = QPushButton(self.groupBox_2)
        self.bt_ReadTemp.setObjectName(u"bt_ReadTemp")
        self.bt_ReadTemp.setGeometry(QRect(268, 30, 101, 25))
        self.groupBox_mon = QGroupBox(self.centralwidget)
        self.groupBox_mon.setTitle("")
        self.groupBox_mon.setGeometry(QRect(10, 195, 411, 61))
        self.mon_run = QSpinBox(self.groupBox_mon)
        self.mon_run.setGeometry(QRect(10, 30, 81, 26))
        self.cb_UseMon = QCheckBox(self.groupBox_mon)
        self.cb_UseMon.setGeometry(QRect(10, 0, 160, 23))
        self.cb_UseMon.setText("Use Monitor Run")
        self.groupBox_mon.setGeometry(QRect(10, 195, 411, 61))
        self.mon_run = QSpinBox(self.groupBox_mon)
        self.mon_run.setGeometry(QRect(10, 30, 81, 26))
        self.groupBox_3 = QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(10, 265, 411, 201))
        self.label_5 = QLabel(self.groupBox_3)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(0, 20, 101, 17))
        self.lam = QDoubleSpinBox(self.groupBox_3)
        self.lam.setObjectName(u"lam")
        self.lam.setGeometry(QRect(100, 20, 81, 20))
        self.label_6 = QLabel(self.groupBox_3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(190, 20, 91, 17))
        self.dl = QDoubleSpinBox(self.groupBox_3)
        self.dl.setObjectName(u"dl")
        self.dl.setGeometry(QRect(280, 20, 81, 20))
        self.label_7 = QLabel(self.groupBox_3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(0, 50, 41, 17))
        self.label_8 = QLabel(self.groupBox_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(105, 50, 81, 17))
        self.label_9 = QLabel(self.groupBox_3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(255, 50, 61, 17))
        self.label_11 = QLabel(self.groupBox_3)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(380, 50, 41, 17))
        self.cb_UserDefinedEBin = QCheckBox(self.groupBox_3)
        self.cb_UserDefinedEBin.setObjectName(u"cb_UserDefinedEBin")
        self.cb_UserDefinedEBin.setGeometry(QRect(10, 80, 151, 23))
        self.bt_UserDefinedEBin = QPushButton(self.groupBox_3)
        self.bt_UserDefinedEBin.setObjectName(u"bt_UserDefinedEBin")
        self.bt_UserDefinedEBin.setGeometry(QRect(160, 80, 89, 25))
        self.txt_EbinFileName = QLabel(self.groupBox_3)
        self.txt_EbinFileName.setObjectName(u"txt_EbinFileName")
        self.txt_EbinFileName.setGeometry(QRect(160, 110, 241, 20))
        self.label_10 = QLabel(self.groupBox_3)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(20, 140, 67, 17))
        self.emin = QSpinBox(self.groupBox_3)
        self.emin.setObjectName(u"emin")
        self.emin.setGeometry(QRect(40, 50, 61, 20))
        self.emax = QSpinBox(self.groupBox_3)
        self.emax.setObjectName(u"emax")
        self.emax.setGeometry(QRect(190, 50, 61, 20))
        self.T0S = QSpinBox(self.groupBox_3)
        self.T0S.setObjectName(u"T0S")
        self.T0S.setGeometry(QRect(80, 140, 81, 20))
        self.label_13 = QLabel(self.groupBox_3)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(170, 140, 67, 17))
        self.label_14 = QLabel(self.groupBox_3)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(10, 170, 81, 17))
        self.Sta = QSpinBox(self.groupBox_3)
        self.Sta.setObjectName(u"Sta")
        self.Sta.setGeometry(QRect(80, 170, 71, 20))
        self.label_15 = QLabel(self.groupBox_3)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(160, 170, 111, 17))
        self.End = QSpinBox(self.groupBox_3)
        self.End.setObjectName(u"End")
        self.End.setGeometry(QRect(270, 170, 91, 20))
        self.label_16 = QLabel(self.groupBox_3)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(360, 170, 51, 17))
        self.de = QDoubleSpinBox(self.groupBox_3)
        self.de.setObjectName(u"de")
        self.de.setGeometry(QRect(315, 50, 61, 20))
        self.groupBox_4 = QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setGeometry(QRect(10, 475, 411, 80))
        self.cb_VanadiumCorr = QCheckBox(self.groupBox_4)
        self.cb_VanadiumCorr.setObjectName(u"cb_VanadiumCorr")
        self.cb_VanadiumCorr.setGeometry(QRect(10, 30, 161, 23))
        self.bt_VanadiumCorr = QPushButton(self.groupBox_4)
        self.bt_VanadiumCorr.setObjectName(u"bt_VanadiumCorr")
        self.bt_VanadiumCorr.setGeometry(QRect(180, 30, 89, 25))
        self.txt_VanadiumCorr = QLabel(self.groupBox_4)
        self.txt_VanadiumCorr.setObjectName(u"txt_VanadiumCorr")
        self.txt_VanadiumCorr.setGeometry(QRect(70, 60, 341, 17))
        self.groupBox_5 = QGroupBox(self.centralwidget)
        self.groupBox_5.setObjectName(u"groupBox_5")
        self.groupBox_5.setGeometry(QRect(10, 565, 411, 61))
        self.bt_MaskFile = QPushButton(self.groupBox_5)
        self.bt_MaskFile.setObjectName(u"bt_MaskFile")
        self.bt_MaskFile.setGeometry(QRect(10, 30, 89, 25))
        self.txt_MaskFile = QLabel(self.groupBox_5)
        self.txt_MaskFile.setObjectName(u"txt_MaskFile")
        self.txt_MaskFile.setGeometry(QRect(110, 35, 291, 17))
        self.bt_LaunchDataRed = QPushButton(self.centralwidget)
        self.bt_LaunchDataRed.setObjectName(u"bt_LaunchDataRed")
        self.bt_LaunchDataRed.setGeometry(QRect(10, 635, 201, 31))
        self.bt_Quit = QPushButton(self.centralwidget)
        self.bt_Quit.setObjectName(u"bt_Quit")
        self.bt_Quit.setGeometry(QRect(378, 645, 51, 25))
        self.background = QWidget(self.centralwidget)
        self.background.setObjectName(u"background")
        self.background.setGeometry(QRect(0, 0, 441, 720))
        self.background.setStyleSheet(u"")
        MainWindow.setCentralWidget(self.centralwidget)
        self.background.raise_()
        self.errorstr.raise_()
        self.groupBox.raise_()
        self.groupBox_2.raise_()
        self.groupBox_3.raise_()
        self.groupBox_4.raise_()
        self.groupBox_5.raise_()
        self.bt_LaunchDataRed.raise_()
        self.bt_Quit.raise_()
        self.groupBox_mon.raise_() #20260130 Yamada add
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 434, 22))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.errorstr.setText(QCoreApplication.translate("MainWindow", u"Low Resolution Mode", None))
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Si", None))
        self.rb_Si111.setText(QCoreApplication.translate("MainWindow", u"Si(111) : dQ = ", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Qmin=0.0 Qmax=2.0", None))
        self.rb_Si333.setText(QCoreApplication.translate("MainWindow", u"Si(333) : dQ = ", None))
        self.rb_Si311.setText(QCoreApplication.translate("MainWindow", u"Si(311) : dQ = ", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Qmin=0.0 Qmax=6.0", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Qmin=0.0 Qmax=4.0", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("MainWindow", u"Run No.", None))
        self.Ch.setItemText(0, QCoreApplication.translate("MainWindow", u"A", None))
        self.Ch.setItemText(1, QCoreApplication.translate("MainWindow", u"B", None))
        self.Ch.setItemText(2, QCoreApplication.translate("MainWindow", u"C", None))
        self.Ch.setItemText(3, QCoreApplication.translate("MainWindow", u"C2", None))
        self.Ch.setItemText(4, QCoreApplication.translate("MainWindow", u"D", None))
        self.Ch.setItemText(5, QCoreApplication.translate("MainWindow", u"D2", None))
        self.Ch.setItemText(6, QCoreApplication.translate("MainWindow", u"D3", None))
        self.Ch.setItemText(7, QCoreApplication.translate("MainWindow", u"D4", None))
        self.Ch.setItemText(8, QCoreApplication.translate("MainWindow", u"D5", None))
        self.Ch.setItemText(9, QCoreApplication.translate("MainWindow", u"C_TC", None))
        self.Ch.setItemText(10, QCoreApplication.translate("MainWindow", u"D_TC", None))
        self.Ch.setItemText(11, QCoreApplication.translate("MainWindow", u"PEACE_U", None))
        self.Ch.setItemText(12, QCoreApplication.translate("MainWindow", u"PEACE_M", None))
        self.Ch.setItemText(13, QCoreApplication.translate("MainWindow", u"PEACE_L", None))
        self.Ch.setItemText(14, QCoreApplication.translate("MainWindow", u"PEACE_UB", None))
        self.Ch.setItemText(15, QCoreApplication.translate("MainWindow", u"PEACE_MB", None))
        self.Ch.setItemText(16, QCoreApplication.translate("MainWindow", u"PEACE_LB", None))

        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Temp. Sensor", None))
        self.bt_ReadTemp.setText(QCoreApplication.translate("MainWindow", u"Read Temp.", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("MainWindow", u"Parameter", None))
        self.label_5.setText(QCoreApplication.translate("MainWindow", u"Lamb Center", None))
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"[A] / dLamb", None))
        self.label_7.setText(QCoreApplication.translate("MainWindow", u"Emin", None))
        self.label_8.setText(QCoreApplication.translate("MainWindow", u"ueV / Emax", None))
        self.label_9.setText(QCoreApplication.translate("MainWindow", u"ueV / dE", None))
        self.label_11.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.cb_UserDefinedEBin.setText(QCoreApplication.translate("MainWindow", u"User defined E Bin", None))
        self.bt_UserDefinedEBin.setText(QCoreApplication.translate("MainWindow", u"Select File", None))
        self.txt_EbinFileName.setText(QCoreApplication.translate("MainWindow", u"not selected", None))
        self.label_10.setText(QCoreApplication.translate("MainWindow", u"T0Shift", None))
        self.label_13.setText(QCoreApplication.translate("MainWindow", u"[us]", None))
        self.label_14.setText(QCoreApplication.translate("MainWindow", u"StartTime", None))
        self.label_15.setText(QCoreApplication.translate("MainWindow", u"[sec.] / EndTime", None))
        self.label_16.setText(QCoreApplication.translate("MainWindow", u"[usec]", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("MainWindow", u"Detector Efficiency Correction", None))
        self.cb_VanadiumCorr.setText(QCoreApplication.translate("MainWindow", u"Vanadium Correction", None))
        self.bt_VanadiumCorr.setText(QCoreApplication.translate("MainWindow", u"Select File", None))
        self.txt_VanadiumCorr.setText(QCoreApplication.translate("MainWindow", u"not selected", None))
        self.groupBox_5.setTitle(QCoreApplication.translate("MainWindow", u"Select mask file", None))
        self.bt_MaskFile.setText(QCoreApplication.translate("MainWindow", u"Select file", None))
        self.txt_MaskFile.setText(QCoreApplication.translate("MainWindow", u"maskTY.txt", None))
        self.bt_LaunchDataRed.setText(QCoreApplication.translate("MainWindow", u"Launch Data Reduction", None))
        self.bt_Quit.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
    # retranslateUi

