#ifndef UTSUSEMISETMASKDNA
#define UTSUSEMISETMASKDNA

#include "Header.hh"

#include "ElementContainer.hh"
#include "ElementContainerArray.hh"
#include "ElementContainerMatrix.hh"

//////////////////////////////////
class UtsusemiSetMaskDNA
{
private:

public:
    UtsusemiSetMaskDNA();

    UtsusemiSetMaskDNA(ElementContainerMatrix *ecm);

    UtsusemiSetMaskDNA(ElementContainerArray *eca);

    UtsusemiSetMaskDNA(ElementContainer *ec);

    ~UtsusemiSetMaskDNA();

    void Execute(ElementContainerMatrix *ecm);

    void Execute(ElementContainerArray *eca);

    void Execute(ElementContainer *ec);
};

#endif
