import Manyo.MLF

e = Manyo.ElementContainer()
e.Add("x",[0,1,2,3,4,5,6,7,8,9])
e.Add("y",[5,2,7,8,2,1,3,4,7])
e.Add("err",[1,1,1,1,1,1,1,1,1])
e.SetKeys("x","y","err")

ConView=Manyo.MLF.ContainerViewer()
ConView.Histogram(e)

