#include <string>
#include "Header.hh"
#include "AdvPeakType.hh"

const AdvPeakType UNKNOWN_PEAK_TYPE={ UNKNOWN_PEAK_TYPE_ID, string("uk") };
const AdvPeakType PEAK             ={ PEAK_ID,              string("p")  };
const AdvPeakType SHOLDER_I        ={ SHOLDER_I_ID,         string("si") };
const AdvPeakType SHOLDER_D        ={ SHOLDER_D_ID,         string("sd") };
