#!/bin/bash

pyside-uic FastPlot.ui           -o ui_FastPlotQt.py
pyside-uic DgPlotPtn.ui          -o ui_tmp1.py
pyside-uic DgScaleLabel.ui       -o ui_tmp2.py
pyside-uic DgSetting.ui          -o ui_tmp3.py
pyside-uic DgTitle.ui            -o ui_tmp4.py
pyside-uic FloatBox.ui           -o ui_tmp5.py
pyside-uic DgSaveAsText.ui       -o ui_tmp6.py
pyside-uic DgFitting.ui          -o ui_tmp7.py
pyside-uic DgGridSetting.ui      -o ui_tmp8.py
pyside-uic DgTickStyleSetting.ui -o ui_tmp9.py
pyside-uic DgCalcDS.ui           -o ui_tmp10.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp3.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp4.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp5.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp6.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp7.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp8.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp9.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py
cat ui_tmp10.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_FastPlotQt.py

rm ui_tmp*.py

