#ifndef UTSUSEMIREDISPUBSUBEVENTMONITORMWPC
#define UTSUSEMIREDISPUBSUBEVENTMONITORMWPC

#include "UtsusemiRedisPubSubEventMonitorTemplate.hh"
#include "UtsusemiEventDataConverterMWPC.hh"

class UtsusemiRedisPubSubEventMonitorMWPC:
    public UtsusemiRedisPubSubEventMonitorTemplate<UtsusemiEventDataConverterMWPC>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiRedisPubSubEventMonitorMWPC();
        //!< Constructor
        /*!<
         */
    ~UtsusemiRedisPubSubEventMonitorMWPC();
        //!< Destructor
        /*!<
         */
    void OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId=1 );
    UtsusemiEventDataConverterMWPC* PutUtsusemiEventDataConverterMWPC(){ return _EDC; }
};
#endif
