/*
$Id: NeutronWriteBinaryData.hh 1562 2006-06-08 10:27:20Z suzuki $
*/


#ifndef NEUTRONWRITEBINARYDATA_HH
#define NEUTRONWRITEBINARYDATA_HH

#include "Header.hh"

template <class T>
class NeutronWriteBinaryData
{

public:
  NeutronWriteBinaryData();
  ~NeutronWriteBinaryData();

  void Write( T* array, UInt4 size, std::string FileName );
  void Write( std::vector<T> v, std::string FileName );
};

#include "NeutronWriteBinaryData.cc"
#endif
