#ifndef UTSUSEMIHISTOGRAMBINTOPOINT
#define UTSUSEMIHISTOGRAMBINTOPOINT

#include "UtsusemiHeader.hh"

//////////////////////////////////
// Histogram Bin to Point
/////////////////////////////////

//! Histogram format to points 
/*! 
 * This makes points data, which means that there is one-to-one correspondence
 * between X-values ans Y-values ( not like Histogram ).
 *
 */
class UtsusemiHistogramBinToPoint
    : public ManipulatorBase< ElementContainerMatrix >
{
private:
    UtsusemiCheckDataProcess* UCP;
    string _MessageTag;
    string _ProcessTag;
    void Initialize();

public:
    UtsusemiHistogramBinToPoint();
        //!< Constructor
        /*!<
         */
    UtsusemiHistogramBinToPoint( ElementContainerMatrix *ecm );
        //!< Constructor
        /*!<
         *   @param ec   target ElementContainer
         */
    ~UtsusemiHistogramBinToPoint();
        //!< Destructor
        /*!<
         */
    void Execute( bool isReverse=false );
};
#endif
