# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ElasticScanGUIQt.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDoubleSpinBox, QGroupBox,
    QLabel, QMainWindow, QMenuBar, QPushButton,
    QRadioButton, QSizePolicy, QSpinBox, QStatusBar,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(452, 719)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.errorstr = QLabel(self.centralwidget)
        self.errorstr.setObjectName(u"errorstr")
        self.errorstr.setGeometry(QRect(100, 0, 241, 31))
        font = QFont()
        font.setPointSize(14)
        font.setBold(True)
        self.errorstr.setFont(font)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 20, 431, 241))
        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 20, 67, 31))
        self.run = QSpinBox(self.groupBox)
        self.run.setObjectName(u"run")
        self.run.setGeometry(QRect(70, 22, 91, 26))
        self.run.setMaximum(999999)
        self.label_3 = QLabel(self.groupBox)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(170, 20, 101, 31))
        self.ChMenu = QComboBox(self.groupBox)
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.setObjectName(u"ChMenu")
        self.ChMenu.setGeometry(QRect(270, 22, 51, 25))
        self.readtemp_bt = QPushButton(self.groupBox)
        self.readtemp_bt.setObjectName(u"readtemp_bt")
        self.readtemp_bt.setGeometry(QRect(330, 22, 89, 25))
        self.groupBox_2 = QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(10, 50, 411, 61))
        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(0, 30, 51, 21))
        self.label_5 = QLabel(self.groupBox_2)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(120, 30, 71, 21))
        self.Sta = QDoubleSpinBox(self.groupBox_2)
        self.Sta.setObjectName(u"Sta")
        self.Sta.setGeometry(QRect(40, 25, 81, 26))
        self.Sta.setDecimals(1)
        self.Sta.setMaximum(9999999.000000000000000)
        self.End = QDoubleSpinBox(self.groupBox_2)
        self.End.setObjectName(u"End")
        self.End.setGeometry(QRect(180, 25, 81, 26))
        self.End.setDecimals(1)
        self.End.setMaximum(9999999.000000000000000)
        self.End.setValue(180.000000000000000)
        self.label_6 = QLabel(self.groupBox_2)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(260, 30, 101, 21))
        self.Bin = QDoubleSpinBox(self.groupBox_2)
        self.Bin.setObjectName(u"Bin")
        self.Bin.setGeometry(QRect(350, 25, 51, 26))
        self.Bin.setDecimals(1)
        self.Bin.setMaximum(9999999.000000000000000)
        self.Bin.setValue(60.000000000000000)
        self.groupBox_3 = QGroupBox(self.groupBox)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(10, 110, 411, 61))
        self.label_7 = QLabel(self.groupBox_3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(0, 30, 51, 21))
        self.label_8 = QLabel(self.groupBox_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(120, 30, 21, 21))
        self.label_9 = QLabel(self.groupBox_3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(300, 30, 51, 21))
        self.label_10 = QLabel(self.groupBox_3)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(150, 30, 51, 21))
        self.label_11 = QLabel(self.groupBox_3)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(270, 30, 21, 21))
        self.Ti = QDoubleSpinBox(self.groupBox_3)
        self.Ti.setObjectName(u"Ti")
        self.Ti.setGeometry(QRect(40, 25, 81, 26))
        self.Ti.setDecimals(1)
        self.Ti.setMaximum(9999999.000000000000000)
        self.Tf = QDoubleSpinBox(self.groupBox_3)
        self.Tf.setObjectName(u"Tf")
        self.Tf.setGeometry(QRect(180, 25, 81, 26))
        self.Tf.setDecimals(1)
        self.Tf.setMaximum(9999999.000000000000000)
        self.Tf.setValue(300.000000000000000)
        self.TBin = QDoubleSpinBox(self.groupBox_3)
        self.TBin.setObjectName(u"TBin")
        self.TBin.setGeometry(QRect(330, 25, 51, 26))
        self.TBin.setDecimals(1)
        self.TBin.setMaximum(9999999.000000000000000)
        self.TBin.setValue(1.000000000000000)
        self.groupBox_4 = QGroupBox(self.groupBox)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setGeometry(QRect(10, 179, 411, 51))
        self.rb1_NFinished = QRadioButton(self.groupBox_4)
        self.rb1_NFinished.setObjectName(u"rb1_NFinished")
        self.rb1_NFinished.setGeometry(QRect(20, 20, 112, 23))
        self.rb1_Finished = QRadioButton(self.groupBox_4)
        self.rb1_Finished.setObjectName(u"rb1_Finished")
        self.rb1_Finished.setGeometry(QRect(170, 20, 112, 23))
        self.groupBox_5 = QGroupBox(self.centralwidget)
        self.groupBox_5.setObjectName(u"groupBox_5")
        self.groupBox_5.setGeometry(QRect(10, 270, 431, 231))
        self.label_12 = QLabel(self.groupBox_5)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(10, 30, 41, 17))
        self.label_13 = QLabel(self.groupBox_5)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(130, 30, 81, 17))
        self.label_15 = QLabel(self.groupBox_5)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(210, 30, 91, 17))
        self.rb2_FCOFF = QRadioButton(self.groupBox_5)
        self.rb2_FCOFF.setObjectName(u"rb2_FCOFF")
        self.rb2_FCOFF.setGeometry(QRect(30, 50, 381, 23))
        self.rb2_FCON_11_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_11_225.setObjectName(u"rb2_FCON_11_225")
        self.rb2_FCON_11_225.setGeometry(QRect(30, 70, 381, 23))
        self.rb2_FCON_33_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_33_225.setObjectName(u"rb2_FCON_33_225")
        self.rb2_FCON_33_225.setGeometry(QRect(30, 90, 381, 23))
        self.rb2_FCON_1S_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_1S_225.setObjectName(u"rb2_FCON_1S_225")
        self.rb2_FCON_1S_225.setGeometry(QRect(30, 110, 381, 23))
        self.rb2_FCON_33_300 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_33_300.setObjectName(u"rb2_FCON_33_300")
        self.rb2_FCON_33_300.setGeometry(QRect(30, 130, 381, 23))
        self.rb2_FCON_11_300 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_11_300.setObjectName(u"rb2_FCON_11_300")
        self.rb2_FCON_11_300.setGeometry(QRect(30, 150, 381, 23))
        self.label_17 = QLabel(self.groupBox_5)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(10, 180, 51, 17))
        self.label_18 = QLabel(self.groupBox_5)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setGeometry(QRect(140, 180, 51, 17))
        self.label_19 = QLabel(self.groupBox_5)
        self.label_19.setObjectName(u"label_19")
        self.label_19.setGeometry(QRect(270, 180, 51, 17))
        self.label_20 = QLabel(self.groupBox_5)
        self.label_20.setObjectName(u"label_20")
        self.label_20.setGeometry(QRect(390, 180, 41, 17))
        self.label_21 = QLabel(self.groupBox_5)
        self.label_21.setObjectName(u"label_21")
        self.label_21.setGeometry(QRect(10, 210, 71, 17))
        self.T0shift = QSpinBox(self.groupBox_5)
        self.T0shift.setObjectName(u"T0shift")
        self.T0shift.setGeometry(QRect(70, 205, 81, 26))
        self.T0shift.setMaximum(400000)
        self.T0shift.setValue(12325)
        self.label_22 = QLabel(self.groupBox_5)
        self.label_22.setObjectName(u"label_22")
        self.label_22.setGeometry(QRect(160, 210, 51, 17))
        self.dqs1 = QDoubleSpinBox(self.groupBox_5)
        self.dqs1.setObjectName(u"dqs1")
        self.dqs1.setGeometry(QRect(50, 25, 81, 26))
        self.dqs1.setDecimals(2)
        self.dqs1.setMinimum(0.010000000000000)
        self.dqs1.setMaximum(9.990000000000000)
        self.dqs1.setSingleStep(0.010000000000000)
        self.dqs1.setValue(0.050000000000000)
        self.emin = QDoubleSpinBox(self.groupBox_5)
        self.emin.setObjectName(u"emin")
        self.emin.setGeometry(QRect(60, 175, 81, 26))
        self.emin.setDecimals(2)
        self.emin.setMinimum(-999.000000000000000)
        self.emin.setMaximum(999.000000000000000)
        self.emin.setSingleStep(1.000000000000000)
        self.emin.setValue(-4.000000000000000)
        self.emax = QDoubleSpinBox(self.groupBox_5)
        self.emax.setObjectName(u"emax")
        self.emax.setGeometry(QRect(190, 175, 81, 26))
        self.emax.setDecimals(2)
        self.emax.setMinimum(-999.000000000000000)
        self.emax.setMaximum(999.000000000000000)
        self.emax.setSingleStep(1.000000000000000)
        self.emax.setValue(4.000000000000000)
        self.de = QDoubleSpinBox(self.groupBox_5)
        self.de.setObjectName(u"de")
        self.de.setGeometry(QRect(310, 175, 81, 26))
        self.de.setDecimals(2)
        self.de.setMinimum(0.010000000000000)
        self.de.setMaximum(99.000000000000000)
        self.de.setSingleStep(0.010000000000000)
        self.de.setValue(2.000000000000000)
        self.groupBox_6 = QGroupBox(self.centralwidget)
        self.groupBox_6.setObjectName(u"groupBox_6")
        self.groupBox_6.setGeometry(QRect(10, 510, 431, 131))
        self.label_23 = QLabel(self.groupBox_6)
        self.label_23.setObjectName(u"label_23")
        self.label_23.setGeometry(QRect(10, 30, 41, 17))
        self.label_14 = QLabel(self.groupBox_6)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(130, 30, 81, 17))
        self.label_16 = QLabel(self.groupBox_6)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(210, 30, 91, 17))
        self.rb3_CHOFF = QRadioButton(self.groupBox_6)
        self.rb3_CHOFF.setObjectName(u"rb3_CHOFF")
        self.rb3_CHOFF.setGeometry(QRect(30, 50, 381, 23))
        self.rb3_CHON_11_225 = QRadioButton(self.groupBox_6)
        self.rb3_CHON_11_225.setObjectName(u"rb3_CHON_11_225")
        self.rb3_CHON_11_225.setGeometry(QRect(30, 70, 381, 23))
        self.label_24 = QLabel(self.groupBox_6)
        self.label_24.setObjectName(u"label_24")
        self.label_24.setGeometry(QRect(390, 100, 41, 20))
        self.label_25 = QLabel(self.groupBox_6)
        self.label_25.setObjectName(u"label_25")
        self.label_25.setGeometry(QRect(270, 100, 51, 17))
        self.label_26 = QLabel(self.groupBox_6)
        self.label_26.setObjectName(u"label_26")
        self.label_26.setGeometry(QRect(10, 100, 51, 17))
        self.label_27 = QLabel(self.groupBox_6)
        self.label_27.setObjectName(u"label_27")
        self.label_27.setGeometry(QRect(140, 100, 51, 17))
        self.dqs3 = QDoubleSpinBox(self.groupBox_6)
        self.dqs3.setObjectName(u"dqs3")
        self.dqs3.setGeometry(QRect(50, 25, 81, 26))
        self.dqs3.setDecimals(2)
        self.dqs3.setMinimum(0.010000000000000)
        self.dqs3.setMaximum(9.990000000000000)
        self.dqs3.setSingleStep(0.010000000000000)
        self.dqs3.setValue(0.050000000000000)
        self.emin3 = QDoubleSpinBox(self.groupBox_6)
        self.emin3.setObjectName(u"emin3")
        self.emin3.setGeometry(QRect(60, 95, 81, 26))
        self.emin3.setDecimals(0)
        self.emin3.setMinimum(-999.000000000000000)
        self.emin3.setMaximum(999.000000000000000)
        self.emin3.setSingleStep(0.010000000000000)
        self.emin3.setValue(-500.000000000000000)
        self.emax3 = QDoubleSpinBox(self.groupBox_6)
        self.emax3.setObjectName(u"emax3")
        self.emax3.setGeometry(QRect(190, 95, 81, 26))
        self.emax3.setDecimals(0)
        self.emax3.setMinimum(0.000000000000000)
        self.emax3.setMaximum(999.000000000000000)
        self.emax3.setSingleStep(4.000000000000000)
        self.emax3.setValue(100.000000000000000)
        self.de3 = QDoubleSpinBox(self.groupBox_6)
        self.de3.setObjectName(u"de3")
        self.de3.setGeometry(QRect(310, 95, 81, 26))
        self.de3.setDecimals(2)
        self.de3.setMinimum(0.010000000000000)
        self.de3.setMaximum(99.000000000000000)
        self.de3.setSingleStep(0.010000000000000)
        self.de3.setValue(50.000000000000000)
        self.button1 = QPushButton(self.centralwidget)
        self.button1.setObjectName(u"button1")
        self.button1.setGeometry(QRect(10, 650, 151, 25))
        self.button3 = QPushButton(self.centralwidget)
        self.button3.setObjectName(u"button3")
        self.button3.setGeometry(QRect(380, 650, 61, 25))
        self.background = QWidget(self.centralwidget)
        self.background.setObjectName(u"background")
        self.background.setGeometry(QRect(0, 0, 441, 691))
        self.background.setStyleSheet(u"")
        MainWindow.setCentralWidget(self.centralwidget)
        self.background.raise_()
        self.errorstr.raise_()
        self.groupBox.raise_()
        self.groupBox_5.raise_()
        self.groupBox_6.raise_()
        self.button1.raise_()
        self.button3.raise_()
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 452, 20))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.run, self.ChMenu)
        QWidget.setTabOrder(self.ChMenu, self.readtemp_bt)
        QWidget.setTabOrder(self.readtemp_bt, self.Sta)
        QWidget.setTabOrder(self.Sta, self.End)
        QWidget.setTabOrder(self.End, self.Bin)
        QWidget.setTabOrder(self.Bin, self.Ti)
        QWidget.setTabOrder(self.Ti, self.Tf)
        QWidget.setTabOrder(self.Tf, self.TBin)
        QWidget.setTabOrder(self.TBin, self.rb1_NFinished)
        QWidget.setTabOrder(self.rb1_NFinished, self.rb1_Finished)
        QWidget.setTabOrder(self.rb1_Finished, self.dqs1)
        QWidget.setTabOrder(self.dqs1, self.rb2_FCOFF)
        QWidget.setTabOrder(self.rb2_FCOFF, self.rb2_FCON_11_225)
        QWidget.setTabOrder(self.rb2_FCON_11_225, self.rb2_FCON_33_225)
        QWidget.setTabOrder(self.rb2_FCON_33_225, self.rb2_FCON_1S_225)
        QWidget.setTabOrder(self.rb2_FCON_1S_225, self.rb2_FCON_33_300)
        QWidget.setTabOrder(self.rb2_FCON_33_300, self.rb2_FCON_11_300)
        QWidget.setTabOrder(self.rb2_FCON_11_300, self.emin)
        QWidget.setTabOrder(self.emin, self.emax)
        QWidget.setTabOrder(self.emax, self.de)
        QWidget.setTabOrder(self.de, self.T0shift)
        QWidget.setTabOrder(self.T0shift, self.dqs3)
        QWidget.setTabOrder(self.dqs3, self.rb3_CHOFF)
        QWidget.setTabOrder(self.rb3_CHOFF, self.rb3_CHON_11_225)
        QWidget.setTabOrder(self.rb3_CHON_11_225, self.emin3)
        QWidget.setTabOrder(self.emin3, self.emax3)
        QWidget.setTabOrder(self.emax3, self.de3)
        QWidget.setTabOrder(self.de3, self.button1)
        QWidget.setTabOrder(self.button1, self.button3)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.errorstr.setText(QCoreApplication.translate("MainWindow", u"Elastic Scan Stand-by.", None))
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Run", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Run No.", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Temp. Sensor", None))
        self.ChMenu.setItemText(0, QCoreApplication.translate("MainWindow", u"A", None))
        self.ChMenu.setItemText(1, QCoreApplication.translate("MainWindow", u"B", None))
        self.ChMenu.setItemText(2, QCoreApplication.translate("MainWindow", u"C", None))
        self.ChMenu.setItemText(3, QCoreApplication.translate("MainWindow", u"C2", None))
        self.ChMenu.setItemText(4, QCoreApplication.translate("MainWindow", u"D", None))
        self.ChMenu.setItemText(5, QCoreApplication.translate("MainWindow", u"D2", None))
        self.ChMenu.setItemText(6, QCoreApplication.translate("MainWindow", u"D3", None))
        self.ChMenu.setItemText(7, QCoreApplication.translate("MainWindow", u"D4", None))
        self.ChMenu.setItemText(8, QCoreApplication.translate("MainWindow", u"D5", None))
        self.ChMenu.setItemText(9, QCoreApplication.translate("MainWindow", u"PEACE_U", None))
        self.ChMenu.setItemText(10, QCoreApplication.translate("MainWindow", u"PEACE_M", None))
        self.ChMenu.setItemText(11, QCoreApplication.translate("MainWindow", u"PEACE_L", None))
        self.ChMenu.setItemText(12, QCoreApplication.translate("MainWindow", u"PEACE_UB", None))
        self.ChMenu.setItemText(13, QCoreApplication.translate("MainWindow", u"PEACE_MB", None))
        self.ChMenu.setItemText(14, QCoreApplication.translate("MainWindow", u"PEACE_LB", None))

        self.readtemp_bt.setText(QCoreApplication.translate("MainWindow", u"Read Temp.", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("MainWindow", u"Time [sec]", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Start:", None))
        self.label_5.setText(QCoreApplication.translate("MainWindow", u"sec. End:", None))
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"sec. Time Bin:", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("MainWindow", u"Temperature", None))
        self.label_7.setText(QCoreApplication.translate("MainWindow", u"Start:", None))
        self.label_8.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_9.setText(QCoreApplication.translate("MainWindow", u"Bin:", None))
        self.label_10.setText(QCoreApplication.translate("MainWindow", u"End:", None))
        self.label_11.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("MainWindow", u"Reduction from event data", None))
        self.rb1_NFinished.setText(QCoreApplication.translate("MainWindow", u"Not Finished", None))
        self.rb1_Finished.setText(QCoreApplication.translate("MainWindow", u"Finished", None))
        self.groupBox_5.setTitle(QCoreApplication.translate("MainWindow", u"Si(111)", None))
        self.label_12.setText(QCoreApplication.translate("MainWindow", u"dQ = ", None))
        self.label_13.setText(QCoreApplication.translate("MainWindow", u"Qmin= 0.0", None))
        self.label_15.setText(QCoreApplication.translate("MainWindow", u"Qmax= 2.0", None))
        self.rb2_FCOFF.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper OFF (Emin=-200,Emax=50,dE=50ueV)", None))
        self.rb2_FCON_11_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,225Hz,Ereso=2.5ueV)", None))
        self.rb2_FCON_33_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (3cm,3cm,225Hz,Ereso=3.5ueV)", None))
        self.rb2_FCON_1S_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,STOP,225Hz,Ereso=3.0ueV)", None))
        self.rb2_FCON_33_300.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (3cm,3cm,300Hz,Ereso=3.1ueV)", None))
        self.rb2_FCON_11_300.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,300Hz,Ereso=2.4ueV)", None))
        self.label_17.setText(QCoreApplication.translate("MainWindow", u"Emin=", None))
        self.label_18.setText(QCoreApplication.translate("MainWindow", u", Emax=", None))
        self.label_19.setText(QCoreApplication.translate("MainWindow", u", dE=", None))
        self.label_20.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.label_21.setText(QCoreApplication.translate("MainWindow", u"T0Shift=", None))
        self.label_22.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.groupBox_6.setTitle(QCoreApplication.translate("MainWindow", u"Si(333)", None))
        self.label_23.setText(QCoreApplication.translate("MainWindow", u"dQ=", None))
        self.label_14.setText(QCoreApplication.translate("MainWindow", u"Qmin= 0.0", None))
        self.label_16.setText(QCoreApplication.translate("MainWindow", u"Qmax= 2.0", None))
        self.rb3_CHOFF.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper OFF (Emin=-500,Emax=100,dE=50ueV)", None))
        self.rb3_CHON_11_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,225Hz)", None))
        self.label_24.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.label_25.setText(QCoreApplication.translate("MainWindow", u", dE=", None))
        self.label_26.setText(QCoreApplication.translate("MainWindow", u"Emin=", None))
        self.label_27.setText(QCoreApplication.translate("MainWindow", u", Emax=", None))
        self.button1.setText(QCoreApplication.translate("MainWindow", u"Reduction Start", None))
        self.button3.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
    # retranslateUi

