#!/bin/bash
UICPROG='pyside6-uic'
OUTPUT='ui6_FastPlotQt.py'
${UICPROG} FastPlot.ui           -o ${OUTPUT}
${UICPROG} DgPlotPtn.ui          -o ui_tmp1.py
${UICPROG} DgScaleLabel.ui       -o ui_tmp2.py
${UICPROG} DgSetting.ui          -o ui_tmp3.py
${UICPROG} DgTitle.ui            -o ui_tmp4.py
${UICPROG} FloatBox.ui           -o ui_tmp5.py
${UICPROG} DgSaveAsText.ui       -o ui_tmp6.py
${UICPROG} DgFitting.ui          -o ui_tmp7.py
${UICPROG} DgGridSetting.ui      -o ui_tmp8.py
${UICPROG} DgTickStyleSetting.ui -o ui_tmp9.py
${UICPROG} DgCalcDS.ui           -o ui_tmp10.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp3.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp4.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp5.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp6.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp7.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp8.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp9.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}
cat ui_tmp10.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ${OUTPUT}

rm ui_tmp*.py

