#!/bin/bash

pyside-uic D4matSlicerQ_Main.ui -o ui_D4matSlicerQ.py
pyside-uic D4matSlicerQ_Sub.ui -o ui_tmp1.py
pyside-uic D4matSlicerQ_MakeNewD4mat.ui -o ui_tmp2.py
pyside-uic D4matSlicerQ_Rebin.ui -o ui_tmp3.py
pyside-uic DgSaveAsText.ui -o ui_tmp4.py

cat ui_tmp1.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_D4matSlicerQ.py
cat ui_tmp2.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_D4matSlicerQ.py
cat ui_tmp3.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_D4matSlicerQ.py
cat ui_tmp4.py | grep -v "coding: utf-8" | grep -v "from PySide import QtCore, QtGui" >> ui_D4matSlicerQ.py

rm ui_tmp1.py ui_tmp2.py ui_tmp3.py ui_tmp4.py
