# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'DlgSetUtsusemiEnv.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(730, 315)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayoutWidget_2 = QWidget(self.centralwidget)
        self.gridLayoutWidget_2.setObjectName(u"gridLayoutWidget_2")
        self.gridLayoutWidget_2.setGeometry(QRect(240, 10, 471, 191))
        self.gridLayout = QGridLayout(self.gridLayoutWidget_2)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.label_3 = QLabel(self.gridLayoutWidget_2)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setMaximumSize(QSize(16777215, 25))

        self.gridLayout.addWidget(self.label_3, 4, 2, 1, 1)

        self.txtUsrDir = QLineEdit(self.gridLayoutWidget_2)
        self.txtUsrDir.setObjectName(u"txtUsrDir")

        self.gridLayout.addWidget(self.txtUsrDir, 3, 2, 1, 2)

        self.btUsrDir = QPushButton(self.gridLayoutWidget_2)
        self.btUsrDir.setObjectName(u"btUsrDir")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btUsrDir.sizePolicy().hasHeightForWidth())
        self.btUsrDir.setSizePolicy(sizePolicy)

        self.gridLayout.addWidget(self.btUsrDir, 2, 3, 1, 1)

        self.label = QLabel(self.gridLayoutWidget_2)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 0, 2, 1, 1)

        self.label_2 = QLabel(self.gridLayoutWidget_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setMaximumSize(QSize(16777215, 25))

        self.gridLayout.addWidget(self.label_2, 2, 2, 1, 1)

        self.btDataDir = QPushButton(self.gridLayoutWidget_2)
        self.btDataDir.setObjectName(u"btDataDir")
        sizePolicy1 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.btDataDir.sizePolicy().hasHeightForWidth())
        self.btDataDir.setSizePolicy(sizePolicy1)
        self.btDataDir.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.btDataDir, 0, 3, 1, 1)

        self.txtDataDir = QLineEdit(self.gridLayoutWidget_2)
        self.txtDataDir.setObjectName(u"txtDataDir")

        self.gridLayout.addWidget(self.txtDataDir, 1, 2, 1, 2)

        self.txtWorkDir = QLineEdit(self.gridLayoutWidget_2)
        self.txtWorkDir.setObjectName(u"txtWorkDir")

        self.gridLayout.addWidget(self.txtWorkDir, 5, 2, 1, 2)

        self.line = QFrame(self.gridLayoutWidget_2)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.VLine)
        self.line.setFrameShadow(QFrame.Sunken)

        self.gridLayout.addWidget(self.line, 0, 1, 7, 1)

        self.btWorkDir = QPushButton(self.gridLayoutWidget_2)
        self.btWorkDir.setObjectName(u"btWorkDir")
        sizePolicy.setHeightForWidth(self.btWorkDir.sizePolicy().hasHeightForWidth())
        self.btWorkDir.setSizePolicy(sizePolicy)

        self.gridLayout.addWidget(self.btWorkDir, 4, 3, 1, 1)

        self.beamList = QListWidget(self.centralwidget)
        self.beamList.setObjectName(u"beamList")
        self.beamList.setGeometry(QRect(20, 40, 201, 101))
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.beamList.sizePolicy().hasHeightForWidth())
        self.beamList.setSizePolicy(sizePolicy2)
        self.label1 = QLabel(self.centralwidget)
        self.label1.setObjectName(u"label1")
        self.label1.setGeometry(QRect(20, 10, 91, 25))
        sizePolicy3 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.label1.sizePolicy().hasHeightForWidth())
        self.label1.setSizePolicy(sizePolicy3)
        self.label2 = QLabel(self.centralwidget)
        self.label2.setObjectName(u"label2")
        self.label2.setGeometry(QRect(60, 145, 81, 25))
        sizePolicy3.setHeightForWidth(self.label2.sizePolicy().hasHeightForWidth())
        self.label2.setSizePolicy(sizePolicy3)
        self.cbLogQuiet = QCheckBox(self.centralwidget)
        self.cbLogQuiet.setObjectName(u"cbLogQuiet")
        self.cbLogQuiet.setGeometry(QRect(160, 150, 49, 23))
        sizePolicy.setHeightForWidth(self.cbLogQuiet.sizePolicy().hasHeightForWidth())
        self.cbLogQuiet.setSizePolicy(sizePolicy)
        self.spnFontSize = QSpinBox(self.centralwidget)
        self.spnFontSize.setObjectName(u"spnFontSize")
        self.spnFontSize.setGeometry(QRect(160, 180, 61, 25))
        sizePolicy.setHeightForWidth(self.spnFontSize.sizePolicy().hasHeightForWidth())
        self.spnFontSize.setSizePolicy(sizePolicy)
        self.spnNumThread = QSpinBox(self.centralwidget)
        self.spnNumThread.setObjectName(u"spnNumThread")
        self.spnNumThread.setGeometry(QRect(160, 210, 61, 25))
        sizePolicy.setHeightForWidth(self.spnNumThread.sizePolicy().hasHeightForWidth())
        self.spnNumThread.setSizePolicy(sizePolicy)
        self.label3 = QLabel(self.centralwidget)
        self.label3.setObjectName(u"label3")
        self.label3.setGeometry(QRect(60, 180, 81, 25))
        sizePolicy3.setHeightForWidth(self.label3.sizePolicy().hasHeightForWidth())
        self.label3.setSizePolicy(sizePolicy3)
        self.label4 = QLabel(self.centralwidget)
        self.label4.setObjectName(u"label4")
        self.label4.setGeometry(QRect(10, 205, 141, 33))
        sizePolicy3.setHeightForWidth(self.label4.sizePolicy().hasHeightForWidth())
        self.label4.setSizePolicy(sizePolicy3)
        self.label5 = QLabel(self.centralwidget)
        self.label5.setObjectName(u"label5")
        self.label5.setGeometry(QRect(40, 235, 111, 25))
        sizePolicy3.setHeightForWidth(self.label5.sizePolicy().hasHeightForWidth())
        self.label5.setSizePolicy(sizePolicy3)
        self.cbDebugMode = QCheckBox(self.centralwidget)
        self.cbDebugMode.setObjectName(u"cbDebugMode")
        self.cbDebugMode.setGeometry(QRect(160, 240, 49, 23))
        sizePolicy.setHeightForWidth(self.cbDebugMode.sizePolicy().hasHeightForWidth())
        self.cbDebugMode.setSizePolicy(sizePolicy)
        self.buttonBox = QDialogButtonBox(self.centralwidget)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setGeometry(QRect(520, 240, 191, 25))
        self.buttonBox.setLocale(QLocale(QLocale.English, QLocale.UnitedStates))
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(True)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 730, 22))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"WORK_DIR", None))
        self.btUsrDir.setText(QCoreApplication.translate("MainWindow", u"Browse", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"DATA_DIR", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"USR_DIR", None))
        self.btDataDir.setText(QCoreApplication.translate("MainWindow", u"Browse", None))
        self.btWorkDir.setText(QCoreApplication.translate("MainWindow", u"Browse", None))
        self.label1.setText(QCoreApplication.translate("MainWindow", u"Instrument :", None))
        self.label2.setText(QCoreApplication.translate("MainWindow", u"Log quiet :", None))
        self.cbLogQuiet.setText(QCoreApplication.translate("MainWindow", u"Yes", None))
        self.label3.setText(QCoreApplication.translate("MainWindow", u"Font Size :", None))
        self.label4.setText(QCoreApplication.translate("MainWindow", u"Number of threads :", None))
        self.label5.setText(QCoreApplication.translate("MainWindow", u"Debug mode :", None))
        self.cbDebugMode.setText(QCoreApplication.translate("MainWindow", u"Yes", None))
    # retranslateUi

