# -*- coding: utf-8 -*-
"""

"""
import vis.DetectMap as dp
import time
#import vis.MPlot as mp
import uGao.MPlot as mp
import uGao.M2PlotPlusCui as mp2
#import vis.M2Plot as mp2
#import vis.VisualContM as vctm
import vis.VisualContQ as vctm
import Manyo

# 予約語の定義
ret = None 
vp1 = None
vp2 = None
vc = None
vc2 = None
vc3 = None
ec = None
earray = None
DAT = None
ECM = None
ECA = None
vcm = None
mpp = None 
###################################################
def ShowM2PlotPlus(dat=ECA,mpp=mpp):
        """
        Show Data of ECA with M2PlotPlus
        
        @param dat [Def:DAT]
        @param mpp [Def:mpp]
        @retval mpp
        """
        if mpp is None:
                mpp = mp2.M2PlotPlusCui(dat)
                mpp.ShowData()
        else:
                if mpp.plot.IsAliveComm():
                        mpp.ChangeData(dat)
                        mpp.ShowData()
                else:
                        del mpp
                        mpp=mp2.M2PlotPlusCui(dat)
                        mpp.ShowData()
        return mpp
# 関数名登録ディクショナリ
_functions = {"ShowM2PlotPlus":ShowM2PlotPlus,
	      }
