#! /bin/bash

#source "${HOME}/mlfscripts/setup.sh"
source ../mlf_build_env.sh

if [[ -a "${MLFDATA}/py_manyo" ]]
then
    echo "${MLFDATA}/py_manyo exists already, skipping"
    exit
else
    /bin/bash "${MLFINIT}/setup_py_manyo_server.sh"
fi

if [[ -a "${MLFDATA}/py_utsusemi" ]]
then
    echo "${MLFDATA}/py_utsusemi exists already, skipping"
    exit
else
    /bin/bash "${MLFINIT}/setup_py_utsusemi_server.sh"
fi
