# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'M2PlotPlus.ui'
#
# Created: Thu Aug  5 16:12:57 2021
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(418, 803)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/ugao_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.SumRangePanel = QtGui.QGroupBox(self.centralwidget)
        self.SumRangePanel.setEnabled(True)
        self.SumRangePanel.setGeometry(QtCore.QRect(30, 125, 381, 91))
        self.SumRangePanel.setObjectName("SumRangePanel")
        self.layoutWidget_1 = QtGui.QWidget(self.SumRangePanel)
        self.layoutWidget_1.setGeometry(QtCore.QRect(21, 56, 334, 29))
        self.layoutWidget_1.setObjectName("layoutWidget_1")
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.layoutWidget_1)
        self.horizontalLayout_2.setSizeConstraint(QtGui.QLayout.SetNoConstraint)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_5 = QtGui.QLabel(self.layoutWidget_1)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setMinimumSize(QtCore.QSize(50, 0))
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.txtR0 = QtGui.QLineEdit(self.layoutWidget_1)
        self.txtR0.setEnabled(False)
        self.txtR0.setObjectName("txtR0")
        self.horizontalLayout_2.addWidget(self.txtR0)
        self.label_4 = QtGui.QLabel(self.layoutWidget_1)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.txtR1 = QtGui.QLineEdit(self.layoutWidget_1)
        self.txtR1.setEnabled(False)
        self.txtR1.setObjectName("txtR1")
        self.horizontalLayout_2.addWidget(self.txtR1)
        self.layoutWidget_2 = QtGui.QWidget(self.SumRangePanel)
        self.layoutWidget_2.setGeometry(QtCore.QRect(21, 21, 211, 29))
        self.layoutWidget_2.setObjectName("layoutWidget_2")
        self.horizontalLayout = QtGui.QHBoxLayout(self.layoutWidget_2)
        self.horizontalLayout.setSizeConstraint(QtGui.QLayout.SetNoConstraint)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_6 = QtGui.QLabel(self.layoutWidget_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setMinimumSize(QtCore.QSize(0, 0))
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.cbR = QtGui.QComboBox(self.layoutWidget_2)
        self.cbR.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbR.sizePolicy().hasHeightForWidth())
        self.cbR.setSizePolicy(sizePolicy)
        self.cbR.setMinimumSize(QtCore.QSize(160, 0))
        self.cbR.setObjectName("cbR")
        self.horizontalLayout.addWidget(self.cbR)
        self.layoutWidget_3 = QtGui.QWidget(self.centralwidget)
        self.layoutWidget_3.setGeometry(QtCore.QRect(10, 0, 401, 118))
        self.layoutWidget_3.setObjectName("layoutWidget_3")
        self.gridLayout = QtGui.QGridLayout(self.layoutWidget_3)
        self.gridLayout.setSizeConstraint(QtGui.QLayout.SetMinimumSize)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.cbX = QtGui.QComboBox(self.layoutWidget_3)
        self.cbX.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbX.sizePolicy().hasHeightForWidth())
        self.cbX.setSizePolicy(sizePolicy)
        self.cbX.setObjectName("cbX")
        self.gridLayout.addWidget(self.cbX, 1, 2, 1, 1)
        self.txt_label_x = QtGui.QLineEdit(self.layoutWidget_3)
        self.txt_label_x.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txt_label_x.sizePolicy().hasHeightForWidth())
        self.txt_label_x.setSizePolicy(sizePolicy)
        self.txt_label_x.setObjectName("txt_label_x")
        self.gridLayout.addWidget(self.txt_label_x, 1, 3, 1, 1)
        self.label_9 = QtGui.QLabel(self.layoutWidget_3)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 0, 3, 1, 1)
        self.label = QtGui.QLabel(self.layoutWidget_3)
        self.label.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.cbY = QtGui.QComboBox(self.layoutWidget_3)
        self.cbY.setEnabled(False)
        self.cbY.setObjectName("cbY")
        self.gridLayout.addWidget(self.cbY, 2, 2, 1, 1)
        self.label_2 = QtGui.QLabel(self.layoutWidget_3)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.label_3 = QtGui.QLabel(self.layoutWidget_3)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(25, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 1, 1, 1)
        self.txt_label_y = QtGui.QLineEdit(self.layoutWidget_3)
        self.txt_label_y.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txt_label_y.sizePolicy().hasHeightForWidth())
        self.txt_label_y.setSizePolicy(sizePolicy)
        self.txt_label_y.setObjectName("txt_label_y")
        self.gridLayout.addWidget(self.txt_label_y, 2, 3, 1, 1)
        self.cbZ = QtGui.QComboBox(self.layoutWidget_3)
        self.cbZ.setEnabled(False)
        self.cbZ.setObjectName("cbZ")
        self.gridLayout.addWidget(self.cbZ, 3, 2, 1, 1)
        self.label_8 = QtGui.QLabel(self.layoutWidget_3)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 0, 2, 1, 1)
        self.txt_unit_x = QtGui.QLineEdit(self.layoutWidget_3)
        self.txt_unit_x.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txt_unit_x.sizePolicy().hasHeightForWidth())
        self.txt_unit_x.setSizePolicy(sizePolicy)
        self.txt_unit_x.setObjectName("txt_unit_x")
        self.gridLayout.addWidget(self.txt_unit_x, 1, 4, 1, 1)
        self.txt_unit_y = QtGui.QLineEdit(self.layoutWidget_3)
        self.txt_unit_y.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txt_unit_y.sizePolicy().hasHeightForWidth())
        self.txt_unit_y.setSizePolicy(sizePolicy)
        self.txt_unit_y.setObjectName("txt_unit_y")
        self.gridLayout.addWidget(self.txt_unit_y, 2, 4, 1, 1)
        self.label_10 = QtGui.QLabel(self.layoutWidget_3)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 0, 4, 1, 1)
        self.showButton = QtGui.QPushButton(self.centralwidget)
        self.showButton.setEnabled(False)
        self.showButton.setGeometry(QtCore.QRect(280, 220, 77, 30))
        self.showButton.setAutoDefault(True)
        self.showButton.setObjectName("showButton")
        self.openButton = QtGui.QPushButton(self.centralwidget)
        self.openButton.setGeometry(QtCore.QRect(10, 220, 77, 30))
        self.openButton.setAutoDefault(True)
        self.openButton.setDefault(False)
        self.openButton.setFlat(False)
        self.openButton.setObjectName("openButton")
        self.groupBox_int = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_int.setGeometry(QtCore.QRect(20, 280, 391, 91))
        self.groupBox_int.setObjectName("groupBox_int")
        self.layoutWidget_4 = QtGui.QWidget(self.groupBox_int)
        self.layoutWidget_4.setGeometry(QtCore.QRect(10, 60, 371, 29))
        self.layoutWidget_4.setObjectName("layoutWidget_4")
        self.horizontalLayout_int_att = QtGui.QHBoxLayout(self.layoutWidget_4)
        self.horizontalLayout_int_att.setSpacing(6)
        self.horizontalLayout_int_att.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_int_att.setObjectName("horizontalLayout_int_att")
        self.chk_int_log = QtGui.QCheckBox(self.layoutWidget_4)
        self.chk_int_log.setObjectName("chk_int_log")
        self.horizontalLayout_int_att.addWidget(self.chk_int_log)
        self.chk_smooth = QtGui.QCheckBox(self.layoutWidget_4)
        self.chk_smooth.setObjectName("chk_smooth")
        self.horizontalLayout_int_att.addWidget(self.chk_smooth)
        self.label_smooth_win = QtGui.QLabel(self.layoutWidget_4)
        self.label_smooth_win.setEnabled(False)
        self.label_smooth_win.setObjectName("label_smooth_win")
        self.horizontalLayout_int_att.addWidget(self.label_smooth_win)
        self.txt_smooth_win = QtGui.QLineEdit(self.layoutWidget_4)
        self.txt_smooth_win.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txt_smooth_win.sizePolicy().hasHeightForWidth())
        self.txt_smooth_win.setSizePolicy(sizePolicy)
        self.txt_smooth_win.setMinimumSize(QtCore.QSize(30, 0))
        self.txt_smooth_win.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.txt_smooth_win.setObjectName("txt_smooth_win")
        self.horizontalLayout_int_att.addWidget(self.txt_smooth_win)
        self.label_smooth_tim = QtGui.QLabel(self.layoutWidget_4)
        self.label_smooth_tim.setEnabled(False)
        self.label_smooth_tim.setObjectName("label_smooth_tim")
        self.horizontalLayout_int_att.addWidget(self.label_smooth_tim)
        self.txt_smooth_tim = QtGui.QLineEdit(self.layoutWidget_4)
        self.txt_smooth_tim.setEnabled(False)
        self.txt_smooth_tim.setMinimumSize(QtCore.QSize(30, 0))
        self.txt_smooth_tim.setObjectName("txt_smooth_tim")
        self.horizontalLayout_int_att.addWidget(self.txt_smooth_tim)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_int_att.addItem(spacerItem1)
        self.horizontalLayout_int_att.setStretch(3, 1)
        self.horizontalLayout_int_att.setStretch(5, 1)
        self.layoutWidget_5 = QtGui.QWidget(self.groupBox_int)
        self.layoutWidget_5.setGeometry(QtCore.QRect(10, 20, 371, 31))
        self.layoutWidget_5.setObjectName("layoutWidget_5")
        self.horizontalLayout_int = QtGui.QHBoxLayout(self.layoutWidget_5)
        self.horizontalLayout_int.setSizeConstraint(QtGui.QLayout.SetNoConstraint)
        self.horizontalLayout_int.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_int.setObjectName("horizontalLayout_int")
        self.chk_int_auto_range = QtGui.QCheckBox(self.layoutWidget_5)
        self.chk_int_auto_range.setChecked(True)
        self.chk_int_auto_range.setObjectName("chk_int_auto_range")
        self.horizontalLayout_int.addWidget(self.chk_int_auto_range)
        self.horizontalLayout_int_range = QtGui.QHBoxLayout()
        self.horizontalLayout_int_range.setObjectName("horizontalLayout_int_range")
        self.label_int_range = QtGui.QLabel(self.layoutWidget_5)
        self.label_int_range.setEnabled(False)
        self.label_int_range.setObjectName("label_int_range")
        self.horizontalLayout_int_range.addWidget(self.label_int_range)
        self.txt_int_min = QtGui.QLineEdit(self.layoutWidget_5)
        self.txt_int_min.setEnabled(False)
        self.txt_int_min.setObjectName("txt_int_min")
        self.horizontalLayout_int_range.addWidget(self.txt_int_min)
        self.label_int_range_h = QtGui.QLabel(self.layoutWidget_5)
        self.label_int_range_h.setEnabled(False)
        self.label_int_range_h.setObjectName("label_int_range_h")
        self.horizontalLayout_int_range.addWidget(self.label_int_range_h)
        self.txt_int_max = QtGui.QLineEdit(self.layoutWidget_5)
        self.txt_int_max.setEnabled(False)
        self.txt_int_max.setObjectName("txt_int_max")
        self.horizontalLayout_int_range.addWidget(self.txt_int_max)
        self.horizontalLayout_int.addLayout(self.horizontalLayout_int_range)
        self.line = QtGui.QFrame(self.centralwidget)
        self.line.setGeometry(QtCore.QRect(10, 245, 401, 20))
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.groupBox_xscale = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_xscale.setGeometry(QtCore.QRect(20, 370, 391, 80))
        self.groupBox_xscale.setObjectName("groupBox_xscale")
        self.layoutWidget_6 = QtGui.QWidget(self.groupBox_xscale)
        self.layoutWidget_6.setGeometry(QtCore.QRect(10, 20, 371, 32))
        self.layoutWidget_6.setObjectName("layoutWidget_6")
        self.horizontalLayout_xslice = QtGui.QHBoxLayout(self.layoutWidget_6)
        self.horizontalLayout_xslice.setSizeConstraint(QtGui.QLayout.SetNoConstraint)
        self.horizontalLayout_xslice.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_xslice.setObjectName("horizontalLayout_xslice")
        self.chk_xscale_auto_range = QtGui.QCheckBox(self.layoutWidget_6)
        self.chk_xscale_auto_range.setChecked(True)
        self.chk_xscale_auto_range.setObjectName("chk_xscale_auto_range")
        self.horizontalLayout_xslice.addWidget(self.chk_xscale_auto_range)
        self.horizontalLayout_xscale_range = QtGui.QHBoxLayout()
        self.horizontalLayout_xscale_range.setObjectName("horizontalLayout_xscale_range")
        self.label_xscale_range = QtGui.QLabel(self.layoutWidget_6)
        self.label_xscale_range.setEnabled(False)
        self.label_xscale_range.setObjectName("label_xscale_range")
        self.horizontalLayout_xscale_range.addWidget(self.label_xscale_range)
        self.txt_xscale_min = QtGui.QLineEdit(self.layoutWidget_6)
        self.txt_xscale_min.setEnabled(False)
        self.txt_xscale_min.setObjectName("txt_xscale_min")
        self.horizontalLayout_xscale_range.addWidget(self.txt_xscale_min)
        self.label_xscale_h = QtGui.QLabel(self.layoutWidget_6)
        self.label_xscale_h.setEnabled(False)
        self.label_xscale_h.setObjectName("label_xscale_h")
        self.horizontalLayout_xscale_range.addWidget(self.label_xscale_h)
        self.txt_xscale_max = QtGui.QLineEdit(self.layoutWidget_6)
        self.txt_xscale_max.setEnabled(False)
        self.txt_xscale_max.setObjectName("txt_xscale_max")
        self.horizontalLayout_xscale_range.addWidget(self.txt_xscale_max)
        self.horizontalLayout_xslice.addLayout(self.horizontalLayout_xscale_range)
        self.horizontalLayoutWidget = QtGui.QWidget(self.groupBox_xscale)
        self.horizontalLayoutWidget.setGeometry(QtCore.QRect(10, 50, 341, 26))
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.chk_xscale_log = QtGui.QCheckBox(self.horizontalLayoutWidget)
        self.chk_xscale_log.setObjectName("chk_xscale_log")
        self.horizontalLayout_3.addWidget(self.chk_xscale_log)
        spacerItem2 = QtGui.QSpacerItem(30, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.chk_xscale_inv = QtGui.QCheckBox(self.horizontalLayoutWidget)
        self.chk_xscale_inv.setObjectName("chk_xscale_inv")
        self.horizontalLayout_3.addWidget(self.chk_xscale_inv)
        self.groupBox_yscale = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_yscale.setGeometry(QtCore.QRect(20, 450, 391, 80))
        self.groupBox_yscale.setObjectName("groupBox_yscale")
        self.layoutWidget_7 = QtGui.QWidget(self.groupBox_yscale)
        self.layoutWidget_7.setGeometry(QtCore.QRect(10, 20, 371, 32))
        self.layoutWidget_7.setObjectName("layoutWidget_7")
        self.horizontalLayout_yscale = QtGui.QHBoxLayout(self.layoutWidget_7)
        self.horizontalLayout_yscale.setSizeConstraint(QtGui.QLayout.SetNoConstraint)
        self.horizontalLayout_yscale.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_yscale.setObjectName("horizontalLayout_yscale")
        self.chk_yscale_auto_range = QtGui.QCheckBox(self.layoutWidget_7)
        self.chk_yscale_auto_range.setChecked(True)
        self.chk_yscale_auto_range.setObjectName("chk_yscale_auto_range")
        self.horizontalLayout_yscale.addWidget(self.chk_yscale_auto_range)
        self.horizontalLayout_yscale_range = QtGui.QHBoxLayout()
        self.horizontalLayout_yscale_range.setObjectName("horizontalLayout_yscale_range")
        self.label_yscale_range = QtGui.QLabel(self.layoutWidget_7)
        self.label_yscale_range.setEnabled(False)
        self.label_yscale_range.setObjectName("label_yscale_range")
        self.horizontalLayout_yscale_range.addWidget(self.label_yscale_range)
        self.txt_yscale_min = QtGui.QLineEdit(self.layoutWidget_7)
        self.txt_yscale_min.setEnabled(False)
        self.txt_yscale_min.setObjectName("txt_yscale_min")
        self.horizontalLayout_yscale_range.addWidget(self.txt_yscale_min)
        self.label_yscale_h = QtGui.QLabel(self.layoutWidget_7)
        self.label_yscale_h.setEnabled(False)
        self.label_yscale_h.setObjectName("label_yscale_h")
        self.horizontalLayout_yscale_range.addWidget(self.label_yscale_h)
        self.txt_yscale_max = QtGui.QLineEdit(self.layoutWidget_7)
        self.txt_yscale_max.setEnabled(False)
        self.txt_yscale_max.setObjectName("txt_yscale_max")
        self.horizontalLayout_yscale_range.addWidget(self.txt_yscale_max)
        self.horizontalLayout_yscale.addLayout(self.horizontalLayout_yscale_range)
        self.horizontalLayoutWidget_2 = QtGui.QWidget(self.groupBox_yscale)
        self.horizontalLayoutWidget_2.setGeometry(QtCore.QRect(10, 50, 341, 26))
        self.horizontalLayoutWidget_2.setObjectName("horizontalLayoutWidget_2")
        self.horizontalLayout_4 = QtGui.QHBoxLayout(self.horizontalLayoutWidget_2)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.chk_yscale_log = QtGui.QCheckBox(self.horizontalLayoutWidget_2)
        self.chk_yscale_log.setObjectName("chk_yscale_log")
        self.horizontalLayout_4.addWidget(self.chk_yscale_log)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.chk_yscale_inv = QtGui.QCheckBox(self.horizontalLayoutWidget_2)
        self.chk_yscale_inv.setObjectName("chk_yscale_inv")
        self.horizontalLayout_4.addWidget(self.chk_yscale_inv)
        self.btIntApply = QtGui.QPushButton(self.centralwidget)
        self.btIntApply.setGeometry(QtCore.QRect(310, 260, 100, 30))
        self.btIntApply.setAutoDefault(True)
        self.btIntApply.setObjectName("btIntApply")
        self.groupBox_slice = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_slice.setGeometry(QtCore.QRect(20, 530, 391, 221))
        self.groupBox_slice.setObjectName("groupBox_slice")
        self.tabWidget_Slice = QtGui.QTabWidget(self.groupBox_slice)
        self.tabWidget_Slice.setEnabled(True)
        self.tabWidget_Slice.setGeometry(QtCore.QRect(10, 20, 376, 191))
        self.tabWidget_Slice.setObjectName("tabWidget_Slice")
        self.tab_xzslice = QtGui.QWidget()
        self.tab_xzslice.setObjectName("tab_xzslice")
        self.gridLayoutWidget = QtGui.QWidget(self.tab_xzslice)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(10, 0, 351, 124))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout_xscale = QtGui.QGridLayout(self.gridLayoutWidget)
        self.gridLayout_xscale.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_xscale.setObjectName("gridLayout_xscale")
        self.cb_xzslice_bin = QtGui.QCheckBox(self.gridLayoutWidget)
        self.cb_xzslice_bin.setObjectName("cb_xzslice_bin")
        self.gridLayout_xscale.addWidget(self.cb_xzslice_bin, 3, 0, 1, 1)
        self.rd_xzslice_range = QtGui.QRadioButton(self.gridLayoutWidget)
        self.rd_xzslice_range.setObjectName("rd_xzslice_range")
        self.gridLayout_xscale.addWidget(self.rd_xzslice_range, 2, 0, 1, 1)
        self.txt_xzslice_center = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_center.setObjectName("txt_xzslice_center")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_center, 0, 1, 1, 1)
        self.label_xzslice_h = QtGui.QLabel(self.gridLayoutWidget)
        self.label_xzslice_h.setEnabled(False)
        self.label_xzslice_h.setAlignment(QtCore.Qt.AlignCenter)
        self.label_xzslice_h.setObjectName("label_xzslice_h")
        self.gridLayout_xscale.addWidget(self.label_xzslice_h, 2, 2, 1, 1)
        self.rd_xzslice_value = QtGui.QRadioButton(self.gridLayoutWidget)
        self.rd_xzslice_value.setChecked(True)
        self.rd_xzslice_value.setObjectName("rd_xzslice_value")
        self.gridLayout_xscale.addWidget(self.rd_xzslice_value, 0, 0, 1, 1)
        self.txt_xzslice_width = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_width.setObjectName("txt_xzslice_width")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_width, 0, 3, 1, 1)
        self.txt_xzslice_end = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_end.setEnabled(False)
        self.txt_xzslice_end.setObjectName("txt_xzslice_end")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_end, 2, 3, 1, 1)
        self.txt_xzslice_delta = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_delta.setEnabled(False)
        self.txt_xzslice_delta.setObjectName("txt_xzslice_delta")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_delta, 2, 5, 1, 1)
        self.txt_xzslice_start = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_start.setEnabled(False)
        self.txt_xzslice_start.setText("")
        self.txt_xzslice_start.setObjectName("txt_xzslice_start")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_start, 2, 1, 1, 1)
        self.label_xzslice_width = QtGui.QLabel(self.gridLayoutWidget)
        self.label_xzslice_width.setAlignment(QtCore.Qt.AlignCenter)
        self.label_xzslice_width.setObjectName("label_xzslice_width")
        self.gridLayout_xscale.addWidget(self.label_xzslice_width, 2, 4, 1, 1)
        self.txt_xzslice_bin = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_bin.setEnabled(False)
        self.txt_xzslice_bin.setObjectName("txt_xzslice_bin")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_bin, 3, 1, 1, 1)
        self.label_xzslice_pm = QtGui.QLabel(self.gridLayoutWidget)
        self.label_xzslice_pm.setAlignment(QtCore.Qt.AlignCenter)
        self.label_xzslice_pm.setObjectName("label_xzslice_pm")
        self.gridLayout_xscale.addWidget(self.label_xzslice_pm, 0, 2, 1, 1)
        self.txt_xzslice_center_to = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_center_to.setObjectName("txt_xzslice_center_to")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_center_to, 1, 3, 1, 1)
        self.txt_xzslice_center_step = QtGui.QLineEdit(self.gridLayoutWidget)
        self.txt_xzslice_center_step.setObjectName("txt_xzslice_center_step")
        self.gridLayout_xscale.addWidget(self.txt_xzslice_center_step, 1, 5, 1, 1)
        self.label_11 = QtGui.QLabel(self.gridLayoutWidget)
        self.label_11.setObjectName("label_11")
        self.gridLayout_xscale.addWidget(self.label_11, 1, 2, 1, 1)
        self.label_12 = QtGui.QLabel(self.gridLayoutWidget)
        self.label_12.setObjectName("label_12")
        self.gridLayout_xscale.addWidget(self.label_12, 1, 4, 1, 1)
        self.btXZSlice1d = QtGui.QPushButton(self.tab_xzslice)
        self.btXZSlice1d.setGeometry(QtCore.QRect(80, 130, 77, 21))
        self.btXZSlice1d.setAutoDefault(True)
        self.btXZSlice1d.setObjectName("btXZSlice1d")
        self.btXZSlice2d = QtGui.QPushButton(self.tab_xzslice)
        self.btXZSlice2d.setGeometry(QtCore.QRect(260, 130, 77, 21))
        self.btXZSlice2d.setAutoDefault(True)
        self.btXZSlice2d.setObjectName("btXZSlice2d")
        self.btXZSlice1dclear = QtGui.QPushButton(self.tab_xzslice)
        self.btXZSlice1dclear.setGeometry(QtCore.QRect(160, 130, 61, 21))
        self.btXZSlice1dclear.setAutoDefault(True)
        self.btXZSlice1dclear.setObjectName("btXZSlice1dclear")
        self.tabWidget_Slice.addTab(self.tab_xzslice, "")
        self.tab_yzslice = QtGui.QWidget()
        self.tab_yzslice.setObjectName("tab_yzslice")
        self.gridLayoutWidget_2 = QtGui.QWidget(self.tab_yzslice)
        self.gridLayoutWidget_2.setGeometry(QtCore.QRect(10, 0, 351, 124))
        self.gridLayoutWidget_2.setObjectName("gridLayoutWidget_2")
        self.gridLayout_yzslice = QtGui.QGridLayout(self.gridLayoutWidget_2)
        self.gridLayout_yzslice.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_yzslice.setObjectName("gridLayout_yzslice")
        self.txt_yzslice_center = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_center.setObjectName("txt_yzslice_center")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_center, 0, 1, 1, 1)
        self.rd_yzslice_range = QtGui.QRadioButton(self.gridLayoutWidget_2)
        self.rd_yzslice_range.setObjectName("rd_yzslice_range")
        self.gridLayout_yzslice.addWidget(self.rd_yzslice_range, 2, 0, 1, 1)
        self.rd_yzslice_value = QtGui.QRadioButton(self.gridLayoutWidget_2)
        self.rd_yzslice_value.setChecked(True)
        self.rd_yzslice_value.setObjectName("rd_yzslice_value")
        self.gridLayout_yzslice.addWidget(self.rd_yzslice_value, 0, 0, 1, 1)
        self.txt_yzslice_end = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_end.setEnabled(False)
        self.txt_yzslice_end.setObjectName("txt_yzslice_end")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_end, 2, 3, 1, 1)
        self.label_yzslice_h = QtGui.QLabel(self.gridLayoutWidget_2)
        self.label_yzslice_h.setEnabled(False)
        self.label_yzslice_h.setAlignment(QtCore.Qt.AlignCenter)
        self.label_yzslice_h.setObjectName("label_yzslice_h")
        self.gridLayout_yzslice.addWidget(self.label_yzslice_h, 2, 2, 1, 1)
        self.txt_yzslice_width = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_width.setObjectName("txt_yzslice_width")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_width, 0, 3, 1, 1)
        self.txt_yzslice_start = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_start.setEnabled(False)
        self.txt_yzslice_start.setText("")
        self.txt_yzslice_start.setObjectName("txt_yzslice_start")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_start, 2, 1, 1, 1)
        self.txt_yzslice_bin = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_bin.setEnabled(False)
        self.txt_yzslice_bin.setObjectName("txt_yzslice_bin")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_bin, 3, 1, 1, 1)
        self.label_xzslice_width_2 = QtGui.QLabel(self.gridLayoutWidget_2)
        self.label_xzslice_width_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_xzslice_width_2.setObjectName("label_xzslice_width_2")
        self.gridLayout_yzslice.addWidget(self.label_xzslice_width_2, 2, 4, 1, 1)
        self.txt_yzslice_delta = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_delta.setEnabled(False)
        self.txt_yzslice_delta.setObjectName("txt_yzslice_delta")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_delta, 2, 5, 1, 1)
        self.cb_yzslice_bin = QtGui.QCheckBox(self.gridLayoutWidget_2)
        self.cb_yzslice_bin.setObjectName("cb_yzslice_bin")
        self.gridLayout_yzslice.addWidget(self.cb_yzslice_bin, 3, 0, 1, 1)
        self.label_yzslice_pm = QtGui.QLabel(self.gridLayoutWidget_2)
        self.label_yzslice_pm.setAlignment(QtCore.Qt.AlignCenter)
        self.label_yzslice_pm.setObjectName("label_yzslice_pm")
        self.gridLayout_yzslice.addWidget(self.label_yzslice_pm, 0, 2, 1, 1)
        self.label_13 = QtGui.QLabel(self.gridLayoutWidget_2)
        self.label_13.setObjectName("label_13")
        self.gridLayout_yzslice.addWidget(self.label_13, 1, 2, 1, 1)
        self.txt_yzslice_center_to = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_center_to.setObjectName("txt_yzslice_center_to")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_center_to, 1, 3, 1, 1)
        self.label_14 = QtGui.QLabel(self.gridLayoutWidget_2)
        self.label_14.setObjectName("label_14")
        self.gridLayout_yzslice.addWidget(self.label_14, 1, 4, 1, 1)
        self.txt_yzslice_center_step = QtGui.QLineEdit(self.gridLayoutWidget_2)
        self.txt_yzslice_center_step.setObjectName("txt_yzslice_center_step")
        self.gridLayout_yzslice.addWidget(self.txt_yzslice_center_step, 1, 5, 1, 1)
        self.btYZSlice1d = QtGui.QPushButton(self.tab_yzslice)
        self.btYZSlice1d.setGeometry(QtCore.QRect(80, 130, 77, 21))
        self.btYZSlice1d.setAutoDefault(True)
        self.btYZSlice1d.setObjectName("btYZSlice1d")
        self.btYZSlice2d = QtGui.QPushButton(self.tab_yzslice)
        self.btYZSlice2d.setGeometry(QtCore.QRect(260, 130, 77, 21))
        self.btYZSlice2d.setAutoDefault(True)
        self.btYZSlice2d.setObjectName("btYZSlice2d")
        self.btYZSlice1dclear = QtGui.QPushButton(self.tab_yzslice)
        self.btYZSlice1dclear.setGeometry(QtCore.QRect(160, 130, 61, 21))
        self.btYZSlice1dclear.setAutoDefault(True)
        self.btYZSlice1dclear.setObjectName("btYZSlice1dclear")
        self.tabWidget_Slice.addTab(self.tab_yzslice, "")
        self.tab_diagonal = QtGui.QWidget()
        self.tab_diagonal.setObjectName("tab_diagonal")
        self.gridLayoutWidget_3 = QtGui.QWidget(self.tab_diagonal)
        self.gridLayoutWidget_3.setGeometry(QtCore.QRect(0, 0, 171, 71))
        self.gridLayoutWidget_3.setObjectName("gridLayoutWidget_3")
        self.gridLayout_yzslice_3 = QtGui.QGridLayout(self.gridLayoutWidget_3)
        self.gridLayout_yzslice_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_yzslice_3.setObjectName("gridLayout_yzslice_3")
        self.label_diagonal_sp = QtGui.QLabel(self.gridLayoutWidget_3)
        self.label_diagonal_sp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_diagonal_sp.setObjectName("label_diagonal_sp")
        self.gridLayout_yzslice_3.addWidget(self.label_diagonal_sp, 0, 0, 1, 1)
        self.label_diagonal_ep = QtGui.QLabel(self.gridLayoutWidget_3)
        self.label_diagonal_ep.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_diagonal_ep.setObjectName("label_diagonal_ep")
        self.gridLayout_yzslice_3.addWidget(self.label_diagonal_ep, 1, 0, 1, 1)
        self.txt_diagonal_ep_y = QtGui.QLineEdit(self.gridLayoutWidget_3)
        self.txt_diagonal_ep_y.setEnabled(True)
        self.txt_diagonal_ep_y.setObjectName("txt_diagonal_ep_y")
        self.gridLayout_yzslice_3.addWidget(self.txt_diagonal_ep_y, 1, 2, 1, 1)
        self.txt_diagonal_sp_x = QtGui.QLineEdit(self.gridLayoutWidget_3)
        self.txt_diagonal_sp_x.setObjectName("txt_diagonal_sp_x")
        self.gridLayout_yzslice_3.addWidget(self.txt_diagonal_sp_x, 0, 1, 1, 1)
        self.txt_diagonal_ep_x = QtGui.QLineEdit(self.gridLayoutWidget_3)
        self.txt_diagonal_ep_x.setEnabled(True)
        self.txt_diagonal_ep_x.setText("")
        self.txt_diagonal_ep_x.setObjectName("txt_diagonal_ep_x")
        self.gridLayout_yzslice_3.addWidget(self.txt_diagonal_ep_x, 1, 1, 1, 1)
        self.txt_diagonal_sp_y = QtGui.QLineEdit(self.gridLayoutWidget_3)
        self.txt_diagonal_sp_y.setObjectName("txt_diagonal_sp_y")
        self.gridLayout_yzslice_3.addWidget(self.txt_diagonal_sp_y, 0, 2, 1, 1)
        self.btDiagonal1d = QtGui.QPushButton(self.tab_diagonal)
        self.btDiagonal1d.setGeometry(QtCore.QRect(310, 10, 61, 21))
        self.btDiagonal1d.setAutoDefault(True)
        self.btDiagonal1d.setObjectName("btDiagonal1d")
        self.btDiagonal2d = QtGui.QPushButton(self.tab_diagonal)
        self.btDiagonal2d.setEnabled(False)
        self.btDiagonal2d.setGeometry(QtCore.QRect(310, 40, 61, 21))
        self.btDiagonal2d.setAutoDefault(True)
        self.btDiagonal2d.setObjectName("btDiagonal2d")
        self.gridLayoutWidget_4 = QtGui.QWidget(self.tab_diagonal)
        self.gridLayoutWidget_4.setGeometry(QtCore.QRect(180, 0, 122, 71))
        self.gridLayoutWidget_4.setObjectName("gridLayoutWidget_4")
        self.gridLayout_yzslice_4 = QtGui.QGridLayout(self.gridLayoutWidget_4)
        self.gridLayout_yzslice_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_yzslice_4.setObjectName("gridLayout_yzslice_4")
        self.label_diagonal_width = QtGui.QLabel(self.gridLayoutWidget_4)
        self.label_diagonal_width.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_diagonal_width.setObjectName("label_diagonal_width")
        self.gridLayout_yzslice_4.addWidget(self.label_diagonal_width, 0, 0, 1, 1)
        self.txt_diagonal_binning = QtGui.QLineEdit(self.gridLayoutWidget_4)
        self.txt_diagonal_binning.setEnabled(True)
        self.txt_diagonal_binning.setText("")
        self.txt_diagonal_binning.setObjectName("txt_diagonal_binning")
        self.gridLayout_yzslice_4.addWidget(self.txt_diagonal_binning, 1, 1, 1, 1)
        self.label_diagonal_binning = QtGui.QLabel(self.gridLayoutWidget_4)
        self.label_diagonal_binning.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_diagonal_binning.setObjectName("label_diagonal_binning")
        self.gridLayout_yzslice_4.addWidget(self.label_diagonal_binning, 1, 0, 1, 1)
        self.txt_diagonal_width = QtGui.QLineEdit(self.gridLayoutWidget_4)
        self.txt_diagonal_width.setObjectName("txt_diagonal_width")
        self.gridLayout_yzslice_4.addWidget(self.txt_diagonal_width, 0, 1, 1, 1)
        self.tabWidget_Slice.addTab(self.tab_diagonal, "")
        self.tab = QtGui.QWidget()
        self.tab.setObjectName("tab")
        self.layoutWidget_8 = QtGui.QWidget(self.tab)
        self.layoutWidget_8.setGeometry(QtCore.QRect(0, 10, 361, 26))
        self.layoutWidget_8.setObjectName("layoutWidget_8")
        self.layout_sliceMethod = QtGui.QHBoxLayout(self.layoutWidget_8)
        self.layout_sliceMethod.setContentsMargins(0, 0, 0, 0)
        self.layout_sliceMethod.setObjectName("layout_sliceMethod")
        self.label_7 = QtGui.QLabel(self.layoutWidget_8)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.layout_sliceMethod.addWidget(self.label_7)
        self.rb_isAveSlice = QtGui.QRadioButton(self.layoutWidget_8)
        self.rb_isAveSlice.setChecked(True)
        self.rb_isAveSlice.setObjectName("rb_isAveSlice")
        self.layout_sliceMethod.addWidget(self.rb_isAveSlice)
        self.rb_isSumSlice = QtGui.QRadioButton(self.layoutWidget_8)
        self.rb_isSumSlice.setObjectName("rb_isSumSlice")
        self.layout_sliceMethod.addWidget(self.rb_isSumSlice)
        self.tabWidget_Slice.addTab(self.tab, "")
        self.btFindPlotter = QtGui.QPushButton(self.centralwidget)
        self.btFindPlotter.setGeometry(QtCore.QRect(370, 220, 30, 30))
        self.btFindPlotter.setText("")
        self.btFindPlotter.setIconSize(QtCore.QSize(32, 32))
        self.btFindPlotter.setAutoDefault(True)
        self.btFindPlotter.setObjectName("btFindPlotter")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 418, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuEdit = QtGui.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuRegion_type = QtGui.QMenu(self.menuEdit)
        self.menuRegion_type.setObjectName("menuRegion_type")
        self.menuFunction = QtGui.QMenu(self.menubar)
        self.menuFunction.setObjectName("menuFunction")
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setStyleSheet("background-color: rgb(255, 255, 255);")
        self.statusbar.setSizeGripEnabled(False)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionECM = QtGui.QAction(MainWindow)
        self.actionECM.setObjectName("actionECM")
        self.actionTest = QtGui.QAction(MainWindow)
        self.actionTest.setObjectName("actionTest")
        self.actionShow = QtGui.QAction(MainWindow)
        self.actionShow.setEnabled(False)
        self.actionShow.setObjectName("actionShow")
        self.actionECA = QtGui.QAction(MainWindow)
        self.actionECA.setObjectName("actionECA")
        self.actionSlice = QtGui.QAction(MainWindow)
        self.actionSlice.setObjectName("actionSlice")
        self.actionRectangle = QtGui.QAction(MainWindow)
        self.actionRectangle.setObjectName("actionRectangle")
        self.actionCircle = QtGui.QAction(MainWindow)
        self.actionCircle.setObjectName("actionCircle")
        self.actionConfirm_region = QtGui.QAction(MainWindow)
        self.actionConfirm_region.setObjectName("actionConfirm_region")
        self.actionAuto_confirm = QtGui.QAction(MainWindow)
        self.actionAuto_confirm.setCheckable(True)
        self.actionAuto_confirm.setChecked(True)
        self.actionAuto_confirm.setObjectName("actionAuto_confirm")
        self.actionECA_mdb = QtGui.QAction(MainWindow)
        self.actionECA_mdb.setObjectName("actionECA_mdb")
        self.actionECM_mdb = QtGui.QAction(MainWindow)
        self.actionECM_mdb.setObjectName("actionECM_mdb")
        self.actionOpen = QtGui.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionOpen_data = QtGui.QAction(MainWindow)
        self.actionOpen_data.setObjectName("actionOpen_data")
        self.actionAverage = QtGui.QAction(MainWindow)
        self.actionAverage.setEnabled(False)
        self.actionAverage.setObjectName("actionAverage")
        self.actionY_axis_2D = QtGui.QAction(MainWindow)
        self.actionY_axis_2D.setEnabled(False)
        self.actionY_axis_2D.setObjectName("actionY_axis_2D")
        self.actionX_axis_2D = QtGui.QAction(MainWindow)
        self.actionX_axis_2D.setEnabled(False)
        self.actionX_axis_2D.setObjectName("actionX_axis_2D")
        self.actionPoint = QtGui.QAction(MainWindow)
        self.actionPoint.setObjectName("actionPoint")
        self.actionExit = QtGui.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionAbout = QtGui.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionChangeLogLevel = QtGui.QAction(MainWindow)
        self.actionChangeLogLevel.setObjectName("actionChangeLogLevel")
        self.actionSave_as_Text = QtGui.QAction(MainWindow)
        self.actionSave_as_Text.setObjectName("actionSave_as_Text")
        self.menuFile.addAction(self.actionOpen_data)
        self.menuFile.addAction(self.actionSave_as_Text)
        self.menuFile.addAction(self.actionExit)
        self.menuRegion_type.addAction(self.actionRectangle)
        self.menuRegion_type.addAction(self.actionCircle)
        self.menuRegion_type.addAction(self.actionPoint)
        self.menuEdit.addAction(self.actionConfirm_region)
        self.menuEdit.addAction(self.menuRegion_type.menuAction())
        self.menuEdit.addAction(self.actionAuto_confirm)
        self.menuFunction.addAction(self.actionShow)
        self.menuFunction.addAction(self.actionTest)
        self.menuHelp.addAction(self.actionChangeLogLevel)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuFunction.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget_Slice.setCurrentIndex(0)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center.setDisabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_width.setDisabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_start.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_end.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_width.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_start.setDisabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_end.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_center.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_width.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_start.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_end.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_start.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_end.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_center.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_width.setDisabled)
        QtCore.QObject.connect(self.chk_smooth, QtCore.SIGNAL("toggled(bool)"), self.label_smooth_win.setEnabled)
        QtCore.QObject.connect(self.chk_smooth, QtCore.SIGNAL("toggled(bool)"), self.label_smooth_tim.setEnabled)
        QtCore.QObject.connect(self.chk_smooth, QtCore.SIGNAL("toggled(bool)"), self.txt_smooth_win.setEnabled)
        QtCore.QObject.connect(self.chk_smooth, QtCore.SIGNAL("toggled(bool)"), self.txt_smooth_tim.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_delta.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_delta.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_delta.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_delta.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center_to.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center_step.setEnabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center_to.setDisabled)
        QtCore.QObject.connect(self.rd_xzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center_step.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_xzslice_center_to.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_value, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_center_step.setEnabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_center_to.setDisabled)
        QtCore.QObject.connect(self.rd_yzslice_range, QtCore.SIGNAL("toggled(bool)"), self.txt_yzslice_center_step.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.openButton, self.cbX)
        MainWindow.setTabOrder(self.cbX, self.txt_label_x)
        MainWindow.setTabOrder(self.txt_label_x, self.txt_unit_x)
        MainWindow.setTabOrder(self.txt_unit_x, self.cbY)
        MainWindow.setTabOrder(self.cbY, self.txt_label_y)
        MainWindow.setTabOrder(self.txt_label_y, self.txt_unit_y)
        MainWindow.setTabOrder(self.txt_unit_y, self.cbZ)
        MainWindow.setTabOrder(self.cbZ, self.cbR)
        MainWindow.setTabOrder(self.cbR, self.txtR0)
        MainWindow.setTabOrder(self.txtR0, self.txtR1)
        MainWindow.setTabOrder(self.txtR1, self.showButton)
        MainWindow.setTabOrder(self.showButton, self.btFindPlotter)
        MainWindow.setTabOrder(self.btFindPlotter, self.chk_int_auto_range)
        MainWindow.setTabOrder(self.chk_int_auto_range, self.txt_int_min)
        MainWindow.setTabOrder(self.txt_int_min, self.txt_int_max)
        MainWindow.setTabOrder(self.txt_int_max, self.chk_int_log)
        MainWindow.setTabOrder(self.chk_int_log, self.chk_smooth)
        MainWindow.setTabOrder(self.chk_smooth, self.txt_smooth_win)
        MainWindow.setTabOrder(self.txt_smooth_win, self.txt_smooth_tim)
        MainWindow.setTabOrder(self.txt_smooth_tim, self.chk_xscale_auto_range)
        MainWindow.setTabOrder(self.chk_xscale_auto_range, self.txt_xscale_min)
        MainWindow.setTabOrder(self.txt_xscale_min, self.txt_xscale_max)
        MainWindow.setTabOrder(self.txt_xscale_max, self.chk_xscale_log)
        MainWindow.setTabOrder(self.chk_xscale_log, self.chk_xscale_inv)
        MainWindow.setTabOrder(self.chk_xscale_inv, self.chk_yscale_auto_range)
        MainWindow.setTabOrder(self.chk_yscale_auto_range, self.txt_yscale_min)
        MainWindow.setTabOrder(self.txt_yscale_min, self.txt_yscale_max)
        MainWindow.setTabOrder(self.txt_yscale_max, self.chk_yscale_log)
        MainWindow.setTabOrder(self.chk_yscale_log, self.chk_yscale_inv)
        MainWindow.setTabOrder(self.chk_yscale_inv, self.btIntApply)
        MainWindow.setTabOrder(self.btIntApply, self.tabWidget_Slice)
        MainWindow.setTabOrder(self.tabWidget_Slice, self.rd_xzslice_value)
        MainWindow.setTabOrder(self.rd_xzslice_value, self.txt_xzslice_center)
        MainWindow.setTabOrder(self.txt_xzslice_center, self.txt_xzslice_width)
        MainWindow.setTabOrder(self.txt_xzslice_width, self.txt_xzslice_center_to)
        MainWindow.setTabOrder(self.txt_xzslice_center_to, self.txt_xzslice_center_step)
        MainWindow.setTabOrder(self.txt_xzslice_center_step, self.rd_xzslice_range)
        MainWindow.setTabOrder(self.rd_xzslice_range, self.txt_xzslice_start)
        MainWindow.setTabOrder(self.txt_xzslice_start, self.txt_xzslice_end)
        MainWindow.setTabOrder(self.txt_xzslice_end, self.txt_xzslice_delta)
        MainWindow.setTabOrder(self.txt_xzslice_delta, self.cb_xzslice_bin)
        MainWindow.setTabOrder(self.cb_xzslice_bin, self.txt_xzslice_bin)
        MainWindow.setTabOrder(self.txt_xzslice_bin, self.btXZSlice1d)
        MainWindow.setTabOrder(self.btXZSlice1d, self.btXZSlice1dclear)
        MainWindow.setTabOrder(self.btXZSlice1dclear, self.btXZSlice2d)
        MainWindow.setTabOrder(self.btXZSlice2d, self.rd_yzslice_value)
        MainWindow.setTabOrder(self.rd_yzslice_value, self.txt_yzslice_center)
        MainWindow.setTabOrder(self.txt_yzslice_center, self.txt_yzslice_width)
        MainWindow.setTabOrder(self.txt_yzslice_width, self.txt_yzslice_center_to)
        MainWindow.setTabOrder(self.txt_yzslice_center_to, self.txt_yzslice_center_step)
        MainWindow.setTabOrder(self.txt_yzslice_center_step, self.rd_yzslice_range)
        MainWindow.setTabOrder(self.rd_yzslice_range, self.txt_yzslice_start)
        MainWindow.setTabOrder(self.txt_yzslice_start, self.txt_yzslice_end)
        MainWindow.setTabOrder(self.txt_yzslice_end, self.txt_yzslice_delta)
        MainWindow.setTabOrder(self.txt_yzslice_delta, self.cb_yzslice_bin)
        MainWindow.setTabOrder(self.cb_yzslice_bin, self.txt_yzslice_bin)
        MainWindow.setTabOrder(self.txt_yzslice_bin, self.btYZSlice1d)
        MainWindow.setTabOrder(self.btYZSlice1d, self.btYZSlice1dclear)
        MainWindow.setTabOrder(self.btYZSlice1dclear, self.btYZSlice2d)
        MainWindow.setTabOrder(self.btYZSlice2d, self.txt_diagonal_sp_x)
        MainWindow.setTabOrder(self.txt_diagonal_sp_x, self.txt_diagonal_sp_y)
        MainWindow.setTabOrder(self.txt_diagonal_sp_y, self.txt_diagonal_width)
        MainWindow.setTabOrder(self.txt_diagonal_width, self.btDiagonal1d)
        MainWindow.setTabOrder(self.btDiagonal1d, self.txt_diagonal_ep_x)
        MainWindow.setTabOrder(self.txt_diagonal_ep_x, self.txt_diagonal_ep_y)
        MainWindow.setTabOrder(self.txt_diagonal_ep_y, self.txt_diagonal_binning)
        MainWindow.setTabOrder(self.txt_diagonal_binning, self.btDiagonal2d)
        MainWindow.setTabOrder(self.btDiagonal2d, self.rb_isAveSlice)
        MainWindow.setTabOrder(self.rb_isAveSlice, self.rb_isSumSlice)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "M2Plot+", None, QtGui.QApplication.UnicodeUTF8))
        self.SumRangePanel.setTitle(QtGui.QApplication.translate("MainWindow", "Summation range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("MainWindow", "Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("MainWindow", "Key", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("MainWindow", "Label", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("MainWindow", "X axis", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("MainWindow", "Y axis", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("MainWindow", "Intensity", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("MainWindow", "Key", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("MainWindow", "Unit", None, QtGui.QApplication.UnicodeUTF8))
        self.showButton.setText(QtGui.QApplication.translate("MainWindow", "Show", None, QtGui.QApplication.UnicodeUTF8))
        self.openButton.setText(QtGui.QApplication.translate("MainWindow", "Open", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_int.setTitle(QtGui.QApplication.translate("MainWindow", "Inteinsity", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_int_log.setText(QtGui.QApplication.translate("MainWindow", "Log scale", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_smooth.setText(QtGui.QApplication.translate("MainWindow", "Smooth", None, QtGui.QApplication.UnicodeUTF8))
        self.label_smooth_win.setText(QtGui.QApplication.translate("MainWindow", "window", None, QtGui.QApplication.UnicodeUTF8))
        self.txt_smooth_win.setText(QtGui.QApplication.translate("MainWindow", "1", None, QtGui.QApplication.UnicodeUTF8))
        self.label_smooth_tim.setText(QtGui.QApplication.translate("MainWindow", "times", None, QtGui.QApplication.UnicodeUTF8))
        self.txt_smooth_tim.setText(QtGui.QApplication.translate("MainWindow", "1", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_int_auto_range.setText(QtGui.QApplication.translate("MainWindow", "Auto Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.label_int_range.setText(QtGui.QApplication.translate("MainWindow", "Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_int_range_h.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_xscale.setTitle(QtGui.QApplication.translate("MainWindow", "X Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_xscale_auto_range.setText(QtGui.QApplication.translate("MainWindow", "Auto Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xscale_range.setText(QtGui.QApplication.translate("MainWindow", "Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xscale_h.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_xscale_log.setText(QtGui.QApplication.translate("MainWindow", "Log Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_xscale_inv.setText(QtGui.QApplication.translate("MainWindow", "Inversion", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_yscale.setTitle(QtGui.QApplication.translate("MainWindow", "Y Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_yscale_auto_range.setText(QtGui.QApplication.translate("MainWindow", "Auto Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.label_yscale_range.setText(QtGui.QApplication.translate("MainWindow", "Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_yscale_h.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_yscale_log.setText(QtGui.QApplication.translate("MainWindow", "Log Scale", None, QtGui.QApplication.UnicodeUTF8))
        self.chk_yscale_inv.setText(QtGui.QApplication.translate("MainWindow", "Inversion", None, QtGui.QApplication.UnicodeUTF8))
        self.btIntApply.setText(QtGui.QApplication.translate("MainWindow", "Apply", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_slice.setTitle(QtGui.QApplication.translate("MainWindow", "Slice", None, QtGui.QApplication.UnicodeUTF8))
        self.cb_xzslice_bin.setText(QtGui.QApplication.translate("MainWindow", "rebin", None, QtGui.QApplication.UnicodeUTF8))
        self.rd_xzslice_range.setText(QtGui.QApplication.translate("MainWindow", "Y range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xzslice_h.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.rd_xzslice_value.setText(QtGui.QApplication.translate("MainWindow", "Y value", None, QtGui.QApplication.UnicodeUTF8))
        self.txt_xzslice_delta.setText(QtGui.QApplication.translate("MainWindow", "0.0", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xzslice_width.setText(QtGui.QApplication.translate("MainWindow", "delta", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xzslice_pm.setText(QtGui.QApplication.translate("MainWindow", "+-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_11.setText(QtGui.QApplication.translate("MainWindow", "To:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_12.setText(QtGui.QApplication.translate("MainWindow", "Step:", None, QtGui.QApplication.UnicodeUTF8))
        self.btXZSlice1d.setText(QtGui.QApplication.translate("MainWindow", "1D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.btXZSlice2d.setText(QtGui.QApplication.translate("MainWindow", "2D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.btXZSlice1dclear.setText(QtGui.QApplication.translate("MainWindow", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget_Slice.setTabText(self.tabWidget_Slice.indexOf(self.tab_xzslice), QtGui.QApplication.translate("MainWindow", "Along X axis", None, QtGui.QApplication.UnicodeUTF8))
        self.rd_yzslice_range.setText(QtGui.QApplication.translate("MainWindow", "X range", None, QtGui.QApplication.UnicodeUTF8))
        self.rd_yzslice_value.setText(QtGui.QApplication.translate("MainWindow", "X value", None, QtGui.QApplication.UnicodeUTF8))
        self.label_yzslice_h.setText(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_xzslice_width_2.setText(QtGui.QApplication.translate("MainWindow", "delta", None, QtGui.QApplication.UnicodeUTF8))
        self.txt_yzslice_delta.setText(QtGui.QApplication.translate("MainWindow", "0.0", None, QtGui.QApplication.UnicodeUTF8))
        self.cb_yzslice_bin.setText(QtGui.QApplication.translate("MainWindow", "rebin", None, QtGui.QApplication.UnicodeUTF8))
        self.label_yzslice_pm.setText(QtGui.QApplication.translate("MainWindow", "+-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_13.setText(QtGui.QApplication.translate("MainWindow", "To:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_14.setText(QtGui.QApplication.translate("MainWindow", "Step:", None, QtGui.QApplication.UnicodeUTF8))
        self.btYZSlice1d.setText(QtGui.QApplication.translate("MainWindow", "1D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.btYZSlice2d.setText(QtGui.QApplication.translate("MainWindow", "2D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.btYZSlice1dclear.setText(QtGui.QApplication.translate("MainWindow", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget_Slice.setTabText(self.tabWidget_Slice.indexOf(self.tab_yzslice), QtGui.QApplication.translate("MainWindow", "Along Y axis", None, QtGui.QApplication.UnicodeUTF8))
        self.label_diagonal_sp.setText(QtGui.QApplication.translate("MainWindow", "Start:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_diagonal_ep.setText(QtGui.QApplication.translate("MainWindow", "End:", None, QtGui.QApplication.UnicodeUTF8))
        self.btDiagonal1d.setText(QtGui.QApplication.translate("MainWindow", "1D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.btDiagonal2d.setText(QtGui.QApplication.translate("MainWindow", "2D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.label_diagonal_width.setText(QtGui.QApplication.translate("MainWindow", "width:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_diagonal_binning.setText(QtGui.QApplication.translate("MainWindow", "binning:", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget_Slice.setTabText(self.tabWidget_Slice.indexOf(self.tab_diagonal), QtGui.QApplication.translate("MainWindow", "Diagonal", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("MainWindow", "Integration :", None, QtGui.QApplication.UnicodeUTF8))
        self.rb_isAveSlice.setText(QtGui.QApplication.translate("MainWindow", "Average", None, QtGui.QApplication.UnicodeUTF8))
        self.rb_isSumSlice.setText(QtGui.QApplication.translate("MainWindow", "Summation", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget_Slice.setTabText(self.tabWidget_Slice.indexOf(self.tab), QtGui.QApplication.translate("MainWindow", "Settings", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.menuEdit.setTitle(QtGui.QApplication.translate("MainWindow", "Edit", None, QtGui.QApplication.UnicodeUTF8))
        self.menuRegion_type.setTitle(QtGui.QApplication.translate("MainWindow", "Region type", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFunction.setTitle(QtGui.QApplication.translate("MainWindow", "Function", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "Help", None, QtGui.QApplication.UnicodeUTF8))
        self.actionECM.setText(QtGui.QApplication.translate("MainWindow", "ECM", None, QtGui.QApplication.UnicodeUTF8))
        self.actionTest.setText(QtGui.QApplication.translate("MainWindow", "Test", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow.setText(QtGui.QApplication.translate("MainWindow", "Show", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+S", None, QtGui.QApplication.UnicodeUTF8))
        self.actionECA.setText(QtGui.QApplication.translate("MainWindow", "ECA", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSlice.setText(QtGui.QApplication.translate("MainWindow", "Slice", None, QtGui.QApplication.UnicodeUTF8))
        self.actionRectangle.setText(QtGui.QApplication.translate("MainWindow", "Rectangle", None, QtGui.QApplication.UnicodeUTF8))
        self.actionRectangle.setProperty("regName", QtGui.QApplication.translate("MainWindow", "Rectangle", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCircle.setText(QtGui.QApplication.translate("MainWindow", "Circle", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCircle.setProperty("regName", QtGui.QApplication.translate("MainWindow", "Circle", None, QtGui.QApplication.UnicodeUTF8))
        self.actionConfirm_region.setText(QtGui.QApplication.translate("MainWindow", "Confirm region", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAuto_confirm.setText(QtGui.QApplication.translate("MainWindow", "Auto confirm", None, QtGui.QApplication.UnicodeUTF8))
        self.actionECA_mdb.setText(QtGui.QApplication.translate("MainWindow", "ECA Manyo Data", None, QtGui.QApplication.UnicodeUTF8))
        self.actionECM_mdb.setText(QtGui.QApplication.translate("MainWindow", "ECM Manyo Data", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen.setText(QtGui.QApplication.translate("MainWindow", "Open data", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_data.setText(QtGui.QApplication.translate("MainWindow", "Open", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_data.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+O", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAverage.setText(QtGui.QApplication.translate("MainWindow", "Average", None, QtGui.QApplication.UnicodeUTF8))
        self.actionY_axis_2D.setText(QtGui.QApplication.translate("MainWindow", "Y axis 2D", None, QtGui.QApplication.UnicodeUTF8))
        self.actionX_axis_2D.setText(QtGui.QApplication.translate("MainWindow", "X axis 2D", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPoint.setText(QtGui.QApplication.translate("MainWindow", "Point", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPoint.setProperty("regName", QtGui.QApplication.translate("MainWindow", "Point", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setText(QtGui.QApplication.translate("MainWindow", "Exit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+X", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbout.setText(QtGui.QApplication.translate("MainWindow", "About M2PlotPlus", None, QtGui.QApplication.UnicodeUTF8))
        self.actionChangeLogLevel.setText(QtGui.QApplication.translate("MainWindow", "Change the log level", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as_Text.setText(QtGui.QApplication.translate("MainWindow", "Save as Text...", None, QtGui.QApplication.UnicodeUTF8))

try:
    import M2PlotPlus_rc
except:
    import uGao.M2PlotPlus_rc

# Form implementation generated from reading ui file 'dg_save_as_text.ui'
#
# Created: Thu Aug  5 16:12:57 2021
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_DlgSaveAsText(object):
    def setupUi(self, DlgSaveAsText):
        DlgSaveAsText.setObjectName("DlgSaveAsText")
        DlgSaveAsText.resize(400, 205)
        self.ckIgnoreMask = QtGui.QCheckBox(DlgSaveAsText)
        self.ckIgnoreMask.setGeometry(QtCore.QRect(30, 10, 201, 22))
        self.ckIgnoreMask.setObjectName("ckIgnoreMask")
        self.label = QtGui.QLabel(DlgSaveAsText)
        self.label.setGeometry(QtCore.QRect(30, 40, 181, 17))
        self.label.setObjectName("label")
        self.chMaskValue = QtGui.QComboBox(DlgSaveAsText)
        self.chMaskValue.setGeometry(QtCore.QRect(70, 70, 151, 27))
        self.chMaskValue.setObjectName("chMaskValue")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.chMaskValue.addItem("")
        self.txMaskValue = QtGui.QLineEdit(DlgSaveAsText)
        self.txMaskValue.setGeometry(QtCore.QRect(240, 70, 113, 27))
        self.txMaskValue.setObjectName("txMaskValue")
        self.txQbin = QtGui.QLineEdit(DlgSaveAsText)
        self.txQbin.setEnabled(False)
        self.txQbin.setGeometry(QtCore.QRect(240, 110, 113, 27))
        self.txQbin.setObjectName("txQbin")
        self.stQbin = QtGui.QLabel(DlgSaveAsText)
        self.stQbin.setGeometry(QtCore.QRect(160, 110, 81, 17))
        self.stQbin.setObjectName("stQbin")
        self.line = QtGui.QFrame(DlgSaveAsText)
        self.line.setGeometry(QtCore.QRect(20, 150, 361, 20))
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.btSave = QtGui.QPushButton(DlgSaveAsText)
        self.btSave.setGeometry(QtCore.QRect(50, 170, 99, 27))
        self.btSave.setObjectName("btSave")
        self.btCancel = QtGui.QPushButton(DlgSaveAsText)
        self.btCancel.setGeometry(QtCore.QRect(260, 170, 99, 27))
        self.btCancel.setObjectName("btCancel")

        self.retranslateUi(DlgSaveAsText)
        QtCore.QMetaObject.connectSlotsByName(DlgSaveAsText)

    def retranslateUi(self, DlgSaveAsText):
        DlgSaveAsText.setWindowTitle(QtGui.QApplication.translate("DlgSaveAsText", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.ckIgnoreMask.setText(QtGui.QApplication.translate("DlgSaveAsText", "Ignore Masked Points", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("DlgSaveAsText", "Replace Mask Value with", None, QtGui.QApplication.UnicodeUTF8))
        self.chMaskValue.setItemText(0, QtGui.QApplication.translate("DlgSaveAsText", "Value in right box", None, QtGui.QApplication.UnicodeUTF8))
        self.chMaskValue.setItemText(1, QtGui.QApplication.translate("DlgSaveAsText", "0", None, QtGui.QApplication.UnicodeUTF8))
        self.chMaskValue.setItemText(2, QtGui.QApplication.translate("DlgSaveAsText", "-1", None, QtGui.QApplication.UnicodeUTF8))
        self.chMaskValue.setItemText(3, QtGui.QApplication.translate("DlgSaveAsText", "Original Value", None, QtGui.QApplication.UnicodeUTF8))
        self.stQbin.setText(QtGui.QApplication.translate("DlgSaveAsText", "Q binning", None, QtGui.QApplication.UnicodeUTF8))
        self.btSave.setText(QtGui.QApplication.translate("DlgSaveAsText", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.btCancel.setText(QtGui.QApplication.translate("DlgSaveAsText", "Cancel", None, QtGui.QApplication.UnicodeUTF8))

