#include "AdvMultiDataLevmarConsts.hh"

/** class name */
const std::string AdvMultiDataLevmarConsts::className=std::string("AdvMultiDataLevmarConsts");

/** key for partial R factor */
const std::string AdvMultiDataLevmarConsts::PARTIAL_R_FACTOR=std::string("partial R factor");
/** key for partial residual error norm */
const std::string AdvMultiDataLevmarConsts::PARTIAL_RESIDUAL_ERR_NORM=std::string("partial residual error norm");
