#include "UtsusemiEventDataMonitorRPMT.hh"
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorRPMT::
UtsusemiEventDataMonitorRPMT(){
    _MessageTag2 = "UtsusemiEventDataMonitorRPMT::";
    isReadout = true;
}
//////////////////////////////////////////////////////////
UtsusemiEventDataMonitorRPMT::
~UtsusemiEventDataMonitorRPMT(){
}
