#include "UtsusemiDetectorInfoEditorNeunet.hh"
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorNeunet::UtsusemiDetectorInfoEditorNeunet(){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorNeunet::UtsusemiDetectorInfoEditorNeunet(std::string dfile, bool workAsReader):UtsusemiDetectorInfoEditorTemplate(dfile, workAsReader){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiDetectorInfoEditorNeunet::~UtsusemiDetectorInfoEditorNeunet(){
}
//////////////////////////////////////////////////////////
void UtsusemiDetectorInfoEditorNeunet::Initialize(){
    _MessageTag = "UtsusemiDetectorInfoEditorNeunet::";

}
