# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'DetectMapQ.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QGroupBox, QLabel, QLineEdit, QListWidget,
    QListWidgetItem, QMainWindow, QMenu, QMenuBar,
    QPushButton, QSizePolicy, QSlider, QStatusBar,
    QWidget)

class Ui_mapframe(object):
    def setupUi(self, mapframe):
        if not mapframe.objectName():
            mapframe.setObjectName(u"mapframe")
        mapframe.resize(985, 778)
        self.actionOpen = QAction(mapframe)
        self.actionOpen.setObjectName(u"actionOpen")
        self.actionSave = QAction(mapframe)
        self.actionSave.setObjectName(u"actionSave")
        self.actionSave_as = QAction(mapframe)
        self.actionSave_as.setObjectName(u"actionSave_as")
        self.actionPrint = QAction(mapframe)
        self.actionPrint.setObjectName(u"actionPrint")
        self.actionExit = QAction(mapframe)
        self.actionExit.setObjectName(u"actionExit")
        self.actionNew_Plotter = QAction(mapframe)
        self.actionNew_Plotter.setObjectName(u"actionNew_Plotter")
        self.actionLog_Scale = QAction(mapframe)
        self.actionLog_Scale.setObjectName(u"actionLog_Scale")
        self.actionLog_Scale.setCheckable(True)
        self.centralwidget = QWidget(mapframe)
        self.centralwidget.setObjectName(u"centralwidget")
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 10, 301, 51))
        self.tof1 = QLineEdit(self.groupBox)
        self.tof1.setObjectName(u"tof1")
        self.tof1.setGeometry(QRect(10, 20, 80, 25))
        self.tof2 = QLineEdit(self.groupBox)
        self.tof2.setObjectName(u"tof2")
        self.tof2.setGeometry(QRect(110, 20, 80, 25))
        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(100, 20, 16, 25))
        self.btRedraw = QPushButton(self.groupBox)
        self.btRedraw.setObjectName(u"btRedraw")
        self.btRedraw.setGeometry(QRect(200, 20, 80, 30))
        self.groupBox_2 = QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(10, 80, 301, 51))
        self.psd0 = QLineEdit(self.groupBox_2)
        self.psd0.setObjectName(u"psd0")
        self.psd0.setGeometry(QRect(80, 20, 80, 25))
        self.bt2dPlot = QPushButton(self.groupBox_2)
        self.bt2dPlot.setObjectName(u"bt2dPlot")
        self.bt2dPlot.setGeometry(QRect(170, 20, 100, 30))
        self.label_2 = QLabel(self.groupBox_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(20, 20, 66, 17))
        self.groupBox_3 = QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(320, 10, 211, 121))
        self.label_3 = QLabel(self.groupBox_3)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(70, 30, 66, 17))
        self.label_4 = QLabel(self.groupBox_3)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(160, 30, 41, 17))
        self.label_5 = QLabel(self.groupBox_3)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(20, 50, 41, 25))
        self.label_6 = QLabel(self.groupBox_3)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(20, 90, 41, 25))
        self.PSD1 = QLineEdit(self.groupBox_3)
        self.PSD1.setObjectName(u"PSD1")
        self.PSD1.setGeometry(QRect(70, 50, 50, 25))
        self.PSD2 = QLineEdit(self.groupBox_3)
        self.PSD2.setObjectName(u"PSD2")
        self.PSD2.setGeometry(QRect(70, 90, 50, 25))
        self.pixel1 = QLineEdit(self.groupBox_3)
        self.pixel1.setObjectName(u"pixel1")
        self.pixel1.setGeometry(QRect(150, 50, 50, 25))
        self.pixel2 = QLineEdit(self.groupBox_3)
        self.pixel2.setObjectName(u"pixel2")
        self.pixel2.setGeometry(QRect(150, 90, 50, 25))
        self.label_7 = QLabel(self.groupBox_3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(130, 50, 16, 25))
        self.label_8 = QLabel(self.groupBox_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(130, 90, 16, 25))
        self.cbXaxisOfPlot = QComboBox(self.centralwidget)
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.setObjectName(u"cbXaxisOfPlot")
        self.cbXaxisOfPlot.setGeometry(QRect(560, 10, 100, 27))
        self.btAppend = QPushButton(self.centralwidget)
        self.btAppend.setObjectName(u"btAppend")
        self.btAppend.setGeometry(QRect(560, 40, 150, 27))
        self.btSumm = QPushButton(self.centralwidget)
        self.btSumm.setObjectName(u"btSumm")
        self.btSumm.setGeometry(QRect(560, 70, 150, 27))
        self.btNewPlot = QPushButton(self.centralwidget)
        self.btNewPlot.setObjectName(u"btNewPlot")
        self.btNewPlot.setGeometry(QRect(560, 100, 150, 27))
        self.label_9 = QLabel(self.centralwidget)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(720, 10, 66, 17))
        self.ckconv = QCheckBox(self.centralwidget)
        self.ckconv.setObjectName(u"ckconv")
        self.ckconv.setGeometry(QRect(810, 10, 151, 22))
        self.plotList = QListWidget(self.centralwidget)
        self.plotList.setObjectName(u"plotList")
        self.plotList.setGeometry(QRect(720, 30, 200, 115))
        self.slider = QSlider(self.centralwidget)
        self.slider.setObjectName(u"slider")
        self.slider.setGeometry(QRect(0, 230, 30, 350))
        self.slider.setValue(99)
        self.slider.setOrientation(Qt.Vertical)
        self.slider.setInvertedAppearance(False)
        self.mapPanel = QWidget(self.centralwidget)
        self.mapPanel.setObjectName(u"mapPanel")
        self.mapPanel.setGeometry(QRect(30, 160, 950, 500))
        self.groupBox_4 = QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setGeometry(QRect(30, 670, 401, 51))
        self.ymin = QLineEdit(self.groupBox_4)
        self.ymin.setObjectName(u"ymin")
        self.ymin.setGeometry(QRect(20, 20, 100, 25))
        self.ymax = QLineEdit(self.groupBox_4)
        self.ymax.setObjectName(u"ymax")
        self.ymax.setGeometry(QRect(130, 20, 100, 25))
        self.label_10 = QLabel(self.groupBox_4)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(120, 20, 16, 25))
        self.btIntSet = QPushButton(self.groupBox_4)
        self.btIntSet.setObjectName(u"btIntSet")
        self.btIntSet.setGeometry(QRect(240, 20, 80, 30))
        self.btIntClear = QPushButton(self.groupBox_4)
        self.btIntClear.setObjectName(u"btIntClear")
        self.btIntClear.setGeometry(QRect(340, 20, 50, 30))
        self.infotxt = QLabel(self.centralwidget)
        self.infotxt.setObjectName(u"infotxt")
        self.infotxt.setGeometry(QRect(450, 670, 500, 55))
        self.infotxt.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.line = QFrame(self.centralwidget)
        self.line.setObjectName(u"line")
        self.line.setGeometry(QRect(0, 140, 981, 16))
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        mapframe.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(mapframe)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 985, 28))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        self.menuDisplay = QMenu(self.menubar)
        self.menuDisplay.setObjectName(u"menuDisplay")
        mapframe.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(mapframe)
        self.statusbar.setObjectName(u"statusbar")
        mapframe.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuDisplay.menuAction())
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPrint)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menuDisplay.addAction(self.actionNew_Plotter)
        self.menuDisplay.addAction(self.actionLog_Scale)

        self.retranslateUi(mapframe)

        QMetaObject.connectSlotsByName(mapframe)
    # setupUi

    def retranslateUi(self, mapframe):
        mapframe.setWindowTitle(QCoreApplication.translate("mapframe", u"Detect Map", None))
        self.actionOpen.setText(QCoreApplication.translate("mapframe", u"Open ...", None))
        self.actionSave.setText(QCoreApplication.translate("mapframe", u"Save", None))
        self.actionSave_as.setText(QCoreApplication.translate("mapframe", u"Save as ...", None))
        self.actionPrint.setText(QCoreApplication.translate("mapframe", u"Print ...", None))
        self.actionExit.setText(QCoreApplication.translate("mapframe", u"Exit", None))
        self.actionNew_Plotter.setText(QCoreApplication.translate("mapframe", u"New Plotter ...", None))
        self.actionLog_Scale.setText(QCoreApplication.translate("mapframe", u"Log Scale", None))
        self.groupBox.setTitle(QCoreApplication.translate("mapframe", u"X-axis Range of Pixel", None))
        self.label.setText(QCoreApplication.translate("mapframe", u"-", None))
        self.btRedraw.setText(QCoreApplication.translate("mapframe", u"Redraw", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("mapframe", u"2D Plot of Detector", None))
        self.bt2dPlot.setText(QCoreApplication.translate("mapframe", u"2D Plot", None))
        self.label_2.setText(QCoreApplication.translate("mapframe", u"DetNo. :", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("mapframe", u"Detector-Pixel Range", None))
        self.label_3.setText(QCoreApplication.translate("mapframe", u"Detector", None))
        self.label_4.setText(QCoreApplication.translate("mapframe", u"Pixel", None))
        self.label_5.setText(QCoreApplication.translate("mapframe", u"From", None))
        self.label_6.setText(QCoreApplication.translate("mapframe", u"To", None))
        self.label_7.setText(QCoreApplication.translate("mapframe", u"-", None))
        self.label_8.setText(QCoreApplication.translate("mapframe", u"-", None))
        self.cbXaxisOfPlot.setItemText(0, QCoreApplication.translate("mapframe", u"TOF", None))
        self.cbXaxisOfPlot.setItemText(1, QCoreApplication.translate("mapframe", u"Detector", None))
        self.cbXaxisOfPlot.setItemText(2, QCoreApplication.translate("mapframe", u"Pixel", None))

        self.btAppend.setText(QCoreApplication.translate("mapframe", u"Append >>", None))
        self.btSumm.setText(QCoreApplication.translate("mapframe", u"Append Sum >>", None))
        self.btNewPlot.setText(QCoreApplication.translate("mapframe", u"New Plotter", None))
        self.label_9.setText(QCoreApplication.translate("mapframe", u"Plotters", None))
        self.ckconv.setText(QCoreApplication.translate("mapframe", u"Convert toEnergy", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("mapframe", u"Intensity Range", None))
        self.label_10.setText(QCoreApplication.translate("mapframe", u"-", None))
        self.btIntSet.setText(QCoreApplication.translate("mapframe", u"Apply", None))
        self.btIntClear.setText(QCoreApplication.translate("mapframe", u"Clear", None))
        self.infotxt.setText(QCoreApplication.translate("mapframe", u"Det-ID", None))
        self.menuFile.setTitle(QCoreApplication.translate("mapframe", u"File", None))
        self.menuDisplay.setTitle(QCoreApplication.translate("mapframe", u"Display", None))
    # retranslateUi

