# -*- coding: utf-8 -*-

import sys
import string
import math

import unittest


class TestManyoFunctions( unittest.TestCase ):

    def test_HistOperations( self ):

        import HistOperations
        ECA = HistOperations.Result
        HistOperations.ElementContainerArrayToText( ECA, "HistOperations.txt" )

        self.assertTrue( HistOperations.ModTest.CompareTextFile(
            "test20160425_r268.txt", "HistOperations.txt" ) )

    def test_HistOperationsHeavy( self ):

        import HistOperationsHeavy
        ECA = HistOperationsHeavy.eca
        HistOperationsHeavy.HistOperations.ElementContainerArrayToText(
            ECA, "HistOperationsHeavy.txt" )

        self.assertTrue( HistOperationsHeavy.HistOperations.ModTest.CompareTextFile(
            "test20160425heavy_r268.txt", "HistOperationsHeavy.txt" ) )


# unittestの継承クラス すべてを実行する
if __name__ == '__main__':
    unittest.main()



