# -*- coding: utf-8 -*-

import sys
import string
import math

import Manyo.core
import ModuleTest as ModTest

ec0 = ModTest.ec0

Mout = Manyo.WriteSerializationFileBinary( "test.bin" )
Mout.Save( ec0 )
del( Mout )

Min = Manyo.ReadSerializationFileBinary( "test.bin" )
e1r = Manyo.ElementContainer()
Min.Load( e1r )
del( Min )


#module.PrintElementContainer( e1r )
