# -*- coding: utf-8 -*-

import sys
import string
import math

FileName = sys.argv[1]
# 引数として 年月日を8桁で入力のこと

import moduleAveraging
moduleAveraging.ModTest.ElementContainerToText(
    moduleAveraging.e2, "Averaging2_" + FileName + ".txt" )
moduleAveraging.ModTest.ElementContainerToText(
    moduleAveraging.e3, "Averaging3_" + FileName + ".txt" )

import moduleBinning
moduleBinning.ModTest.ElementContainerToText(
    moduleBinning.e2, "Binning2_" + FileName + ".txt" )
moduleBinning.ModTest.ElementContainerToText(
    moduleBinning.e3, "Binning3_" + FileName + ".txt" )


import moduleBoostSerial
moduleBoostSerial.ModTest.ElementContainerToText(
    moduleBoostSerial.e1r, "BoostSerial_" + FileName + ".txt" )

import moduleNeXus
moduleNeXus.ModTest.ElementContainerToText(
    moduleNeXus.e2, "NeXus_" + FileName + ".txt" )


import moduleArithmeticOperations

moduleArithmeticOperations.ModTest.ElementContainerToText(
    moduleArithmeticOperations.eca, "ArithAdd_" + FileName + ".txt" )

moduleArithmeticOperations.ModTest.ElementContainerToText(
    moduleArithmeticOperations.ecs, "ArithSub_" + FileName + ".txt" )

moduleArithmeticOperations.ModTest.ElementContainerToText(
    moduleArithmeticOperations.ecm, "ArithMul_" + FileName + ".txt" )

moduleArithmeticOperations.ModTest.ElementContainerToText(
    moduleArithmeticOperations.ecd, "ArithDiv_" + FileName + ".txt" )

moduleArithmeticOperations.ModTest.ElementContainerToText(
    moduleArithmeticOperations.e2, "Arith2_" + FileName + ".txt" )


