# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'SliceViewerMain.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(800, 600)
        self.data_load_view_action = QAction(MainWindow)
        self.data_load_view_action.setObjectName(u"data_load_view_action")
        self.data_load_view_action.setCheckable(False)
        self.data_load_view_action.setChecked(False)
        icon = QIcon()
        icon.addFile(u"icons/SliceViewerMain/OpenFile.png", QSize(), QIcon.Normal, QIcon.Off)
        self.data_load_view_action.setIcon(icon)
        self.data_load_view_action.setIconVisibleInMenu(True)
        self.basic_slice_view_action = QAction(MainWindow)
        self.basic_slice_view_action.setObjectName(u"basic_slice_view_action")
        self.basic_slice_view_action.setCheckable(True)
        self.basic_slice_view_action.setChecked(True)
        icon1 = QIcon()
        icon1.addFile(u"icons/SliceViewerMain/3d_effect.png", QSize(), QIcon.Normal, QIcon.Off)
        self.basic_slice_view_action.setIcon(icon1)
        self.basic_slice_slider_view_action = QAction(MainWindow)
        self.basic_slice_slider_view_action.setObjectName(u"basic_slice_slider_view_action")
        self.basic_slice_slider_view_action.setCheckable(True)
        self.basic_slice_slider_view_action.setChecked(False)
        icon2 = QIcon()
        icon2.addFile(u"icons/SliceViewerMain/hslider.png", QSize(), QIcon.Normal, QIcon.Off)
        self.basic_slice_slider_view_action.setIcon(icon2)
        self.free_slice_view_action = QAction(MainWindow)
        self.free_slice_view_action.setObjectName(u"free_slice_view_action")
        self.free_slice_view_action.setCheckable(True)
        self.free_slice_view_action.setChecked(False)
        icon3 = QIcon()
        icon3.addFile(u"icons/SliceViewerMain/3d_free.png", QSize(), QIcon.Normal, QIcon.Off)
        self.free_slice_view_action.setIcon(icon3)
        self.view_setting_view_action = QAction(MainWindow)
        self.view_setting_view_action.setObjectName(u"view_setting_view_action")
        self.view_setting_view_action.setCheckable(True)
        self.view_setting_view_action.setChecked(False)
        icon4 = QIcon()
        icon4.addFile(u"icons/SliceViewerMain/samsung__04.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_setting_view_action.setIcon(icon4)
        self.color_bar_view_action = QAction(MainWindow)
        self.color_bar_view_action.setObjectName(u"color_bar_view_action")
        self.color_bar_view_action.setCheckable(True)
        self.color_bar_view_action.setChecked(True)
        icon5 = QIcon()
        icon5.addFile(u"icons/SliceViewerMain/color-table.png", QSize(), QIcon.Normal, QIcon.Off)
        self.color_bar_view_action.setIcon(icon5)
        self.color_bar_setting_view_action = QAction(MainWindow)
        self.color_bar_setting_view_action.setObjectName(u"color_bar_setting_view_action")
        self.color_bar_setting_view_action.setCheckable(True)
        self.color_bar_setting_view_action.setChecked(False)
        icon6 = QIcon()
        icon6.addFile(u"icons/SliceViewerMain/color.png", QSize(), QIcon.Normal, QIcon.Off)
        self.color_bar_setting_view_action.setIcon(icon6)
        self.label_setting_view_action = QAction(MainWindow)
        self.label_setting_view_action.setObjectName(u"label_setting_view_action")
        self.label_setting_view_action.setCheckable(True)
        self.label_setting_view_action.setChecked(False)
        icon7 = QIcon()
        icon7.addFile(u"icons/SliceViewerMain/str.png", QSize(), QIcon.Normal, QIcon.Off)
        self.label_setting_view_action.setIcon(icon7)
        self.zoom_best_fit_action = QAction(MainWindow)
        self.zoom_best_fit_action.setObjectName(u"zoom_best_fit_action")
        self.zoom_best_fit_action.setCheckable(False)
        self.zoom_best_fit_action.setChecked(False)
        icon8 = QIcon()
        icon8.addFile(u"icons/SliceViewerMain/zoom-best-fit.png", QSize(), QIcon.Normal, QIcon.Off)
        self.zoom_best_fit_action.setIcon(icon8)
        self.zoom_best_fit_action.setIconVisibleInMenu(True)
        self.view_direction_to_plus_x_action = QAction(MainWindow)
        self.view_direction_to_plus_x_action.setObjectName(u"view_direction_to_plus_x_action")
        self.view_direction_to_plus_x_action.setChecked(False)
        icon9 = QIcon()
        icon9.addFile(u"icons/SliceViewerMain/view_nyz.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_plus_x_action.setIcon(icon9)
        self.view_direction_to_plus_x_action.setIconVisibleInMenu(True)
        self.view_direction_to_minus_x_action = QAction(MainWindow)
        self.view_direction_to_minus_x_action.setObjectName(u"view_direction_to_minus_x_action")
        self.view_direction_to_minus_x_action.setChecked(False)
        icon10 = QIcon()
        icon10.addFile(u"icons/SliceViewerMain/view_yz.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_minus_x_action.setIcon(icon10)
        self.view_direction_to_minus_x_action.setIconVisibleInMenu(True)
        self.view_direction_to_plus_y_action = QAction(MainWindow)
        self.view_direction_to_plus_y_action.setObjectName(u"view_direction_to_plus_y_action")
        self.view_direction_to_plus_y_action.setChecked(False)
        icon11 = QIcon()
        icon11.addFile(u"icons/SliceViewerMain/view_xz.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_plus_y_action.setIcon(icon11)
        self.view_direction_to_plus_y_action.setIconVisibleInMenu(True)
        self.view_direction_to_minus_y_action = QAction(MainWindow)
        self.view_direction_to_minus_y_action.setObjectName(u"view_direction_to_minus_y_action")
        self.view_direction_to_minus_y_action.setChecked(False)
        icon12 = QIcon()
        icon12.addFile(u"icons/SliceViewerMain/view_nxz.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_minus_y_action.setIcon(icon12)
        self.view_direction_to_minus_y_action.setIconVisibleInMenu(True)
        self.view_direction_to_plus_z_action = QAction(MainWindow)
        self.view_direction_to_plus_z_action.setObjectName(u"view_direction_to_plus_z_action")
        self.view_direction_to_plus_z_action.setChecked(False)
        icon13 = QIcon()
        icon13.addFile(u"icons/SliceViewerMain/view_nxy.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_plus_z_action.setIcon(icon13)
        self.view_direction_to_plus_z_action.setIconVisibleInMenu(True)
        self.view_direction_to_minus_z_action = QAction(MainWindow)
        self.view_direction_to_minus_z_action.setObjectName(u"view_direction_to_minus_z_action")
        self.view_direction_to_minus_z_action.setChecked(False)
        icon14 = QIcon()
        icon14.addFile(u"icons/SliceViewerMain/view_xy.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_direction_to_minus_z_action.setIcon(icon14)
        self.view_direction_to_minus_z_action.setIconVisibleInMenu(True)
        self.data_info_view_action = QAction(MainWindow)
        self.data_info_view_action.setObjectName(u"data_info_view_action")
        self.data_info_view_action.setCheckable(True)
        self.data_info_view_action.setChecked(False)
        icon15 = QIcon()
        icon15.addFile(u"icons/SliceViewerMain/Information.png", QSize(), QIcon.Normal, QIcon.Off)
        self.data_info_view_action.setIcon(icon15)
        self.data_info_view_action.setIconVisibleInMenu(True)
        self.axis_grid_view_action = QAction(MainWindow)
        self.axis_grid_view_action.setObjectName(u"axis_grid_view_action")
        self.axis_grid_view_action.setCheckable(True)
        self.axis_grid_view_action.setChecked(False)
        icon16 = QIcon()
        icon16.addFile(u"icons/SliceViewerMain/axis.png", QSize(), QIcon.Normal, QIcon.Off)
        self.axis_grid_view_action.setIcon(icon16)
        self.axis_grid_view_action.setIconVisibleInMenu(True)
        self.increase_vertical_size_action = QAction(MainWindow)
        self.increase_vertical_size_action.setObjectName(u"increase_vertical_size_action")
        self.increase_vertical_size_action.setCheckable(False)
        self.increase_vertical_size_action.setChecked(False)
        icon17 = QIcon()
        icon17.addFile(u"icons/SliceViewerMain/IncVertSize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.increase_vertical_size_action.setIcon(icon17)
        self.increase_vertical_size_action.setIconVisibleInMenu(True)
        self.decrease_vertical_size_action = QAction(MainWindow)
        self.decrease_vertical_size_action.setObjectName(u"decrease_vertical_size_action")
        self.decrease_vertical_size_action.setCheckable(False)
        self.decrease_vertical_size_action.setChecked(False)
        icon18 = QIcon()
        icon18.addFile(u"icons/SliceViewerMain/DecVertSize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.decrease_vertical_size_action.setIcon(icon18)
        self.decrease_vertical_size_action.setIconVisibleInMenu(True)
        self.increase_horizontal_size_action = QAction(MainWindow)
        self.increase_horizontal_size_action.setObjectName(u"increase_horizontal_size_action")
        self.increase_horizontal_size_action.setCheckable(False)
        self.increase_horizontal_size_action.setChecked(False)
        icon19 = QIcon()
        icon19.addFile(u"icons/SliceViewerMain/IncHoriSize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.increase_horizontal_size_action.setIcon(icon19)
        self.increase_horizontal_size_action.setIconVisibleInMenu(True)
        self.decrease_horizontal_size_action = QAction(MainWindow)
        self.decrease_horizontal_size_action.setObjectName(u"decrease_horizontal_size_action")
        self.decrease_horizontal_size_action.setCheckable(False)
        self.decrease_horizontal_size_action.setChecked(False)
        icon20 = QIcon()
        icon20.addFile(u"icons/SliceViewerMain/DecHoriSize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.decrease_horizontal_size_action.setIcon(icon20)
        self.decrease_horizontal_size_action.setIconVisibleInMenu(True)
        self.view_screenshot_action = QAction(MainWindow)
        self.view_screenshot_action.setObjectName(u"view_screenshot_action")
        self.view_screenshot_action.setChecked(False)
        icon21 = QIcon()
        icon21.addFile(u"icons/SliceViewerMain/Camera.png", QSize(), QIcon.Normal, QIcon.Off)
        self.view_screenshot_action.setIcon(icon21)
        self.translucent_action = QAction(MainWindow)
        self.translucent_action.setObjectName(u"translucent_action")
        self.translucent_action.setCheckable(True)
        self.translucent_action.setChecked(True)
        icon22 = QIcon()
        icon22.addFile(u"icons/SliceViewerMain/Transparency.png", QSize(), QIcon.Normal, QIcon.Off)
        self.translucent_action.setIcon(icon22)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.topLayout = QHBoxLayout()
        self.topLayout.setObjectName(u"topLayout")
        self.parameterLayout = QVBoxLayout()
        self.parameterLayout.setObjectName(u"parameterLayout")

        self.topLayout.addLayout(self.parameterLayout)

        self.mainViewLayout = QVBoxLayout()
        self.mainViewLayout.setObjectName(u"mainViewLayout")

        self.topLayout.addLayout(self.mainViewLayout)

        self.colorBarLayout = QHBoxLayout()
        self.colorBarLayout.setObjectName(u"colorBarLayout")

        self.topLayout.addLayout(self.colorBarLayout)


        self.gridLayout.addLayout(self.topLayout, 0, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.menuBar = QMenuBar(MainWindow)
        self.menuBar.setObjectName(u"menuBar")
        self.menuBar.setGeometry(QRect(0, 0, 800, 20))
        self.menu = QMenu(self.menuBar)
        self.menu.setObjectName(u"menu")
        self.menu_2 = QMenu(self.menuBar)
        self.menu_2.setObjectName(u"menu_2")
        MainWindow.setMenuBar(self.menuBar)
        self.toolBar = QToolBar(MainWindow)
        self.toolBar.setObjectName(u"toolBar")
        MainWindow.addToolBar(Qt.TopToolBarArea, self.toolBar)

        self.menuBar.addAction(self.menu.menuAction())
        self.menuBar.addAction(self.menu_2.menuAction())
        self.menu.addAction(self.data_load_view_action)
        self.menu.addAction(self.data_info_view_action)
        self.menu_2.addSeparator()
        self.menu_2.addAction(self.basic_slice_view_action)
        self.menu_2.addAction(self.free_slice_view_action)
        self.menu_2.addSeparator()
        self.menu_2.addAction(self.color_bar_view_action)
        self.menu_2.addAction(self.axis_grid_view_action)
        self.menu_2.addAction(self.view_setting_view_action)
        self.menu_2.addAction(self.label_setting_view_action)
        self.menu_2.addSeparator()
        self.menu_2.addAction(self.view_screenshot_action)
        self.menu_2.addAction(self.translucent_action)
        self.menu_2.addSeparator()
        self.menu_2.addAction(self.zoom_best_fit_action)
        self.menu_2.addAction(self.view_direction_to_plus_x_action)
        self.menu_2.addAction(self.view_direction_to_minus_x_action)
        self.menu_2.addAction(self.view_direction_to_plus_y_action)
        self.menu_2.addAction(self.view_direction_to_minus_y_action)
        self.menu_2.addAction(self.view_direction_to_plus_z_action)
        self.menu_2.addAction(self.view_direction_to_minus_z_action)
        self.menu_2.addSeparator()
        self.menu_2.addAction(self.increase_vertical_size_action)
        self.menu_2.addAction(self.decrease_vertical_size_action)
        self.menu_2.addAction(self.increase_horizontal_size_action)
        self.menu_2.addAction(self.decrease_horizontal_size_action)
        self.toolBar.addAction(self.data_load_view_action)
        self.toolBar.addAction(self.data_info_view_action)
        self.toolBar.addAction(self.basic_slice_view_action)
        self.toolBar.addAction(self.free_slice_view_action)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.color_bar_view_action)
        self.toolBar.addAction(self.axis_grid_view_action)
        self.toolBar.addAction(self.view_setting_view_action)
        self.toolBar.addAction(self.label_setting_view_action)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.view_screenshot_action)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.translucent_action)
        self.toolBar.addAction(self.zoom_best_fit_action)
        self.toolBar.addAction(self.view_direction_to_plus_x_action)
        self.toolBar.addAction(self.view_direction_to_minus_x_action)
        self.toolBar.addAction(self.view_direction_to_plus_y_action)
        self.toolBar.addAction(self.view_direction_to_minus_y_action)
        self.toolBar.addAction(self.view_direction_to_plus_z_action)
        self.toolBar.addAction(self.view_direction_to_minus_z_action)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.increase_vertical_size_action)
        self.toolBar.addAction(self.decrease_vertical_size_action)
        self.toolBar.addAction(self.increase_horizontal_size_action)
        self.toolBar.addAction(self.decrease_horizontal_size_action)
        self.toolBar.addSeparator()

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.data_load_view_action.setText(QCoreApplication.translate("MainWindow", u"Load Data", None))
        self.basic_slice_view_action.setText(QCoreApplication.translate("MainWindow", u"Basic Slice", None))
        self.basic_slice_slider_view_action.setText(QCoreApplication.translate("MainWindow", u"Slider For Basic Slice", None))
        self.free_slice_view_action.setText(QCoreApplication.translate("MainWindow", u"Free Slice", None))
        self.view_setting_view_action.setText(QCoreApplication.translate("MainWindow", u"View Settings", None))
        self.color_bar_view_action.setText(QCoreApplication.translate("MainWindow", u"Color Bar", None))
        self.color_bar_setting_view_action.setText(QCoreApplication.translate("MainWindow", u"Color Bar Settings", None))
#if QT_CONFIG(tooltip)
        self.color_bar_setting_view_action.setToolTip(QCoreApplication.translate("MainWindow", u"Color Bar Settings", None))
#endif // QT_CONFIG(tooltip)
        self.label_setting_view_action.setText(QCoreApplication.translate("MainWindow", u"label Settings", None))
#if QT_CONFIG(tooltip)
        self.label_setting_view_action.setToolTip(QCoreApplication.translate("MainWindow", u"Label Settings", None))
#endif // QT_CONFIG(tooltip)
        self.zoom_best_fit_action.setText(QCoreApplication.translate("MainWindow", u"Zoom best fit", None))
#if QT_CONFIG(tooltip)
        self.zoom_best_fit_action.setToolTip(QCoreApplication.translate("MainWindow", u"Zoom best fit", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_plus_x_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to +X", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_plus_x_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to +X", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_minus_x_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to -X", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_minus_x_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to -X", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_plus_y_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to +Y", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_plus_y_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to +Y", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_minus_y_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to -Y", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_minus_y_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to -X", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_plus_z_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to +Z", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_plus_z_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to +Z", None))
#endif // QT_CONFIG(tooltip)
        self.view_direction_to_minus_z_action.setText(QCoreApplication.translate("MainWindow", u"Set View direction to -Z", None))
#if QT_CONFIG(tooltip)
        self.view_direction_to_minus_z_action.setToolTip(QCoreApplication.translate("MainWindow", u"View direction to -Z", None))
#endif // QT_CONFIG(tooltip)
        self.data_info_view_action.setText(QCoreApplication.translate("MainWindow", u"Data Information", None))
#if QT_CONFIG(tooltip)
        self.data_info_view_action.setToolTip(QCoreApplication.translate("MainWindow", u"Data Information", None))
#endif // QT_CONFIG(tooltip)
        self.axis_grid_view_action.setText(QCoreApplication.translate("MainWindow", u"Axis/Grid", None))
#if QT_CONFIG(tooltip)
        self.axis_grid_view_action.setToolTip(QCoreApplication.translate("MainWindow", u"Axis/Grid", None))
#endif // QT_CONFIG(tooltip)
        self.increase_vertical_size_action.setText(QCoreApplication.translate("MainWindow", u"Increase Vertical Size ", None))
#if QT_CONFIG(tooltip)
        self.increase_vertical_size_action.setToolTip(QCoreApplication.translate("MainWindow", u"Increase Vertical Size", None))
#endif // QT_CONFIG(tooltip)
        self.decrease_vertical_size_action.setText(QCoreApplication.translate("MainWindow", u"Decrease Vertical Size", None))
#if QT_CONFIG(tooltip)
        self.decrease_vertical_size_action.setToolTip(QCoreApplication.translate("MainWindow", u"Decrease Vertical Size", None))
#endif // QT_CONFIG(tooltip)
        self.increase_horizontal_size_action.setText(QCoreApplication.translate("MainWindow", u"Increase Horizontal Size", None))
#if QT_CONFIG(tooltip)
        self.increase_horizontal_size_action.setToolTip(QCoreApplication.translate("MainWindow", u"Decrease Horizontal Size", None))
#endif // QT_CONFIG(tooltip)
        self.decrease_horizontal_size_action.setText(QCoreApplication.translate("MainWindow", u"Decrease Horizontal Size", None))
#if QT_CONFIG(tooltip)
        self.decrease_horizontal_size_action.setToolTip(QCoreApplication.translate("MainWindow", u"Decrease Horizontal Size", None))
#endif // QT_CONFIG(tooltip)
        self.view_screenshot_action.setText(QCoreApplication.translate("MainWindow", u"Screenshot", None))
#if QT_CONFIG(tooltip)
        self.view_screenshot_action.setToolTip(QCoreApplication.translate("MainWindow", u"Screenshot", None))
#endif // QT_CONFIG(tooltip)
        self.translucent_action.setText(QCoreApplication.translate("MainWindow", u"Translucent", None))
#if QT_CONFIG(tooltip)
        self.translucent_action.setToolTip(QCoreApplication.translate("MainWindow", u"translucent", None))
#endif // QT_CONFIG(tooltip)
        self.menu.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menu_2.setTitle(QCoreApplication.translate("MainWindow", u"View", None))
        self.toolBar.setWindowTitle(QCoreApplication.translate("MainWindow", u"toolBar", None))
    # retranslateUi

