# -*- coding: utf-8 -*-

import sys
import string
import math

import Manyo.core
import ModuleTest as ModTest

ec0 = ModTest.ec0
e2 = ec0.Averaging( 2 )
e3 = ec0.Averaging( 3 )

#module.PrintElementContainer( e2 )
#module.PrintElementContainer( e3 )
