# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'VisContQ_Main.ui'
##
## Created by: Qt User Interface Compiler version 5.14.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QMetaObject, QObject, QPoint,
    QRect, QSize, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QLinearGradient, QPalette, QPainter, QPixmap,
    QRadialGradient)
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(574, 683)
        self.menu_load = QAction(MainWindow)
        self.menu_load.setObjectName(u"menu_load")
        self.menu_legacy = QAction(MainWindow)
        self.menu_legacy.setObjectName(u"menu_legacy")
        self.actionSave_as = QAction(MainWindow)
        self.actionSave_as.setObjectName(u"actionSave_as")
        self.menu_load_qxqyqz_text = QAction(MainWindow)
        self.menu_load_qxqyqz_text.setObjectName(u"menu_load_qxqyqz_text")
        self.menu_setting = QAction(MainWindow)
        self.menu_setting.setObjectName(u"menu_setting")
        self.menu_exit = QAction(MainWindow)
        self.menu_exit.setObjectName(u"menu_exit")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.btApply = QPushButton(self.centralwidget)
        self.btApply.setObjectName(u"btApply")
        self.btApply.setGeometry(QRect(220, 10, 211, 31))
        self.btOpperate = QPushButton(self.centralwidget)
        self.btOpperate.setObjectName(u"btOpperate")
        self.btOpperate.setGeometry(QRect(460, 10, 99, 27))
        self.tabWidget = QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tabWidget.setGeometry(QRect(0, 50, 571, 586))
        self.splash_label = QLabel(self.centralwidget)
        self.splash_label.setObjectName(u"splash_label")
        self.splash_label.setGeometry(QRect(5, 5, 200, 40))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 574, 25))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.menu_load)
        self.menuFile.addAction(self.menu_legacy)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addAction(self.menu_load_qxqyqz_text)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menu_setting)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menu_exit)

        self.retranslateUi(MainWindow)

        self.tabWidget.setCurrentIndex(-1)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.menu_load.setText(QCoreApplication.translate("MainWindow", u"Load", None))
        self.menu_legacy.setText(QCoreApplication.translate("MainWindow", u"Load Legacy...", None))
        self.actionSave_as.setText(QCoreApplication.translate("MainWindow", u"Save as ...", None))
        self.menu_load_qxqyqz_text.setText(QCoreApplication.translate("MainWindow", u"Load QxQyQz..", None))
        self.menu_setting.setText(QCoreApplication.translate("MainWindow", u"Setting ...", None))
        self.menu_exit.setText(QCoreApplication.translate("MainWindow", u"Exit", None))
        self.btApply.setText(QCoreApplication.translate("MainWindow", u"Apply params to other Tabs", None))
        self.btOpperate.setText(QCoreApplication.translate("MainWindow", u"CalcBar", None))
        self.splash_label.setText("")
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File...", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'VisContQ_DataOperationDialog.ui'
##
## Created by: Qt User Interface Compiler version 5.14.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QMetaObject, QObject, QPoint,
    QRect, QSize, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QLinearGradient, QPalette, QPainter, QPixmap,
    QRadialGradient)
from PySide2.QtWidgets import *


class Ui_DataOperationDialog(object):
    def setupUi(self, DataOperationDialog):
        if DataOperationDialog.objectName():
            DataOperationDialog.setObjectName(u"DataOperationDialog")
        DataOperationDialog.resize(560, 160)
        self.groupBox = QGroupBox(DataOperationDialog)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(20, 10, 521, 101))
        self.ckPr = QCheckBox(self.groupBox)
        self.ckPr.setObjectName(u"ckPr")
        self.ckPr.setGeometry(QRect(10, 30, 201, 22))
        self.cb0 = QComboBox(self.groupBox)
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.setObjectName(u"cb0")
        self.cb0.setGeometry(QRect(10, 60, 40, 26))
        self.stEQ = QLabel(self.groupBox)
        self.stEQ.setObjectName(u"stEQ")
        self.stEQ.setGeometry(QRect(60, 60, 21, 26))
        self.m1 = QLineEdit(self.groupBox)
        self.m1.setObjectName(u"m1")
        self.m1.setGeometry(QRect(80, 60, 50, 26))
        self.asta1 = QLabel(self.groupBox)
        self.asta1.setObjectName(u"asta1")
        self.asta1.setGeometry(QRect(140, 60, 21, 26))
        self.cb1 = QComboBox(self.groupBox)
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.setObjectName(u"cb1")
        self.cb1.setGeometry(QRect(160, 60, 40, 26))
        self.cbOP = QComboBox(self.groupBox)
        self.cbOP.addItem("")
        self.cbOP.addItem("")
        self.cbOP.addItem("")
        self.cbOP.setObjectName(u"cbOP")
        self.cbOP.setGeometry(QRect(220, 60, 40, 26))
        self.asta2 = QLabel(self.groupBox)
        self.asta2.setObjectName(u"asta2")
        self.asta2.setGeometry(QRect(340, 60, 21, 26))
        self.cb2 = QComboBox(self.groupBox)
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.setObjectName(u"cb2")
        self.cb2.setGeometry(QRect(360, 60, 40, 26))
        self.m2 = QLineEdit(self.groupBox)
        self.m2.setObjectName(u"m2")
        self.m2.setGeometry(QRect(280, 60, 50, 26))
        self.btCalc = QPushButton(self.groupBox)
        self.btCalc.setObjectName(u"btCalc")
        self.btCalc.setGeometry(QRect(420, 60, 91, 27))
        self.btClose = QPushButton(DataOperationDialog)
        self.btClose.setObjectName(u"btClose")
        self.btClose.setGeometry(QRect(210, 120, 99, 27))
        QWidget.setTabOrder(self.ckPr, self.cb0)
        QWidget.setTabOrder(self.cb0, self.m1)
        QWidget.setTabOrder(self.m1, self.cb1)
        QWidget.setTabOrder(self.cb1, self.cbOP)
        QWidget.setTabOrder(self.cbOP, self.m2)
        QWidget.setTabOrder(self.m2, self.cb2)
        QWidget.setTabOrder(self.cb2, self.btCalc)
        QWidget.setTabOrder(self.btCalc, self.btClose)

        self.retranslateUi(DataOperationDialog)

        QMetaObject.connectSlotsByName(DataOperationDialog)
    # setupUi

    def retranslateUi(self, DataOperationDialog):
        DataOperationDialog.setWindowTitle(QCoreApplication.translate("DataOperationDialog", u"Dialog", None))
        self.groupBox.setTitle(QCoreApplication.translate("DataOperationDialog", u"GroupBox", None))
        self.ckPr.setText(QCoreApplication.translate("DataOperationDialog", u"Calculate Slicing Dta", None))
        self.cb0.setItemText(0, QCoreApplication.translate("DataOperationDialog", u"A", None))
        self.cb0.setItemText(1, QCoreApplication.translate("DataOperationDialog", u"B", None))
        self.cb0.setItemText(2, QCoreApplication.translate("DataOperationDialog", u"C", None))
        self.cb0.setItemText(3, QCoreApplication.translate("DataOperationDialog", u"D", None))
        self.cb0.setItemText(4, QCoreApplication.translate("DataOperationDialog", u"E", None))
        self.cb0.setItemText(5, QCoreApplication.translate("DataOperationDialog", u"F", None))
        self.cb0.setItemText(6, QCoreApplication.translate("DataOperationDialog", u"G", None))
        self.cb0.setItemText(7, QCoreApplication.translate("DataOperationDialog", u"H", None))
        self.cb0.setItemText(8, QCoreApplication.translate("DataOperationDialog", u"I", None))
        self.cb0.setItemText(9, QCoreApplication.translate("DataOperationDialog", u"J", None))

        self.stEQ.setText(QCoreApplication.translate("DataOperationDialog", u"=", None))
        self.asta1.setText(QCoreApplication.translate("DataOperationDialog", u"*", None))
        self.cb1.setItemText(0, QCoreApplication.translate("DataOperationDialog", u"A", None))
        self.cb1.setItemText(1, QCoreApplication.translate("DataOperationDialog", u"B", None))
        self.cb1.setItemText(2, QCoreApplication.translate("DataOperationDialog", u"C", None))
        self.cb1.setItemText(3, QCoreApplication.translate("DataOperationDialog", u"D", None))
        self.cb1.setItemText(4, QCoreApplication.translate("DataOperationDialog", u"E", None))
        self.cb1.setItemText(5, QCoreApplication.translate("DataOperationDialog", u"F", None))
        self.cb1.setItemText(6, QCoreApplication.translate("DataOperationDialog", u"G", None))
        self.cb1.setItemText(7, QCoreApplication.translate("DataOperationDialog", u"H", None))
        self.cb1.setItemText(8, QCoreApplication.translate("DataOperationDialog", u"I", None))
        self.cb1.setItemText(9, QCoreApplication.translate("DataOperationDialog", u"J", None))

        self.cbOP.setItemText(0, QCoreApplication.translate("DataOperationDialog", u"-", None))
        self.cbOP.setItemText(1, QCoreApplication.translate("DataOperationDialog", u"+", None))
        self.cbOP.setItemText(2, "")

        self.asta2.setText(QCoreApplication.translate("DataOperationDialog", u"*", None))
        self.cb2.setItemText(0, QCoreApplication.translate("DataOperationDialog", u"A", None))
        self.cb2.setItemText(1, QCoreApplication.translate("DataOperationDialog", u"B", None))
        self.cb2.setItemText(2, QCoreApplication.translate("DataOperationDialog", u"C", None))
        self.cb2.setItemText(3, QCoreApplication.translate("DataOperationDialog", u"D", None))
        self.cb2.setItemText(4, QCoreApplication.translate("DataOperationDialog", u"E", None))
        self.cb2.setItemText(5, QCoreApplication.translate("DataOperationDialog", u"F", None))
        self.cb2.setItemText(6, QCoreApplication.translate("DataOperationDialog", u"G", None))
        self.cb2.setItemText(7, QCoreApplication.translate("DataOperationDialog", u"H", None))
        self.cb2.setItemText(8, QCoreApplication.translate("DataOperationDialog", u"I", None))
        self.cb2.setItemText(9, QCoreApplication.translate("DataOperationDialog", u"J", None))

        self.btCalc.setText(QCoreApplication.translate("DataOperationDialog", u"Calculate", None))
        self.btClose.setText(QCoreApplication.translate("DataOperationDialog", u"Close", None))
    # retranslateUi


################################################################################
## Form generated from reading UI file 'VisContQ_SettingDialog.ui'
##
## Created by: Qt User Interface Compiler version 5.14.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QMetaObject, QObject, QPoint,
    QRect, QSize, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QLinearGradient, QPalette, QPainter, QPixmap,
    QRadialGradient)
from PySide2.QtWidgets import *


class Ui_SettingDialog(object):
    def setupUi(self, SettingDialog):
        if SettingDialog.objectName():
            SettingDialog.setObjectName(u"SettingDialog")
        SettingDialog.resize(400, 85)
        self.groupBox = QGroupBox(SettingDialog)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(0, 0, 371, 80))
        self.ckSliceAverageMode = QCheckBox(self.groupBox)
        self.ckSliceAverageMode.setObjectName(u"ckSliceAverageMode")
        self.ckSliceAverageMode.setGeometry(QRect(20, 30, 371, 22))

        self.retranslateUi(SettingDialog)

        QMetaObject.connectSlotsByName(SettingDialog)
    # setupUi

    def retranslateUi(self, SettingDialog):
        SettingDialog.setWindowTitle(QCoreApplication.translate("SettingDialog", u"Dialog", None))
        self.groupBox.setTitle(QCoreApplication.translate("SettingDialog", u"Slice", None))
        self.ckSliceAverageMode.setText(QCoreApplication.translate("SettingDialog", u"Average Mode    On : Average  / Off : Sum", None))
    # retranslateUi

