#ifndef UTSUSEMIREDISPUBSUBEVENTMONITORNEUNET
#define UTSUSEMIREDISPUBSUBEVENTMONITORNEUNET

#include "UtsusemiRedisPubSubEventMonitorTemplate.hh"
#include "UtsusemiEventDataConverterNeunet.hh"
#include "T0TreatToolsNeunet.hh"

class UtsusemiRedisPubSubEventMonitorNeunet:
    public UtsusemiRedisPubSubEventMonitorTemplate<UtsusemiEventDataConverterNeunet>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiRedisPubSubEventMonitorNeunet();
        //!< Constructor
        /*!<
         */
    ~UtsusemiRedisPubSubEventMonitorNeunet();
        //!< Destructor
        /*!<
         */
    void OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId=1 );
    UtsusemiEventDataConverterNeunet* PutUtsusemiEventDataConverterNeunet(){ return _EDC; }
};
#endif
