#include "UtsusemiRedisPubSubEventMonitorMWPC.hh"
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorMWPC::
UtsusemiRedisPubSubEventMonitorMWPC(){
    _MessageTag2 = "UtsusemiRedisPubSubEventMonitorMWPC::";
}
//////////////////////////////////////////////////////////
UtsusemiRedisPubSubEventMonitorMWPC::
~UtsusemiRedisPubSubEventMonitorMWPC(){
}
//////////////////////////////////////////////////////////
void UtsusemiRedisPubSubEventMonitorMWPC::
OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId ){
    _EDC->SetElementContainerReadout( pixelId, ec, caseId );
    SetMeasTimeSliced( ec->PutHeaderPointer() );
    SetKickers( ec->PutHeaderPointer(), caseId );
}
