#ifndef UTSUSEMIREDISPUBSUBEVENTMONITOREREADOUT1D
#define UTSUSEMIREDISPUBSUBEVENTMONITOREREADOUT1D

#include "UtsusemiRedisPubSubEventMonitorTemplate.hh"
#include "EmakiEventDataConverterReadout1d.hh"
#include "T0TreatToolsReadout.hh"

class UtsusemiRedisPubSubEventMonitorReadout1d:
    public UtsusemiRedisPubSubEventMonitorTemplate<EmakiEventDataConverterReadout1d>
{
private:
    std::string _MessageTag2;
public:
    UtsusemiRedisPubSubEventMonitorReadout1d();
        //!< Constructor
        /*!<
         */
    ~UtsusemiRedisPubSubEventMonitorReadout1d();
        //!< Destructor
        /*!<
         */
    void OutputEC( ElementContainer* ec, UInt4 pixelId, UInt4 caseId=1 );
    EmakiEventDataConverterReadout1d* PutUtsusemiEventDataConverterReadout1d(){ return _EDC; }
};
#endif
