# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'SequencerQ_LDR_Main.ui'
#
# Created: Mon Jan 15 19:09:36 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1174, 718)
        MainWindow.setUnifiedTitleAndToolBarOnMac(False)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.SplashPanel = QtGui.QLabel(self.centralwidget)
        self.SplashPanel.setGeometry(QtCore.QRect(10, 10, 311, 50))
        self.SplashPanel.setText("")
        self.SplashPanel.setObjectName("SplashPanel")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(330, 10, 261, 51))
        self.groupBox.setObjectName("groupBox")
        self.btClear = QtGui.QPushButton(self.groupBox)
        self.btClear.setGeometry(QtCore.QRect(10, 20, 51, 32))
        self.btClear.setObjectName("btClear")
        self.btOpen = QtGui.QPushButton(self.groupBox)
        self.btOpen.setGeometry(QtCore.QRect(80, 20, 32, 32))
        self.btOpen.setText("")
        self.btOpen.setObjectName("btOpen")
        self.btSaveas = QtGui.QPushButton(self.groupBox)
        self.btSaveas.setGeometry(QtCore.QRect(120, 20, 32, 32))
        self.btSaveas.setText("")
        self.btSaveas.setObjectName("btSaveas")
        self.btCCD = QtGui.QPushButton(self.groupBox)
        self.btCCD.setGeometry(QtCore.QRect(160, 20, 99, 32))
        self.btCCD.setObjectName("btCCD")
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setGeometry(QtCore.QRect(800, 10, 371, 51))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btBegin = QtGui.QPushButton(self.groupBox_2)
        self.btBegin.setGeometry(QtCore.QRect(10, 20, 60, 32))
        self.btBegin.setObjectName("btBegin")
        self.btStop = QtGui.QPushButton(self.groupBox_2)
        self.btStop.setGeometry(QtCore.QRect(80, 20, 60, 32))
        self.btStop.setObjectName("btStop")
        self.btResume = QtGui.QPushButton(self.groupBox_2)
        self.btResume.setGeometry(QtCore.QRect(150, 20, 101, 32))
        self.btResume.setObjectName("btResume")
        self.btVis = QtGui.QPushButton(self.groupBox_2)
        self.btVis.setGeometry(QtCore.QRect(280, 20, 81, 32))
        self.btVis.setObjectName("btVis")
        self.line = QtGui.QFrame(self.groupBox_2)
        self.line.setGeometry(QtCore.QRect(260, 20, 20, 31))
        self.line.setFrameShape(QtGui.QFrame.VLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.tree0 = QtGui.QTreeWidget(self.centralwidget)
        self.tree0.setGeometry(QtCore.QRect(10, 259, 300, 241))
        self.tree0.setHeaderHidden(True)
        self.tree0.setObjectName("tree0")
        self.tree0.headerItem().setText(0, "1")
        self.list1 = QtGui.QTreeWidget(self.centralwidget)
        self.list1.setGeometry(QtCore.QRect(360, 259, 800, 201))
        self.list1.setObjectName("list1")
        self.list1.headerItem().setText(0, "1")
        self.st0 = QtGui.QLabel(self.centralwidget)
        self.st0.setGeometry(QtCore.QRect(380, 70, 91, 17))
        self.st0.setObjectName("st0")
        self.btAdd = QtGui.QPushButton(self.centralwidget)
        self.btAdd.setGeometry(QtCore.QRect(320, 270, 32, 32))
        self.btAdd.setText("")
        self.btAdd.setObjectName("btAdd")
        self.btRemove = QtGui.QPushButton(self.centralwidget)
        self.btRemove.setGeometry(QtCore.QRect(320, 310, 32, 32))
        self.btRemove.setText("")
        self.btRemove.setObjectName("btRemove")
        self.btCopyAdd = QtGui.QPushButton(self.centralwidget)
        self.btCopyAdd.setGeometry(QtCore.QRect(320, 390, 32, 32))
        self.btCopyAdd.setText("")
        self.btCopyAdd.setObjectName("btCopyAdd")
        self.btUp = QtGui.QPushButton(self.centralwidget)
        self.btUp.setGeometry(QtCore.QRect(320, 430, 32, 32))
        self.btUp.setText("")
        self.btUp.setObjectName("btUp")
        self.btDown = QtGui.QPushButton(self.centralwidget)
        self.btDown.setGeometry(QtCore.QRect(320, 470, 32, 32))
        self.btDown.setText("")
        self.btDown.setObjectName("btDown")
        self.stCurrentDir = QtGui.QLabel(self.centralwidget)
        self.stCurrentDir.setGeometry(QtCore.QRect(470, 70, 671, 17))
        self.stCurrentDir.setText("")
        self.stCurrentDir.setObjectName("stCurrentDir")
        self.funcdoc = QtGui.QTextBrowser(self.centralwidget)
        self.funcdoc.setGeometry(QtCore.QRect(10, 509, 341, 171))
        self.funcdoc.setObjectName("funcdoc")
        self.btSet = QtGui.QPushButton(self.centralwidget)
        self.btSet.setGeometry(QtCore.QRect(930, 470, 120, 32))
        self.btSet.setObjectName("btSet")
        self.retval = QtGui.QLineEdit(self.centralwidget)
        self.retval.setGeometry(QtCore.QRect(610, 470, 150, 32))
        self.retval.setObjectName("retval")
        self.st2 = QtGui.QLabel(self.centralwidget)
        self.st2.setGeometry(QtCore.QRect(500, 480, 101, 17))
        self.st2.setObjectName("st2")
        self.sca_Args = QtGui.QScrollArea(self.centralwidget)
        self.sca_Args.setGeometry(QtCore.QRect(360, 510, 801, 171))
        self.sca_Args.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.sca_Args.setWidgetResizable(False)
        self.sca_Args.setObjectName("sca_Args")
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 799, 269))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.grb_Args = QtGui.QGroupBox(self.scrollAreaWidgetContents)
        self.grb_Args.setEnabled(True)
        self.grb_Args.setGeometry(QtCore.QRect(10, 0, 780, 270))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.grb_Args.sizePolicy().hasHeightForWidth())
        self.grb_Args.setSizePolicy(sizePolicy)
        self.grb_Args.setAutoFillBackground(True)
        self.grb_Args.setTitle("")
        self.grb_Args.setFlat(True)
        self.grb_Args.setCheckable(False)
        self.grb_Args.setObjectName("grb_Args")
        self.arg1 = QtGui.QLineEdit(self.grb_Args)
        self.arg1.setGeometry(QtCore.QRect(10, 30, 240, 24))
        self.arg1.setObjectName("arg1")
        self.arg2 = QtGui.QLineEdit(self.grb_Args)
        self.arg2.setGeometry(QtCore.QRect(270, 30, 240, 24))
        self.arg2.setObjectName("arg2")
        self.arg3 = QtGui.QLineEdit(self.grb_Args)
        self.arg3.setGeometry(QtCore.QRect(530, 30, 240, 24))
        self.arg3.setObjectName("arg3")
        self.arg4 = QtGui.QLineEdit(self.grb_Args)
        self.arg4.setGeometry(QtCore.QRect(10, 80, 240, 24))
        self.arg4.setObjectName("arg4")
        self.arg5 = QtGui.QLineEdit(self.grb_Args)
        self.arg5.setGeometry(QtCore.QRect(270, 80, 240, 24))
        self.arg5.setObjectName("arg5")
        self.arg6 = QtGui.QLineEdit(self.grb_Args)
        self.arg6.setGeometry(QtCore.QRect(530, 80, 240, 24))
        self.arg6.setObjectName("arg6")
        self.arg7 = QtGui.QLineEdit(self.grb_Args)
        self.arg7.setGeometry(QtCore.QRect(10, 130, 240, 24))
        self.arg7.setObjectName("arg7")
        self.arg8 = QtGui.QLineEdit(self.grb_Args)
        self.arg8.setGeometry(QtCore.QRect(270, 130, 240, 24))
        self.arg8.setObjectName("arg8")
        self.arg9 = QtGui.QLineEdit(self.grb_Args)
        self.arg9.setGeometry(QtCore.QRect(530, 130, 240, 24))
        self.arg9.setObjectName("arg9")
        self.arg10 = QtGui.QLineEdit(self.grb_Args)
        self.arg10.setGeometry(QtCore.QRect(10, 180, 240, 24))
        self.arg10.setObjectName("arg10")
        self.arg11 = QtGui.QLineEdit(self.grb_Args)
        self.arg11.setGeometry(QtCore.QRect(270, 180, 240, 24))
        self.arg11.setObjectName("arg11")
        self.arg12 = QtGui.QLineEdit(self.grb_Args)
        self.arg12.setGeometry(QtCore.QRect(530, 180, 240, 24))
        self.arg12.setObjectName("arg12")
        self.arg13 = QtGui.QLineEdit(self.grb_Args)
        self.arg13.setGeometry(QtCore.QRect(10, 230, 240, 24))
        self.arg13.setObjectName("arg13")
        self.arg14 = QtGui.QLineEdit(self.grb_Args)
        self.arg14.setGeometry(QtCore.QRect(270, 230, 240, 24))
        self.arg14.setObjectName("arg14")
        self.arg15 = QtGui.QLineEdit(self.grb_Args)
        self.arg15.setGeometry(QtCore.QRect(530, 230, 240, 24))
        self.arg15.setObjectName("arg15")
        self.argst1 = QtGui.QLabel(self.grb_Args)
        self.argst1.setGeometry(QtCore.QRect(20, 10, 231, 17))
        self.argst1.setObjectName("argst1")
        self.argst2 = QtGui.QLabel(self.grb_Args)
        self.argst2.setGeometry(QtCore.QRect(280, 10, 231, 17))
        self.argst2.setObjectName("argst2")
        self.argst3 = QtGui.QLabel(self.grb_Args)
        self.argst3.setGeometry(QtCore.QRect(540, 10, 231, 17))
        self.argst3.setObjectName("argst3")
        self.argst4 = QtGui.QLabel(self.grb_Args)
        self.argst4.setGeometry(QtCore.QRect(20, 60, 231, 17))
        self.argst4.setObjectName("argst4")
        self.argst5 = QtGui.QLabel(self.grb_Args)
        self.argst5.setGeometry(QtCore.QRect(280, 60, 231, 17))
        self.argst5.setObjectName("argst5")
        self.argst6 = QtGui.QLabel(self.grb_Args)
        self.argst6.setGeometry(QtCore.QRect(540, 60, 231, 17))
        self.argst6.setObjectName("argst6")
        self.argst7 = QtGui.QLabel(self.grb_Args)
        self.argst7.setGeometry(QtCore.QRect(20, 110, 231, 17))
        self.argst7.setObjectName("argst7")
        self.argst8 = QtGui.QLabel(self.grb_Args)
        self.argst8.setGeometry(QtCore.QRect(280, 110, 231, 17))
        self.argst8.setObjectName("argst8")
        self.argst9 = QtGui.QLabel(self.grb_Args)
        self.argst9.setGeometry(QtCore.QRect(540, 110, 231, 17))
        self.argst9.setObjectName("argst9")
        self.argst10 = QtGui.QLabel(self.grb_Args)
        self.argst10.setGeometry(QtCore.QRect(20, 160, 231, 17))
        self.argst10.setObjectName("argst10")
        self.argst11 = QtGui.QLabel(self.grb_Args)
        self.argst11.setGeometry(QtCore.QRect(280, 160, 231, 17))
        self.argst11.setObjectName("argst11")
        self.argst12 = QtGui.QLabel(self.grb_Args)
        self.argst12.setGeometry(QtCore.QRect(540, 160, 231, 17))
        self.argst12.setObjectName("argst12")
        self.argst13 = QtGui.QLabel(self.grb_Args)
        self.argst13.setGeometry(QtCore.QRect(20, 210, 231, 17))
        self.argst13.setObjectName("argst13")
        self.argst14 = QtGui.QLabel(self.grb_Args)
        self.argst14.setGeometry(QtCore.QRect(280, 210, 231, 17))
        self.argst14.setObjectName("argst14")
        self.argst15 = QtGui.QLabel(self.grb_Args)
        self.argst15.setGeometry(QtCore.QRect(540, 210, 231, 17))
        self.argst15.setObjectName("argst15")
        self.sca_Args.setWidget(self.scrollAreaWidgetContents)
        self.gbLDRbox = QtGui.QGroupBox(self.centralwidget)
        self.gbLDRbox.setGeometry(QtCore.QRect(10, 90, 1151, 161))
        self.gbLDRbox.setFlat(False)
        self.gbLDRbox.setCheckable(True)
        self.gbLDRbox.setObjectName("gbLDRbox")
        self.sca_HistInfo = QtGui.QScrollArea(self.gbLDRbox)
        self.sca_HistInfo.setGeometry(QtCore.QRect(0, 20, 811, 141))
        self.sca_HistInfo.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.sca_HistInfo.setWidgetResizable(False)
        self.sca_HistInfo.setObjectName("sca_HistInfo")
        self.scrollAreaLDR = QtGui.QWidget()
        self.scrollAreaLDR.setEnabled(False)
        self.scrollAreaLDR.setGeometry(QtCore.QRect(0, 0, 789, 250))
        self.scrollAreaLDR.setObjectName("scrollAreaLDR")
        self.label = QtGui.QLabel(self.scrollAreaLDR)
        self.label.setGeometry(QtCore.QRect(10, 10, 91, 20))
        self.label.setObjectName("label")
        self.label_2 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_2.setGeometry(QtCore.QRect(250, 10, 121, 17))
        self.label_2.setObjectName("label_2")
        self.txHistInfo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txHistInfo.setGeometry(QtCore.QRect(380, 10, 391, 27))
        self.txHistInfo.setObjectName("txHistInfo")
        self.label_3 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_3.setGeometry(QtCore.QRect(10, 50, 91, 17))
        self.label_3.setObjectName("label_3")
        self.txDetParam = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txDetParam.setGeometry(QtCore.QRect(100, 50, 271, 27))
        self.txDetParam.setObjectName("txDetParam")
        self.label_4 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_4.setGeometry(QtCore.QRect(420, 50, 91, 17))
        self.label_4.setObjectName("label_4")
        self.txDetRange = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txDetRange.setGeometry(QtCore.QRect(500, 50, 271, 27))
        self.txDetRange.setObjectName("txDetRange")
        self.label_5 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_5.setGeometry(QtCore.QRect(10, 90, 91, 17))
        self.label_5.setObjectName("label_5")
        self.txTimeRange = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txTimeRange.setGeometry(QtCore.QRect(100, 90, 271, 27))
        self.txTimeRange.setObjectName("txTimeRange")
        self.txMaskInfo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txMaskInfo.setGeometry(QtCore.QRect(100, 130, 271, 27))
        self.txMaskInfo.setObjectName("txMaskInfo")
        self.label_6 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_6.setGeometry(QtCore.QRect(10, 130, 91, 17))
        self.label_6.setObjectName("label_6")
        self.label_7 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_7.setGeometry(QtCore.QRect(420, 130, 71, 17))
        self.label_7.setObjectName("label_7")
        self.txCaseInfo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txCaseInfo.setGeometry(QtCore.QRect(500, 130, 271, 27))
        self.txCaseInfo.setObjectName("txCaseInfo")
        self.label_8 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_8.setGeometry(QtCore.QRect(10, 170, 91, 17))
        self.label_8.setObjectName("label_8")
        self.txBgInfo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txBgInfo.setGeometry(QtCore.QRect(100, 170, 271, 27))
        self.txBgInfo.setObjectName("txBgInfo")
        self.txTofShift = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txTofShift.setGeometry(QtCore.QRect(500, 170, 271, 27))
        self.txTofShift.setObjectName("txTofShift")
        self.txAnaMode = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txAnaMode.setGeometry(QtCore.QRect(100, 210, 271, 27))
        self.txAnaMode.setObjectName("txAnaMode")
        self.label_9 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_9.setGeometry(QtCore.QRect(420, 170, 71, 17))
        self.label_9.setObjectName("label_9")
        self.label_10 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_10.setGeometry(QtCore.QRect(10, 210, 91, 17))
        self.label_10.setObjectName("label_10")
        self.txRunNo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txRunNo.setGeometry(QtCore.QRect(100, 10, 113, 27))
        self.txRunNo.setObjectName("txRunNo")
        self.label_11 = QtGui.QLabel(self.scrollAreaLDR)
        self.label_11.setGeometry(QtCore.QRect(420, 90, 71, 17))
        self.label_11.setObjectName("label_11")
        self.txFrameInfo = QtGui.QLineEdit(self.scrollAreaLDR)
        self.txFrameInfo.setGeometry(QtCore.QRect(500, 90, 271, 27))
        self.txFrameInfo.setObjectName("txFrameInfo")
        self.sca_HistInfo.setWidget(self.scrollAreaLDR)
        self.status = QtGui.QLabel(self.gbLDRbox)
        self.status.setGeometry(QtCore.QRect(950, 140, 201, 20))
        self.status.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.status.setObjectName("status")
        self.st1 = QtGui.QLabel(self.gbLDRbox)
        self.st1.setGeometry(QtCore.QRect(830, 0, 111, 17))
        self.st1.setObjectName("st1")
        self.txRetvalLDR = QtGui.QLineEdit(self.gbLDRbox)
        self.txRetvalLDR.setGeometry(QtCore.QRect(1080, 70, 61, 27))
        self.txRetvalLDR.setObjectName("txRetvalLDR")
        self.stRetValLDR = QtGui.QLabel(self.gbLDRbox)
        self.stRetValLDR.setGeometry(QtCore.QRect(980, 70, 101, 17))
        self.stRetValLDR.setObjectName("stRetValLDR")
        self.btConnectServ = QtGui.QPushButton(self.gbLDRbox)
        self.btConnectServ.setGeometry(QtCore.QRect(840, 70, 111, 51))
        self.btConnectServ.setObjectName("btConnectServ")
        self.ckEditParams = QtGui.QCheckBox(self.gbLDRbox)
        self.ckEditParams.setGeometry(QtCore.QRect(830, 50, 121, 22))
        self.ckEditParams.setObjectName("ckEditParams")
        self.label_12 = QtGui.QLabel(self.gbLDRbox)
        self.label_12.setGeometry(QtCore.QRect(980, 100, 101, 21))
        self.label_12.setObjectName("label_12")
        self.txIntervalSec = QtGui.QLineEdit(self.gbLDRbox)
        self.txIntervalSec.setGeometry(QtCore.QRect(1080, 100, 61, 27))
        self.txIntervalSec.setObjectName("txIntervalSec")
        self.statusLDRserver = QtGui.QLabel(self.gbLDRbox)
        self.statusLDRserver.setGeometry(QtCore.QRect(870, 20, 271, 20))
        self.statusLDRserver.setObjectName("statusLDRserver")
        self.st1_2 = QtGui.QLabel(self.gbLDRbox)
        self.st1_2.setGeometry(QtCore.QRect(840, 140, 111, 17))
        self.st1_2.setObjectName("st1_2")
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setGeometry(QtCore.QRect(610, 10, 171, 51))
        self.groupBox_3.setObjectName("groupBox_3")
        self.btLDRBegin = QtGui.QPushButton(self.groupBox_3)
        self.btLDRBegin.setGeometry(QtCore.QRect(10, 20, 60, 31))
        self.btLDRBegin.setObjectName("btLDRBegin")
        self.btLDREnd = QtGui.QPushButton(self.groupBox_3)
        self.btLDREnd.setGeometry(QtCore.QRect(90, 20, 70, 31))
        self.btLDREnd.setObjectName("btLDREnd")
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionFile = QtGui.QAction(MainWindow)
        self.actionFile.setObjectName("actionFile")

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "MainWindow", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.btClear.setText(QtGui.QApplication.translate("MainWindow", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.btCCD.setText(QtGui.QApplication.translate("MainWindow", "Current Dir", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("MainWindow", "Control", None, QtGui.QApplication.UnicodeUTF8))
        self.btBegin.setText(QtGui.QApplication.translate("MainWindow", "Start", None, QtGui.QApplication.UnicodeUTF8))
        self.btStop.setText(QtGui.QApplication.translate("MainWindow", "Stop", None, QtGui.QApplication.UnicodeUTF8))
        self.btResume.setText(QtGui.QApplication.translate("MainWindow", "Resume", None, QtGui.QApplication.UnicodeUTF8))
        self.btVis.setText(QtGui.QApplication.translate("MainWindow", "Visualize", None, QtGui.QApplication.UnicodeUTF8))
        self.st0.setText(QtGui.QApplication.translate("MainWindow", "Current Dir :", None, QtGui.QApplication.UnicodeUTF8))
        self.btSet.setText(QtGui.QApplication.translate("MainWindow", "Set", None, QtGui.QApplication.UnicodeUTF8))
        self.st2.setText(QtGui.QApplication.translate("MainWindow", "Return Label :", None, QtGui.QApplication.UnicodeUTF8))
        self.argst1.setText(QtGui.QApplication.translate("MainWindow", "Argument1", None, QtGui.QApplication.UnicodeUTF8))
        self.argst2.setText(QtGui.QApplication.translate("MainWindow", "Argument2", None, QtGui.QApplication.UnicodeUTF8))
        self.argst3.setText(QtGui.QApplication.translate("MainWindow", "Argument3", None, QtGui.QApplication.UnicodeUTF8))
        self.argst4.setText(QtGui.QApplication.translate("MainWindow", "Argument4", None, QtGui.QApplication.UnicodeUTF8))
        self.argst5.setText(QtGui.QApplication.translate("MainWindow", "Argument5", None, QtGui.QApplication.UnicodeUTF8))
        self.argst6.setText(QtGui.QApplication.translate("MainWindow", "Argument6", None, QtGui.QApplication.UnicodeUTF8))
        self.argst7.setText(QtGui.QApplication.translate("MainWindow", "Argument7", None, QtGui.QApplication.UnicodeUTF8))
        self.argst8.setText(QtGui.QApplication.translate("MainWindow", "Argument8", None, QtGui.QApplication.UnicodeUTF8))
        self.argst9.setText(QtGui.QApplication.translate("MainWindow", "Argument9", None, QtGui.QApplication.UnicodeUTF8))
        self.argst10.setText(QtGui.QApplication.translate("MainWindow", "Argument10", None, QtGui.QApplication.UnicodeUTF8))
        self.argst11.setText(QtGui.QApplication.translate("MainWindow", "Argument11", None, QtGui.QApplication.UnicodeUTF8))
        self.argst12.setText(QtGui.QApplication.translate("MainWindow", "Argument12", None, QtGui.QApplication.UnicodeUTF8))
        self.argst13.setText(QtGui.QApplication.translate("MainWindow", "Argument13", None, QtGui.QApplication.UnicodeUTF8))
        self.argst14.setText(QtGui.QApplication.translate("MainWindow", "Argument14", None, QtGui.QApplication.UnicodeUTF8))
        self.argst15.setText(QtGui.QApplication.translate("MainWindow", "Argument15", None, QtGui.QApplication.UnicodeUTF8))
        self.gbLDRbox.setTitle(QtGui.QApplication.translate("MainWindow", "Live Data Reduction Mode", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("MainWindow", "Run Number", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("MainWindow", "Histogram param", None, QtGui.QApplication.UnicodeUTF8))
        self.txHistInfo.setText(QtGui.QApplication.translate("MainWindow", "tof,0,40000,100", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("MainWindow", "DetParam", None, QtGui.QApplication.UnicodeUTF8))
        self.txDetParam.setText(QtGui.QApplication.translate("MainWindow", "PSD", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("MainWindow", "DetRange", None, QtGui.QApplication.UnicodeUTF8))
        self.txDetRange.setText(QtGui.QApplication.translate("MainWindow", "All", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("MainWindow", "TimeRange", None, QtGui.QApplication.UnicodeUTF8))
        self.txTimeRange.setText(QtGui.QApplication.translate("MainWindow", "All", None, QtGui.QApplication.UnicodeUTF8))
        self.txMaskInfo.setText(QtGui.QApplication.translate("MainWindow", "NoFile", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("MainWindow", "MaskInfo", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("MainWindow", "CaseInfo", None, QtGui.QApplication.UnicodeUTF8))
        self.txCaseInfo.setText(QtGui.QApplication.translate("MainWindow", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("MainWindow", "BGInfo", None, QtGui.QApplication.UnicodeUTF8))
        self.txBgInfo.setText(QtGui.QApplication.translate("MainWindow", "TimeDep:None", None, QtGui.QApplication.UnicodeUTF8))
        self.txTofShift.setText(QtGui.QApplication.translate("MainWindow", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.txAnaMode.setText(QtGui.QApplication.translate("MainWindow", "0", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("MainWindow", "TOF Shift", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("MainWindow", "AnaMode", None, QtGui.QApplication.UnicodeUTF8))
        self.txRunNo.setText(QtGui.QApplication.translate("MainWindow", "0", None, QtGui.QApplication.UnicodeUTF8))
        self.label_11.setText(QtGui.QApplication.translate("MainWindow", "FrameInfo", None, QtGui.QApplication.UnicodeUTF8))
        self.txFrameInfo.setText(QtGui.QApplication.translate("MainWindow", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.status.setText(QtGui.QApplication.translate("MainWindow", "Executable", None, QtGui.QApplication.UnicodeUTF8))
        self.st1.setText(QtGui.QApplication.translate("MainWindow", "Server  Status :", None, QtGui.QApplication.UnicodeUTF8))
        self.txRetvalLDR.setText(QtGui.QApplication.translate("MainWindow", "DAT", None, QtGui.QApplication.UnicodeUTF8))
        self.stRetValLDR.setText(QtGui.QApplication.translate("MainWindow", "Return Label :", None, QtGui.QApplication.UnicodeUTF8))
        self.btConnectServ.setText(QtGui.QApplication.translate("MainWindow", "Connect Serv", None, QtGui.QApplication.UnicodeUTF8))
        self.ckEditParams.setText(QtGui.QApplication.translate("MainWindow", "Edit Params", None, QtGui.QApplication.UnicodeUTF8))
        self.label_12.setText(QtGui.QApplication.translate("MainWindow", "Interval [sec] :", None, QtGui.QApplication.UnicodeUTF8))
        self.txIntervalSec.setText(QtGui.QApplication.translate("MainWindow", "20", None, QtGui.QApplication.UnicodeUTF8))
        self.statusLDRserver.setText(QtGui.QApplication.translate("MainWindow", "Not Connected", None, QtGui.QApplication.UnicodeUTF8))
        self.st1_2.setText(QtGui.QApplication.translate("MainWindow", "Script Status :", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("MainWindow", "LiveDataReduction", None, QtGui.QApplication.UnicodeUTF8))
        self.btLDRBegin.setText(QtGui.QApplication.translate("MainWindow", "Begin", None, QtGui.QApplication.UnicodeUTF8))
        self.btLDREnd.setText(QtGui.QApplication.translate("MainWindow", "End", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFile.setText(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'SequencerQ_Save.ui'
#
# Created: Mon Jan 15 19:09:36 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(342, 170)
        self.comment = QtGui.QTextEdit(Dialog)
        self.comment.setGeometry(QtCore.QRect(10, 40, 320, 80))
        self.comment.setObjectName("comment")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(10, 10, 261, 17))
        self.label.setObjectName("label")
        self.btOk = QtGui.QPushButton(Dialog)
        self.btOk.setGeometry(QtCore.QRect(50, 130, 100, 32))
        self.btOk.setObjectName("btOk")
        self.btCancel = QtGui.QPushButton(Dialog)
        self.btCancel.setGeometry(QtCore.QRect(180, 130, 100, 32))
        self.btCancel.setObjectName("btCancel")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Comments for the sequence :", None, QtGui.QApplication.UnicodeUTF8))
        self.btOk.setText(QtGui.QApplication.translate("Dialog", "OK", None, QtGui.QApplication.UnicodeUTF8))
        self.btCancel.setText(QtGui.QApplication.translate("Dialog", "Cancel", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'SequencerQ_Vis.ui'
#
# Created: Mon Jan 15 19:09:36 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_visframe(object):
    def setupUi(self, visframe):
        visframe.setObjectName("visframe")
        visframe.resize(558, 333)
        self.centralwidget = QtGui.QWidget(visframe)
        self.centralwidget.setObjectName("centralwidget")
        self.tree = QtGui.QTreeWidget(self.centralwidget)
        self.tree.setGeometry(QtCore.QRect(10, 10, 200, 265))
        self.tree.setHeaderHidden(True)
        self.tree.setObjectName("tree")
        self.tree.headerItem().setText(0, "1")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(220, 10, 221, 261))
        self.groupBox.setObjectName("groupBox")
        self.arg1 = QtGui.QLineEdit(self.groupBox)
        self.arg1.setGeometry(QtCore.QRect(110, 20, 100, 28))
        self.arg1.setObjectName("arg1")
        self.arg2 = QtGui.QLineEdit(self.groupBox)
        self.arg2.setGeometry(QtCore.QRect(110, 50, 100, 28))
        self.arg2.setObjectName("arg2")
        self.arg3 = QtGui.QLineEdit(self.groupBox)
        self.arg3.setGeometry(QtCore.QRect(110, 80, 100, 28))
        self.arg3.setObjectName("arg3")
        self.arg4 = QtGui.QLineEdit(self.groupBox)
        self.arg4.setGeometry(QtCore.QRect(110, 110, 100, 28))
        self.arg4.setObjectName("arg4")
        self.arg5 = QtGui.QLineEdit(self.groupBox)
        self.arg5.setGeometry(QtCore.QRect(110, 140, 100, 28))
        self.arg5.setObjectName("arg5")
        self.arg6 = QtGui.QLineEdit(self.groupBox)
        self.arg6.setGeometry(QtCore.QRect(110, 170, 100, 28))
        self.arg6.setObjectName("arg6")
        self.arg7 = QtGui.QLineEdit(self.groupBox)
        self.arg7.setGeometry(QtCore.QRect(110, 200, 100, 28))
        self.arg7.setObjectName("arg7")
        self.arg8 = QtGui.QLineEdit(self.groupBox)
        self.arg8.setGeometry(QtCore.QRect(110, 230, 100, 28))
        self.arg8.setObjectName("arg8")
        self.argst1 = QtGui.QLabel(self.groupBox)
        self.argst1.setGeometry(QtCore.QRect(0, 30, 101, 20))
        self.argst1.setObjectName("argst1")
        self.argst2 = QtGui.QLabel(self.groupBox)
        self.argst2.setGeometry(QtCore.QRect(0, 60, 101, 20))
        self.argst2.setObjectName("argst2")
        self.argst3 = QtGui.QLabel(self.groupBox)
        self.argst3.setGeometry(QtCore.QRect(0, 90, 101, 20))
        self.argst3.setObjectName("argst3")
        self.argst4 = QtGui.QLabel(self.groupBox)
        self.argst4.setGeometry(QtCore.QRect(0, 120, 101, 20))
        self.argst4.setObjectName("argst4")
        self.argst5 = QtGui.QLabel(self.groupBox)
        self.argst5.setGeometry(QtCore.QRect(0, 150, 101, 20))
        self.argst5.setObjectName("argst5")
        self.argst6 = QtGui.QLabel(self.groupBox)
        self.argst6.setGeometry(QtCore.QRect(0, 180, 101, 20))
        self.argst6.setObjectName("argst6")
        self.argst7 = QtGui.QLabel(self.groupBox)
        self.argst7.setGeometry(QtCore.QRect(0, 210, 101, 20))
        self.argst7.setObjectName("argst7")
        self.argst8 = QtGui.QLabel(self.groupBox)
        self.argst8.setGeometry(QtCore.QRect(0, 240, 101, 20))
        self.argst8.setObjectName("argst8")
        self.retval = QtGui.QLineEdit(self.centralwidget)
        self.retval.setGeometry(QtCore.QRect(450, 60, 100, 28))
        self.retval.setObjectName("retval")
        self.label = QtGui.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(450, 40, 91, 17))
        self.label.setObjectName("label")
        self.btExec = QtGui.QPushButton(self.centralwidget)
        self.btExec.setGeometry(QtCore.QRect(468, 110, 80, 32))
        self.btExec.setObjectName("btExec")
        self.btClose = QtGui.QPushButton(self.centralwidget)
        self.btClose.setGeometry(QtCore.QRect(470, 170, 80, 32))
        self.btClose.setObjectName("btClose")
        visframe.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(visframe)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 558, 25))
        self.menubar.setObjectName("menubar")
        visframe.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(visframe)
        self.statusbar.setObjectName("statusbar")
        visframe.setStatusBar(self.statusbar)

        self.retranslateUi(visframe)
        QtCore.QMetaObject.connectSlotsByName(visframe)

    def retranslateUi(self, visframe):
        visframe.setWindowTitle(QtGui.QApplication.translate("visframe", "MainWindow", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("visframe", "Arguments", None, QtGui.QApplication.UnicodeUTF8))
        self.argst1.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst2.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst3.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst4.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst5.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst6.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst7.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.argst8.setText(QtGui.QApplication.translate("visframe", "Argument", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("visframe", "Return Label", None, QtGui.QApplication.UnicodeUTF8))
        self.btExec.setText(QtGui.QApplication.translate("visframe", "Execute", None, QtGui.QApplication.UnicodeUTF8))
        self.btClose.setText(QtGui.QApplication.translate("visframe", "Close", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'SequencerQ_LDR_Serv.ui'
#
# Created: Mon Jan 15 19:09:36 2018
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_DialogServerConnect(object):
    def setupUi(self, DialogServerConnect):
        DialogServerConnect.setObjectName("DialogServerConnect")
        DialogServerConnect.resize(363, 325)
        self.groupBox = QtGui.QGroupBox(DialogServerConnect)
        self.groupBox.setGeometry(QtCore.QRect(20, 0, 331, 181))
        self.groupBox.setObjectName("groupBox")
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setGeometry(QtCore.QRect(20, 120, 91, 17))
        self.label.setObjectName("label")
        self.txServerAddress = QtGui.QLineEdit(self.groupBox)
        self.txServerAddress.setGeometry(QtCore.QRect(130, 120, 181, 27))
        self.txServerAddress.setObjectName("txServerAddress")
        self.label_2 = QtGui.QLabel(self.groupBox)
        self.label_2.setGeometry(QtCore.QRect(20, 140, 68, 17))
        self.label_2.setObjectName("label_2")
        self.txPortNo = QtGui.QLineEdit(self.groupBox)
        self.txPortNo.setGeometry(QtCore.QRect(130, 150, 113, 27))
        self.txPortNo.setObjectName("txPortNo")
        self.label_3 = QtGui.QLabel(self.groupBox)
        self.label_3.setGeometry(QtCore.QRect(20, 20, 101, 17))
        self.label_3.setObjectName("label_3")
        self.txtRunNo = QtGui.QLabel(self.groupBox)
        self.txtRunNo.setGeometry(QtCore.QRect(170, 20, 111, 17))
        self.txtRunNo.setObjectName("txtRunNo")
        self.label_5 = QtGui.QLabel(self.groupBox)
        self.label_5.setGeometry(QtCore.QRect(20, 40, 141, 17))
        self.label_5.setObjectName("label_5")
        self.txtDetectModules = QtGui.QLabel(self.groupBox)
        self.txtDetectModules.setGeometry(QtCore.QRect(50, 60, 281, 51))
        self.txtDetectModules.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.txtDetectModules.setObjectName("txtDetectModules")
        self.btConnectToServer = QtGui.QPushButton(DialogServerConnect)
        self.btConnectToServer.setGeometry(QtCore.QRect(20, 190, 131, 51))
        self.btConnectToServer.setObjectName("btConnectToServer")
        self.txtMessage = QtGui.QLabel(DialogServerConnect)
        self.txtMessage.setGeometry(QtCore.QRect(180, 190, 161, 101))
        self.txtMessage.setObjectName("txtMessage")
        self.btClose = QtGui.QPushButton(DialogServerConnect)
        self.btClose.setGeometry(QtCore.QRect(270, 290, 71, 27))
        self.btClose.setObjectName("btClose")
        self.btDisconnect = QtGui.QPushButton(DialogServerConnect)
        self.btDisconnect.setGeometry(QtCore.QRect(40, 290, 99, 27))
        self.btDisconnect.setObjectName("btDisconnect")

        self.retranslateUi(DialogServerConnect)
        QtCore.QMetaObject.connectSlotsByName(DialogServerConnect)

    def retranslateUi(self, DialogServerConnect):
        DialogServerConnect.setWindowTitle(QtGui.QApplication.translate("DialogServerConnect", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("DialogServerConnect", "Server Information", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("DialogServerConnect", "IPAddress : ", None, QtGui.QApplication.UnicodeUTF8))
        self.txServerAddress.setText(QtGui.QApplication.translate("DialogServerConnect", "localhost", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("DialogServerConnect", "Port No :", None, QtGui.QApplication.UnicodeUTF8))
        self.txPortNo.setText(QtGui.QApplication.translate("DialogServerConnect", "2223", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("DialogServerConnect", "Run Number : ", None, QtGui.QApplication.UnicodeUTF8))
        self.txtRunNo.setText(QtGui.QApplication.translate("DialogServerConnect", "XXX000000", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("DialogServerConnect", "Detector Modules :", None, QtGui.QApplication.UnicodeUTF8))
        self.txtDetectModules.setText(QtGui.QApplication.translate("DialogServerConnect", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.btConnectToServer.setText(QtGui.QApplication.translate("DialogServerConnect", "Connect", None, QtGui.QApplication.UnicodeUTF8))
        self.txtMessage.setText(QtGui.QApplication.translate("DialogServerConnect", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.btClose.setText(QtGui.QApplication.translate("DialogServerConnect", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.btDisconnect.setText(QtGui.QApplication.translate("DialogServerConnect", "Disconnect", None, QtGui.QApplication.UnicodeUTF8))

