# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'VisContQ_Main.ui'
#
# Created: Fri Jun 12 18:58:24 2020
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(574, 683)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.btApply = QtGui.QPushButton(self.centralwidget)
        self.btApply.setGeometry(QtCore.QRect(220, 10, 211, 31))
        self.btApply.setObjectName("btApply")
        self.btOpperate = QtGui.QPushButton(self.centralwidget)
        self.btOpperate.setGeometry(QtCore.QRect(460, 10, 99, 27))
        self.btOpperate.setObjectName("btOpperate")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setGeometry(QtCore.QRect(0, 50, 571, 586))
        self.tabWidget.setObjectName("tabWidget")
        self.splash_label = QtGui.QLabel(self.centralwidget)
        self.splash_label.setGeometry(QtCore.QRect(5, 5, 200, 40))
        self.splash_label.setText("")
        self.splash_label.setObjectName("splash_label")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 574, 25))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.menu_load = QtGui.QAction(MainWindow)
        self.menu_load.setObjectName("menu_load")
        self.menu_legacy = QtGui.QAction(MainWindow)
        self.menu_legacy.setObjectName("menu_legacy")
        self.actionSave_as = QtGui.QAction(MainWindow)
        self.actionSave_as.setObjectName("actionSave_as")
        self.menu_load_qxqyqz_text = QtGui.QAction(MainWindow)
        self.menu_load_qxqyqz_text.setObjectName("menu_load_qxqyqz_text")
        self.menu_setting = QtGui.QAction(MainWindow)
        self.menu_setting.setObjectName("menu_setting")
        self.menu_exit = QtGui.QAction(MainWindow)
        self.menu_exit.setObjectName("menu_exit")
        self.menuFile.addAction(self.menu_load)
        self.menuFile.addAction(self.menu_legacy)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addAction(self.menu_load_qxqyqz_text)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menu_setting)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menu_exit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "MainWindow", None, QtGui.QApplication.UnicodeUTF8))
        self.btApply.setText(QtGui.QApplication.translate("MainWindow", "Apply params to other Tabs", None, QtGui.QApplication.UnicodeUTF8))
        self.btOpperate.setText(QtGui.QApplication.translate("MainWindow", "CalcBar", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "File...", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_load.setText(QtGui.QApplication.translate("MainWindow", "Load", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_legacy.setText(QtGui.QApplication.translate("MainWindow", "Load Legacy...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as.setText(QtGui.QApplication.translate("MainWindow", "Save as ...", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_load_qxqyqz_text.setText(QtGui.QApplication.translate("MainWindow", "Load QxQyQz..", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_setting.setText(QtGui.QApplication.translate("MainWindow", "Setting ...", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_exit.setText(QtGui.QApplication.translate("MainWindow", "Exit", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'VisContQ_DataOperationDialog.ui'
#
# Created: Fri Jun 12 18:58:24 2020
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_DataOperationDialog(object):
    def setupUi(self, DataOperationDialog):
        DataOperationDialog.setObjectName("DataOperationDialog")
        DataOperationDialog.resize(560, 160)
        self.groupBox = QtGui.QGroupBox(DataOperationDialog)
        self.groupBox.setGeometry(QtCore.QRect(20, 10, 521, 101))
        self.groupBox.setObjectName("groupBox")
        self.ckPr = QtGui.QCheckBox(self.groupBox)
        self.ckPr.setGeometry(QtCore.QRect(10, 30, 201, 22))
        self.ckPr.setObjectName("ckPr")
        self.cb0 = QtGui.QComboBox(self.groupBox)
        self.cb0.setGeometry(QtCore.QRect(10, 60, 40, 26))
        self.cb0.setObjectName("cb0")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.cb0.addItem("")
        self.stEQ = QtGui.QLabel(self.groupBox)
        self.stEQ.setGeometry(QtCore.QRect(60, 60, 21, 26))
        self.stEQ.setObjectName("stEQ")
        self.m1 = QtGui.QLineEdit(self.groupBox)
        self.m1.setGeometry(QtCore.QRect(80, 60, 50, 26))
        self.m1.setObjectName("m1")
        self.asta1 = QtGui.QLabel(self.groupBox)
        self.asta1.setGeometry(QtCore.QRect(140, 60, 21, 26))
        self.asta1.setObjectName("asta1")
        self.cb1 = QtGui.QComboBox(self.groupBox)
        self.cb1.setGeometry(QtCore.QRect(160, 60, 40, 26))
        self.cb1.setObjectName("cb1")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cb1.addItem("")
        self.cbOP = QtGui.QComboBox(self.groupBox)
        self.cbOP.setGeometry(QtCore.QRect(220, 60, 40, 26))
        self.cbOP.setObjectName("cbOP")
        self.cbOP.addItem("")
        self.cbOP.addItem("")
        self.cbOP.addItem("")
        self.cbOP.setItemText(2, "")
        self.asta2 = QtGui.QLabel(self.groupBox)
        self.asta2.setGeometry(QtCore.QRect(340, 60, 21, 26))
        self.asta2.setObjectName("asta2")
        self.cb2 = QtGui.QComboBox(self.groupBox)
        self.cb2.setGeometry(QtCore.QRect(360, 60, 40, 26))
        self.cb2.setObjectName("cb2")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.cb2.addItem("")
        self.m2 = QtGui.QLineEdit(self.groupBox)
        self.m2.setGeometry(QtCore.QRect(280, 60, 50, 26))
        self.m2.setObjectName("m2")
        self.btCalc = QtGui.QPushButton(self.groupBox)
        self.btCalc.setGeometry(QtCore.QRect(420, 60, 91, 27))
        self.btCalc.setObjectName("btCalc")
        self.btClose = QtGui.QPushButton(DataOperationDialog)
        self.btClose.setGeometry(QtCore.QRect(210, 120, 99, 27))
        self.btClose.setObjectName("btClose")

        self.retranslateUi(DataOperationDialog)
        QtCore.QMetaObject.connectSlotsByName(DataOperationDialog)
        DataOperationDialog.setTabOrder(self.ckPr, self.cb0)
        DataOperationDialog.setTabOrder(self.cb0, self.m1)
        DataOperationDialog.setTabOrder(self.m1, self.cb1)
        DataOperationDialog.setTabOrder(self.cb1, self.cbOP)
        DataOperationDialog.setTabOrder(self.cbOP, self.m2)
        DataOperationDialog.setTabOrder(self.m2, self.cb2)
        DataOperationDialog.setTabOrder(self.cb2, self.btCalc)
        DataOperationDialog.setTabOrder(self.btCalc, self.btClose)

    def retranslateUi(self, DataOperationDialog):
        DataOperationDialog.setWindowTitle(QtGui.QApplication.translate("DataOperationDialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("DataOperationDialog", "GroupBox", None, QtGui.QApplication.UnicodeUTF8))
        self.ckPr.setText(QtGui.QApplication.translate("DataOperationDialog", "Calculate Slicing Dta", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(0, QtGui.QApplication.translate("DataOperationDialog", "A", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(1, QtGui.QApplication.translate("DataOperationDialog", "B", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(2, QtGui.QApplication.translate("DataOperationDialog", "C", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(3, QtGui.QApplication.translate("DataOperationDialog", "D", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(4, QtGui.QApplication.translate("DataOperationDialog", "E", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(5, QtGui.QApplication.translate("DataOperationDialog", "F", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(6, QtGui.QApplication.translate("DataOperationDialog", "G", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(7, QtGui.QApplication.translate("DataOperationDialog", "H", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(8, QtGui.QApplication.translate("DataOperationDialog", "I", None, QtGui.QApplication.UnicodeUTF8))
        self.cb0.setItemText(9, QtGui.QApplication.translate("DataOperationDialog", "J", None, QtGui.QApplication.UnicodeUTF8))
        self.stEQ.setText(QtGui.QApplication.translate("DataOperationDialog", "=", None, QtGui.QApplication.UnicodeUTF8))
        self.asta1.setText(QtGui.QApplication.translate("DataOperationDialog", "*", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(0, QtGui.QApplication.translate("DataOperationDialog", "A", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(1, QtGui.QApplication.translate("DataOperationDialog", "B", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(2, QtGui.QApplication.translate("DataOperationDialog", "C", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(3, QtGui.QApplication.translate("DataOperationDialog", "D", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(4, QtGui.QApplication.translate("DataOperationDialog", "E", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(5, QtGui.QApplication.translate("DataOperationDialog", "F", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(6, QtGui.QApplication.translate("DataOperationDialog", "G", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(7, QtGui.QApplication.translate("DataOperationDialog", "H", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(8, QtGui.QApplication.translate("DataOperationDialog", "I", None, QtGui.QApplication.UnicodeUTF8))
        self.cb1.setItemText(9, QtGui.QApplication.translate("DataOperationDialog", "J", None, QtGui.QApplication.UnicodeUTF8))
        self.cbOP.setItemText(0, QtGui.QApplication.translate("DataOperationDialog", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.cbOP.setItemText(1, QtGui.QApplication.translate("DataOperationDialog", "+", None, QtGui.QApplication.UnicodeUTF8))
        self.asta2.setText(QtGui.QApplication.translate("DataOperationDialog", "*", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(0, QtGui.QApplication.translate("DataOperationDialog", "A", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(1, QtGui.QApplication.translate("DataOperationDialog", "B", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(2, QtGui.QApplication.translate("DataOperationDialog", "C", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(3, QtGui.QApplication.translate("DataOperationDialog", "D", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(4, QtGui.QApplication.translate("DataOperationDialog", "E", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(5, QtGui.QApplication.translate("DataOperationDialog", "F", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(6, QtGui.QApplication.translate("DataOperationDialog", "G", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(7, QtGui.QApplication.translate("DataOperationDialog", "H", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(8, QtGui.QApplication.translate("DataOperationDialog", "I", None, QtGui.QApplication.UnicodeUTF8))
        self.cb2.setItemText(9, QtGui.QApplication.translate("DataOperationDialog", "J", None, QtGui.QApplication.UnicodeUTF8))
        self.btCalc.setText(QtGui.QApplication.translate("DataOperationDialog", "Calculate", None, QtGui.QApplication.UnicodeUTF8))
        self.btClose.setText(QtGui.QApplication.translate("DataOperationDialog", "Close", None, QtGui.QApplication.UnicodeUTF8))


# Form implementation generated from reading ui file 'VisContQ_SettingDialog.ui'
#
# Created: Fri Jun 12 18:58:24 2020
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!


class Ui_SettingDialog(object):
    def setupUi(self, SettingDialog):
        SettingDialog.setObjectName("SettingDialog")
        SettingDialog.resize(400, 85)
        self.groupBox = QtGui.QGroupBox(SettingDialog)
        self.groupBox.setGeometry(QtCore.QRect(0, 0, 371, 80))
        self.groupBox.setObjectName("groupBox")
        self.ckSliceAverageMode = QtGui.QCheckBox(self.groupBox)
        self.ckSliceAverageMode.setGeometry(QtCore.QRect(20, 30, 371, 22))
        self.ckSliceAverageMode.setObjectName("ckSliceAverageMode")

        self.retranslateUi(SettingDialog)
        QtCore.QMetaObject.connectSlotsByName(SettingDialog)

    def retranslateUi(self, SettingDialog):
        SettingDialog.setWindowTitle(QtGui.QApplication.translate("SettingDialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("SettingDialog", "Slice", None, QtGui.QApplication.UnicodeUTF8))
        self.ckSliceAverageMode.setText(QtGui.QApplication.translate("SettingDialog", "Average Mode    On : Average  / Off : Sum", None, QtGui.QApplication.UnicodeUTF8))

