# -*- coding: utf-8 -*-

import sys
import string
import math

import Manyo.core
import ModuleTest as ModTest

ec0 = ModTest.ec0

ne1 = Manyo.NeXusFileIO()
ne1.Write( ec0, "test.nx", "suzuki", 0 )
del ne1


ne2 = Manyo.NeXusFileIO()
e2 = ne2.ReadElementContainer( "test.nx" )
del ne2

#module.PrintElementContainer( e2 )
