#include "UtsusemiWiringInfoEditorBase.hh"
//////////////////////////////////////////////////////////
UtsusemiWiringInfoEditorBase::UtsusemiWiringInfoEditorBase(){
    Initialize();
}
//////////////////////////////////////////////////////////
UtsusemiWiringInfoEditorBase::~UtsusemiWiringInfoEditorBase(){
}
//////////////////////////////////////////////////////////
void UtsusemiWiringInfoEditorBase::Initialize(){
    _MessageTag = "UtsusemiWiringInfoEditorBase::";
}
