#!/bin/zsh

cd source/Utsusemi/manyo

export UTSUSEMI_REV=`git log -1 --pretty=format:'%cd' --date=format:"%y%m%d"`

make clean -f Makefile.mac
make uninstall -f Makefile.mac
chmod +x depend.mac
./depend.mac
make install -f Makefile.mac NUM_MULTH=2 CXX=clang++
